@@ -149,9 +149,6 @@ static void throw_out_all_chest_content(int obst_index)
 		}
 
 		// Empty the item list
-		for (i = 0; i < drop_count; i++) {
-			dynarray_del(item_list, 0, sizeof(item));
-		}
 		dynarray_free(item_list);
 
 		// Remove the chest items obstacle extension
@@ -161,7 +158,7 @@ static void throw_out_all_chest_content(int obst_index)
 	// If the chest was empty, maybe generate a random item to be dropped
 	if (!drop_count) {
 		find_dropable_position_near_chest(&item_x, &item_y, obst_index, lvl);
-		DropRandomItem(Me.pos.z, item_x, item_y, 0, FALSE);
+		DropRandomItem(Me.pos.z, item_x, item_y, lvl->drop_class, FALSE);
 	}
 }
 
@@ -199,7 +196,7 @@ static int mouse_cursor_is_on_that_obstacle(level *lvl, int obst_index)
  * If there is an obstacle below the mouse cursor, the function will
  * return its obstacle index.  Else -1 will be returned.
  */
-int clickable_obstacle_below_mouse_cursor(level **obst_lvl)
+int clickable_obstacle_below_mouse_cursor(level **obst_lvl, int clickable_only)
 {
 #define HOVER_CHECK_DIST 2
 #define SIDE_LENGTH (HOVER_CHECK_DIST * 2 + 1)
@@ -244,7 +241,7 @@ int clickable_obstacle_below_mouse_cursor(level **obst_lvl)
 			obst_normal = lvl->obstacle_list[obst_index].pos.x + lvl->obstacle_list[obst_index].pos.y;
 
 			if (obst_normal > max_normal &&
-				get_obstacle_spec(lvl->obstacle_list[obst_index].type)->flags & IS_CLICKABLE &&
+				(!clickable_only || (get_obstacle_spec(lvl->obstacle_list[obst_index].type)->flags & IS_CLICKABLE)) &&
 				mouse_cursor_is_on_that_obstacle(lvl, obst_index)) {
 
 				max_normal = obst_normal;
@@ -431,7 +428,7 @@ static int reach_obstacle_from_specific_direction(level *obst_lvl, int obst_inde
 		Me.mouse_move_target.y -= spec->block_area_parm_2;
 		break;
 	default:
-		ErrorMessage(__FUNCTION__, "Invalid direction!!", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "Invalid direction!!", PLEASE_INFORM | IS_FATAL);
 		break;
 	}
     return 0;
@@ -462,6 +459,10 @@ static int approach(int type)
 			{ ISO_CONSOLE_S, NORTH },
 			{ ISO_CONSOLE_E, WEST },
 			{ ISO_CONSOLE_W, EAST },
+		        { ISO_CONSOLE_SECURE_E, EAST },
+		        { ISO_CONSOLE_SECURE_S, SOUTH },
+		        { ISO_CONSOLE_SECURE_W, WEST },
+        		{ ISO_CONSOLE_SECURE_N, NORTH },
 			{ ISO_WALL_TERMINAL_S, SOUTH},
 			{ ISO_WALL_TERMINAL_E, EAST},
 			{ ISO_WALL_TERMINAL_N, NORTH},
@@ -472,6 +473,26 @@ static int approach(int type)
 			{ ISO_SIGN_2_FLASH, SOUTH },
 			{ ISO_SIGN_3, EAST },
 			{ ISO_SIGN_3_FLASH, EAST },
+			{ ISO_VENDING_MACHINE_1_E, EAST },
+			{ ISO_VENDING_MACHINE_1_N, NORTH },
+			{ ISO_VENDING_MACHINE_1_W, WEST },
+			{ ISO_VENDING_MACHINE_1_S, SOUTH },
+			{ ISO_VENDING_MACHINE_2_E, EAST },
+			{ ISO_VENDING_MACHINE_2_N, NORTH },
+			{ ISO_VENDING_MACHINE_2_W, WEST },
+			{ ISO_VENDING_MACHINE_2_S, SOUTH },
+			{ ISO_VENDING_MACHINE_3_E, EAST },
+			{ ISO_VENDING_MACHINE_3_N, NORTH },
+			{ ISO_VENDING_MACHINE_3_W, WEST },
+			{ ISO_VENDING_MACHINE_3_S, SOUTH },
+			{ ISO_BOOKSHELF_LOOTABLE_E, EAST },
+			{ ISO_BOOKSHELF_LOOTABLE_S, SOUTH },
+			{ ISO_BOOKSHELF_LOOTABLE_W, WEST },
+			{ ISO_BOOKSHELF_LOOTABLE_N, NORTH },
+			{ ISO_BOOKSHELF_LONG_LOOTABLE_E, EAST },
+			{ ISO_BOOKSHELF_LONG_LOOTABLE_S, SOUTH },
+			{ ISO_BOOKSHELF_LONG_LOOTABLE_W, WEST },
+			{ ISO_BOOKSHELF_LONG_LOOTABLE_N, NORTH },
 	};
 
 	for (i = 0; i < sizeof(lookup)/sizeof(lookup[0]); i++) {
@@ -480,7 +501,7 @@ static int approach(int type)
 	}
 
 
-	ErrorMessage(__FUNCTION__, "Tried to approach obstacle type %d, but this is not a known type.", PLEASE_INFORM, IS_WARNING_ONLY, type);
+	error_message(__FUNCTION__, "Tried to approach obstacle type %d, but this is not a known type.", PLEASE_INFORM, type);
 
 	return UNDEFINED;
 }
@@ -489,18 +510,12 @@ static void act_chest(level *l, obstacle *o)
 {
 	throw_out_all_chest_content(get_obstacle_index(l, o));
 
-	if (o->type == ISO_H_CHEST_CLOSED)
-		o->type = ISO_H_CHEST_OPEN;
-	if (o->type == ISO_V_CHEST_CLOSED)
-		o->type = ISO_V_CHEST_OPEN;
-	if (o->type == ISO_N_CHEST2_CLOSED)
-		o->type = ISO_N_CHEST2_OPEN;
-	if (o->type == ISO_E_CHEST2_CLOSED)
-		o->type = ISO_E_CHEST2_OPEN;
-	if (o->type == ISO_S_CHEST2_CLOSED)
-		o->type = ISO_S_CHEST2_OPEN;
-	if (o->type == ISO_W_CHEST2_CLOSED)
-		o->type = ISO_W_CHEST2_OPEN;
+	// After emptying the chest we change it to its corresponding "open" type.
+	// If such a type was not defined, the chest will stay "closed" and can be
+	// looted indefinitely.
+	int new_type = get_obstacle_spec(o->type)->result_type_after_looting;
+	if (new_type != -1)
+		o->type = new_type;
 }
 
 static void act_barrel(level *l, obstacle *o)
@@ -518,26 +533,30 @@ static void act_terminal(level *l, obstacle *o)
 {
 	char *dialog = get_obstacle_extension(l, o, OBSTACLE_EXTENSION_DIALOGFILE);
 	if (!dialog) {
-		append_new_game_message("Colored patterns appear on the screen, but they do not look like any computer interface you have ever seen. Perhaps is this what they call a screen \"saver\".");
+		append_new_game_message(_("Colored patterns appear on the screen, but they do not look like any computer interface you have ever seen. Perhaps is this what they call a screen \"saver\"."));
 		return;
 	}
 
 	enemy dummy_enemy;
-	dummy_enemy.type = get_droid_type("TRM");
+	// There are currently two terminal "droids", with different in-dialog images.
+	if ((o->type >= ISO_CONSOLE_SECURE_E) && (o->type <= ISO_CONSOLE_SECURE_N))
+		dummy_enemy.type = get_droid_type("STM");
+	else
+		dummy_enemy.type = get_droid_type("TRM");
 	enemy_reset(&dummy_enemy);
 	dummy_enemy.dialog_section_name = dialog;
 	dummy_enemy.will_rush_tux = FALSE;
-	ChatWithFriendlyDroid(&dummy_enemy);
+	chat_with_droid(&dummy_enemy);
 }
 
 static void act_sign(level *l, obstacle *o)
 {
 	const char *message = get_obstacle_extension(l, o, OBSTACLE_EXTENSION_SIGNMESSAGE);
 	if (!message) {
-		message = "There is nothing on this sign.";
+		message = _("There is nothing on this sign.");
 	}
 
-	append_new_game_message("%s", message);
+	append_new_game_message("%s", D_(message));
 }
 
 static void __obstacle_action(level *lvl, int index, enum interactive_obstacle_type act)
@@ -594,22 +613,25 @@ static void __obstacle_action(level *lvl, int index, enum interactive_obstacle_t
 	colldet_filter filter = ObstacleByIdPassFilter;
 	filter.data = &index;
 
-	if (DirectLineColldet(Me.pos.x, Me.pos.y, vpos.x, vpos.y, Me.pos.z, &filter)) {
-		// Do the specific action
-		switch (act) {
-			case ACT_CHEST:
-				act_chest(lvl, o);
-				break;
-			case ACT_TERMINAL:
-				act_terminal(lvl, o);
-				break;
-			case ACT_SIGN:
-				act_sign(lvl, o);
-				break;
-			case ACT_BARREL:
-				act_barrel(lvl, o);
-				break;
-		}
+	if (!DirectLineColldet(Me.pos.x, Me.pos.y, vpos.x, vpos.y, Me.pos.z, &filter))
+		return;
+
+	event_obstacle_action(o);
+
+	// Do the specific action
+	switch (act) {
+		case ACT_CHEST:
+			act_chest(lvl, o);
+			break;
+		case ACT_TERMINAL:
+			act_terminal(lvl, o);
+			break;
+		case ACT_SIGN:
+			act_sign(lvl, o);
+			break;
+		case ACT_BARREL:
+			act_barrel(lvl, o);
+			break;
 	}
 }
 
@@ -712,7 +734,7 @@ action_fptr get_action_by_name(const char *action_name)
 			return action_map[i].action;
 	}
 
-	ErrorMessage(__FUNCTION__, "\nUnknown obstacle action '%s'.\n", PLEASE_INFORM, IS_FATAL, action_name);
+	error_message(__FUNCTION__, "\nUnknown obstacle action '%s'.", PLEASE_INFORM | IS_FATAL, action_name);
 	return NULL;
 }
 
