@@ -94,7 +94,7 @@ static void select_recipe(int index)
 	// Copy the item description to the text widget.
 	struct auto_string *desc = ui.description.text;
 	type = arr[index].item_type;
-	autostr_printf(desc, "%s", ItemMap[type].item_description);
+	autostr_printf(desc, "%s", D_(ItemMap[type].item_description));
 
 	// Append the add-on tooltip to the text widget.
 	autostr_append(desc, "\n\n%s%s%s\n", font_switchto_msgstat, _("Features:"), font_switchto_msgvar);
@@ -103,7 +103,7 @@ static void select_recipe(int index)
 	//clean the array
 	memset(&ui.materials_for_selected, 0, sizeof(struct material) * 5);
 	for (i = 0; i < spec->materials.size; i++) {
-		ui.materials_for_selected[i].item_type = GetItemIndexByName(materials[i].name);
+		ui.materials_for_selected[i].item_type = get_item_type_by_id(materials[i].name);
 		ui.materials_for_selected[i].required = materials[i].value;
 		ui.materials_for_selected[i].available = CountItemtypeInInventory(ui.materials_for_selected[i].item_type);
 	}
@@ -130,7 +130,7 @@ static void check_recipe_requirements()
 
 		// Check if the player has enough materials.
 		for (j = 0; j < spec->materials.size; j++) {
-			int type = GetItemIndexByName(materials[j].name);
+			int type = get_item_type_by_id(materials[j].name);
 			int count = CountItemtypeInInventory(type);
 			if (count < materials[j].value) {
 				recipes[i].available = FALSE;
@@ -153,14 +153,14 @@ static void craft_item()
 
 		// Subtract materials.
 		for (i = 0; i < spec->materials.size; i++) {
-			int type = GetItemIndexByName(materials[i].name);
+			int type = get_item_type_by_id(materials[i].name);
 			int count = materials[i].value;
 			DeleteInventoryItemsOfType(type, count);
 		}
 
 		// Create the item and add it to the inventory.
 		int type = arr[ui.selection].item_type;
-		it = create_item_with_name(ItemMap[type].item_name, TRUE, 1);
+		it = create_item_with_id(ItemMap[type].id, TRUE, 1);
 		give_item(&it);
 
 		// The player lost some materials so some of the recipes might have become
@@ -255,16 +255,16 @@ void show_addon_crafting_ui()
 	blit_background("item_upgrade_crafting.png");
 
 	// Draw the title.
-	SetCurrentFont(Menu_BFont);
-	display_text(_("Craft Addons"), rects.title_text.x, rects.title_text.y, NULL);
+	set_current_font(Menu_Font);
+	display_text(_("Craft Addons"), rects.title_text.x, rects.title_text.y, NULL, 1.0);
 
 	// Draw the details string.
-	SetCurrentFont(Blue_BFont);
-	display_text(_("Details"), rects.details_text.x, rects.details_text.y, NULL);
+	set_current_font(Blue_Font);
+	display_text(_("Details"), rects.details_text.x, rects.details_text.y, NULL, 1.0);
 
 	// Draw the parts text
-	SetCurrentFont(Blue_BFont);
-	display_text(_("Parts"), rects.materials_list.x, rects.materials_list.y, NULL);
+	set_current_font(Blue_Font);
+	display_text(_("Parts"), rects.materials_list.x, rects.materials_list.y, NULL, 1.0);
 
 	// Draw the apply and close buttons.
 	if (arr[ui.selection].available) {
@@ -285,12 +285,12 @@ void show_addon_crafting_ui()
 			HighlightRectangle(Screen, rect);
 		}
 		if (arr[i].available) {
-			SetCurrentFont(Blue_BFont);
+			set_current_font(Blue_Font);
 		} else {
-			SetCurrentFont(Red_BFont);
+			set_current_font(Red_Font);
 		}
 		int type = arr[i].item_type;
-		display_text(ItemMap[type].item_name, rect.x + rect.h, rect.y + 4, NULL);
+		display_text(D_(item_specs_get_name(type)), rect.x + rect.h, rect.y + 4, NULL, 1.0);
 		struct image *img = get_item_inventory_image(type);
 		if (img) {
 			float scale = (float)(RECIPE_LIST_IMG_WIDTH + RECIPE_LIST_IMG_HEIGHT) / (img->w + img->h);
@@ -334,18 +334,18 @@ void show_addon_crafting_ui()
 
 		//first the divisor  (11 part in the diagram)
 		sprintf( text ,"%s%02d", font_switchto_neon, ui.materials_for_selected[i].required);
-		int w_text = TextWidth(text);
+		int w_text = text_width(get_current_font(), text);
 		x = rect.x + rect.w  // the right border
 			 - WIDTH_BORDER // the blue border
 			 - w_text; // right justified
-		display_text(text, x, y, NULL);
+		display_text(text, x, y, NULL, 1.0);
 
 		//then the '/' center
-		w_text = TextWidth("/");
+		w_text = text_width(get_current_font(), "/");
 		int half = w_text / 2;
 		x = rect.x + (rect.w / 2) // center of the column
 			 - half; // center placed
-		display_text("/", x, y, NULL);
+		display_text("/", x, y, NULL, 1.0);
 
 		//last dividend (00 part in the diagram)
 		if (ui.materials_for_selected[i].available < ui.materials_for_selected[i].required) {
@@ -356,11 +356,11 @@ void show_addon_crafting_ui()
 			else
 				sprintf( text ,"%s%02d", font_switchto_neon, ui.materials_for_selected[i].available);
 		}
-		w_text = TextWidth(text);
+		w_text = text_width(get_current_font(), text);
 		x = rect.x + (rect.w / 2) // center of the column
 			 - half // the part used by /
 			 - w_text; // right justified
-		display_text(text, x , y , NULL);
+		display_text(text, x , y , NULL, 1.0);
 
 		rect.y += MARGIN_SPACE + img->h + 20;
 	}
@@ -465,7 +465,7 @@ void addon_crafting_ui()
 	// Clear the struct and build the recipe list.
 	memset(&ui, 0, sizeof(ui));
 	widget_text_init(&ui.description, "");
-	ui.description.font = Messagevar_BFont;
+	ui.description.font = Messagevar_Font;
 	WIDGET(&ui.description)->rect = rects.recipe_desc;
 	ui.description.content_above_func = draw_scroll_desc_up_button;
 	ui.description.content_below_func = draw_scroll_desc_down_button;
