@@ -35,7 +35,7 @@
 #include "global.h"
 #include "proto.h"
 
-/* Distances for hitting a druid */
+/* Distances for hitting a droid */
 #define DROIDHITDIST  (0.25)
 #define DROIDHITDIST2 (DROIDHITDIST*DROIDHITDIST)
 
@@ -98,9 +98,7 @@ static void move_this_bullet_and_check_its_collisions(int num)
 
 /**
  * Whenever a new bullet is generated, we need to find a free index in 
- * the array of bullets.  This function automates the process and 
- * also is secure against too many bullets in the game (with a rather
- * ungraceful exit, but that event shouldn't ever occur in a normal game.
+ * the array of bullets.  This function automates the process.
  */
 int find_free_melee_shot_index(void)
 {
@@ -109,16 +107,16 @@ int find_free_melee_shot_index(void)
 	for (j = 0; j < MAX_MELEE_SHOTS; j++) {
 		if (AllMeleeShots[j].attack_target_type == ATTACK_TARGET_IS_NOTHING) {
 			return (j);
-			break;
 		}
 	}
 
-	ErrorMessage(__FUNCTION__, "\
-I seem to have run out of free melee shot entries.", PLEASE_INFORM, IS_WARNING_ONLY);
+	// TODO use a dynarray rather than a static array, to avoid amount limitation
+	error_once_message(ONCE_PER_GAME, __FUNCTION__,
+	                   "I seem to have run out of free melee shot entries.",
+	                   PLEASE_INFORM);
 
-	return (0);
-
-};				// void find_free_bullet_entry_pointer ( void )
+	return -1;
+}
 
 void delete_melee_shot(melee_shot * t)
 {
@@ -149,9 +147,9 @@ void DoMeleeDamage(void)
 			/* Attack an enemy */
 			enemy *tg = enemy_resolve_address(CurMelS->bot_target_n, &CurMelS->bot_target_addr);
 			if (!tg) {
-				ErrorMessage(__FUNCTION__,
-					     "Melee shot was set to ATTACK_TARGET_IS_ENEMY but had no targeted enemy. Deleting.\n",
-					     NO_NEED_TO_INFORM, IS_WARNING_ONLY);
+				error_message(__FUNCTION__,
+					     "Melee shot was set to ATTACK_TARGET_IS_ENEMY but had no targeted enemy. Deleting.",
+					     NO_REPORT);
 				delete_melee_shot(CurMelS);
 				continue;
 			}
@@ -175,7 +173,7 @@ void DoMeleeDamage(void)
 			/* hit player */
 			if (MyRandom(100) < CurMelS->to_hit) {
 				float real_damage = CurMelS->damage * get_player_damage_factor();
-				hit_tux(real_damage, CurMelS->owner);
+				hit_tux(real_damage);
 				DamageProtectiveEquipment();
 			}
 		}
@@ -202,9 +200,6 @@ void MoveBullets(void)
 		if (CurBullet->type == INFOUT)
 			continue;
 
-		if (CurBullet->time_to_hide_still > 0)
-			continue;
-
 		move_this_bullet_and_check_its_collisions(i);
 
 		// WARNING!  The bullet collision check might have deleted the bullet, so 
@@ -240,8 +235,10 @@ void DeleteBullet(int Bulletnumber, int ShallWeStartABlast)
 	// cause later, after the bullet is deleted, it will be hard to know
 	// the correct location ;)
 	//
-	if (ShallWeStartABlast)
-		StartBlast(CurBullet->pos.x, CurBullet->pos.y, CurBullet->pos.z, Bulletmap[CurBullet->type].blast_type, CurBullet->damage, CurBullet->faction);
+	if (ShallWeStartABlast) {
+		struct bulletspec *bullet_spec = dynarray_member(&bullet_specs, CurBullet->type, sizeof(struct bulletspec));
+		StartBlast(CurBullet->pos.x, CurBullet->pos.y, CurBullet->pos.z, bullet_spec->blast_type, CurBullet->damage, CurBullet->faction, NULL);
+	}
 
 	CurBullet->type = INFOUT;
 	CurBullet->time_in_seconds = 0;
@@ -263,26 +260,28 @@ void DeleteBullet(int Bulletnumber, int ShallWeStartABlast)
  * DROIDBLAST,       (explosion of a dying droid)
  * OWNBLAST          (not implemented)
  *
+ * StartBlast will either use sound_name if passed, or if NULL will use
+ * the default sound for the blast. 
  */
-void StartBlast(float x, float y, int level, int type, int dmg, int faction)
+void StartBlast(float x, float y, int lvl, int type, int dmg, int faction, char *sound_name)
 {
 	int i;
 	blast *NewBlast;
 
 	// Resolve blast real position, if possible
 	//
-	gps blast_vpos = { x, y, level };
+	gps blast_vpos = { x, y, lvl };
 	gps blast_pos;
 
 	if (!resolve_virtual_position(&blast_pos, &blast_vpos)) {
 		// The blast position is nowhere....
-		ErrorMessage(__FUNCTION__, "\
+		error_message(__FUNCTION__, "\
 A BLAST VIRTUAL POSITION WAS FOUND TO BE INCONSISTENT.\n\
 \n\
 However, the error is not fatal and will be silently compensated for now.\n\
 When reporting a problem to the FreedroidRPG developers, please note if this\n\
 warning message was created prior to the error in your report.\n\
-However, it should NOT cause any serious trouble for FreedroidRPG.", NO_NEED_TO_INFORM, IS_WARNING_ONLY);
+However, it should NOT cause any serious trouble for FreedroidRPG.", NO_REPORT);
 		return;
 	}
 	// If the blast is not on a visible level, we do not take care of it
@@ -318,7 +317,11 @@ However, it should NOT cause any serious trouble for FreedroidRPG.", NO_NEED_TO_
 
 	NewBlast->faction = faction;
 
-	play_blast_sound(type, &NewBlast->pos);
+	if(sound_name) {
+		play_blast_sound(sound_name, &NewBlast->pos);
+	} else {
+		play_blast_sound(Blastmap[type].sound_file, &NewBlast->pos);
+	}
 }
 
 /**
@@ -338,14 +341,14 @@ void animate_blasts(void)
 			// Therefore we introduce some extra security here...
 			//
 			if (!pos_inside_level(CurBlast->pos.x, CurBlast->pos.y, curShip.AllLevels[CurBlast->pos.z])) {
-				ErrorMessage(__FUNCTION__, "\
+				error_message(__FUNCTION__, "\
 A BLAST WAS FOUND TO EXIST OUTSIDE THE BOUNDS OF THE MAP.\n\
 This is an indication of an inconsistency in FreedroidRPG.\n\
 \n\
 However, the error is not fatal and will be silently compensated for now.\n\
 When reporting a problem to the FreedroidRPG developers, please note if this\n\
 warning message was created prior to the error in your report.\n\
-However, it should NOT cause any serious trouble for FreedroidRPG.", NO_NEED_TO_INFORM, IS_WARNING_ONLY);
+However, it should NOT cause any serious trouble for FreedroidRPG.", NO_REPORT);
 				CurBlast->pos.x = 0;
 				CurBlast->pos.y = 0;
 				CurBlast->pos.z = 0;
@@ -409,7 +412,7 @@ void MoveActiveSpells(void)
 
 		// We do some collision checking with the obstacles in each
 		// 'active_direction' of the spell and deactivate those directions,
-		// where some collision with solid material has happend.
+		// where some collision with solid material has happened.
 		//
 		for (direction_index = 0; direction_index < RADIAL_SPELL_DIRECTIONS; direction_index++) {
 			if (AllActiveSpells[i].active_directions[direction_index] == FALSE)
@@ -456,8 +459,8 @@ void MoveActiveSpells(void)
 											      erot->virt_pos.y);
 
 				if (DistanceFromCenter < minDist) {
-					if ((AllActiveSpells[i].hit_type == ATTACK_HIT_BOTS && Druidmap[erot->type].is_human) ||
-					    (AllActiveSpells[i].hit_type == ATTACK_HIT_HUMANS && !Druidmap[erot->type].is_human))
+					if ((AllActiveSpells[i].hit_type == ATTACK_HIT_BOTS && Droidmap[erot->type].is_human) ||
+					    (AllActiveSpells[i].hit_type == ATTACK_HIT_HUMANS && !Droidmap[erot->type].is_human))
 						continue;
 
 					// Let's see if that enemy has a direction, that is still
@@ -554,9 +557,7 @@ void clear_active_bullets()
 
 /**
  * Whenever a new bullet is generated, we need to find a free index in 
- * the array of bullets.  This function automates the process and 
- * also is secure against too many bullets in the game (with a rather
- * ungraceful exit, but that event shouldn't ever occur in a normal game.
+ * the array of bullets.  This function automates the process.
  */
 int find_free_bullet_index(void)
 {
@@ -565,18 +566,90 @@ int find_free_bullet_index(void)
 	for (j = 0; j < MAXBULLETS; j++) {
 		if (AllBullets[j].type == INFOUT) {
 			return (j);
-			break;
 		}
 	}
 
-	// If this point is ever reached, there's a severe bug in here...
-	//
-	ErrorMessage(__FUNCTION__, "\
-I seem to have run out of free bullet entries.  This can't normally happen.  --> some bug in here, oh no...", PLEASE_INFORM, IS_FATAL);
+	// TODO use a dynarray rather than a static array, to avoid amount limitation
+	error_once_message(ONCE_PER_GAME, __FUNCTION__,
+	                   "I seem to have run out of free bullet entries. "
+	                   "This can't normally happen.  --> some bug in here, oh no...",
+	                   PLEASE_INFORM);
 
-	return (-1);		// can't happen.  just to make compilers happy (no warnings)
+	return -1;
+}
 
-};				// void find_free_bullet_entry_pointer ( void )
+/**
+ * \brief Initialize a bullet - Common part
+ *
+ * \param bullet            Pointer to the bullet struct to fill in
+ * \param bullet_type       Type of the bullet (i.e. its graphical representation)
+ * \param from_pos          Starting position of the bullet
+ * \param height            Vertical offset applied when drawing the bullet, to simulate a shot starting from the gun's muzzle
+ * \param weapon_item_type  Type of the weapon used to shoot the bullet (defines
+ *                          the behavior of the bullet)
+ */
+static void _bullet_init(struct bullet *bullet, int bullet_type, gps *from_pos, int height, short int weapon_item_type)
+{
+	memset(bullet, 0, sizeof(struct bullet));
+
+	// Fill in non-zero field
+	bullet->type = bullet_type;
+	bullet->pos.x = from_pos->x;
+	bullet->pos.y = from_pos->y;
+	bullet->pos.z = from_pos->z;
+	bullet->height = height;
+
+	// Bullet characteristics defined by the weapon used to fire the bullet.
+	bullet->bullet_lifetime = ItemMap[weapon_item_type].weapon_bullet_lifetime;
+	bullet->pass_through_hit_bodies = ItemMap[weapon_item_type].weapon_bullet_pass_through_hit_bodies;
+}
+
+
+/**
+ * \brief Initialize a bullet, shot by the player
+ *
+ * \param bullet            Pointer to the bullet struct to fill in
+ * \param bullet_type       Type of the bullet (i.e. its graphical representation)
+ * \param weapon_item_type  Type of the weapon used to shoot the bullet (defines
+ *                          the behavior of the bullet)
+ */
+void bullet_init_for_player(struct bullet *bullet, int bullet_type, short int weapon_item_type)
+{
+	// Initialize the characteristics that do not depend on the shooter
+	_bullet_init(bullet, bullet_type, &Me.pos, tux_rendering.gun_muzzle_height, weapon_item_type);
+
+	// The damage depends on the weapon's type + the player's stats.
+	// It is computed and stored in the Me struct.
+	bullet->damage = Me.base_damage + MyRandom(Me.damage_modifier);
+
+	// Remember that the bullet was shot by Tux.
+	bullet->mine = TRUE;
+	bullet->owner = -1;
+	bullet->faction = FACTION_SELF;
+}
+
+/**
+ * \brief Initialize a bullet, shot by a bot
+ *
+ * \param bullet            Pointer to the bullet struct to fill in
+ * \param bullet_type       Type of the bullet (i.e. its graphical representation)
+ * \param weapon_item_type  Type of the weapon used to shoot the bullet (defines
+ *                          the behavior of the bullet)
+ * \param bot               Pointer to the bot that fired the bullet
+ */
+void bullet_init_for_enemy(struct bullet *bullet, int bullet_type, short int weapon_item_type, struct enemy *bot)
+{
+	// Initialize the characteristics that do not depend on the shooter
+	_bullet_init(bullet, bullet_type, &(bot->virt_pos), Droidmap[bot->type].gun_muzzle_height, weapon_item_type);
+
+	// The damage depends on the weapon's type only.
+	bullet->damage = ItemMap[weapon_item_type].weapon_base_damage +
+	                 MyRandom(ItemMap[weapon_item_type].weapon_damage_modifier);
+
+	// Remember that the bullet was shot by this bot.
+	bullet->owner = bot->id;
+	bullet->faction = bot->faction;
+}
 
 /**
  *
@@ -599,7 +672,7 @@ void apply_bullet_damage_to_player(int damage, int owner)
 {
 	float real_damage = damage * get_player_damage_factor();
 
-	hit_tux(real_damage, owner);	// loose some energy
+	hit_tux(real_damage);           // loose some energy
 	DamageProtectiveEquipment();    // chance worn items gets damaged on each hit
 }
 
@@ -653,12 +726,12 @@ void check_bullet_player_collisions(bullet * CurBullet, int num)
 void check_bullet_enemy_collisions(bullet * CurBullet, int num)
 {
 	double xdist, ydist;
-	int level = CurBullet->pos.z;
+	int lvl = CurBullet->pos.z;
 
 	// Check for collision with enemys
 	//
 	enemy *ThisRobot, *nerot;
-	BROWSE_LEVEL_BOTS_SAFE(ThisRobot, nerot, level) {
+	BROWSE_LEVEL_BOTS_SAFE(ThisRobot, nerot, lvl) {
 		// Check several hitting conditions
 		//
 		xdist = CurBullet->pos.x - ThisRobot->pos.x;
@@ -667,9 +740,9 @@ void check_bullet_enemy_collisions(bullet * CurBullet, int num)
 		if ((xdist * xdist + ydist * ydist) >= DROIDHITDIST2)
 			continue;
 
-		if (CurBullet->hit_type == ATTACK_HIT_BOTS && Druidmap[ThisRobot->type].is_human)
+		if (CurBullet->hit_type == ATTACK_HIT_BOTS && Droidmap[ThisRobot->type].is_human)
 			continue;
-		if (CurBullet->hit_type == ATTACK_HIT_HUMANS && !Druidmap[ThisRobot->type].is_human)
+		if (CurBullet->hit_type == ATTACK_HIT_HUMANS && !Droidmap[ThisRobot->type].is_human)
 			continue;
 		if (is_friendly(ThisRobot->faction, CurBullet->faction))
 			continue;
@@ -691,7 +764,7 @@ void check_bullet_enemy_collisions(bullet * CurBullet, int num)
 		// be completely deleted of course, with the same small explosion as well
 		//
 		if (CurBullet->pass_through_hit_bodies)
-			StartBlast(CurBullet->pos.x, CurBullet->pos.y, CurBullet->pos.z, BULLETBLAST, 0, CurBullet->faction);
+			StartBlast(CurBullet->pos.x, CurBullet->pos.y, CurBullet->pos.z, BULLETBLAST, 0, CurBullet->faction, NULL);
 		else
 			DeleteBullet(num, TRUE);	// we want a bullet-explosion
 
@@ -772,7 +845,7 @@ void CheckBulletCollisions(int num)
 };				// CheckBulletCollisions( ... )
 
 /**
- * This function checks for collisions of blasts with bullets and druids
+ * This function checks for collisions of blasts with bullets and droids
  * and delivers damage to the hit objects according to how long they have
  * been in the blast.
  * 
@@ -827,7 +900,7 @@ void CheckBlastCollisions(int num)
 		if (blast_vpos.z != -1) {
 			if ((fabsf(Me.pos.x - blast_vpos.x) < Blast_Radius) && (fabsf(Me.pos.y - blast_vpos.y) < Blast_Radius)) {
 				float real_damage = CurBlast->damage_per_second * Frame_Time() * get_player_damage_factor();
-				hit_tux(real_damage, -100);
+				hit_tux(real_damage);
 			}
 		}
 	}
@@ -842,15 +915,17 @@ void CheckBlastCollisions(int num)
 int GetBulletByName(const char *bullet_name)
 {
 	int i;
-	for (i=0; i < Number_Of_Bullet_Types; i++) {
-		if (!Bulletmap[i].name)
+	for (i=0; i < bullet_specs.size; i++) {
+		struct bulletspec *bullet_spec = dynarray_member(&bullet_specs, i, sizeof(struct bulletspec));
+
+		if (!bullet_spec->name)
 			return 0;
 
-		if (!strcmp(bullet_name, Bulletmap[i].name))
+		if (!strcmp(bullet_name, bullet_spec->name))
 			return i;
 	}
-	ErrorMessage(__FUNCTION__, "\
-The bullet name \"%s\" lacks a definition.", PLEASE_INFORM, IS_WARNING_ONLY, bullet_name);
+	error_message(__FUNCTION__, "\
+The bullet name \"%s\" lacks a definition.", PLEASE_INFORM, bullet_name);
 	return 0;
 }
 
