@@ -124,11 +124,11 @@ static void AddInfluencerItemSecondaryBonus(item * BonusItem)
  * \brief Get the experience required for next level.
  * \param level Current level.
  */
-int get_experience_required(int level)
+unsigned int get_experience_required(int lvl)
 {
-	if (level < 0)
+	if (lvl < 0)
 		return 0;
-	return BASE_EXP_REQUIRED * (level * level) + BASE_EXP_REQUIRED;
+	return BASE_EXP_REQUIRED * (lvl * lvl) + BASE_EXP_REQUIRED;
 }
 
 /**
@@ -136,7 +136,7 @@ int get_experience_required(int level)
  */
 static void check_for_new_experience_level_reached()
 {
-	int exp_required = get_experience_required(Me.exp_level);
+	unsigned int exp_required = get_experience_required(Me.exp_level);
 
 	if (Me.Experience >= exp_required) {
 		Me.exp_level++;
@@ -224,7 +224,7 @@ static void update_secondary_stats_from_primary_stats()
 	//
 	Me.to_hit = 100 - 40 * exp(- 0.03 * Me.dexterity);
 
-	// How many life points can this character aquire currently
+	// How many life points can this character acquire currently
 	//
 	Me.maxenergy = BASE_HPMAX + (Me.exp_level) * HPMAX_GAIN_PER_LEVEL
                                   + (Me.physique) * HPMAX_GAIN_PER_PHY_POINT;
@@ -252,7 +252,7 @@ static void update_secondary_stats_from_primary_stats()
 static void update_damage_tux_can_do()
 {
 	if (Me.weapon_item.type != (-1)) {
-		if (ItemMap[Me.weapon_item.type].item_weapon_is_melee != 0) {
+		if (ItemMap[Me.weapon_item.type].weapon_is_melee != 0) {
 			// Damage modifier in case of MELEE WEAPON is computed:  
 			// weapon's modifier * (100+strength)%
 			//
@@ -368,42 +368,42 @@ static void show_character_screen_skills()
 	//
 	if ((Me.melee_weapon_skill < 0) || (Me.melee_weapon_skill >= NUMBER_OF_SKILL_LEVELS)) {
 		fprintf(stderr, "\nmelee_weapon_skill: %d.", Me.melee_weapon_skill);
-		ErrorMessage(__FUNCTION__, "\
-Error: melee weapon skill seems out of bounds.", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "\
+Error: melee weapon skill seems out of bounds.", PLEASE_INFORM | IS_FATAL);
 	}
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Melee"), RIGHT_TXT_X + CharacterRect.x, MELEE_SKILL_Y + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Melee"), RIGHT_TXT_X + CharacterRect.x, MELEE_SKILL_Y + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 	display_text(_(AllSkillTexts[Me.melee_weapon_skill]),
-		    SKILLS_VALUE_X + CharacterRect.x, MELEE_SKILL_Y + 17 + CharacterRect.y, &CharacterRect);
+		    SKILLS_VALUE_X + CharacterRect.x, MELEE_SKILL_Y + 17 + CharacterRect.y, &CharacterRect, 1.0);
 
 	// We add some security against skill values out of allowed
 	// bounds.
 	//
 	if ((Me.ranged_weapon_skill < 0) || (Me.ranged_weapon_skill >= NUMBER_OF_SKILL_LEVELS)) {
 		fprintf(stderr, "\nranged_weapon_skill: %d.", Me.ranged_weapon_skill);
-		ErrorMessage(__FUNCTION__, "\
-Error: ranged weapon skill seems out of bounds.", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "\
+Error: ranged weapon skill seems out of bounds.", PLEASE_INFORM | IS_FATAL);
 	}
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Ranged"), RIGHT_TXT_X + CharacterRect.x, RANGED_SKILL_Y + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Ranged"), RIGHT_TXT_X + CharacterRect.x, RANGED_SKILL_Y + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 	display_text(_(AllSkillTexts[Me.ranged_weapon_skill]),
-		    SKILLS_VALUE_X + CharacterRect.x, RANGED_SKILL_Y + 17 + CharacterRect.y, &CharacterRect);
+		    SKILLS_VALUE_X + CharacterRect.x, RANGED_SKILL_Y + 17 + CharacterRect.y, &CharacterRect, 1.0);
 
 	// We add some security against skill values out of allowed
 	// bounds.
 	//
 	if ((Me.spellcasting_skill < 0) || (Me.spellcasting_skill >= NUMBER_OF_SKILL_LEVELS)) {
 		fprintf(stderr, "\nProgramming_Skill: %d.", Me.spellcasting_skill);
-		ErrorMessage(__FUNCTION__, "\
-Error: Programming_Skill skill seems out of bounds.", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "\
+Error: Programming_Skill skill seems out of bounds.", PLEASE_INFORM | IS_FATAL);
 	}
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Programming"), RIGHT_TXT_X + CharacterRect.x, SPELLCASTING_SKILL_Y + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Programming"), RIGHT_TXT_X + CharacterRect.x, SPELLCASTING_SKILL_Y + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 	display_text(_(AllSkillTexts[Me.spellcasting_skill]),
-		    SKILLS_VALUE_X + CharacterRect.x, SPELLCASTING_SKILL_Y + 17 + CharacterRect.y, &CharacterRect);
+		    SKILLS_VALUE_X + CharacterRect.x, SPELLCASTING_SKILL_Y + 17 + CharacterRect.y, &CharacterRect, 1.0);
 
 };				// void show_character_screen_skills ( )
 
@@ -423,7 +423,7 @@ void ShowCharacterScreen()
 	if (GameConfig.CharacterScreen_Visible == FALSE)
 		return;
 
-	SetCurrentFont(Messagestat_BFont);
+	set_current_font(Messagestat_Font);
 	
 	// We define the right side of the user screen as the rectangle
 	// for our inventory screen.
@@ -435,101 +435,101 @@ void ShowCharacterScreen()
 
 	blit_background("character.png");
 
-	SetCurrentFont(Blue_BFont);
+	set_current_font(Blue_Font);
 
-	SetCurrentFont(Menu_BFont);
-	display_text(Me.character_name, 20 + CharacterRect.x, 12 + CharacterRect.y, &CharacterRect);
+	set_current_font(Menu_Font);
+	display_text(Me.character_name, 20 + CharacterRect.x, 12 + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Level"), LEFT_TXT_X + CharacterRect.x, 73 + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Level"), LEFT_TXT_X + CharacterRect.x, 73 + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%d", Me.exp_level);
-	display_text(CharText, LEVEL_NR_X + CharacterRect.x, 73 + CharacterRect.y, &CharacterRect);
+	display_text(CharText, LEVEL_NR_X + CharacterRect.x, 73 + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Experience"), LEFT_TXT_X + CharacterRect.x, 89 + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Experience"), LEFT_TXT_X + CharacterRect.x, 89 + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%u", Me.Experience);
-	display_text(CharText, LEVEL_NR_X + CharacterRect.x, 89 + CharacterRect.y, &CharacterRect);
+	display_text(CharText, LEVEL_NR_X + CharacterRect.x, 89 + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Next level"), LEFT_TXT_X + CharacterRect.x, 107 + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Next level"), LEFT_TXT_X + CharacterRect.x, 107 + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%u", get_experience_required(Me.exp_level));
-	display_text(CharText, LEVEL_NR_X + CharacterRect.x, 107 + CharacterRect.y, &CharacterRect);
+	display_text(CharText, LEVEL_NR_X + CharacterRect.x, 107 + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Circuits"), RIGHT_TXT_X + CharacterRect.x, 71 + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
-	sprintf(CharText, "%6d", Me.Gold);
-	display_text(CharText, 240 + CharacterRect.x, 71 + CharacterRect.y, &CharacterRect);
+	set_current_font(Messagestat_Font);
+	display_text(_("Circuits"), RIGHT_TXT_X + CharacterRect.x, 71 + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
+	sprintf(CharText, "%6u", Me.Gold);
+	display_text(CharText, 240 + CharacterRect.x, 71 + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Strength"), LEFT_TXT_X + CharacterRect.x, STR_Y + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Strength"), LEFT_TXT_X + CharacterRect.x, STR_Y + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%d", Me.strength);
 	if (Me.strength != Me.base_strength)
 		sprintf(CharText + strlen(CharText), " (%+d)", Me.strength - Me.base_strength);
-	display_text(CharText, STR_X + CharacterRect.x, STR_Y + CharacterRect.y, &CharacterRect);
+	display_text(CharText, STR_X + CharacterRect.x, STR_Y + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Dexterity"), LEFT_TXT_X + CharacterRect.x, DEX_Y + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Dexterity"), LEFT_TXT_X + CharacterRect.x, DEX_Y + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%d", Me.dexterity);
 	if (Me.dexterity != Me.base_dexterity)
 		sprintf(CharText + strlen(CharText), " (%+d)", Me.dexterity - Me.base_dexterity);
-	display_text(CharText, STR_X + CharacterRect.x, DEX_Y + CharacterRect.y, &CharacterRect);
+	display_text(CharText, STR_X + CharacterRect.x, DEX_Y + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Physique"), LEFT_TXT_X + CharacterRect.x, VIT_Y + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Physique"), LEFT_TXT_X + CharacterRect.x, VIT_Y + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%d", Me.physique);
 	if (Me.physique != Me.base_physique)
 		sprintf(CharText + strlen(CharText), " (%+d)", Me.physique - Me.base_physique);
-	display_text(CharText, STR_X + CharacterRect.x, VIT_Y + CharacterRect.y, &CharacterRect);
+	display_text(CharText, STR_X + CharacterRect.x, VIT_Y + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Cooling"), LEFT_TXT_X + CharacterRect.x, MAG_Y + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Cooling"), LEFT_TXT_X + CharacterRect.x, MAG_Y + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%d", Me.cooling);
 	if (Me.cooling != Me.base_cooling)
 		sprintf(CharText + strlen(CharText), " (%+d)", Me.cooling - Me.base_cooling);
-	display_text(CharText, STR_X + CharacterRect.x, MAG_Y + CharacterRect.y, &CharacterRect);
+	display_text(CharText, STR_X + CharacterRect.x, MAG_Y + CharacterRect.y, &CharacterRect, 1.0);
 
 	SDL_Rect tmprect = CharacterRect;
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Training points"), LEFT_TXT_X + CharacterRect.x, POINTS_Y + CharacterRect.y, &tmprect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Training points"), LEFT_TXT_X + CharacterRect.x, POINTS_Y + CharacterRect.y, &tmprect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%d", Me.points_to_distribute);
-	display_text(CharText, 155 + CharacterRect.x, POINTS_Y + CharacterRect.y, &CharacterRect);
+	display_text(CharText, 155 + CharacterRect.x, POINTS_Y + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Health"), LEFT_TXT_X + CharacterRect.x, HEALTH_STAT_Y + CharacterRect.y, &tmprect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Health"), LEFT_TXT_X + CharacterRect.x, HEALTH_STAT_Y + CharacterRect.y, &tmprect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%d/%d", (int)Me.energy, (int)Me.maxenergy);
-	display_text(CharText, HEALTH_NR_X + CharacterRect.x, HEALTH_STAT_Y + CharacterRect.y, &CharacterRect);
+	display_text(CharText, HEALTH_NR_X + CharacterRect.x, HEALTH_STAT_Y + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Temperature"), LEFT_TXT_X + CharacterRect.x, TEMP_STAT_Y + CharacterRect.y, &tmprect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Temperature"), LEFT_TXT_X + CharacterRect.x, TEMP_STAT_Y + CharacterRect.y, &tmprect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%d/%d", (int)Me.temperature, (int)Me.max_temperature);
-	display_text(CharText, HEALTH_NR_X + CharacterRect.x, TEMP_STAT_Y + CharacterRect.y, &CharacterRect);
+	display_text(CharText, HEALTH_NR_X + CharacterRect.x, TEMP_STAT_Y + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Stamina"), LEFT_TXT_X + CharacterRect.x, STAMINA_STAT_Y + CharacterRect.y, &tmprect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Stamina"), LEFT_TXT_X + CharacterRect.x, STAMINA_STAT_Y + CharacterRect.y, &tmprect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%d/%d", (int)Me.running_power, (int)Me.max_running_power);
-	display_text(CharText, HEALTH_NR_X + CharacterRect.x, STAMINA_STAT_Y + CharacterRect.y, &CharacterRect);
+	display_text(CharText, HEALTH_NR_X + CharacterRect.x, STAMINA_STAT_Y + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Attack"), RIGHT_TXT_X + CharacterRect.x, TOHIT_Y + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Attack"), RIGHT_TXT_X + CharacterRect.x, TOHIT_Y + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%d%%", (int)Me.to_hit);
-	display_text(CharText, ARMOR_NR_X + CharacterRect.x, TOHIT_Y + CharacterRect.y, &CharacterRect);
+	display_text(CharText, ARMOR_NR_X + CharacterRect.x, TOHIT_Y + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Damage"), RIGHT_TXT_X + CharacterRect.x, DAMAGE_Y + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Damage"), RIGHT_TXT_X + CharacterRect.x, DAMAGE_Y + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 
 	// Display range of damage, or a single value if there is no range
 	if (Me.damage_modifier)
@@ -537,13 +537,13 @@ void ShowCharacterScreen()
 	else
 		sprintf(CharText, "%d", (int)Me.base_damage);
 
-	display_text(CharText, ARMOR_NR_X + CharacterRect.x, DAMAGE_Y + CharacterRect.y, &CharacterRect);
+	display_text(CharText, ARMOR_NR_X + CharacterRect.x, DAMAGE_Y + CharacterRect.y, &CharacterRect, 1.0);
 
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Armor"), RIGHT_TXT_X + CharacterRect.x, DAMRED_Y + CharacterRect.y, &CharacterRect);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagestat_Font);
+	display_text(_("Armor"), RIGHT_TXT_X + CharacterRect.x, DAMRED_Y + CharacterRect.y, &CharacterRect, 1.0);
+	set_current_font(Messagevar_Font);
 	sprintf(CharText, "%d", (int)Me.armor_class);
-	display_text(CharText, ARMOR_NR_X + CharacterRect.x, DAMRED_Y + CharacterRect.y, &CharacterRect);
+	display_text(CharText, ARMOR_NR_X + CharacterRect.x, DAMRED_Y + CharacterRect.y, &CharacterRect, 1.0);
 
 	// Now we print out the current skill levels in hacking skill, 
 	// spellcasting, melee combat, ranged weapon combat and repairing things
