@@ -1,4 +1,4 @@
-/* 
+/*
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
@@ -39,8 +39,8 @@
 #define FALSE (0)
 #endif
 
-#define ERR 	-1
-#define OK		0
+#define ERR -1
+#define OK 0
 
 #ifndef max
 #define max(x,y) ((x) < (y) ? (y) : (x) )
@@ -49,29 +49,36 @@
 #define min(x,y) ((x) > (y) ? (y) : (x) )
 #endif
 
+// On Linux and MacOS, 'z' is the length modifier to be used in printf() for
+// a size_t data. On Windows, 'I' is to be used.
+#if __WIN32__
+#define SIZE_T_F "%Iu"
+#else
+#define SIZE_T_F "%zu"
+#endif
+
 //--------------------
 // Here you can control the debug level for the various modules, or at least
 // that's the way it is planned to be.  Turn on and off debugging as you like.
 //
 #define SAVE_LOAD_GAME_DEBUG 1
 enum {
-	NO_NEED_TO_INFORM = 0,
-	PLEASE_INFORM = 1
+	NO_REPORT     = 0,
+	PLEASE_INFORM = 1 << 1,   // Print a message inviting the user to inform us about the error
+	IS_FATAL      = 1 << 2,   // Inform the user that the game will stop, and call Terminate()
+	NO_TERMINATE  = 1 << 3,   // Do not call Terminate() (needed when things have to be cleaned before to call Terminate(), for instance)
+	SILENT        = 1 << 4    // Special flag, used with find_file() to disable any output
 };
+
+// Define how alert_once_window() has to remember for past alerts
 enum {
-	IS_WARNING_ONLY = 0,
-	IS_FATAL = 1
-};
-enum {
-	MENU_MODE_FAST = 2,
-	MENU_MODE_DEFAULT,
-	MENU_MODE_DOUBLE,
+	ONCE_PER_RUN  = 0,  // Never redisplay the alert.
+	ONCE_PER_GAME = 1   // Redisplay the alert if a new savegame was loaded since last time.
 };
 
 enum {
 	COLLISION_TYPE_NONE = 1,
 	COLLISION_TYPE_RECTANGLE = 2,
-	COLLISION_TYPE_CIRCLE = 3
 };
 
 enum {
@@ -84,10 +91,17 @@ enum {
 	ALL_PART_GROUPS = 6
 };
 
-#define ENEMY_ROTATION_MODELS_AVAILABLE 42
+enum {
+	SOUND_OUTPUT_FMT_STEREO = 0,
+	SOUND_OUTPUT_FMT_SURROUND40,
+	SOUND_OUTPUT_FMT_SURROUND51,
+	ALL_SOUND_OUTPUTS
+};
+
+#define ENEMY_ROTATION_MODELS_AVAILABLE 43
 #define ROTATION_ANGLES_PER_ROTATION_MODEL 8
 
-#define MAX_ENEMY_MOVEMENT_PHASES 70
+#define MAX_ENEMY_MOVEMENT_PHASES 999
 #define WALK_ANIMATION 113
 #define ATTACK_ANIMATION 114
 #define GETHIT_ANIMATION 115
@@ -98,13 +112,11 @@ enum {
 #define NUMBER_OF_SHADOW_IMAGES 20
 
 #define MAX_OBSTACLES_ON_MAP 4000
-#define MAX_OBSTACLE_NAMES_PER_LEVEL 100
-#define MAX_OBSTACLE_DESCRIPTIONS_PER_LEVEL 100
 
 #define FLOOR_TILES_VISIBLE_AROUND_TUX ((GameConfig . screen_width >= 1024 ? 13 : GameConfig . screen_width >= 800 ? 9 : 7))
 #define MAX_ITEMS_PER_LEVEL 300
 #define MAX_ITEMS_IN_INVENTORY 100
-#define MAX_ITEMS_IN_NPC_SHOPLIST 50
+#define MAX_ITEMS_IN_NPC_SHOPLIST 200
 #define INVENTORY_GRID_WIDTH 10
 #define INVENTORY_GRID_HEIGHT 6
 #define INV_SUBSQUARE_WIDTH (31)
@@ -112,6 +124,7 @@ enum {
 #define NUMBER_OF_MOUSE_CURSOR_PICTURES 2
 
 // Balancing of loot and items.
+#define MAX_DROP_CLASS 9
 #define MONEY_PER_BOT_CLASS 12
 #define ITEM_DROP_PERCENTAGE 10
 #define GOLD_DROP_PERCENTAGE 40
@@ -122,8 +135,6 @@ enum {
 #define ARMOUR_DURABILITYLOSS_PERCENTAGE_WHEN_HIT 30
 #define WEAPON_DURABILITYLOSS_PERCENTAGE_WHEN_USED 10
 
-#define CHAT_DEBUG_LEVEL 1
-
 #define MAX_BIG_SCREEN_MESSAGES 10
 
 #define NUMBER_OF_SKILL_LEVELS 10
@@ -139,9 +150,10 @@ enum upgrade_socket_types {
 
 enum skill_forms {
 	PROGRAM_FORM_INSTANT = 10,	//instant hit whatever behind cursor
-	PROGRAM_FORM_RADIAL,	//nova
-	PROGRAM_FORM_BULLET,	//fire a bullet
-	PROGRAM_FORM_SELF,	//instant hit player
+	PROGRAM_FORM_RADIAL,		//nova
+	PROGRAM_FORM_BULLET,		//fire a bullet
+	PROGRAM_FORM_SELF,			//instant hit player
+	PROGRAM_FORM_SPECIAL,		//special effect
 };
 
 enum _game_mouse_cursors {
@@ -160,6 +172,12 @@ enum _game_config_screen_visible_parameters {
 	GAME_CONFIG_SCREEN_VISIBLE_SKILL_EXPLANATION = 8004,
 };
 
+enum _game_status {
+	INSIDE_MENU = 0,
+	INSIDE_GAME,
+	INSIDE_LVLEDITOR
+};
+
 #define END_OF_OFFSET_FILE_STRING "** End of iso_image offset file **"
 #define OFFSET_FILE_OFFSETX_STRING "OffsetX="
 #define OFFSET_FILE_OFFSETY_STRING "OffsetY="
@@ -204,7 +222,6 @@ enum {
 	IS_VOLATILE = 1024	/* This obstacle vanishes when level is respawned. */
 };
 
-#define AUTOMAP_ZOOM_OUT_FACT 8.0
 
 //--------------------
 // Constants for Paths and names of Data-files
@@ -215,21 +232,16 @@ enum {
 #define FD_DATADIR ".."		// our local fallback
 #endif
 
-#define GRAPHICS_DIR		"graphics/"
-#define SOUND_DIR		"sound/"
-#define MAP_DIR			"map/"
-#define TITLES_DIR		"map/titles/"
-#define DIALOG_DIR		"dialogs/"
-
-#define ICON_FILE		"paraicon.bmp"
+#if __WIN32__
+#define ICON_FILE "paraicon_32x32.bmp"
+#else
+#define ICON_FILE "paraicon.bmp"
+#endif
 
 //--------------------
 #define SKILL_LEVEL_BUTTON_HEIGHT 32
 #define SKILL_LEVEL_BUTTON_WIDTH 30
 
-#define NUMBER_OF_ITEMS_ON_ONE_SCREEN 4
-#define ITEM_MENU_DISTANCE 80
-#define ITEM_FIRST_POS_Y 130
 
 #define INITIAL_BLOCK_WIDTH	64
 #define INITIAL_BLOCK_HEIGHT	64
@@ -245,7 +257,6 @@ enum {
 #define RIGHT_INFO_Y	(8*2)
 #define LEFT_TEXT_LEN 20
 #define RIGHT_TEXT_LEN 6
-#define LINE_HEIGHT_FACTOR (6.0/5.0)
 
 #define BACKGROUND_TEXT_RECT_ALPHA 110
 #define TEXT_BANNER_HORIZONTAL_MARGIN 4
@@ -254,25 +265,17 @@ enum {
 // dimensions of the droids as required for functions NotPassable() and CheckEnemyCollsion() )
 //
 #define DROIDRADIUSXY	        (7*2/64.0)
-#define BULLET_BULLET_COLLISION_DIST (10/64.0)
 
-#define TAKEOVER_BACKGROUND_MUSIC_SOUND "Bleostrada.ogg"
-#define MENU_BACKGROUND_MUSIC_SOUND "menu.ogg"
-#define BIGFIGHT_BACKGROUND_MUSIC_SOUND "hellforce.ogg"
 #define SILENCE (NULL)
-enum _sounds {
-	ERRORSOUND = 0,
-};
 
 //--------------------
 // The sounds when the influencers energy is low or when he is in transfer mode
 // occur periodically.  These constants specify which intervals are to be used
 // for these periodic happenings...
-#define CRY_SOUND_INTERVAL 2
 
 #define MAX_DIALOGUE_OPTIONS_IN_ROSTER 100
-#define MAX_ANSWERS_PER_PERSON MAX_DIALOGUE_OPTIONS_IN_ROSTER
-#define END_ANSWER (MAX_ANSWERS_PER_PERSON-1)
+#define CHAT_TOPIC_STACK_SIZE 10
+#define CHAT_CONTEXT_STACK_SIZE 10
 
 enum {
 	UP_BUTTON,
@@ -358,6 +361,10 @@ enum {
 	LEVEL_EDITOR_TOGGLE_OBSTACLES_BUTTON_PUSHED,
 	LEVEL_EDITOR_TOGGLE_OBSTACLES_BUTTON_OFF,
 	LEVEL_EDITOR_TOGGLE_OBSTACLES_BUTTON_OFF_PUSHED,
+	LEVEL_EDITOR_TOGGLE_MAP_LABELS_BUTTON,
+	LEVEL_EDITOR_TOGGLE_MAP_LABELS_BUTTON_PUSHED,
+	LEVEL_EDITOR_TOGGLE_MAP_LABELS_BUTTON_OFF,
+	LEVEL_EDITOR_TOGGLE_MAP_LABELS_BUTTON_OFF_PUSHED,
 	LEVEL_EDITOR_TOGGLE_TOOLTIPS_BUTTON,
 	LEVEL_EDITOR_TOGGLE_TOOLTIPS_BUTTON_PUSHED,
 	LEVEL_EDITOR_TOGGLE_TOOLTIPS_BUTTON_OFF,
@@ -395,9 +402,6 @@ enum {
 	HELMET_RECT_BUTTON,
 	ARMOUR_RECT_BUTTON,
 
-	SCROLL_DIALOG_MENU_UP_BUTTON,
-	SCROLL_DIALOG_MENU_DOWN_BUTTON,
-
 	MORE_STR_BUTTON,
 	MORE_MAG_BUTTON,
 	MORE_DEX_BUTTON,
@@ -411,11 +415,6 @@ enum {
 
 	DROID_SHOW_EXIT_BUTTON,
 
-	CHAT_LOG_SCROLL_UP_BUTTON,
-	CHAT_LOG_SCROLL_DOWN_BUTTON,
-	CHAT_LOG_SCROLL_OFF_BUTTON,
-	CHAT_LOG_SCROLL_OFF2_BUTTON,
-
 	QUEST_BROWSER_ITEM_SHORT_BUTTON,
 	QUEST_BROWSER_ITEM_LONG_BUTTON,
 
@@ -444,10 +443,6 @@ enum {
 	MAX_MOUSE_PRESS_BUTTONS
 };
 
-#define CHAT_SUBDIALOG_WINDOW_X ((260)*GameConfig . screen_width/640.0)
-#define CHAT_SUBDIALOG_WINDOW_Y ((35)*GameConfig . screen_height/480.0)
-#define CHAT_SUBDIALOG_WINDOW_W ((337)*GameConfig . screen_width/640.0)
-#define CHAT_SUBDIALOG_WINDOW_H ((225)*GameConfig . screen_height/480.0)
 
 #define STR_X 90
 #define STR_Y 140
@@ -457,6 +452,7 @@ enum {
 
 #define BUTTON_WIDTH 35
 #define BUTTON_HEIGHT 19
+#define ENERGYBAR_MAXWIDTH 50
 
 #define DEFAULT_SCREEN_WIDTH  (800)
 #define DEFAULT_SCREEN_HEIGHT (600)
@@ -604,7 +600,6 @@ enum item_quality {
 	GOOD_QUALITY = 2
 };
 
-#define MAX_BULLET_DEFS 16
 
 //--------------------
 // explosion types
@@ -626,6 +621,7 @@ enum _busytype {
 	WEAPON_RELOAD,
 	THROWING_GRENADE,
 	RUNNING_PROGRAM,
+	TAKING_PILL,
 };
 
 enum _attackhit {
@@ -663,9 +659,7 @@ enum _enemy_combat_target_codes {
 	ATTACK_TARGET_IS_PLAYER = 102,
 	ATTACK_TARGET_IS_NOTHING = 103
 };
-enum {
-	MOUSE_CURSOR_ARROW_SHAPE = -402
-};
+
 enum {
 	TRANSPARENCY_NONE = 0,
 	TRANSPARENCY_FOR_WALLS = 10
@@ -717,8 +711,12 @@ enum obstacle_types {
 	ISO_V_DOOR_100_OPEN = 15,
 
 	ISO_TELEPORTER_1 = 16,
+	ISO_TELEPORTER_2 = 17,
 
-	ISO_REFRESH_1 = 21,
+	ISO_DROID_NEST_RED = 18,
+	ISO_DROID_NEST_BLUE,
+	ISO_DROID_NEST_YELLOW,
+	ISO_DROID_NEST_GREEN,
 
 	ISO_H_DOOR_LOCKED = 26,
 	ISO_V_DOOR_LOCKED = 27,
@@ -766,8 +764,7 @@ enum obstacle_types {
 
 	ISO_V_WOOD_FENCE = 59,
 	ISO_H_WOOD_FENCE = 60,
-	ISO_V_DENSE_FENCE = 61,
-	ISO_H_DENSE_FENCE = 62,
+
 	ISO_V_MESH_FENCE = 63,
 	ISO_H_MESH_FENCE = 64,
 	ISO_V_WIRE_FENCE = 65,
@@ -1157,14 +1154,14 @@ enum obstacle_types {
 	ISO_EXIT_5,
 	ISO_EXIT_6,
 	
-	ISO_E_CHEST2_CLOSED,
+	ISO_E_CHEST2_CLOSED,  // = 372
 	ISO_S_CHEST2_CLOSED,
 	ISO_E_CHEST2_OPEN,
 	ISO_S_CHEST2_OPEN,
-	ISO_N_CHEST2_CLOSED,
+	ISO_N_CHEST2_CLOSED, // = 376
 	ISO_W_CHEST2_CLOSED,
 	ISO_N_CHEST2_OPEN,
-	ISO_W_CHEST2_OPEN,
+	ISO_W_CHEST2_OPEN, // = 379
 
 	ISO_SECURITY_GATE_GREEN_E,
 	ISO_SECURITY_GATE_GREEN_S,
@@ -1237,6 +1234,45 @@ enum obstacle_types {
 	ISO_SIGN_2_FLASH,
 	ISO_SIGN_3_FLASH,
 	ISO_BARREL_RADIOACTIVE,
+	ISO_VENDING_MACHINE_1_W,
+	ISO_VENDING_MACHINE_1_S,
+	ISO_VENDING_MACHINE_1_E,
+	ISO_VENDING_MACHINE_1_N,
+	ISO_VENDING_MACHINE_2_W,
+	ISO_VENDING_MACHINE_2_S,
+	ISO_VENDING_MACHINE_2_E,
+	ISO_VENDING_MACHINE_2_N,
+	ISO_VENDING_MACHINE_3_W,
+	ISO_VENDING_MACHINE_3_S,
+	ISO_VENDING_MACHINE_3_E,
+	ISO_VENDING_MACHINE_3_N,
+
+	ISO_TRANSFORMER_1,
+	ISO_TRANSFORMER_2,
+	ISO_TRANSFORMER_3,
+
+	ISO_BOOKSHELF_LOOTABLE_E, // = 466
+	ISO_BOOKSHELF_LOOTED_E,
+	ISO_BOOKSHELF_LOOTABLE_S,
+	ISO_BOOKSHELF_LOOTED_S,
+	ISO_BOOKSHELF_LOOTABLE_W,
+	ISO_BOOKSHELF_LOOTED_W,
+	ISO_BOOKSHELF_LOOTABLE_N,
+	ISO_BOOKSHELF_LOOTED_N,
+	ISO_BOOKSHELF_LONG_LOOTABLE_E, // = 474
+	ISO_BOOKSHELF_LONG_LOOTED_E,
+	ISO_BOOKSHELF_LONG_LOOTABLE_S,
+	ISO_BOOKSHELF_LONG_LOOTED_S,
+	ISO_BOOKSHELF_LONG_LOOTABLE_W,
+	ISO_BOOKSHELF_LONG_LOOTED_W,
+	ISO_BOOKSHELF_LONG_LOOTABLE_N,
+	ISO_BOOKSHELF_LONG_LOOTED_N,
+	ISO_COLLAPSINGFLOOR_VISIBLE, // = 482
+
+	ISO_CONSOLE_SECURE_E = 492,
+	ISO_CONSOLE_SECURE_S = 493,
+	ISO_CONSOLE_SECURE_W = 494,
+	ISO_CONSOLE_SECURE_N = 495
 };
 
 enum {
@@ -1361,7 +1397,11 @@ enum {
 
 	ISO_TWOSQUARE_0001,
 	ISO_TWOSQUARE_0002,
-	ISO_TWOSQUARE_0003
+	ISO_TWOSQUARE_0003,
+
+	ISO_WATER_ANIMATED_SHALLOW,
+	ISO_WATER_ANIMATED_DARK,
+	ISO_DISCO
 };
 
 #define MAX_UNDERLAY_FLOOR_TILES 500
@@ -1432,7 +1472,21 @@ enum {
 	ISO_GROUNDMARKER_RED_Y_2,
 	ISO_GROUNDMARKER_RED_Y_3,
 	ISO_GROUNDMARKER_RED_Y_4,
-	ISO_GROUNDMARKER_RED_X_1
+	ISO_GROUNDMARKER_RED_X_1,
+	ISO_SAND_EDGE_0001,
+	ISO_SAND_EDGE_0002,
+	ISO_SAND_EDGE_0003,
+	ISO_SAND_EDGE_0004,
+	ISO_SAND_EDGE_0005,
+	ISO_SAND_EDGE_0006,
+	ISO_SAND_EDGE_0007,
+	ISO_SAND_EDGE_0008,
+	ISO_SAND_EDGE_0009,
+	ISO_SAND_EDGE_0010,
+	ISO_SAND_EDGE_0011,
+	ISO_SAND_EDGE_0012,
+	ISO_SAND_EDGE_0013,
+	ISO_SAND_EDGE_0014
 };
 
 #define MAX_FLOOR_LAYERS 2
@@ -1478,6 +1532,7 @@ enum faction_id {
 	FACTION_CRAZY,
 	FACTION_SINGULARITY,
 	FACTION_NEUTRAL,
+	FACTION_TEST,
 
 	FACTION_NUMBER_OF_FACTIONS, //keep this last
 };
@@ -1491,10 +1546,10 @@ enum faction_state {
 #define ACCESS_MAP_LABEL(X,Y) ((struct map_label *)(X.arr))[Y]
 
 enum obstacle_extension_type {
-	OBSTACLE_EXTENSION_LABEL = 30, // This extension indicates an obstacle label. The associated data is a const char * containing the label.
+	OBSTACLE_EXTENSION_LABEL = 30,  // This extension indicates an obstacle label. The associated data is a const char * containing the label.
 	OBSTACLE_EXTENSION_CHEST_ITEMS, // This extension indicates an item list. The associated data is a struct dynarray containing struct item.
-	OBSTACLE_EXTENSION_DIALOGFILE, // This extension indicates the dialog filename associated to an interactive obstacle. Contains the dialog basename.
-	OBSTACLE_EXTENSION_SIGNMESSAGE = OBSTACLE_EXTENSION_DIALOGFILE, // This extension contains the message written on a signpost. It is exclusive with a dialogfile specification
+	OBSTACLE_EXTENSION_DIALOGFILE,  // This extension indicates the dialog filename associated to an interactive obstacle. Contains the dialog basename.
+	OBSTACLE_EXTENSION_SIGNMESSAGE  // This extension contains the message written on a signpost.
 };
 
 enum lua_target {
@@ -1502,4 +1557,69 @@ enum lua_target {
 	LUA_DIALOG
 };
 
+enum slot_type {
+	NO_SLOT = 0,
+	WEAPON_SLOT = 1,
+	SHIELD_SLOT = 2,
+	HELM_SLOT = 4,
+	ARMOR_SLOT = 8,
+	BOOT_SLOT = 16,
+};
+
+// These are flags to change the level's environment
+enum level_flags {
+	TELEPORT_BLOCKED 	= 1 << 0,
+	NO_RESPAWN 			= 1 << 1,
+};
+
+// Current step of the dialog Finite state Machine
+enum chat_context_state {
+	LOAD_INIT_SCRIPT = 0,
+	RUN_INIT_SCRIPT = 1,
+	LOAD_STARTUP_SCRIPT = 2,
+	RUN_STARTUP_SCRIPT = 3,
+	SELECT_NEXT_NODE = 4,
+	RUN_NODE_SCRIPT = 5
+};
+
+// Indexes of known data dirs (used in conjunction with data_dirs[])
+enum data_dir_handle {
+	GRAPHICS_DIR = 0,
+	FONT_DIR,
+	SOUND_DIR,
+	MUSIC_DIR,
+	MAP_DIR,
+	TITLES_DIR,
+	DIALOG_DIR,
+#ifdef ENABLE_NLS
+	LOCALE_DIR,
+#endif
+	LUA_MOD_DIR,
+	LAST_DATA_DIR
+};
+
+// Add the sensor features (for droids).
+enum sensor_features {
+    SENSOR_FEATURELESS = 0,		// Any sensor without any feature, featureless.
+    SENSOR_DETECT_INVISIBLE = 1 << 1,	// Sensor that allows the droid to see invisible tux.
+    SENSOR_THROUGH_WALLS = 1 << 2	// Sensor that allows the droid to see through walls.
+};
+
+// Flags defining the type of the 'hardware' texture in a struct image
+enum texture_type {
+	NO_TEXTURE = 0,
+	TEXTURE_CREATED = 1 << 1,
+	IS_SUBTEXTURE = 1 << 2
+};
+
+// ORed flags defining the modifications to apply when loading an image (used by load_image())
+enum load_image_mod_flags {
+	NO_MOD = 0,
+	USE_OFFSET = 1 << 1,  // Use the offset file to translate the image
+	USE_WIDE   = 1 << 2   // Use the wide version of the file (if it exists)
+};
+
+// Draw quads borders (OpenGL only)
+#define DEBUG_QUAD_BORDER 0
+
 #endif
