@@ -63,7 +63,11 @@ struct dynarray *dynarray_alloc(int membernum, size_t membersize)
  */
 void dynarray_resize(struct dynarray *array, int membernum, size_t membersize)
 {
-	array->arr = realloc(array->arr, membernum * membersize);
+	void *buffer = realloc(array->arr, membernum * membersize);
+	if (!buffer) {
+		error_message(__FUNCTION__, "Not enough memory to realloc a dynarray (requested size: " SIZE_T_F ")", IS_FATAL, membernum * membersize);
+	}
+	array->arr = buffer;
 	array->capacity = membernum;
 }
 
@@ -88,10 +92,11 @@ void dynarray_add(struct dynarray *array, void *data, size_t membersize)
 {
 	array->size++;
 
-	if (array->size >= array->capacity) {
+	if (array->size > array->capacity) {
 		dynarray_resize(array, array->size * 2, membersize);
 	}
 
+	// cppcheck-suppress arithOperationsOnVoidPointer
 	memcpy(array->arr + membersize * (array->size - 1), data, membersize);
 }
 
@@ -109,7 +114,9 @@ void dynarray_del(struct dynarray *array, int index, size_t membersize)
 		array->size--;
 		return;
 	} else {
+		// cppcheck-suppress arithOperationsOnVoidPointer
 		void *addr = array->arr + membersize * index;
+		// cppcheck-suppress arithOperationsOnVoidPointer
 		void *next = addr + membersize;
 		int nb = membersize * (array->size - index - 1);
 
@@ -126,5 +133,6 @@ void dynarray_del(struct dynarray *array, int index, size_t membersize)
  */
 void *dynarray_member(struct dynarray *array, int index, size_t membersize)
 {
+	// cppcheck-suppress arithOperationsOnVoidPointer
 	return array->arr + membersize * index;
 }
