@@ -50,7 +50,8 @@ static struct {
 		{ FACTION_CIVILIAN, "civilian" },
 		{ FACTION_CRAZY, "crazy" },
 		{ FACTION_SINGULARITY, "singularity"},
-		{ FACTION_NEUTRAL, "neutral"}
+		{ FACTION_NEUTRAL, "neutral"},
+		{ FACTION_TEST, "test"} // extra faction for level 24, behaves neutral
 };
 
 /**
@@ -65,7 +66,7 @@ enum faction_id get_faction_id(const char *name)
 			return factions[i].id;
 	}
 
-	ErrorMessage(__FUNCTION__, "Faction name %s does not exist.", PLEASE_INFORM, IS_WARNING_ONLY, name);
+	error_message(__FUNCTION__, "Faction name %s does not exist.", PLEASE_INFORM, name);
 	return FACTION_SELF;
 }
 
@@ -77,7 +78,7 @@ enum faction_id get_faction_id(const char *name)
 const char *get_faction_from_id(int faction_id)
 {
 	if ((faction_id < 0) || (faction_id >= (sizeof(factions) / sizeof(factions[0])))) {
-		ErrorMessage(__FUNCTION__, "Malformed faction id! Faction %d does not exist.", PLEASE_INFORM, IS_WARNING_ONLY, faction_id);
+		error_message(__FUNCTION__, "Malformed faction id! Faction %d does not exist.", PLEASE_INFORM, faction_id);
 		return factions[FACTION_SELF].name;
 	}
 
@@ -170,6 +171,11 @@ void init_factions()
 		set_faction_state(FACTION_NEUTRAL, i, FRIENDLY);
 	}
 
+	/* Test as well does not attack anyone, neither is it being attacked */
+	for (i = 0; i < FACTION_NUMBER_OF_FACTIONS; i++) {
+		set_faction_state(FACTION_TEST, i, FRIENDLY);
+	}
+
 	/* Each faction is friendly towards itself. Otherwise, its members attack each other and/or commit suicide. */
 	for (i = 0; i < FACTION_NUMBER_OF_FACTIONS; i++) {
 		set_faction_state(i, i, FRIENDLY);
