@@ -48,7 +48,8 @@ static char *get_current_skill_button_tooltip()
 	if (!buffer)
 		buffer = alloc_autostr(64);
 
-	autostr_printf(buffer, _("%s\nHeat: %d\nRevision: %d\n"), SpellSkillMap[Me.readied_skill].name, calculate_program_heat_cost(Me.readied_skill), Me.skill_level[Me.readied_skill]);
+	// TRANSLATORS: Tooltip of skill button: <skill name>\n Heat: <skill heat cost>\n Revision: <skill revision>
+	autostr_printf(buffer, _("%s\nHeat: %d\nRevision: %d\n"), D_(SpellSkillMap[Me.readied_skill].name), calculate_program_heat_cost(Me.readied_skill), Me.skill_level[Me.readied_skill]);
 
 	return buffer->value;
 }
@@ -94,15 +95,15 @@ static void current_ammo_update(struct widget *w)
 
 	if (Me.weapon_item.type == -1 || &Me.weapon_item == item_held_in_hand)
 		return;
-	if (!ItemMap[Me.weapon_item.type].item_gun_use_ammunition)
+	if (!ItemMap[Me.weapon_item.type].weapon_ammo_type)
 		return;
 
 	if (Me.busy_type == WEAPON_RELOAD)
 		autostr_printf(text, _("reloading"));
 	else if (!Me.weapon_item.ammo_clip)
-		autostr_printf(text, _("%s EMPTY"), font_switchto_red);
+		autostr_printf(text, _("[r]EMPTY"));
 	else
-		autostr_printf(text, "%5d / %2d", Me.weapon_item.ammo_clip, ItemMap[Me.weapon_item.type].item_gun_ammo_clip_size);
+		autostr_printf(text, "%5d / %2d", Me.weapon_item.ammo_clip, ItemMap[Me.weapon_item.type].weapon_ammo_clip_size);
 }
 
 /** Toggle inventory screen on/off. */
@@ -165,7 +166,7 @@ static void stamina_bar_display(struct widget *w)
 	blit_vertical_status_bar(max_value, current_value, filled_color, empty_color, w->rect.x, w->rect.y, w->rect.w, w->rect.h);
 
 	if (GameConfig.cheat_running_stamina)
-		PutStringFont(Screen, Messagestat_BFont, w->rect.x, w->rect.y, "C");
+		put_string(Messagestat_Font, w->rect.x, w->rect.y, "C");
 }
 
 /** Computes the tooltip text displayed when hovering the stamina bar. */
@@ -185,8 +186,8 @@ static void experience_bar_display(struct widget *w)
 {
 	static Uint32 fill_color = 0;
 	static Uint32 empty_color = 0;
-	int exp_required = get_experience_required(Me.exp_level);
-	int exp_required_previously = get_experience_required(Me.exp_level - 1);
+	int exp_required = (int)get_experience_required(Me.exp_level);
+	int exp_required_previously = (int)get_experience_required(Me.exp_level - 1);
 	int exp_range = exp_required - exp_required_previously;
 	int exp_achieved = Me.Experience - exp_required_previously;
 
@@ -212,7 +213,7 @@ static char *get_experience_bar_tooltip(struct widget *w)
 	if (!buffer)
 		buffer = alloc_autostr(64);
 
-	autostr_printf(buffer, "%s\n%d/%d\n", _("XP"), Me.Experience, get_experience_required(Me.exp_level));
+	autostr_printf(buffer, "%s\n%u/%u\n", _("XP"), Me.Experience, get_experience_required(Me.exp_level));
 
 	return buffer->value;
 }
@@ -232,7 +233,7 @@ static void health_bar_display(struct widget *w)
 	blit_vertical_status_bar(Me.maxenergy, Me.energy, fill_color, empty_color, w->rect.x, w->rect.y, w->rect.w, w->rect.h);
 
 	if (Me.god_mode)
-		PutStringFont(Screen, Messagestat_BFont,  w->rect.x, w->rect.y, "C");
+		put_string(Messagestat_Font,  w->rect.x, w->rect.y, "C");
 }
 
 /** Computes the tooltip text displayed when hovering the health bar. */
@@ -333,7 +334,7 @@ static void quick_inventory_display(struct widget *w)
 
 	for (i = 0; i < 10; i++) {
 		sprintf(text, "%d", i < 9 ? i + 1: 0);
-		PutStringFont(Screen, Messagestat_BFont, w->rect.x + i * step, w->rect.y + 16, text);
+		put_string(Messagestat_Font, w->rect.x + i * step, w->rect.y + 16, text);
 		if (((index = GetInventoryItemAt(i, INVENTORY_GRID_HEIGHT - 1)) != -1)
 			&& (Me.Inventory[index].inventory_position.x == i)
 			&& (Me.Inventory[index].inventory_position.y == INVENTORY_GRID_HEIGHT - 1))
@@ -377,6 +378,60 @@ static void item_alarm_display(struct widget *w)
 	}
 }
 
+//
+// Widgets update callbacks
+//
+
+static void _enable_if_quest_browser_open(struct widget *w)
+{
+	w->enabled = quest_browser_activated;
+}
+
+static void _activate_if_quest_browser_changed(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = Me.quest_browser_changed;
+}
+
+static void _activate_if_points_to_distribute(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = (Me.points_to_distribute > 0);
+}
+
+static void _enable_if_skill_help_panel_open(struct widget *w)
+{
+	w->enabled = GameConfig.skill_explanation_screen_visible;
+}
+
+static void _enable_if_char_panel_open(struct widget *w)
+{
+	w->enabled = GameConfig.CharacterScreen_Visible;
+}
+
+static void _enable_if_skill_panel_open(struct widget *w)
+{
+	w->enabled = GameConfig.SkillScreen_Visible;
+}
+
+static void _enable_if_map_covered(struct widget *w)
+{
+	w->enabled = (addon_crafting_ui_visible() || item_upgrade_ui_visible());
+}
+
+static void _enable_if_addon_crafting_panel_open(struct widget *w)
+{
+	w->enabled = addon_crafting_ui_visible();
+}
+
+static void _enable_if_item_upgrade_panel_open(struct widget *w)
+{
+	w->enabled = item_upgrade_ui_visible();
+}
+
+static void _enable_if_inventory_panel_open(struct widget *w)
+{
+	w->enabled = GameConfig.Inventory_Visible;
+}
+
 /**
  * This function builds the hud bar widgets.
  */
@@ -390,43 +445,43 @@ static struct widget_group *create_hud_bar()
 	WIDGET(panel)->rect = WIDGET(hud_bar)->rect;
 
 	// Fixed size tiles
-	struct image *img_1 = widget_load_image_resource("widgets/hud_background_1.png", 0);
+	struct image *img_1 = widget_load_image_resource("widgets/hud_background_1.png", NO_MOD);
 	int left_panel_x = WIDGET(panel)->rect.x;
-	widget_background_add(panel, img_1, left_panel_x, WIDGET(panel)->rect.y, img_1->w, img_1->h);
+	widget_background_add(panel, img_1, left_panel_x, WIDGET(panel)->rect.y, img_1->w, img_1->h, 0);
 
-	struct image *img_3 = widget_load_image_resource("widgets/hud_background_3.png", 0);
+	struct image *img_3 = widget_load_image_resource("widgets/hud_background_3.png", NO_MOD);
 	int center_panel_x = WIDGET(panel)->rect.x + (WIDGET(panel)->rect.w - img_3->w) / 2;
-	widget_background_add(panel, img_3, center_panel_x, WIDGET(panel)->rect.y, img_3->w, img_3->h);
+	widget_background_add(panel, img_3, center_panel_x, WIDGET(panel)->rect.y, img_3->w, img_3->h, 0);
 
-	struct image *img_5 = widget_load_image_resource("widgets/hud_background_5.png", 0);
+	struct image *img_5 = widget_load_image_resource("widgets/hud_background_5.png", NO_MOD);
 	int right_panel_x = WIDGET(panel)->rect.x + WIDGET(panel)->rect.w - img_5->w;
-	widget_background_add(panel, img_5, right_panel_x, WIDGET(panel)->rect.y, img_5->w, img_5->h);
+	widget_background_add(panel, img_5, right_panel_x, WIDGET(panel)->rect.y, img_5->w, img_5->h, 0);
 
 	// Scalable tiles.
 	// These tiles will expand to fill the gap between the 3 fixed tiles.
-	struct image *img_2 = widget_load_image_resource("widgets/hud_background_2.png", 0);
+	struct image *img_2 = widget_load_image_resource("widgets/hud_background_2.png", NO_MOD);
 	int left_scaling_panel_x = left_panel_x + img_1->w;
 	int fill = center_panel_x - left_scaling_panel_x;
 	if (fill < 0)
-		ErrorMessage(__FUNCTION__,
+		error_message(__FUNCTION__,
 			"No space left to place the following background spacer image: %s.\n\n"
 			"You need to run the game in a higher resolution mode",
-			PLEASE_INFORM, IS_FATAL,
+			PLEASE_INFORM | IS_FATAL,
 			"widgets/hud_background_2.png");
 	if (fill > 0)
-		widget_background_add(panel, img_2, left_scaling_panel_x, WIDGET(panel)->rect.y, fill, img_2->h);
+		widget_background_add(panel, img_2, left_scaling_panel_x, WIDGET(panel)->rect.y, fill, img_2->h, 0);
 
-	struct image *img_4 = widget_load_image_resource("widgets/hud_background_4.png", 0);
+	struct image *img_4 = widget_load_image_resource("widgets/hud_background_4.png", NO_MOD);
 	int right_scaling_panel_x = center_panel_x + img_3->w;
 	fill = right_panel_x - right_scaling_panel_x;
 	if (fill < 0)
-		ErrorMessage(__FUNCTION__,
+		error_message(__FUNCTION__,
 			"No space left to place the following background spacer image: %s.\n\n"
 			"You need to run the game in a higher resolution mode",
-			PLEASE_INFORM, IS_FATAL,
+			PLEASE_INFORM | IS_FATAL,
 			"widgets/hud_background_4.png");
 	if (fill > 0)
-		widget_background_add(panel, img_4, right_scaling_panel_x, WIDGET(panel)->rect.y, fill, img_4->h);
+		widget_background_add(panel, img_4, right_scaling_panel_x, WIDGET(panel)->rect.y, fill, img_4->h, 0);
 
 	// item alarm
 	// This widget must be added before the hud bar background.
@@ -459,7 +514,7 @@ static struct widget_group *create_hud_bar()
 			{{NULL}},
 			{left_panel_x + 40, WIDGET(panel)->rect.y + 25, 56, 56},
 			NULL,
-			(void *)TuxReloadWeapon,
+			(void(*)(struct widget_button *))TuxReloadWeapon,
 			get_current_weapon_button_tooltip,
 			current_weapon_button_update
 		},
@@ -469,9 +524,9 @@ static struct widget_group *create_hud_bar()
 			{"mouse_buttons/log_button_red.png","mouse_buttons/log_button_yellow.png", NULL}},
 			{center_panel_x + 55, WIDGET(panel)->rect.y + 49, 32, 19},
 			NULL,
-			(void *)toggle_quest_browser,
+			(void(*)(struct widget_button *))toggle_quest_browser,
 			NULL,
-			WIDGET_UPDATE_FLAG_ON_DATA(WIDGET_BUTTON, active, Me.quest_browser_changed)
+			_activate_if_quest_browser_changed
 		},
 		// Inventory button
 		{
@@ -491,7 +546,7 @@ static struct widget_group *create_hud_bar()
 			NULL,
 			toggle_character_screen,
 			NULL,
-			WIDGET_UPDATE_FLAG_ON_DATA(WIDGET_BUTTON, active,(Me.points_to_distribute > 0))
+			_activate_if_points_to_distribute
 		},
 		// Skill button
 		{
@@ -536,19 +591,20 @@ static struct widget_group *create_hud_bar()
 			{right_panel_x + 169, WIDGET(panel)->rect.y + 11, 6, 50},
 			heat_bar_display,
 			NULL,
-			get_heat_bar_tooltip, NULL
+			get_heat_bar_tooltip,
+			NULL
 		},
 	};
 
 	int i, j, k;
 	for (i = 0; i < sizeof(b) / sizeof(b[0]); i++) {
-		struct widget_button * wb = widget_button_create();
+		struct widget_button *wb = widget_button_create();
 
 		// Load button images
 		for (j = 0; j < 2; j++)
 			for (k = 0; k < 3; k++)
 				if (b[i].image[j][k])
-					wb->image[j][k] = widget_load_image_resource(b[i].image[j][k], 0);
+					wb->image[j][k] = widget_load_image_resource(b[i].image[j][k], NO_MOD);
 
 		widget_set_rect(WIDGET(wb), b[i].rect.x, b[i].rect.y, b[i].rect.w, b[i].rect.h);
 
@@ -570,13 +626,13 @@ static struct widget_group *create_hud_bar()
 	// Ammo indicator
 	struct widget_text *ammo = widget_text_create();
 	widget_set_rect(WIDGET(ammo), left_panel_x + 36, WIDGET(panel)->rect.y + 77, 100, 22);
-	ammo->font = FPS_Display_BFont;
+	ammo->font = FPS_Display_Font;
 	WIDGET(ammo)->update = current_ammo_update;
 
 	// Message log.
 	init_message_log();
-	widget_set_rect(WIDGET(message_log), 120,  WIDGET(panel)->rect.y + 7, 415, 46);
-	message_log->font = Messagevar_BFont;
+	widget_set_rect(WIDGET(message_log), left_scaling_panel_x - 51,  WIDGET(panel)->rect.y + 7, right_panel_x - left_scaling_panel_x + 102, 46);
+	message_log->font = Messagevar_Font;
 	widget_group_add(hud_bar, WIDGET(message_log));
 
 	widget_group_add(hud_bar, WIDGET(ammo));
@@ -599,6 +655,7 @@ struct widget_group *get_game_ui()
 	// Create map widget.
 	// This widget is required to know when a mouse event should be handled
 	// by AnalyzePlayerMouseClicks.
+	// TODO: this should definitely not be a widget_button !
 	game_map = widget_button_create();
 	widget_set_rect(WIDGET(game_map), 0, 0, GameConfig.screen_width, GameConfig.screen_height - 98); 
 	widget_group_add(game_widget_group, WIDGET(game_map));
@@ -615,34 +672,34 @@ struct widget_group *get_game_ui()
 		{
 			{0, 0, 320, 480},
 			NULL,
-			WIDGET_UPDATE_FLAG_ON_DATA(WIDGET, enabled, GameConfig.skill_explanation_screen_visible)
+			_enable_if_skill_help_panel_open
 		},
 		{
 			{GameConfig.screen_width - 320, 0, 320, 480},
 			(void *)ShowCharacterScreen,
-			WIDGET_UPDATE_FLAG_ON_DATA(WIDGET, enabled, GameConfig.CharacterScreen_Visible)
+			_enable_if_char_panel_open
 		},
 		{
 			{GameConfig.screen_width - 320, 0, 320, 480},
 			(void *)ShowSkillsScreen,
-			WIDGET_UPDATE_FLAG_ON_DATA(WIDGET, enabled, GameConfig.SkillScreen_Visible)
+			_enable_if_skill_panel_open
 		},
 		// This widget, covering the whole screen, avoid mouse event propagation to the other widgets
 		// when the addon crafting or item upgrade panels are opened.
 		{
 			{0, 0, GameConfig.screen_width, GameConfig.screen_height},
 			NULL,
-			WIDGET_UPDATE_FLAG_ON_DATA(WIDGET, enabled, (addon_crafting_ui_visible() || item_upgrade_ui_visible()))
+			_enable_if_map_covered
 		},
 		{
 			{0, 0, 320, 480},
 			(void *)show_addon_crafting_ui,
-			WIDGET_UPDATE_FLAG_ON_DATA(WIDGET, enabled, addon_crafting_ui_visible())
+			_enable_if_addon_crafting_panel_open
 		},
 		{
 			{320, 0, 320, 480},
 			(void *)show_item_upgrade_ui,
-			WIDGET_UPDATE_FLAG_ON_DATA(WIDGET, enabled, item_upgrade_ui_visible())
+			_enable_if_item_upgrade_panel_open
 		},
 		// The item held in hand is displayed by show_inventory_screen().
 		// In order to have this item displayed in front of all widgets,
@@ -650,7 +707,7 @@ struct widget_group *get_game_ui()
 		{
 			{0, 0, 320, 480},
 			(void *)show_inventory_screen,
-			WIDGET_UPDATE_FLAG_ON_DATA(WIDGET, enabled, GameConfig.Inventory_Visible)
+			_enable_if_inventory_panel_open
 		}
 	};
 
@@ -665,8 +722,22 @@ struct widget_group *get_game_ui()
 
 	// Create the quest browser.
 	struct widget_group *quest_browser = create_quest_browser();
-	WIDGET(quest_browser)->update = WIDGET_UPDATE_FLAG_ON_DATA(WIDGET, enabled, quest_browser_activated);
+	WIDGET(quest_browser)->update = _enable_if_quest_browser_open;
 	widget_group_add(game_widget_group, WIDGET(quest_browser));
 
+	// Create the chat dialog.
+	// TODO: to be inserted in the global UI tree when everything is fully widgetized.
+	create_chat_dialog();
+
 	return game_widget_group;
 }
+
+void free_game_ui()
+{
+	if (game_widget_group)
+	{
+		struct widget *w = WIDGET(game_widget_group);
+		w->free(w);
+		free(game_widget_group);
+	}
+}
