@@ -32,6 +32,7 @@ import sys, re
 dump_structs = [
                  "gps", "point", "moderately_finepoint", "finepoint",
                  "tux_t", "item", "enemy", "bullet", "blast",
+                 "obstacle", "volatile_obstacle",
                  "spell_active", "melee_shot", "mission", "npc",
                  "upgrade_socket", "keybind_t", "configuration_for_freedroid"
                ]
@@ -218,7 +219,7 @@ def main():
 
     # Writing loop
 
-    for s_name in data.keys():
+    for s_name in sorted(data.keys()):
 
         func_name = s_name.replace('struct ', '')
 
@@ -231,14 +232,14 @@ def main():
 
         write_str  = 'void write_%s(struct auto_string *strout, %s *data)\n'  % (func_name, s_name)
         write_str += '{\n'
-        write_str += '    autostr_append(strout, "{\\n");\n'
+        write_str += '    autostr_append(strout, "{\\n" '
         for (type, size, field) in data[s_name]:
             if not '*' in type:
                 # Pointers are not saved (it does not make sense).
-                write_str += '    autostr_append(strout, "%s = ");\n' % field
+                write_str += '"%s = ");\n' % field
                 write_str += '    write_%s(strout, %sdata->%s%s);\n' % (type, '' if size else '&', field, (', %s' % size) if size else '')
-                write_str += '    autostr_append(strout, \",\\n\");\n'
-        write_str += '    autostr_append(strout, "}");\n'
+                write_str += '    autostr_append(strout, ",\\n" '
+        write_str += '"}");\n'
         write_str += '}\n\n'
 
         output_c.write(write_str)
@@ -265,7 +266,7 @@ def main():
     header_str = ""
     impl_str = ""
 
-    for s_name in tracked_types.keys():
+    for s_name in sorted(tracked_types.keys()):
 
         if '_array' in s_name:
 
