@@ -24,14 +24,12 @@
  */
 /**
  * This file contains graphics primitives, such as initialisation of SDL
- * and video modes and fonts.
- */
-/*
- * This file has been checked for remains of german comments in the code
- * I you still find some, please just kill it mercilessly.
+ * and video modes.
  */
 #define _graphics_c
 
+#include "pngfuncs.h"
+
 #include "system.h"
 
 #include "defs.h"
@@ -61,7 +59,7 @@ void blit_mouse_cursor(void)
 	if (!loaded) {
 		for (i = 0; i < 10; i++) {
 			sprintf(constructed_filename, "cursors/mouse_cursor_%04d.png", i);
-			load_image(&mouse_cursors[i], constructed_filename, FALSE);
+			load_image(&mouse_cursors[i], constructed_filename, NO_MOD);
 		}
 		loaded = TRUE;
 	}
@@ -74,6 +72,7 @@ void blit_mouse_cursor(void)
 	case MOUSE_CURSOR_SCROLL_DOWN:
 		cursor_index = 5;
 		cursoff.x = -12;
+		cursoff.y = -30;
 		break;
 	case MOUSE_CURSOR_NORMAL:
 		cursor_index = 0;
@@ -92,8 +91,8 @@ void blit_mouse_cursor(void)
 		cursor_index = 3;
 		break;
 	default:
-		ErrorMessage(__FUNCTION__, "Illegal mouse cursor encountered: %d",
-					 PLEASE_INFORM, IS_FATAL, mouse_cursor);
+		error_message(__FUNCTION__, "Illegal mouse cursor encountered: %d",
+					 PLEASE_INFORM | IS_FATAL, mouse_cursor);
 		break;
 	}
 
@@ -141,6 +140,9 @@ static void fade(int fade_delay, int direction)
 		our_SDL_flip_wrapper();
 		now = SDL_GetTicks();
 	}
+
+	if (!use_open_gl && bg)
+		SDL_FreeSurface(bg);
 }
 
 /**
@@ -191,20 +193,22 @@ int do_graphical_number_selection_in_range(int lower_range, int upper_range, int
 	int old_knob_at = 0;
 	SDL_Event event;
 	SDL_Rect knob_target_rect;
+	int wait_before_repeat = 0; // 0: apply once only - 1: apply once and wait before repeat - 2: repeat after delay
+	Uint32 time_before_repeat = 0;
 
 	/* Initialize the text widget. */
 	static struct widget_text item_description;
 	widget_text_init(&item_description, "");
 	widget_set_rect(WIDGET(&item_description), UNIVERSAL_COORD_W(310), UNIVERSAL_COORD_H(180), UNIVERSAL_COORD_W(75), UNIVERSAL_COORD_H(45));
-	item_description.font = FPS_Display_BFont;
-	item_description.line_height_factor = LINE_HEIGHT_FACTOR;
+	item_description.font = FPS_Display_Font;
+	item_description.line_height_factor = 1.0;
 
 	int old_game_status = game_status;
 
 	StoreMenuBackground(1);
 
 	if (!image_loaded(&selection_knob)) {
-		load_image(&selection_knob, "mouse_buttons/number_selector_selection_knob.png", FALSE);
+		load_image(&selection_knob, "mouse_buttons/number_selector_selection_knob.png", NO_MOD);
 	}
 
 	knob_target_rect.w = selection_knob.w;
@@ -224,6 +228,7 @@ int do_graphical_number_selection_in_range(int lower_range, int upper_range, int
 			item_description.text = alloc_autostr(100);
 			autostr_append(item_description.text, "%d\n", knob_at);
 			if (unit_price) {
+				// TRANSLATORS: Used on object selector's widget to display unit price * number of objects
 				autostr_append(item_description.text, _("%d price\n"), unit_price * knob_at);
 			}
 			old_knob_at = knob_at;
@@ -237,14 +242,16 @@ int do_graphical_number_selection_in_range(int lower_range, int upper_range, int
 		while (SDL_PollEvent(&event)) {
 
 			if (event.type == SDL_QUIT) {
-				Terminate(EXIT_SUCCESS, TRUE);
+				Terminate(EXIT_SUCCESS);
 			} else if (event.type == SDL_KEYDOWN) {
 				switch(event.key.keysym.sym) {
 					case SDLK_LEFT:
 						delta = -1;
+						wait_before_repeat = 0; // apply once only
 						break;
 					case SDLK_RIGHT:
 						delta = 1;
+						wait_before_repeat = 0; // apply once only
 						break;
 					case SDLK_DOWN:
 						knob_at = lower_range;
@@ -282,8 +289,10 @@ int do_graphical_number_selection_in_range(int lower_range, int upper_range, int
 						ok_button_was_pressed = TRUE;
 					} else if (MouseCursorIsOnButton(NUMBER_SELECTOR_LEFT_BUTTON, event.button.x, event.button.y)) {
 						delta = -1;
+						wait_before_repeat = 1; // apply once and wait before repeat
 					} else if (MouseCursorIsOnButton(NUMBER_SELECTOR_RIGHT_BUTTON, event.button.x, event.button.y)) {
 						delta = 1;
+						wait_before_repeat = 1; // apply once and wait before repeat
 					}
 				}
 			} else if (event.type == SDL_MOUSEBUTTONUP) {
@@ -309,8 +318,17 @@ int do_graphical_number_selection_in_range(int lower_range, int upper_range, int
 		}
 
 		if (delta != 0) {
-			knob_at = max(lower_range, min(upper_range, knob_at + delta));
-			SDL_Delay(80);
+			if (wait_before_repeat == 0) { // apply once only (when numkeys are used to change the value)
+				knob_at = max(lower_range, min(upper_range, knob_at + delta));
+				delta = 0;
+			} else if (wait_before_repeat == 1) { // apply once and wait before repeat
+				knob_at = max(lower_range, min(upper_range, knob_at + delta));
+				wait_before_repeat = 2; // repeat after delay
+				time_before_repeat = SDL_GetTicks() + 500; // 500ms before starting to repeat
+			} else if ((wait_before_repeat == 2) && (SDL_GetTicks() > time_before_repeat)) { // repeat apply
+				knob_at = max(lower_range, min(upper_range, knob_at + delta));
+				SDL_Delay(50); // 50ms interval between 2 repeats
+			}
 		}
 		
 		knob_offset_x = (knob_at - lower_range) * knob_step_size;
@@ -321,6 +339,43 @@ int do_graphical_number_selection_in_range(int lower_range, int upper_range, int
 
 };				// int do_graphical_number_selection_in_range ( int lower_range , int upper_range )
 
+/**
+ * Return the pixel value at (x, y).
+ *
+ * NOTE: The surface must be locked before calling this!
+ *
+ * @param surf	The surface where we want to retrieve the pixel.
+ * @param x	The X position of the pixel.
+ * @param y	The Y position of the pixel.
+ * @return The pixel value at (x, y) on success, zero on failure.
+ */
+uint32_t sdl_get_pixel(SDL_Surface *surf, int x, int y)
+{
+	int bpp = surf->format->BytesPerPixel;
+	uint8_t *p = (uint8_t *)surf->pixels + y * surf->pitch + x * bpp;
+
+	switch (bpp) {
+	case 1:
+		return *p;
+		break;
+	case 2:
+		return *(uint16_t *)p;
+		break;
+	case 3:
+		if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
+			return p[0] << 16 | p[1] << 8 | p[2];
+		else
+			return p[0] | p[1] << 8 | p[2] << 16;
+		break;
+	case 4:
+		return *(uint32_t *)p;
+		break;
+	default:
+		/* shouldn't happen, but avoids warnings */
+		return 0;
+	}
+}
+
 void sdl_put_pixel(SDL_Surface *surf, int x, int y, uint8_t red, uint8_t green, uint8_t blue, uint8_t alpha)
 {
 	int bpp = surf->format->BytesPerPixel;
@@ -468,6 +523,7 @@ void InitPictures(void)
 
 	if (!GameConfig.lazyload) {
 		load_all_items();
+		// maybe also load all enemy models here?
 	}
 
 	Load_Blast_Surfaces();
@@ -481,68 +537,6 @@ void InitPictures(void)
 	next_startup_percentage(3);
 }
 
-/**
- * This function should load all the fonts we'll be using via the SDL
- * BFont library in FreedroidRPG.
- */
-void InitOurBFonts(void)
-{
-#define ALL_BFONTS_WE_LOAD 7
-
-#define PARA_FONT_FILE 		"font/parafont"
-#define MENU_FONT_FILE 		"font/cpuFont"
-#define MESSAGEVAR_FONT_FILE 	"font/small_white"
-#define MESSAGESTAT_FONT_FILE 	"font/small_blue"
-#define RED_FONT_FILE 		"font/font05_red"
-#define BLUE_FONT_FILE 		"font/font05_white"
-#define FPS_FONT_FILE 		"font/font05"
-
-	char fpath[2048];
-	int i;
-	const char *MenuFontFiles[ALL_BFONTS_WE_LOAD] = {
-		MENU_FONT_FILE,
-		MESSAGEVAR_FONT_FILE,
-		MESSAGESTAT_FONT_FILE,
-		PARA_FONT_FILE,
-		FPS_FONT_FILE,
-		RED_FONT_FILE,
-		BLUE_FONT_FILE,
-	};
-	BFont_Info **MenuFontPointers[ALL_BFONTS_WE_LOAD] = {
-		&Menu_BFont,
-		&Messagevar_BFont,
-		&Messagestat_BFont,
-		&Para_BFont,
-		&FPS_Display_BFont,
-		&Red_BFont,
-		&Blue_BFont
-	};
-
-	for (i = 0; i < ALL_BFONTS_WE_LOAD; i++) {
-		char constructed_fname[2048];
-		sprintf(constructed_fname, "%s", MenuFontFiles[i]);
-		strcat(constructed_fname, ".png");
-
-		if (find_file(constructed_fname, GRAPHICS_DIR, fpath, 0) != 0) {	//if the file wasn't found, default to the standard ASCII7bit file
-			sprintf(constructed_fname, "%s.png", MenuFontFiles[i]);
-			if (find_file(constructed_fname, GRAPHICS_DIR, fpath, 0) != 0) {
-				fprintf(stderr, "\n\nFont file: '%s'.\n", MenuFontFiles[i]);
-				ErrorMessage(__FUNCTION__, "\
-A font file for the BFont library was not found.", PLEASE_INFORM, IS_FATAL);
-			}
-		}
-
-		if ((*MenuFontPointers[i] = LoadFont(constructed_fname)) == NULL) {
-			fprintf(stderr, "\n\nFont file: '%s'.\n", MenuFontFiles[i]);
-			ErrorMessage(__FUNCTION__, "\
-A font file for the BFont library could not be loaded.", PLEASE_INFORM, IS_FATAL);
-		} else {
-			DebugPrintf(1, "\nSDL Menu Font initialisation successful.\n");
-		}
-	}
-
-};				// InitOurBFonts ( void )
-
 /**
  * This function initializes the timer subsystem.
  */
@@ -552,7 +546,7 @@ void init_timer(void)
 	//
 	if (SDL_InitSubSystem(SDL_INIT_TIMER) == -1) {
 		fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
-		Terminate(EXIT_FAILURE, FALSE);
+		Terminate(EXIT_FAILURE);
 	}
 }
 
@@ -602,9 +596,9 @@ static void check_open_gl_libraries_present(void)
  * and that seem to be occurring so frequently are not coming from this 
  * chunk of code.
  */
+#ifdef HAVE_LIBGL
 static void show_open_gl_driver_info(void)
 {
-#ifdef HAVE_LIBGL
 	// Since we want to use openGl, it might be good to check the OpenGL vendor string
 	// provided by the graphics driver.  Let's see...
 	//
@@ -618,33 +612,29 @@ static void show_open_gl_driver_info(void)
 	fprintf(stderr, "\nExtensions : %s", glGetString(GL_EXTENSIONS));
 	open_gl_check_error_status(__FUNCTION__);
 	fprintf(stderr, "\n\n");
+}
 #endif
-};				// void safely_show_open_gl_driver_info ( void )
 
 /**
  * This function sets the OpenGL double buffering attribute.  We do this
  * in a separate function, so that eventual errors (and bug reports) from
  * the OpenGL error checking can be attributed to a source more easily.
  */
+#ifdef HAVE_LIBGL
 static void set_double_buffering_attribute(void)
 {
-
-#ifdef HAVE_LIBGL
-
 	if (SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1)) {
-		ErrorMessage(__FUNCTION__, "\
-Unable to set SDL_GL_DOUBLEBUFFER attribute!", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "\
+Unable to set SDL_GL_DOUBLEBUFFER attribute!", PLEASE_INFORM | IS_FATAL);
 	}
 	// Since the OpenGL stuff hasn't been initialized yet, it's normal
 	// to get an GL_INVALID_OPERATION here, if we would really do the
 	// check.  So better refrain from OpenGL error checking here...
 	//
 	// open_gl_check_error_status ( __FUNCTION__ );
-
+}
 #endif
 
-};				// void safely_set_double_buffering_attribute ( void )
-
 /**
  * This function is supposed to set the video mode in the case that 
  * OpenGL is used for graphics output.  The function is highly split up
@@ -656,6 +646,7 @@ Unable to set SDL_GL_DOUBLEBUFFER attribute!", PLEASE_INFORM, IS_FATAL);
 static void set_video_mode_for_open_gl(void)
 {
 #ifdef HAVE_LIBGL
+
 	Uint32 video_flags = 0;	// flags for SDL video mode 
 	int video_mode_ok_check_result;
 	int buffer_size, depth_size, red_size, green_size, blue_size, alpha_size;
@@ -698,16 +689,16 @@ static void set_video_mode_for_open_gl(void)
 	video_mode_ok_check_result = SDL_VideoModeOK(GameConfig.screen_width, GameConfig.screen_height, vid_bpp, video_flags);
 	switch (video_mode_ok_check_result) {
 	case 0:
-		ErrorMessage(__FUNCTION__,
+		error_message(__FUNCTION__,
 				     "SDL reported that the video mode (%d x %d) mentioned above is not supported\n"
                      "To see all possible resolutions please run 'freedroidRPG -r99'\n"
-                     "Resetting to default resolution (%d x %d)...",
-                     NO_NEED_TO_INFORM, IS_WARNING_ONLY,
+                     "Resetting to current resolution (%d x %d)...",
+                     NO_REPORT,
                      GameConfig.screen_width, GameConfig.screen_height,
-                     DEFAULT_SCREEN_WIDTH, DEFAULT_SCREEN_HEIGHT);
-		//resetting configuration file to default settings
-		GameConfig.screen_width =  DEFAULT_SCREEN_WIDTH;
-		GameConfig.screen_height = DEFAULT_SCREEN_HEIGHT;
+                     vid_info->current_w, vid_info->current_h);
+		//resetting configuration file to current (desktop) settings
+		GameConfig.screen_width =  vid_info->current_w;
+		GameConfig.screen_height = vid_info->current_h;
 		GameConfig.next_time_width_of_screen = GameConfig.screen_width;
 		GameConfig.next_time_height_of_screen = GameConfig.screen_height;
 		break;
@@ -721,7 +712,7 @@ static void set_video_mode_for_open_gl(void)
 			   ErrorMessage ( __FUNCTION__  , "\
 			   SDL reported, that the video mode mentioned \nabove is not supported UNDER THE COLOR DEPTH MENTIONED ABOVE!\n\
 			   We'll be using the alternate color depth given above instead...",
-			   PLEASE_INFORM, IS_WARNING_ONLY );
+			   PLEASE_INFORM );
 			 */
 			vid_bpp = video_mode_ok_check_result;
 		}
@@ -734,7 +725,7 @@ static void set_video_mode_for_open_gl(void)
 	Screen = SDL_SetVideoMode(GameConfig.screen_width, GameConfig.screen_height, vid_bpp, video_flags);
 	if (!Screen) {
 		fprintf(stderr, "Video mode set failed: %s\n", SDL_GetError());
-		Terminate(EXIT_FAILURE, FALSE);
+		Terminate(EXIT_FAILURE);
 	} else {
 		//      open_gl_check_error_status ( __FUNCTION__ );
 		SDL_GL_GetAttribute(SDL_GL_BUFFER_SIZE, &buffer_size);
@@ -766,10 +757,8 @@ static void set_video_mode_for_open_gl(void)
 	our_SDL_flip_wrapper();
 	clear_screen();
 	our_SDL_flip_wrapper();
-
-#endif				// HAVE_LIBGL
-
-};				// void set_video_mode_for_open_gl ( void )
+#endif
+}
 
 /**
  * This function initialises the video display and opens up a 
@@ -779,8 +768,7 @@ void InitVideo(void)
 {
 	char vid_driver[81];
 	Uint32 video_flags = 0;	// flags for SDL video mode 
-	char fpath[2048];
-	char window_title_string[200];
+	char fpath[PATH_MAX];
 
 	// Tell SDL to center the window once we make it
 	putenv("SDL_VIDEO_CENTERED=1");
@@ -794,7 +782,7 @@ void InitVideo(void)
 		DebugPrintf(-4, "\nVideo system type: %s.", vid_driver);
 	} else {
 		fprintf(stderr, "Video driver seems not to exist or initialization failure!\nError code: %s\n", SDL_GetError());
-		Terminate(EXIT_FAILURE, FALSE);
+		Terminate(EXIT_FAILURE);
 	}
 
 	// We check if the program has been compiled with OpenGL libraries present
@@ -812,7 +800,7 @@ void InitVideo(void)
 	vid_info = SDL_GetVideoInfo();
 	if (!vid_info) {
 		fprintf(stderr, "Could not obtain video info via SDL: %s\n", SDL_GetError());
-		Terminate(EXIT_FAILURE, FALSE);
+		Terminate(EXIT_FAILURE);
 	}
 
 	if (use_open_gl) {
@@ -823,40 +811,42 @@ void InitVideo(void)
 
 		if (!SDL_VideoModeOK(GameConfig.screen_width, GameConfig.screen_height, 32, video_flags))
 		{
-			ErrorMessage(__FUNCTION__,
+			error_message(__FUNCTION__,
 			             "SDL reported that the video mode (%d x %d) mentioned above is not supported\n"
 			             "To see all possible resolutions please run 'freedroidRPG -r99'\n"
-			             "Resetting to default resolution (%d x %d)...",
-			             NO_NEED_TO_INFORM, IS_WARNING_ONLY,
+			             "Resetting to current resolution (%d x %d)...",
+			             NO_REPORT,
 			             GameConfig.screen_width, GameConfig.screen_height,
-			             DEFAULT_SCREEN_WIDTH, DEFAULT_SCREEN_HEIGHT);
-			//resetting configuration file to default settings
-			GameConfig.screen_width =  DEFAULT_SCREEN_WIDTH;
-			GameConfig.screen_height = DEFAULT_SCREEN_HEIGHT;
+			             vid_info->current_w, vid_info->current_h);
+			//resetting configuration file to current (desktop) settings
+			GameConfig.screen_width =  vid_info->current_w;
+			GameConfig.screen_height = vid_info->current_h;
 			GameConfig.next_time_width_of_screen = GameConfig.screen_width;
 			GameConfig.next_time_height_of_screen = GameConfig.screen_height;
 		}
 		if (!(Screen = SDL_SetVideoMode(GameConfig.screen_width, GameConfig.screen_height, 0, video_flags))) {
 			fprintf(stderr, "Video mode set failed: %s\n", SDL_GetError());
-			Terminate(EXIT_FAILURE, FALSE);
+			Terminate(EXIT_FAILURE);
 		}
 	}
 
 	vid_bpp = 32;		/* start with the simplest */
 
-	// End of possibly open-gl dependant initialisation stuff...
+	// End of possibly open-gl dependent initialisation stuff...
 	//
-	sprintf(window_title_string, "FreedroidRPG %s", VERSION);
 	if (vid_info->wm_available) {	/* if there's a window-manager */
-		SDL_Surface *icon;
+		char window_title_string[200];
+		sprintf(window_title_string, "FreedroidRPG %s", VERSION);
 		SDL_WM_SetCaption(window_title_string, "");
-		find_file(ICON_FILE, GRAPHICS_DIR, fpath, 0);
-		icon = IMG_Load(fpath);
-		SDL_WM_SetIcon(icon, NULL);
-		SDL_FreeSurface(icon);
+
+		if (find_file(ICON_FILE, GRAPHICS_DIR, fpath, PLEASE_INFORM)) {
+			SDL_Surface *icon = IMG_Load(fpath);
+			SDL_WM_SetIcon(icon, NULL);
+			SDL_FreeSurface(icon);
+		}
 	}
 
-	InitOurBFonts();
+	init_fonts();
 
 	blit_background("startup1.jpg");
 	our_SDL_flip_wrapper();
@@ -1078,28 +1068,23 @@ static void draw_line_opengl(int x1, int y1, int x2, int y2, int r, int g, int b
 #endif
 }
 
-void draw_line(float x1, float y1, float x2, float y2, Uint32 color, int width)
+void draw_line(float x1, float y1, float x2, float y2, uint8_t r, uint8_t g, uint8_t b, int width)
 {
-	float rr, gg, bb;
-	rr = (color & 0xff0000) >> 16;
-	gg = (color & 0xff00) >> 8;
-	bb = color & 0xff;
-
 	if (!use_open_gl) {
-		draw_line_sdl(Screen, x1, y1, x2, y2, rr, gg, bb, width);
+		draw_line_sdl(Screen, x1, y1, x2, y2, r, g, b, width);
 	} else {
-		draw_line_opengl(x1, y1, x2, y2, rr, gg, bb, width);
+		draw_line_opengl(x1, y1, x2, y2, r, g, b, width);
 	}
 }
 
-void draw_line_on_map(float x1, float y1, float x2, float y2, Uint32 color, int width)
+void draw_line_on_map(float x1, float y1, float x2, float y2, uint8_t r, uint8_t g, uint8_t b, int width)
 {
 	int c1, c2, r1, r2;
 
 	translate_map_point_to_screen_pixel(x1, y1, &r1, &c1);
 	translate_map_point_to_screen_pixel(x2, y2, &r2, &c2);
 
-	draw_line(r1, c1, r2, c2, color, width);
+	draw_line(r1, c1, r2, c2, r, g, b, width);
 }
 
 /**
@@ -1109,43 +1094,42 @@ void draw_line_on_map(float x1, float y1, float x2, float y2, Uint32 color, int
 void save_screenshot(const char *filename, int width)
 {
 	Activate_Conservative_Frame_Computation();
-	SDL_Surface *NewThumbnail = NULL;
-	SDL_Surface *FullView = Screen;
-	void *imgdata = malloc((GameConfig.screen_width + 2) * (GameConfig.screen_height + 2) * 4);
-	float scale_factor = 1;
-	if (width)
-		scale_factor = (float) width / GameConfig.screen_width;
+	float scale_factor = width ? (float) width / GameConfig.screen_width : 1.0;
+	SDL_Surface *screenshot = NULL;
+
 #ifdef HAVE_LIBGL
 	if (use_open_gl) {
-		// We need to make a copy in processor memory. 
-		glReadPixels(0, 1, GameConfig.screen_width, GameConfig.screen_height - 1, GL_RGB, GL_UNSIGNED_BYTE, imgdata);
+		void *imgdata = malloc(GameConfig.screen_width * GameConfig.screen_height * 3);
+		glPixelStorei(GL_PACK_ALIGNMENT, 1);
+		glReadPixels(0, 0, GameConfig.screen_width, GameConfig.screen_height, GL_RGB, GL_UNSIGNED_BYTE, imgdata);
 
 		// Now we need to make a real SDL surface from the raw image data we
 		// have just extracted.
 		//
-		SDL_FreeSurface(FullView);
-		FullView =
+		SDL_Surface *screen_surf =
 		    SDL_CreateRGBSurfaceFrom(imgdata, GameConfig.screen_width, GameConfig.screen_height, 24, 3 * GameConfig.screen_width,
 					     bmask, gmask, rmask, 0);
-
+		screenshot = zoomSurface(screen_surf, scale_factor, scale_factor, 0);
+		SDL_FreeSurface(screen_surf);
+		free(imgdata);
 	}
 #endif
+	if (!use_open_gl)
+		screenshot = zoomSurface(Screen, scale_factor, scale_factor, 0);
 
-	NewThumbnail = zoomSurface(FullView, scale_factor, scale_factor, 0);
-	free(imgdata);
-	SDL_FreeSurface(FullView);
-	if (NewThumbnail == NULL) {
-		ErrorMessage(__FUNCTION__, "Cannot save image: %s\n", PLEASE_INFORM, IS_WARNING_ONLY, filename);
+	if (screenshot == NULL) {
+		error_message(__FUNCTION__, "Cannot save image: %s", PLEASE_INFORM, filename);
 		return;
 	}
 
 #ifdef HAVE_LIBGL
 	if (use_open_gl) {
-		flip_image_vertically(NewThumbnail);
+		flip_image_vertically(screenshot);
 	}
 #endif
-	SDL_SaveBMP(NewThumbnail, filename);
-	SDL_FreeSurface(NewThumbnail);
+
+	png_save_surface(filename, screenshot);
+	SDL_FreeSurface(screenshot);
 }
 
 void reload_graphics(void)
