@@ -38,9 +38,7 @@
 #include "global.h"
 #include "widgets/widgets.h"
 
-#define TEXT_BANNER_DEFAULT_FONT FPS_Display_BFont
-
-int best_banner_pos_x, best_banner_pos_y;
+#define TEXT_BANNER_DEFAULT_FONT FPS_Display_Font
 
 /**
  * The HUD contains several status graphs.  These graphs appear as 
@@ -56,10 +54,15 @@ void blit_vertical_status_bar(float max_value, float current_value, Uint32 fille
 	SDL_Rect un_running_power_rect;
 	uint8_t r, g, b, a;
 
-	if (!max_value) {
-		max_value = 1;
-		current_value = 1;
+	// Preconditions. Can happen when Tux dies, for instance.
+	if (max_value < 0 || current_value < 0)
+		return;
+
+	if (max_value == 0.0) {
+		max_value = 1.0;
+		current_value = 1.0;
 	}
+
 	// Now we might get the case of current value exceeding the max value by far.  This 
 	// does happen when we set ridiculously high energy values for invincible Tux in the
 	// course of testing and convenient debugging.  To prevent arithmetic exceptions, we
@@ -110,9 +113,9 @@ void append_item_description(struct auto_string *str, item *item)
 		return;
 
 	if (item->type == (-1)) {
-		ErrorMessage(__FUNCTION__, "\
+		error_message(__FUNCTION__, "\
 An item description was requested for an item, that does not seem to \n\
-exist really (i.e. has a type = (-1) ).", PLEASE_INFORM, IS_FATAL);
+exist really (i.e. has a type = (-1) ).", PLEASE_INFORM | IS_FATAL);
 		return;
 	}
 
@@ -120,50 +123,60 @@ exist really (i.e. has a type = (-1) ).", PLEASE_INFORM, IS_FATAL);
 	append_item_name(item, str);
 
 	// We don't want any more information for Valuable Circuits
-	if (MatchItemWithName(item->type, "Valuable Circuits"))
+	if (item_spec_eq_id(item->type, "Valuable Circuits"))
 		return;
 	
 	autostr_append(str, "\n");
 
 	// Weapon damage
-	if (ItemMap[item->type].item_can_be_installed_in_weapon_slot) {
-		autostr_append(str, _("Damage: %d"), item->damage);
-		if (item->damage_modifier)
-			autostr_append(str, _(" to %d"), item->damage_modifier + item->damage);
-		autostr_append(str, "\n");
+	if (ItemMap[item->type].slot == WEAPON_SLOT) {
+		if (item->damage_modifier) {
+			// TRANSLATORS: On item description: range
+			autostr_append(str, _("Damage: %d to %d\n"), item->damage, item->damage_modifier + item->damage);
+		} else {
+			// TRANSLATORS: On item description
+			autostr_append(str, _("Damage: %d\n"), item->damage);
+		}
 	}
 	// Multiplicity
 	if (ItemMap[item->type].item_group_together_in_inventory) {
+		// TRANSLATORS: On item description
 		autostr_append(str, _("Multiplicity: %d\n"), item->multiplicity);
 	}
 	// Armor bonus
 	if (item->armor_class) {
-			autostr_append(str, _("Armor: %d\n"), item->armor_class);
+		// TRANSLATORS: On item description: Armor bonus
+		autostr_append(str, _("Armor: %d\n"), item->armor_class);
 	}
 	// Durability or indestructible status
 	if (item->max_durability != (-1)) {
+		// TRANSLATORS: On item description: 'current' of 'maximum'
 		autostr_append(str, _("Durability: %d of %d\n"), (int)item->current_durability, (int)item->max_durability);
 	} else if (ItemMap[item->type].base_item_durability != (-1)) {
+		// TRANSLATORS: On item description : durability
 		autostr_append(str, _("Indestructible\n"));
 	}
 	// Ranged weapon ammunition
-	if (ItemMap[item->type].item_gun_ammo_clip_size) {
-		autostr_append(str, _("Ammo: %d of %d\n"), item->ammo_clip, ItemMap[item->type].item_gun_ammo_clip_size);
+	if (ItemMap[item->type].weapon_ammo_type && ItemMap[item->type].weapon_ammo_clip_size) {
+		// TRANSLATORS: On item description: 'current' of 'maximum' amount of ammo
+		autostr_append(str, _("Ammo: %d of %d\n"), item->ammo_clip, ItemMap[item->type].weapon_ammo_clip_size);
 	}
 	// Strength, dexterity or cooling requirements
 	if ((ItemMap[item->type].item_require_strength != (-1)) || (ItemMap[item->type].item_require_dexterity != (-1))) {
 		if (ItemMap[item->type].item_require_strength != (-1)) {
+			// TRANSLATORS: On item description
 			autostr_append(str, _("Required strength: %d\n"), ItemMap[item->type].item_require_strength);
 		}
 		if (ItemMap[item->type].item_require_dexterity != (-1)) {
+			// TRANSLATORS: On item description
 			autostr_append(str, _("Required dexterity: %d\n"), ItemMap[item->type].item_require_dexterity);
 		}
 	}
 
 	// Usable items should say that they can be used via right-clicking on it
-	if (ItemMap[item->type].item_combat_use_description) {
+	if (ItemMap[item->type].right_use.tooltip) {
 		if (game_status == INSIDE_GAME) {
-			autostr_append(str, "%s\n", ItemMap[item->type].item_combat_use_description);
+			autostr_append(str, "%s\n", D_(ItemMap[item->type].right_use.tooltip));
 
 			// Show text only if item is in inventory
 			if (item->inventory_position.x != -1) {
@@ -171,12 +184,13 @@ exist really (i.e. has a type = (-1) ).", PLEASE_INFORM, IS_FATAL);
 				autostr_append(str, _("Right click to use\n"));
 			}
 		} else {
-			autostr_append(str, "Item use: %s\n", ItemMap[item->type].item_combat_use_description);
+			autostr_append(str, _("Item use: %s\n"), D_(ItemMap[item->type].right_use.tooltip));
 		}
 	}
 
 	// Socket count
 	if (item->upgrade_sockets.size) {
+		// TRANSLATORS: On item description: 'used sockets' / 'number of sockets'
 		autostr_append(str, _("Sockets: used %d/%d\n"), count_used_sockets(item), 
 							     item->upgrade_sockets.size);
 	}
@@ -203,12 +217,12 @@ static void show_droid_description(enemy *cur_enemy, gps *description_pos)
 	int bar_y;
 	int bar_x;
 	SDL_Rect rect;
-	BFont_Info *BFont_to_use = Blue_BFont;
+	struct font *BFont_to_use = Blue_Font;
 	Uint8 r, g, b;
 
-	text_length = TextWidthFont(BFont_to_use, cur_enemy->short_description_text);
+	text_length = text_width(BFont_to_use, D_(cur_enemy->short_description_text));
 
-	rect.h = FontHeight(BFont_to_use);
+	rect.h = get_font_height(BFont_to_use);
 
 	// Hostile droids' bars are shown in red, friendly in green.
 	if (!is_friendly(cur_enemy->faction, FACTION_SELF)) {
@@ -225,15 +239,15 @@ static void show_droid_description(enemy *cur_enemy, gps *description_pos)
 	bar_x = translate_map_point_to_screen_pixel_x(description_pos->x, description_pos->y) - text_length / 2;
 	bar_y =
 	    translate_map_point_to_screen_pixel_y(description_pos->x,
-	    		description_pos->y) + enemy_images[Druidmap[cur_enemy->type].individual_shape_nr][0][0].offset_y -
-	    2.5 * FontHeight(BFont_to_use);
+	    		description_pos->y) + enemy_images[Droidmap[cur_enemy->type].individual_shape_nr][0][0].offset_y -
+	    2.5 * get_font_height(BFont_to_use);
 
 	// Calculates the width of the remaining health bar. Rounds the
 	// width up to the nearest integer to ensure that at least one
 	// pixel of health is always shown.
 	//
-	bar_width = (int) ceil((text_length) * (cur_enemy->energy / Druidmap[cur_enemy->type].maxenergy));
-	barc_width = (int) floor((text_length) * (1.0 - cur_enemy->energy / Druidmap[cur_enemy->type].maxenergy));
+	bar_width = (int) ceil((text_length) * (cur_enemy->energy / Droidmap[cur_enemy->type].maxenergy));
+	barc_width = (int) floor((text_length) * (1.0 - cur_enemy->energy / Droidmap[cur_enemy->type].maxenergy));
 	if (bar_width < 0)
 		bar_width = 0;
 	if (barc_width < 0)
@@ -254,7 +268,7 @@ static void show_droid_description(enemy *cur_enemy, gps *description_pos)
 
 	// Display droid's short description text
 	rect.x = translate_map_point_to_screen_pixel_x(description_pos->x, description_pos->y) - text_length / 2;
-	PutStringFont(Screen, BFont_to_use, rect.x, rect.y, cur_enemy->short_description_text);
+	put_string(BFont_to_use, rect.x, rect.y, D_(cur_enemy->short_description_text));
 }
 
 /**
@@ -262,7 +276,7 @@ static void show_droid_description(enemy *cur_enemy, gps *description_pos)
  * rectangle, possibly next to the mouse cursor, e.g. when the mouse is
  * hovering over an item or barrel or crate or teleporter.
  */
-static void prepare_text_window_content(struct auto_string *str)
+static void prepare_text_window_content(struct auto_string *str, SDL_Rect *rect)
 {
 	point CurPos;
 	point inv_square;
@@ -273,8 +287,8 @@ static void prepare_text_window_content(struct auto_string *str)
 	CurPos.x = GetMousePos_x();
 	CurPos.y = GetMousePos_y();
 
-	best_banner_pos_x = CurPos.x + 20;
-	best_banner_pos_y = CurPos.y;
+	rect->x = CurPos.x + 20;
+	rect->y = CurPos.y;
 
 	autostr_printf(str, "");
 
@@ -282,7 +296,7 @@ static void prepare_text_window_content(struct auto_string *str)
 	 * description field.  If the requirements for this item are not met, we
 	 * show a text. */
 	if (item_held_in_hand != NULL) {
-		autostr_printf(str, "%s%s", font_switchto_neon, D_(ItemMap[item_held_in_hand->type].item_name));
+		autostr_printf(str, "%s%s", font_switchto_neon, D_(item_specs_get_name(item_held_in_hand->type)));
 
 		if (!ItemUsageRequirementsMet(item_held_in_hand, FALSE)) {
 			autostr_append(str, "\n%s%s", font_switchto_red, _("REQUIREMENTS NOT MET"));
@@ -303,46 +317,46 @@ static void prepare_text_window_content(struct auto_string *str)
 			InvIndex = GetInventoryItemAt(inv_square.x, inv_square.y);
 			if (InvIndex != (-1)) {
 				append_item_description(str, &(Me.Inventory[InvIndex]));
-				best_banner_pos_x =
+				rect->x =
 				    (Me.Inventory[InvIndex].inventory_position.x +
 				     ItemMap[Me.Inventory[InvIndex].type].inv_size.x) * 30 + 16;
-				best_banner_pos_y = 300;
+				rect->y = 300;
 			}
 		} else if (MouseCursorIsOnButton(WEAPON_RECT_BUTTON, CurPos.x, CurPos.y)) {
 			if (Me.weapon_item.type > 0) {
 				append_item_description(str, &(Me.weapon_item));
-				best_banner_pos_x = WEAPON_RECT_X + 30 + WEAPON_RECT_WIDTH;
-				best_banner_pos_y = WEAPON_RECT_Y - 30;
+				rect->x = WEAPON_RECT_X + 30 + WEAPON_RECT_WIDTH;
+				rect->y = WEAPON_RECT_Y - 30;
 			}
 		} else if (MouseCursorIsOnButton(DRIVE_RECT_BUTTON, CurPos.x, CurPos.y)) {
 			if (Me.drive_item.type > 0) {
 				append_item_description(str, &(Me.drive_item));
-				best_banner_pos_x = DRIVE_RECT_X + 30 + DRIVE_RECT_WIDTH;
-				best_banner_pos_y = DRIVE_RECT_Y - 30;
+				rect->x = DRIVE_RECT_X + 30 + DRIVE_RECT_WIDTH;
+				rect->y = DRIVE_RECT_Y - 30;
 			}
 		} else if (MouseCursorIsOnButton(SHIELD_RECT_BUTTON, CurPos.x, CurPos.y)) {
 			if (Me.shield_item.type > 0) {
 				append_item_description(str, &(Me.shield_item));
-				best_banner_pos_x = SHIELD_RECT_X + 30 + SHIELD_RECT_WIDTH;
-				best_banner_pos_y = SHIELD_RECT_Y - 30;
+				rect->x = SHIELD_RECT_X + 30 + SHIELD_RECT_WIDTH;
+				rect->y = SHIELD_RECT_Y - 30;
 			} else if (Me.weapon_item.type > 0) {
-				if (ItemMap[Me.weapon_item.type].item_gun_requires_both_hands) {
+				if (ItemMap[Me.weapon_item.type].weapon_needs_two_hands) {
 					append_item_description(str, &(Me.weapon_item));
-					best_banner_pos_x = SHIELD_RECT_X + 30 + SHIELD_RECT_WIDTH;
-					best_banner_pos_y = SHIELD_RECT_Y - 30;
+					rect->x = SHIELD_RECT_X + 30 + SHIELD_RECT_WIDTH;
+					rect->y = SHIELD_RECT_Y - 30;
 				}
 			}
 		} else if (MouseCursorIsOnButton(ARMOUR_RECT_BUTTON, CurPos.x, CurPos.y)) {
 			if (Me.armour_item.type > 0) {
 				append_item_description(str, &(Me.armour_item));
-				best_banner_pos_x = ARMOUR_RECT_X + 30 + ARMOUR_RECT_WIDTH;
-				best_banner_pos_y = ARMOUR_RECT_Y - 30;
+				rect->x = ARMOUR_RECT_X + 30 + ARMOUR_RECT_WIDTH;
+				rect->y = ARMOUR_RECT_Y - 30;
 			}
 		} else if (MouseCursorIsOnButton(HELMET_RECT_BUTTON, CurPos.x, CurPos.y)) {
 			if (Me.special_item.type > 0) {
 				append_item_description(str, &(Me.special_item));
-				best_banner_pos_x = HELMET_RECT_X + 30 + HELMET_RECT_WIDTH;
-				best_banner_pos_y = HELMET_RECT_Y - 30;
+				rect->x = HELMET_RECT_X + 30 + HELMET_RECT_WIDTH;
+				rect->y = HELMET_RECT_Y - 30;
 			}
 		}
 	}			// if nothing is 'held in hand' && inventory-screen visible
@@ -375,28 +389,28 @@ static void prepare_text_window_content(struct auto_string *str)
 			update_virtual_position(&item_vpos, &(obj_lvl->ItemList[index_of_floor_item_below_mouse_cursor].pos), Me.pos.z);
 			if (item_vpos.x != -1) {
 				append_item_description(str, &(obj_lvl->ItemList[index_of_floor_item_below_mouse_cursor]));
-				best_banner_pos_x =	translate_map_point_to_screen_pixel_x(item_vpos.x, item_vpos.y) + 80;
-				best_banner_pos_y =	translate_map_point_to_screen_pixel_y(item_vpos.x, item_vpos.y) - 30;
+				rect->x =	translate_map_point_to_screen_pixel_x(item_vpos.x, item_vpos.y) + 80;
+				rect->y =	translate_map_point_to_screen_pixel_y(item_vpos.x, item_vpos.y) - 30;
 			}
 		}
 
 		// Display Clickable Obstacle label  in the top status banner.
 		//
 		obj_lvl = NULL;
-		index_of_obst_below_mouse_cursor = clickable_obstacle_below_mouse_cursor(&obj_lvl);
+		index_of_obst_below_mouse_cursor = clickable_obstacle_below_mouse_cursor(&obj_lvl, TRUE);
 		if (index_of_obst_below_mouse_cursor != (-1)) {
 			gps obst_vpos;
 			update_virtual_position(&obst_vpos, &(obj_lvl->obstacle_list[index_of_obst_below_mouse_cursor].pos), Me.pos.z);
 			if (obst_vpos.x != -1) {
-				const char *label =  _(get_obstacle_spec(obj_lvl->obstacle_list[index_of_obst_below_mouse_cursor].type)->label);
+				char *label =  D_(get_obstacle_spec(obj_lvl->obstacle_list[index_of_obst_below_mouse_cursor].type)->label);
 				if (!label) {
-					ErrorMessage(__FUNCTION__, "Obstacle type %d is clickable, and as such requires a label to be displayed on mouseover.\n", PLEASE_INFORM, IS_WARNING_ONLY, obj_lvl->obstacle_list[index_of_obst_below_mouse_cursor].type);
-					label = "No label for this obstacle";
+					error_message(__FUNCTION__, "Obstacle type %d is clickable, and as such requires a label to be displayed on mouseover.", PLEASE_INFORM, obj_lvl->obstacle_list[index_of_obst_below_mouse_cursor].type);
+					label = _("No label for this obstacle");
 				}
 
 				autostr_printf(str, "%s", label);
-				best_banner_pos_x = translate_map_point_to_screen_pixel_x(obst_vpos.x, obst_vpos.y) + 50;
-				best_banner_pos_y = translate_map_point_to_screen_pixel_y(obst_vpos.x, obst_vpos.y) - 20;
+				rect->x = translate_map_point_to_screen_pixel_x(obst_vpos.x, obst_vpos.y) + 50;
+				rect->y = translate_map_point_to_screen_pixel_y(obst_vpos.x, obst_vpos.y) - 20;
 			}
 		}
 
@@ -434,13 +448,8 @@ void show_current_text_banner(void)
 		txt = alloc_autostr(200);
 
 	// Prepare the string, that is to be displayed inside the text rectangle
-	prepare_text_window_content(txt);
-
-	SDL_Rect rect;
-	rect.x = best_banner_pos_x;
-	rect.y = best_banner_pos_y;
-	rect.w = 0;
-	rect.h = 0;
+	SDL_Rect rect = { 0, 0, 0 };
+	prepare_text_window_content(txt, &rect);
 
 	display_tooltip(txt->value, 1, rect);
 }
@@ -450,8 +459,8 @@ void show_current_text_banner(void)
  * The rectangle size can be specified or left to 0, meaning the rectangle will be
  * expanded to fit the text. 
  * @param text Text to be displayed.
- * @centered Flag marking whether the text will be centered or not.
- * @SDL_Rect Rectangle in which the tooltip will be displayed.
+ * @param centered Flag marking whether the text will be centered or not.
+ * @param rect Rectangle in which the tooltip will be displayed.
  */
 void display_tooltip(const char *text, int centered, SDL_Rect rect)
 {
@@ -461,7 +470,7 @@ void display_tooltip(const char *text, int centered, SDL_Rect rect)
 	strcpy(buffer, text);
 
 	// Set font before making any font specific calculations.
-	SetCurrentFont(TEXT_BANNER_DEFAULT_FONT);
+	set_current_font(TEXT_BANNER_DEFAULT_FONT);
 
 	// If the width is not specified, expand the rectangle to fit
 	// the longest line width.
@@ -469,8 +478,8 @@ void display_tooltip(const char *text, int centered, SDL_Rect rect)
 		rect.w = longest_line_width(buffer) + 2 * TEXT_BANNER_HORIZONTAL_MARGIN;
 
 	// Compute the required height.
-	int lines_in_text = get_lines_needed(buffer, rect, LINE_HEIGHT_FACTOR);
-	rect.h = lines_in_text * FontHeight(GetCurrentFont());
+	int lines_in_text = get_lines_needed(buffer, rect, 1.0);
+	rect.h = lines_in_text * get_font_height(get_current_font());
 	
 	// Add extra correction to ensure the banner rectangle stays inside
 	// the visible screen.
@@ -494,12 +503,12 @@ void display_tooltip(const char *text, int centered, SDL_Rect rect)
 	
 	// Print the text.
 	if (!centered) {
-		display_text_using_line_height(buffer, text_rect.x, text_rect.y, &text_rect, 1.0);
+		display_text(buffer, text_rect.x, text_rect.y, &text_rect, 1.0);
 		return;
 	}
 
 	// Print the text centered.
-	int line_spacing = (text_rect.h - lines_in_text * FontHeight(GetCurrentFont())) / (lines_in_text + 1);
+	int line_spacing = (text_rect.h - lines_in_text * get_font_height(get_current_font())) / (lines_in_text + 1);
 	char *ptr = buffer;
 	int i;
 	for (i = 0; i < lines_in_text; i++) {
@@ -510,10 +519,10 @@ void display_tooltip(const char *text, int centered, SDL_Rect rect)
 			this_line[pos] = '\0';
 			ptr += pos + 1;
 		}
-		int offset = (text_rect.w - TextWidth(this_line)) / 2;
-		PutString(Screen,
-			  text_rect.x + offset,
-			  text_rect.y + line_spacing + i * (line_spacing + FontHeight(GetCurrentFont())), this_line);
+		int offset = (text_rect.w - text_width(get_current_font(), this_line)) / 2;
+		put_string(get_current_font(),
+			      text_rect.x + offset,
+			      text_rect.y + line_spacing + i * (line_spacing + get_font_height(get_current_font())), this_line);
 	}
 }
 
@@ -594,8 +603,6 @@ int get_current_fps(void)
 static void show_top_left_text(void)
 {
 	SDL_Rect clip;
-	int minutes;
-	int seconds;
 	int i;
 	int remaining_bots;
 	static struct auto_string *txt;
@@ -612,17 +619,7 @@ static void show_top_left_text(void)
 		if (!Me.AllMissions[i].MissionWasAssigned)
 			continue;
 
-		if (Me.AllMissions[i].MustLiveTime != (-1)) {
-			minutes = floor((Me.AllMissions[i].MustLiveTime - Me.MissionTimeElapsed) / 60);
-			seconds = rintf(Me.AllMissions[i].MustLiveTime - Me.MissionTimeElapsed) - 60 * minutes;
-			if (minutes < 0) {
-				minutes = 0;
-				seconds = 0;
-			}
-			autostr_append(txt, _("Time to hold out still: %2d:%2d\n"), minutes, seconds);
-		}
-
-		if ((Me.AllMissions[i].must_clear_first_level == Me.pos.z) || (Me.AllMissions[i].must_clear_second_level == Me.pos.z)) {
+		if (Me.AllMissions[i].must_clear_level == Me.pos.z) {
 			remaining_bots = 0;
 
 			enemy *erot, *nerot;
@@ -640,8 +637,8 @@ static void show_top_left_text(void)
 	clip.w = GameConfig.screen_width;
 	clip.h = GameConfig.screen_height;
 
-	SetCurrentFont(FPS_Display_BFont);
-	display_text_using_line_height(txt->value, clip.x, clip.y, &clip, 1.0);
+	set_current_font(FPS_Display_Font);
+	display_text(txt->value, clip.x, clip.y, &clip, 1.0);
 }
 
 /**
@@ -659,6 +656,7 @@ static void show_top_right_text(void)
 		if (GameConfig.Draw_Position) {
 			sprintf(level_name_and_time, "%s (%03.1f:%03.1f:%d)  ",
 				D_(curShip.AllLevels[Me.pos.z]->Levelname), Me.pos.x, Me.pos.y, Me.pos.z);
+			// TRANSLATORS: In-game date: Day <day number> <hour>:<minute>
 			sprintf(temp_text, _("Day %d  %02d:%02d"),
 				get_days_of_game_duration(Me.current_game_date),
 				get_hours_of_game_duration(Me.current_game_date), get_minutes_of_game_duration(Me.current_game_date));
@@ -672,7 +670,7 @@ static void show_top_right_text(void)
 			strcat(level_name_and_time, temp_text);
 			strcat(level_name_and_time, " ");
 		}
-		RightPutStringFont(Screen, FPS_Display_BFont, 2, level_name_and_time);
+		put_string_right(FPS_Display_Font, 0.3 * get_font_height(get_current_font()), level_name_and_time);
 	}
 }
 
@@ -685,6 +683,9 @@ void show_texts_and_banner(void) {
 	show_current_text_banner();
 	show_top_left_text();
 	show_top_right_text();
+	if (GameConfig.effect_countdowns_visible) {
+		display_effect_countdowns();
+	}
 	DisplayBigScreenMessage();
 }
 
@@ -717,8 +718,8 @@ void toggle_game_config_screen_visibility(int screen_visible)
 		GameConfig.Inventory_Visible = FALSE;
 		break;
 	default:
-		ErrorMessage(__FUNCTION__, "\
-unhandled skill screen code received.  something is going VERY wrong!", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "\
+unhandled skill screen code received.  something is going VERY wrong!", PLEASE_INFORM | IS_FATAL);
 		break;
 	}
 
