@@ -24,14 +24,11 @@
  *
  */
 /**
- * This file contains (all) functions for keyboard and joystick handling
+ * This file contains functions for keyboard and mouse events.
  */
 
 #define _input_c
 
-#include "system.h"
-
-#include "defs.h"
 #include "struct.h"
 #include "global.h"
 #include "proto.h"
@@ -44,14 +41,14 @@
 #define SDL_BUTTON_WHEELDOWN 5
 #endif
 
-Uint8 *key_state_array;		/* SDL keyboard state internal arrey */
-int key_state_array_size;
+static Uint8 *key_state_array;		/* SDL keyboard state internal array */
+static int key_state_array_size;
 
-Uint8 mouse_state_last_frame;	/* mouse states reference */
-Uint8 mouse_state_this_frame;	/* current mouse state */
+static Uint8 mouse_state_last_frame;	/* mouse states reference */
+static Uint8 mouse_state_this_frame;	/* current mouse state */
 
-int MouseWheelUpMovesRecorded;
-int MouseWheelDownMovesRecorded;
+static int MouseWheelUpMovesRecorded;
+static int MouseWheelDownMovesRecorded;
 
 void init_keyboard_input_array()
 {
@@ -115,7 +112,7 @@ int input_handle(void)
 		switch (event.type) {
 		case SDL_QUIT:
 			printf("\n\nUser requested termination...\n\nTerminating...");
-			Terminate(EXIT_SUCCESS, TRUE);
+			Terminate(EXIT_SUCCESS);
 			break;
 
 		case SDL_KEYDOWN:
@@ -158,113 +155,76 @@ int MouseWheelDownPressed(void)
 		return (FALSE);
 }
 
+static int key_is_pressed(int key)
+{
+	SDL_PumpEvents();
+	return key_state_array[key];
+}
+
 /* *Pressed functions return the current state of the keyboard/mouse button in question */
 int LeftPressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_LEFT]));
+	return key_is_pressed(SDLK_LEFT);
 }
 
 int RightPressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_RIGHT]));
+	return key_is_pressed(SDLK_RIGHT);
 }
 
 int UpPressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_UP]));
+	return key_is_pressed(SDLK_UP);
 }
 
 int DownPressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_DOWN]));
+	return key_is_pressed(SDLK_DOWN);
 }
 
 int SpacePressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_SPACE]));
+	return key_is_pressed(SDLK_SPACE);
 }
 
 int EnterPressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_RETURN]));
+	return key_is_pressed(SDLK_RETURN);
 }
 
 int EscapePressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_ESCAPE]));
+	return key_is_pressed(SDLK_ESCAPE);
 }
 
 int LeftCtrlPressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_LCTRL]));
+	return key_is_pressed(SDLK_LCTRL);
 }
 
 int CtrlPressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_LCTRL]) || key_state_array[SDLK_RCTRL]);
+	return key_is_pressed(SDLK_LCTRL) || key_is_pressed(SDLK_RCTRL);
 }
 
 int ShiftPressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_RSHIFT] || key_state_array[SDLK_LSHIFT]));
-}
-
-int AltPressed()
-{
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_RALT] || key_state_array[SDLK_LALT]));
+	return key_is_pressed(SDLK_RSHIFT) || key_is_pressed(SDLK_LSHIFT);
 }
 
 int APressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_a]));
-}
-
-int HPressed()
-{
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_h]));
-}
-
-int MPressed()
-{
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_m]));
+	return key_is_pressed(SDLK_a);
 }
 
 int QPressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_q]));
-}
-
-int SPressed()
-{
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_s]));
-}
-
-int WPressed()
-{
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_w]));
+	return key_is_pressed(SDLK_q);
 }
 
 int XPressed()
 {
-	SDL_PumpEvents();
-	return ((key_state_array[SDLK_x]));
+	return key_is_pressed(SDLK_x);
 }
 
 int MouseRightPressed()
@@ -292,14 +252,32 @@ int MouseLeftClicked()
 	return (!(mouse_state_last_frame & SDL_BUTTON(1)) && (mouse_state_this_frame & SDL_BUTTON(1)));
 }
 
-int MouseRightUnclicked()
+/**
+ * \brief Wait until there is no more interaction event pending.
+ *
+ * \details Some UI screens (the dialog screen, for instance) have their own
+ * interaction loop, and do not use the main input_handle() function.
+ * When such a UI screen is closed, the state of the input devices (pressed
+ * keys, cursor position and so on) can be incoherent with the interaction
+ * state internally computed and stored by input_handle().
+ * This function waits for a 'neutral' interaction state, and is to be used
+ * before to return to the main game screen.
+ */
+void WaitNoEvent()
 {
-	return ((mouse_state_last_frame & SDL_BUTTON(3)) && !(mouse_state_this_frame & SDL_BUTTON(3)));
-}
+	SDL_Event event;
+	int mouse_x, mouse_y;
 
-int MouseLeftUnclicked()
-{
-	return ((mouse_state_last_frame & SDL_BUTTON(1)) && !(mouse_state_this_frame & SDL_BUTTON(1)));
+	// Because we are still using a mix of event-based and state-based
+	// interaction, it is not easy to define what a 'neutral' state is.
+	// We at least wait for the release of the 'common' used keys, and
+	// set the cursor position to the current mouse position.
+
+	while ( (SDL_PumpEvents(), SDL_GetMouseState(&mouse_x, &mouse_y) & (SDL_BUTTON(1) | SDL_BUTTON(3))) ||
+			EnterPressed() || EscapePressed() || SDL_PollEvent(&event))
+		SDL_Delay(1);
+	input_axis.x = mouse_x - UserCenter_x;
+	input_axis.y = mouse_y - UserCenter_y;
 }
 
 #undef _input_c
