@@ -96,16 +96,12 @@ int item_can_be_customized(item *it)
 {
 	itemspec *spec = &ItemMap[it->type];
 
-	return spec->item_can_be_installed_in_weapon_slot ||
-	       spec->item_can_be_installed_in_drive_slot ||
-	       spec->item_can_be_installed_in_armour_slot ||
-	       spec->item_can_be_installed_in_shield_slot ||
-	       spec->item_can_be_installed_in_special_slot;
+	return spec->slot != NO_SLOT;
 }
 
 /**
  * \brief Checks if the spec of an add-on is compatible with an item.
- * \param addonspec Spec of the add-on whose compatiblity to test.
+ * \param addonspec Spec of the add-on whose compatibility to test.
  * \param it Item to which the add-on would be installed.
  * \return TRUE if compatible, FALSE if incompatible.
  */
@@ -117,24 +113,21 @@ static int addon_is_compatible_with_item(struct addon_spec *addonspec, item *it)
 	if (str) {
 		itemspec* spec = &ItemMap[it->type];
 		if (!strcmp(str, "melee weapon")) {
-			ret = spec->item_can_be_installed_in_weapon_slot &&
-			      spec->item_weapon_is_melee;
+			ret = (spec->slot == WEAPON_SLOT) &&
+			      spec->weapon_is_melee;
 		} else if (!strcmp(str, "ranged weapon")) {
-			ret = spec->item_can_be_installed_in_weapon_slot &&
-			      !spec->item_weapon_is_melee;
+			ret = (spec->slot == WEAPON_SLOT) &&
+			      !spec->weapon_is_melee;
 		} else if (!strcmp(str, "armor")) {
-			ret = spec->item_can_be_installed_in_armour_slot ||
-			      spec->item_can_be_installed_in_drive_slot ||
-			      spec->item_can_be_installed_in_shield_slot ||
-			      spec->item_can_be_installed_in_special_slot;
+			ret = (spec->slot & (SHIELD_SLOT | HELM_SLOT | ARMOR_SLOT | BOOT_SLOT));
 		} else if (!strcmp(str, "boots")) {
-			ret = spec->item_can_be_installed_in_drive_slot;
+			ret = (spec->slot == BOOT_SLOT);
 		} else if (!strcmp(str, "jacket")) {
-			ret = spec->item_can_be_installed_in_armour_slot;
+			ret = (spec->slot == ARMOR_SLOT);
 		} else if (!strcmp(str, "shield")) {
-			ret = spec->item_can_be_installed_in_shield_slot;
+			ret = (spec->slot == SHIELD_SLOT);
 		} else if (!strcmp(str, "helmet")) {
-			ret = spec->item_can_be_installed_in_special_slot;
+			ret = (spec->slot == HELM_SLOT);
 		} else {
 			ret = FALSE;
 		}
@@ -159,11 +152,11 @@ int item_can_be_installed_to_socket(item *dstitem, item *addon, int socketid)
 
 	// Make sure that the socket and the add-on exist.
 	if (socketid >= dstitem->upgrade_sockets.size) {
-		ErrorMessage(__FUNCTION__, "Socket index out of bounds", PLEASE_INFORM, IS_WARNING_ONLY);
+		error_message(__FUNCTION__, "Socket index out of bounds", PLEASE_INFORM);
 		return FALSE;
 	}
 	if (addon->type == -1) {
-		ErrorMessage(__FUNCTION__, "Add-on type was -1", PLEASE_INFORM, IS_WARNING_ONLY);
+		error_message(__FUNCTION__, "Add-on type was -1", PLEASE_INFORM);
 		return FALSE;
 	}
 
@@ -248,52 +241,84 @@ void get_item_bonus_string(item *it, const char *separator, struct auto_string *
 {
 	// Append the bonuses to the string.
 	if (it->bonus_to_str) {
+		// TRANSLATORS: Item's bonus: '10 to strength' means 'add 10 to player's strength'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d to strength%s"), it->bonus_to_str, separator);
 	}
 	if (it->bonus_to_dex) {
+		// TRANSLATORS: Item's bonus: '10 to dexterity' means 'add 10 to player's dexterity'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d to dexterity%s"), it->bonus_to_dex, separator);
 	}
 	if (it->bonus_to_cooling) {
+		// TRANSLATORS: Item's bonus: '10 to cooling' means 'add 10 to player's cooling capability'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d to cooling%s"), it->bonus_to_cooling, separator);
 	}
 	if (it->bonus_to_physique) {
+		// TRANSLATORS: Item's bonus: '10 to physique' means 'add 10 to player's physique'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d to physique%s"), it->bonus_to_physique, separator);
 	}
 	if (it->bonus_to_health_points) {
+		// TRANSLATORS: Item's bonus: '10 to health points' means 'add 10 to player's health'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d health points%s"), it->bonus_to_health_points, separator);
 	}
 	if (it->bonus_to_health_recovery) {
+		// TRANSLATORS: Item's bonus: '0.1 to health points per seconds' means 'add 0.1 to player's recovery rate'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+0.1f health points per second%s"), it->bonus_to_health_recovery, separator);
 	}
 	if (it->bonus_to_cooling_rate) {
 		if (it->bonus_to_cooling_rate > 0) {
+			// TRANSLATORS: Item's bonus: '0.1 cooling per seconds' means 'add 0.1 to player's cooling rate'
+			// TRANSLATORS: (trailing %s contains a text separator)
 			autostr_append(desc, _("%0.1f cooling per second%s"), it->bonus_to_cooling_rate, separator);
 		} else {
+			// TRANSLATORS: Item's bonus: '0.1 heating per seconds' means 'remove 0.1 to player's cooling rate'
+			// TRANSLATORS: (trailing %s contains a text separator)
 			autostr_append(desc, _("%0.1f heating per second%s"), -it->bonus_to_cooling_rate, separator);
 		}
 	}
 	if (it->bonus_to_attack) {
+		// TRANSLATORS: Item's bonus: '10 to attacks' means 'add 10 to player's attack capability'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d%% to attack%s"), it->bonus_to_attack, separator);
 	}
 	if (it->bonus_to_all_attributes) {
+		// TRANSLATORS: Item's bonus: '10 to all attributes' means 'add 10 to all player's capabilities'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d to all attributes%s"), it->bonus_to_all_attributes, separator);
 	}
 	if (it->bonus_to_damage) {
+		// TRANSLATORS: Item's bonus: '10 to damage' means 'add 10 to player's damage'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d to damage%s"), it->bonus_to_damage, separator);
 	}
 	if (it->bonus_to_armor_class) {
+		// TRANSLATORS: Item's bonus: '10 to armor' means 'add 10 to player's armor'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d to armor%s"), it->bonus_to_armor_class, separator);
 	}
 	if (it->bonus_to_paralyze_enemy) {
+		// TRANSLATORS: Item's bonus: '10 to paralyze enemy' means 'add 10 to player's paralyze capability'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d to paralyze enemy%s"), it->bonus_to_paralyze_enemy, separator);
 	}
 	if (it->bonus_to_slow_enemy) {
+		// TRANSLATORS: Item's bonus: '10 to slow enemy' means 'add 10 to player's capability to slow an ennemy'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d to slow enemy%s"), it->bonus_to_slow_enemy, separator);
 	}
 	if (it->bonus_to_light_radius) {
+		// TRANSLATORS: Item's bonus: '10 to light radius' means 'add 10 to the light radius around the player'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d to light radius%s"), it->bonus_to_light_radius, separator);
 	}
 	if (it->bonus_to_experience_gain) {
+		// TRANSLATORS: Item's bonus: '10 to experience gain' means 'add 10 to the player's experience multiplier'
+		// TRANSLATORS: (trailing %s contains a text separator)
 		autostr_append(desc, _("%+d%% to experience gain%s"), it->bonus_to_experience_gain, separator);
 	}
 }
@@ -344,7 +369,7 @@ void print_addon_description(struct addon_spec *spec, struct auto_string *desc)
 	// an arbitrary type to calculate the effects of the add-on.
 	init_item(&temp_item);
 	temp_item.type = 1;
-	create_upgrade_socket(&temp_item, UPGRADE_SOCKET_TYPE_UNIVERSAL, ItemMap[spec->type].item_name);
+	create_upgrade_socket(&temp_item, UPGRADE_SOCKET_TYPE_UNIVERSAL, ItemMap[spec->type].id);
 	calculate_item_bonuses(&temp_item);
 	get_item_bonus_string(&temp_item, "\n", desc);
 	DeleteItem(&temp_item);
@@ -421,17 +446,20 @@ void calculate_item_bonuses(item *it)
 	it->bonus_to_slow_enemy = 0;
 	it->bonus_to_light_radius = 0;
 	it->bonus_to_experience_gain = 0;
-	it->damage = ItemMap[it->type].base_item_gun_damage;
-	it->damage_modifier = ItemMap[it->type].item_gun_damage_modifier;
-	if (it->type < 0) {
-		return;
+	if (it->type > 0) {
+		it->damage = ItemMap[it->type].weapon_base_damage;
+		it->damage_modifier = ItemMap[it->type].weapon_damage_modifier;
+	} else {
+		it->damage = 0;
+		it->damage_modifier = 0;
+		return; // End when the item type don't exist.
 	}
 
 	// Apply bonuses from add-ons.
 	for (i = 0; i < it->upgrade_sockets.size; i++) {
 		addon = it->upgrade_sockets.arr[i].addon;
 		if (addon) {
-			spec = get_addon_spec(GetItemIndexByName(addon));
+			spec = get_addon_spec(get_item_type_by_id(addon));
 			for (j = 0; j < spec->bonuses.size; j++) {
 				apply_addon_bonus(it, &((struct addon_bonus*) spec->bonuses.arr)[j]);
 			}
