@@ -106,7 +106,7 @@ static void load_images()
 
 	for (i = 0; i < IMAGE_MAX; i++) {
 		memset(&images[i], 0, sizeof(struct image));
-		load_image(&images[i], fnames[i], FALSE);
+		load_image(&images[i], fnames[i], NO_MOD);
 	}
 }
 
@@ -168,7 +168,15 @@ static void show_socket_creation_menu()
 	int i;
 	char buffer[64];
 	point cursor;
-	const char *names[] = { _("Electric"), _("Mechanical"), _("Universal"), _("Cancel") };
+	const char *names[] = {
+			// TRANSLATORS: Socket's type
+			_("Electric"),
+			// TRANSLATORS: Socket's type
+			_("Mechanical"),
+			// TRANSLATORS: Socket's type
+			_("Universal"),
+			// TRANSLATORS: Cancel socket's type choice menu
+			_("Cancel") };
 	const int costs[] = { ELECTRIC_SOCKET_COST, MECHANICAL_SOCKET_COST, UNIVERSAL_SOCKET_COST, -1 };
 
 	// Get the position of the cursor for mouse hover tests.
@@ -176,12 +184,12 @@ static void show_socket_creation_menu()
 	cursor.y = GetMousePos_y();
 
 	// Draw the menu title and the column headers.
-	SetCurrentFont(Blue_BFont);
-	display_text(_("Add socket"), rects.prompt.x, rects.prompt.y, NULL);
-	SetCurrentFont(Messagestat_BFont);
-	display_text(_("Type"), rects.table[0][0].x, rects.table[0][0].y, NULL);
-	display_text(_("Cost"), rects.table[1][0].x, rects.table[1][0].y, NULL);
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Blue_Font);
+	display_text(_("Add socket"), rects.prompt.x, rects.prompt.y, NULL, 1.0);
+	set_current_font(Messagestat_Font);
+	display_text(_("Type"), rects.table[0][0].x, rects.table[0][0].y, NULL, 1.0);
+	display_text(_("Cost"), rects.table[1][0].x, rects.table[1][0].y, NULL, 1.0);
+	set_current_font(Messagevar_Font);
 
 	// Draw the selectable rows whose contents were given as parameters.
 	for (i = 0; i < 4; i++) {
@@ -191,12 +199,12 @@ static void show_socket_creation_menu()
 		}
 
 		// Draw the name column.
-		display_text(names[i], rects.table[0][i + 1].x, rects.table[0][i + 1].y, NULL);
+		display_text(names[i], rects.table[0][i + 1].x, rects.table[0][i + 1].y, NULL, 1.0);
 
 		// Draw the cost column if the row has a cost assigned to it.
 		if (costs[i] != -1) {
 			sprintf(buffer, "%d", costs[i]);
-			display_text(buffer, rects.table[1][i + 1].x, rects.table[1][i + 1].y, NULL);
+			display_text(buffer, rects.table[1][i + 1].x, rects.table[1][i + 1].y, NULL, 1.0);
 		}
 	}
 }
@@ -246,33 +254,33 @@ void show_item_upgrade_ui()
 	if (ui.custom_item.type != -1) {
 		ShowItemPicture(rects.custom_slot.x, rects.custom_slot.y, ui.custom_item.type);
 	} else {
-		SetCurrentFont(Messagestat_BFont);
+		set_current_font(Messagestat_Font);
 		display_text(_("Place an item here"), rects.custom_slot.x,
-		            rects.custom_slot.y + rects.custom_slot.h / 2, NULL);
+		            rects.custom_slot.y + rects.custom_slot.h / 2, NULL, 1.0);
 	}
 
 	// Draw the title.
-	SetCurrentFont(Menu_BFont);
-	display_text(_("Upgrade Items"), rects.title_text.x, rects.title_text.y, NULL);
+	set_current_font(Menu_Font);
+	display_text(_("Upgrade Items"), rects.title_text.x, rects.title_text.y, NULL, 1.0);
 
 	// Draw the text labels.
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagevar_Font);
 	sprintf(buffer, _("Circuits: %d"), Me.Gold);
-	display_text(buffer, rects.money_text.x, rects.money_text.y, NULL);
+	display_text(buffer, rects.money_text.x, rects.money_text.y, NULL, 1.0);
 	if (ui.cost > Me.Gold) {
-		SetCurrentFont(Messagestat_BFont);
+		set_current_font(Messagered_Font);
 	} else {
-		SetCurrentFont(Messagevar_BFont);
+		set_current_font(Messagevar_Font);
 	}
 	sprintf(buffer, _("Cost: %d"), ui.cost);
-	display_text(buffer, rects.cost_text.x, rects.cost_text.y, NULL);
-	SetCurrentFont(Blue_BFont);
-	display_text(_("Bonuses"), rects.bonus_text.x, rects.bonus_text.y, NULL);
+	display_text(buffer, rects.cost_text.x, rects.cost_text.y, NULL, 1.0);
+	set_current_font(Blue_Font);
+	display_text(_("Bonuses"), rects.bonus_text.x, rects.bonus_text.y, NULL, 1.0);
 
 	// Draw the item bonus string.
-	SetCurrentFont(Messagevar_BFont);
+	set_current_font(Messagevar_Font);
 	display_text(ui.bonus_text->value, rects.bonus_list.x, rects.bonus_list.y,
-	            &rects.bonus_list);
+	            &rects.bonus_list, 1.0);
 
 	if (ui.create_socket_active) {
 
@@ -399,7 +407,7 @@ static void calculate_cost_and_bonuses(void)
 			if (addon->type != -1) {
 				socket = &temp.upgrade_sockets.arr[i];
 				free(socket->addon);
-				socket->addon = strdup(ItemMap[addon->type].item_name);
+				socket->addon = strdup(ItemMap[addon->type].id);
 			}
 		}
 	}
@@ -510,7 +518,7 @@ static int set_customized_item(item *it)
 	for (i = 0; i < ADDON_ITEMS_MAX && i < ui.custom_item.upgrade_sockets.size; i++) {
 		socket = &ui.custom_item.upgrade_sockets.arr[i];
 		if (socket->addon) {
-			ui.addon_items[i] = create_item_with_name(socket->addon, TRUE, 1);
+			ui.addon_items[i] = create_item_with_id(socket->addon, TRUE, 1);
 		}
 	}
 
@@ -534,7 +542,13 @@ static int set_addon_item(item *it, int index)
 	// customized item remains unchanged; the player needs to press the
 	// apply button before any modifications to the item take place.
 	play_item_sound(it->type, &Me.pos);
-	MoveItem(it, &ui.addon_items[index]);
+	if (it->multiplicity == 1) {
+		MoveItem(it, &ui.addon_items[index]);
+	} else {
+		CopyItem(it, &ui.addon_items[index]);
+		it->multiplicity--;
+		ui.addon_items[index].multiplicity = 1;
+	}
 
 	// Recalculate the cost and the bonus string and enable the apply button
 	// if the player has enough gold to pay for the upgrade.
@@ -561,7 +575,7 @@ static void apply_customization(void)
 		socket = &ui.custom_item.upgrade_sockets.arr[i];
 		if (!socket->addon && addon->type != -1) {
 			free(socket->addon);
-			socket->addon = strdup(ItemMap[addon->type].item_name);
+			socket->addon = strdup(ItemMap[addon->type].id);
 		}
 	}
 
