@@ -36,15 +36,6 @@
 #include "global.h"
 #include "proto.h"
 
-enum {
-	WEAPON_SLOT = 0,
-	SHIELD_SLOT,
-	SPECIAL_SLOT,
-	ARMOUR_SLOT,
-	DRIVE_SLOT,
-	FIRST_INV_SLOT
-};
-
 /**
  * \brief Initializes an empty item.
  * \param item_ Item.
@@ -60,14 +51,19 @@ void init_item(item *it)
 	it->inventory_position.y = -1;
 }
 
-item create_item_with_name(const char *item_name, int full_durability, int multiplicity)
+item create_item_with_id(const char *item_id, int full_durability, int multiplicity)
 {
 	item new_item;
-	
+
 	init_item(&new_item);
-	new_item.type = GetItemIndexByName(item_name);
+	new_item.type = get_item_type_by_id(item_id);
+	if (new_item.type < 0 || new_item.type >= Number_Of_Item_Types) {
+		error_message(__FUNCTION__, "No items with the name \"%s\" exist in the game. Cannot create item.\nChanging item type to bug item to avoid crash.", PLEASE_INFORM, item_id);
+		new_item.type = 0;
+	}
+
 	FillInItemProperties(&new_item, full_durability, multiplicity);
-	
+
 	return new_item;
 }
 
@@ -78,17 +74,7 @@ item create_item_with_name(const char *item_name, int full_durability, int multi
  */
 static int equippable_item(item *it)
 {
-	if (ItemMap[it->type].item_can_be_installed_in_weapon_slot)
-		return TRUE;
-	if (ItemMap[it->type].item_can_be_installed_in_drive_slot)
-		return TRUE;
-	if (ItemMap[it->type].item_can_be_installed_in_armour_slot)
-		return TRUE;
-	if (ItemMap[it->type].item_can_be_installed_in_shield_slot)
-		return TRUE;
-	if (ItemMap[it->type].item_can_be_installed_in_special_slot)
-		return TRUE;
-	return FALSE;
+	return ItemMap[it->type].slot != NO_SLOT;
 }
 
 void equip_item(item *new_item)
@@ -98,11 +84,12 @@ void equip_item(item *new_item)
 	
 	new_itemspec = &ItemMap[new_item->type];
 
-	// If we're equipping a two-handed weapon, we need to unequip the shield
-	// as well. We drop the shield to the inventory or to the floor.
-	if (new_itemspec->item_gun_requires_both_hands && Me.shield_item.type != -1) {
-		give_item(&Me.shield_item);
-	}
+	// If the item can't be equipped, stop now and throw a warning.
+	if (!equippable_item(new_item)) {
+		error_message(__FUNCTION__, "Tried to equip the item \"%s\" which can't be equipped.",
+				PLEASE_INFORM, new_itemspec->id);
+		return;
+ 	}
 
 	// If there's an existing item in the equipment slot, drop it to the
 	// inventory or to the floor.
@@ -111,6 +98,18 @@ void equip_item(item *new_item)
 		give_item(old_item);
 	}
 
+	// If we're equipping a two-handed weapon, we need to unequip the shield
+	// as well. We drop the shield to the inventory or to the floor.
+	if (new_itemspec->weapon_needs_two_hands && Me.shield_item.type != -1) {
+		give_item(&Me.shield_item);
+	}
+
+	// Before equipping a shield, if a two-handed weapon is equipped, you need to drop it.
+	if (new_itemspec->slot == SHIELD_SLOT && Me.weapon_item.type != -1 
+			&& ItemMap[Me.weapon_item.type].weapon_needs_two_hands) {
+		give_item(&Me.weapon_item);
+	}
+
 	// Move the new item to the now empty equipment slot.
 	MoveItem(new_item, old_item);
 }
@@ -127,19 +126,19 @@ item *get_equipped_item_in_slot_for(int item_type)
 
 	spec = &ItemMap[item_type];
 	
-	if (spec->item_can_be_installed_in_weapon_slot) {
+	if (spec->slot == WEAPON_SLOT) {
 		equipped_item = &Me.weapon_item;
 	}
-	else if (spec->item_can_be_installed_in_drive_slot) {
+	else if (spec->slot == BOOT_SLOT) {
 		equipped_item = &Me.drive_item;
 	}
-	else if (spec->item_can_be_installed_in_armour_slot) {
+	else if (spec->slot == ARMOR_SLOT) {
 		equipped_item = &Me.armour_item;
 	}
-	else if (spec->item_can_be_installed_in_shield_slot) {
+	else if (spec->slot == SHIELD_SLOT) {
 		equipped_item = &Me.shield_item;
 	}
-	else if (spec->item_can_be_installed_in_special_slot) {
+	else if (spec->slot == HELM_SLOT) {
 		equipped_item = &Me.special_item;
 	}
 	else
@@ -158,24 +157,49 @@ item *get_equipped_item_in_slot_for(int item_type)
 static void self_repair_item(item *it)
 {
 	int wear = 0, used = 0;
+	int my_skill_level = 0;
+	int percent_chance = MyRandom(100);
 
 	if (it->max_durability == -1) {
 		play_sound("effects/tux_ingame_comments/Tux_Item_Cant_Be_0.ogg");
 		return;
 	} 
 
-	used = it->max_durability - it->current_durability;
-	/* Self repair formula: decrease max_durability between 1 and 11-skill_level*/
-	wear = 1 + MyRandom(10 - Me.skill_level[get_program_index_with_name("Repair equipment")]);
-	//never decrease more than current missing durability
-	it->max_durability -= min(wear, used);
-	if (it->max_durability < 1) {
-		it->max_durability = 1;
+	my_skill_level = Me.skill_level[get_program_index_with_name("Repair equipment")];
+
+/* if Tux's repair ability is at the full skill level 
+ * make it so that most of the time the item is fully repaired. Give it a 5 % chance 
+ * of improving the item and a 3% chance of making it worse.
+ * otherwise give him a skill level *10 percent chance of repairing the item completely
+ */
+	if(my_skill_level >= NUMBER_OF_SKILL_LEVELS-1) {
+		if(percent_chance > 94)
+		{
+			it->max_durability++;
+		}
+		else if (percent_chance < 3)
+		{
+			it->max_durability--;
+		}
+	}
+	else if(my_skill_level*10 < percent_chance) {		
+		used = it->max_durability - it->current_durability;
+		/* Self repair formula: decrease max_durability between 1 and 11-skill_level*/
+		wear = 1 + MyRandom(10 - my_skill_level);
+		//never decrease more than current missing durability
+		it->max_durability -= min(wear, used);
+		if (it->max_durability < 1) {
+			it->max_durability = 1;
+		}
 	}
 	//when you wear off all extra durability, the item become normal again
 	if (it->quality == GOOD_QUALITY && it->max_durability < ItemMap[it->type].base_item_durability) {
 		it->quality = NORMAL_QUALITY;
 	}
+	//if you add extra durability, the item becomes good
+	if (it->quality == NORMAL_QUALITY && it->max_durability > ItemMap[it->type].base_item_durability) {
+		it->quality = GOOD_QUALITY;
+	}
 
 	it->current_durability = it->max_durability;
 	play_sound("effects/tux_ingame_comments/Tux_This_Quick_Fix_0.ogg");
@@ -195,7 +219,7 @@ unsigned long calculate_item_buy_price(item * BuyItem)
 	for (i = 0; i < BuyItem->upgrade_sockets.size; i++) {
 		const char *addon = BuyItem->upgrade_sockets.arr[i].addon;
 		if (addon) {
-			int type = GetItemIndexByName(addon);
+			int type = get_item_type_by_id(addon);
 			price += ItemMap[type].base_list_price;
 		}
 	}
@@ -282,6 +306,12 @@ static enum item_quality random_item_quality()
  */
 void FillInItemProperties(item *it, int full_durability, int multiplicity)
 {
+	// Some basic error checking of item type.
+	if ((it->type < 0) || (it->type >= Number_Of_Item_Types)) {
+		error_message(__FUNCTION__, "Cannot fill in information for item with invalid type.", PLEASE_INFORM | IS_FATAL);
+		return;
+	}	
+
 	itemspec *spec = &ItemMap[it->type];
 
 	it->multiplicity = multiplicity;
@@ -294,18 +324,16 @@ void FillInItemProperties(item *it, int full_durability, int multiplicity)
 		return;
 	}
 
-
 	it->quality = random_item_quality();
 
 	// Add random bullets to the clip if the item is a gun.
-	if (spec->item_gun_ammo_clip_size) {
-		it->ammo_clip = MyRandom(spec->item_gun_ammo_clip_size);
+	if (spec->weapon_ammo_type && spec->weapon_ammo_clip_size) {
+		it->ammo_clip = MyRandom(spec->weapon_ammo_clip_size);
 	}
 
 	// Set the base damage reduction by using the item spec and a random multiplier.
 	it->armor_class = spec->base_armor_class + MyRandom(spec->armor_class_modifier);
 
-
 	// Set the maximum and current durabilities of the item.
 	if (spec->base_item_durability != -1) {
 		// The maximum durability is within the range specified by the item spec.
@@ -347,10 +375,10 @@ void append_item_name(item * ShowItem, struct auto_string *str)
 		autostr_append(str, "%s", font_switchto_neon);
 	}
 
-	if (MatchItemWithName(ShowItem->type, "Valuable Circuits"))
+	if (item_spec_eq_id(ShowItem->type, "Valuable Circuits"))
 		autostr_append(str, "%d ", ShowItem->multiplicity);
 
-	autostr_append(str, "%s", D_(ItemMap[ShowItem->type].item_name));
+	autostr_append(str, "%s", D_(item_specs_get_name(ShowItem->type)));
 
 	if (ShowItem->quality == GOOD_QUALITY) {
 		autostr_append(str, "\n%s", font_switchto_blue);
@@ -372,9 +400,9 @@ item *DropItemAt(int ItemType, int level_num, float x, float y, int multiplicity
 	gps item_pos;
 
 	if (ItemType < 0 || ItemType >= Number_Of_Item_Types)
-		ErrorMessage(__FUNCTION__, "\
+		error_message(__FUNCTION__, "\
 Received item type %d that is outside the range of allowed item types.",
-			     PLEASE_INFORM, IS_FATAL, ItemType);
+			     PLEASE_INFORM | IS_FATAL, ItemType);
 
 	// Fix virtual position (e.g. from a dying robot)
 	item_pos.x = x;
@@ -396,11 +424,6 @@ Received item type %d that is outside the range of allowed item types.",
 
 static int get_random_item_type(int class)
 {
-
-	if (class > 9) {
-		ErrorMessage(__FUNCTION__, "Random item class %d exceeds 9.\n", class, PLEASE_INFORM, IS_FATAL);
-	}
-
 	int a = MyRandom(item_count_per_class[class] - 1) + 1;
 
 	//printf("Choosing in class %d among %d items, taking item %d\n", class, item_count_per_class[class], a);
@@ -416,7 +439,7 @@ static int get_random_item_type(int class)
 	}
 
 	if (a) {
-		ErrorMessage(__FUNCTION__, "Looking for random item with class %d, a = %d after the loop.", PLEASE_INFORM, IS_FATAL, class,
+		error_message(__FUNCTION__, "Looking for random item with class %d, a = %d after the loop.", PLEASE_INFORM | IS_FATAL, class,
 			     a);
 	}
 	//printf("Dropping item %s (%d <= class <= %d), class %d\n", ItemMap[i].item_name, ItemMap[i].min_drop_class, ItemMap[i].max_drop_class, class);
@@ -437,6 +460,15 @@ void DropRandomItem(int level_num, float x, float y, int class, int force_magica
 	int drop_item_type = 1;
 	int drop_item_multiplicity = 1;
 
+	if (class == -1) 
+		return; // -1 is the value to not drop item.
+
+	// Drop item only if the class is between 0 and MAX_DROP_CLASS.
+	if (class < 0 || class > MAX_DROP_CLASS) {
+		error_message(__FUNCTION__, "The drop class %d is invalid (out of 0 to MAX_DROP_CLASS: %d).\n", PLEASE_INFORM, class, MAX_DROP_CLASS);
+		return;
+	}
+
 	// First we determine if there is something dropped at all or not,
 	// cause in the later case, we can return immediately.  If a drop is
 	// forced, we need not check for not do drop.
@@ -455,7 +487,7 @@ void DropRandomItem(int level_num, float x, float y, int class, int force_magica
 		// If class == 0, we want to avoid to drop 0-1 valuable circuits
 		int how_many = (class == 0) ? 2 : 0;
 		how_many += MONEY_PER_BOT_CLASS * class + MyRandom(MONEY_PER_BOT_CLASS - 1);
-		DropItemAt(GetItemIndexByName("Valuable Circuits"), level_num, x, y, how_many);
+		DropItemAt(get_item_type_by_id("Valuable Circuits"), level_num, x, y, how_many);
 	}
 
 	if ((DropDecision < ITEM_DROP_PERCENTAGE)) {
@@ -568,7 +600,7 @@ static void MakeHeldFloorItemOutOf(item * SourceItem)
 	if (i >= MAX_ITEMS_PER_LEVEL) {
 		DebugPrintf(0, "\n No free position to drop item!!! ");
 		i = 0;
-		Terminate(EXIT_FAILURE, TRUE);
+		Terminate(EXIT_FAILURE);
 	}
 	// Now we enter the item into the item list of this level
 	//
@@ -636,53 +668,24 @@ void Quick_ApplyItem(int ItemKey)
 	if (FoundItemNr == (-1))
 		return;
 
-	ApplyItem(&(Me.Inventory[FoundItemNr]));
+	apply_item(&(Me.Inventory[FoundItemNr]));
 
 };				// void Quick_ApplyItem( item* CurItem )
 
-/**
- * There are many spellbooks.  These are handled inside the code via 
- * their item_type, an integer.  Now these spellbooks affect skills.  The
- * skills are handled internally via their index in the skill list.  So
- * at some points, it's convenient to look up the skill associated with
- * the spellbook.  So we do this once and have it available all the time
- * in a convenient and easy to maintain fashion.
- */
-static int associate_skill_with_item(int item_type)
-{
-	int associated_skill = (-1);
-	if (!item_type || item_type == -1)
-		return 0;	//we have a problem here...
-
-	if (!strstr(ItemMap[item_type].item_name, "Source Book of")) {
-		fprintf(stderr, "Game took item %d for a source book - this is a bug\n", item_type);
-		return 0;
-	}
-
-	char *pos = strstr(ItemMap[item_type].item_name, "Source Book of");
-	pos += strlen("Source Book of ");
-
-	associated_skill = get_program_index_with_name(pos);
-
-	return (associated_skill);
-
-};				// int associate_skill_with_item ( int item_type )
-
 /**
  * This function checks whether a given item has the name specified. This is
  * used to match an item which its type in a flexible way (match by name instead
  * of matching by index value)
  */
-int GetItemIndexByName(const char *name)
+int get_item_type_by_id(const char *id)
 {
 	int cidx = 0;
-
 	for (; cidx < Number_Of_Item_Types; cidx++) {
-		if (!strcmp(ItemMap[cidx].item_name, name))
+		if (!strcmp(ItemMap[cidx].id, id))
 			return cidx;
 	}
 
-	ErrorMessage(__FUNCTION__, "Unable to find item name %s\n", PLEASE_INFORM, IS_WARNING_ONLY, name);
+	error_message(__FUNCTION__, "Unable to find item id %s", PLEASE_INFORM, id);
 	return -1;
 }
 
@@ -691,22 +694,22 @@ int GetItemIndexByName(const char *name)
  * used to match an item which its type in a flexible way (match by name instead
  * of matching by index value)
  */
-int MatchItemWithName(int type, const char *name)
+int item_spec_eq_id(int type, const char *id)
 {
 	if (type < 0 || type >= Number_Of_Item_Types)
-		return 0;
+		return FALSE;
 
-	if (!strcmp(ItemMap[type].item_name, name))
-		return 1;
+	if (!strcmp(ItemMap[type].id, id))
+		return TRUE;
 	else
-		return 0;
+		return FALSE;
 }
 
 /**
  * This function applies a given item (to the influencer) and maybe 
  * eliminates the item after that, if it's an item that gets used up.
  */
-void ApplyItem(item * CurItem)
+void apply_item(item * CurItem)
 {
 	int failed_usage = 0; // if an item cannot be applied, to not remove it from inventory
 
@@ -714,7 +717,7 @@ void ApplyItem(item * CurItem)
 	if (CurItem->type < 0)
 		return;
 
-	if (!ItemMap[CurItem->type].item_combat_use_description) {
+	if (!ItemMap[CurItem->type].right_use.tooltip) {
 		Me.TextVisibleTime = 0;
 		Me.TextToBeDisplayed = _("I can't use this item here.");
 		return;
@@ -744,9 +747,13 @@ void ApplyItem(item * CurItem)
 		case RUNNING_PROGRAM:
 			msg = _("You are running a program!");
 			break;
+		case TAKING_PILL:
+			msg = _("You are taking a pill!");
+			break;
 		default:
 			msg = _("You are doing something so weird the game does not understand what it is");
 		}
+		// TRANSLATORS: the trailing %s is what the player is currently already doing (drinking, running a program, ...)
 		append_new_game_message(_("How do you expect to do two things at a time? %s"), msg);
 		return;		//if the player is busy reloading or anything
 	}
@@ -754,115 +761,81 @@ void ApplyItem(item * CurItem)
 	// and therefore all we need to do from here on is execute the item effect
 	// upon the influencer or his environment.
 	//
-	if (MatchItemWithName(CurItem->type, "Barf's Energy Drink")) {
+	if (item_spec_eq_id(CurItem->type, "Barf's Energy Drink")) {
 		Me.energy += 15;
 		Me.temperature -= 15;
 		Me.running_power += 15;
-		Me.busy_time = 1;
-		Me.busy_type = DRINKING_POTION;
-	} else if (MatchItemWithName(CurItem->type, "Diet supplement")) {
+	} else if (item_spec_eq_id(CurItem->type, "Diet supplement")) {
 		Me.energy += 25;
-		Me.busy_type = DRINKING_POTION;
-		Me.busy_time = 1;
-	} else if (MatchItemWithName(CurItem->type, "Antibiotic")) {
+		play_sound("effects/new_healing_sound.ogg");
+	} else if (item_spec_eq_id(CurItem->type, "Antibiotic")) {
 		Me.energy += 50;
-		Me.busy_time = 1;
-		Me.busy_type = DRINKING_POTION;
-	} else if (MatchItemWithName(CurItem->type, "Doc-in-a-can")) {
+		play_sound("effects/new_healing_sound.ogg");
+	} else if (item_spec_eq_id(CurItem->type, "Doc-in-a-can")) {
 		Me.energy += Me.maxenergy;
-		Me.busy_time = 1;
-		Me.busy_type = DRINKING_POTION;
-	} else if (MatchItemWithName(CurItem->type, "Bottled ice")) {
+		play_sound("effects/new_healing_sound.ogg");
+	} else if (item_spec_eq_id(CurItem->type, "Bottled ice")) {
 		Me.temperature -= 50;
-		Me.busy_time = 1;
-		Me.busy_type = DRINKING_POTION;
-	} else if (MatchItemWithName(CurItem->type, "Industrial coolant")) {
+	} else if (item_spec_eq_id(CurItem->type, "Industrial coolant")) {
 		Me.temperature -= 100;
-		Me.busy_time = 1;
-		Me.busy_type = DRINKING_POTION;
-	} else if (MatchItemWithName(CurItem->type, "Liquid nitrogen")) {
+	} else if (item_spec_eq_id(CurItem->type, "Liquid nitrogen")) {
 		Me.temperature = 0;
-		Me.busy_time = 1;
-		Me.busy_type = DRINKING_POTION;
-	} else if (MatchItemWithName(CurItem->type, "Running Power Capsule")) {
+	} else if (item_spec_eq_id(CurItem->type, "Running Power Capsule")) {
 		Me.running_power = Me.max_running_power;
 		Me.running_must_rest = FALSE;
-		Me.busy_time = 1;
-		Me.busy_type = DRINKING_POTION;
-	} else if (MatchItemWithName(CurItem->type, "Strength Capsule")) {
+	} else if (item_spec_eq_id(CurItem->type, "Strength Capsule")) {
 		Me.current_power_bonus = 30;
 		Me.power_bonus_end_date = Me.current_game_date + 2.0 * 60;
-		Me.busy_time = 1;
-		Me.busy_type = DRINKING_POTION;
-	} else if (MatchItemWithName(CurItem->type, "Dexterity Capsule")) {
+	} else if (item_spec_eq_id(CurItem->type, "Dexterity Capsule")) {
 		Me.current_dexterity_bonus = 30;
 		Me.dexterity_bonus_end_date = Me.current_game_date + 2.0 * 60;
-		Me.busy_time = 1;
-		Me.busy_type = DRINKING_POTION;
-	} else if (MatchItemWithName(CurItem->type, "Map Maker")) {
+	} else if (item_spec_eq_id(CurItem->type, "Map Maker")) {
 		Me.map_maker_is_present = TRUE;
 		GameConfig.Automap_Visible = TRUE;
 		Play_Spell_ForceToEnergy_Sound();
-	} else if (MatchItemWithName(CurItem->type, "VMX Gas Grenade")) {
-		DoSkill(get_program_index_with_name("Gas grenade"), 0);
-		Me.busy_time = 1;
-		Me.busy_type = THROWING_GRENADE;
-	} else if (MatchItemWithName(CurItem->type, "Small EMP Shockwave Generator")) {
-		DoSkill(get_program_index_with_name("Small EMP grenade"), 0);
-		Me.busy_time = 1;
-		Me.busy_type = THROWING_GRENADE;
-	} else if (MatchItemWithName(CurItem->type, "EMP Shockwave Generator")) {
-		DoSkill(get_program_index_with_name("EMP grenade"), 0);
-		Me.busy_time = 1;
-		Me.busy_type = THROWING_GRENADE;
-	} else if (MatchItemWithName(CurItem->type, "Electronic Noise Generator")) {
-		DoSkill(get_program_index_with_name("Electronic Noise"), 0);
-		Me.busy_time = 1;
-		Me.busy_type = THROWING_GRENADE;
-	} else if (MatchItemWithName(CurItem->type, "Small Plasma Shockwave Emitter")) {
-		DoSkill(get_program_index_with_name("Small Plasma grenade"), 0);
-		Me.busy_time = 1;
-		Me.busy_type = THROWING_GRENADE;
-	} else if (MatchItemWithName(CurItem->type, "Plasma Shockwave Emitter")) {
-		DoSkill(get_program_index_with_name("Plasma grenade"), 0);
-		Me.busy_time = 1;
-		Me.busy_type = THROWING_GRENADE;
-	} else if (MatchItemWithName(CurItem->type, "Strength Pill")) {
+	} else if (item_spec_eq_id(CurItem->type, "Strength Pill")) {
 		Me.base_strength++;
-	} else if (MatchItemWithName(CurItem->type, "Dexterity Pill")) {
+	} else if (item_spec_eq_id(CurItem->type, "Dexterity Pill")) {
 		Me.base_dexterity++;
-	} else if (MatchItemWithName(CurItem->type, "Code Pill")) {
+	} else if (item_spec_eq_id(CurItem->type, "Code Pill")) {
 		Me.base_cooling++;
-	} else if (MatchItemWithName(CurItem->type, "Brain Enlargement Pill")) {
+	} else if (item_spec_eq_id(CurItem->type, "Brain Enlargement Pill")) {
 		Me.base_cooling = 5;
 		Me.base_strength = 5;
 		Me.base_dexterity = 5;
 		Me.base_physique = 5;
 		Takeover_Game_Lost_Sound();
 		append_new_game_message(_("The doctor warned you. You are now weak and sickly."));
-	} else if (MatchItemWithName(CurItem->type, "Teleporter homing beacon")) {
-		DoSkill(get_program_index_with_name("Sanctuary"), 0);
-	} else if (strstr(ItemMap[CurItem->type].item_name, "Source Book of")) {
-		int sidx = associate_skill_with_item(CurItem->type);
-		failed_usage = improve_program(sidx);
+	}
+
+	// Do the skill
+	if (ItemMap[CurItem->type].right_use.skill) {
+		failed_usage = !DoSkill(get_program_index_with_name(ItemMap[CurItem->type].right_use.skill), 0);
+	// Improve the skill
+	} else if (ItemMap[CurItem->type].right_use.add_skill) {
+		failed_usage = improve_program(get_program_index_with_name(ItemMap[CurItem->type].right_use.add_skill));
 
 		if(failed_usage == 0) {
 			Play_Spell_ForceToEnergy_Sound();
 		} else {
-			append_new_game_message(_("You have reached the maximum skill level for %s"),ItemMap[CurItem->type].item_name + strlen("Source Book of "));
+			// TRANSLATORS: the trailing %s is a program name
+			append_new_game_message(_("You have reached the maximum skill level for %s"), 
+									D_(ItemMap[CurItem->type].right_use.add_skill));
 			Takeover_Game_Deadlock_Sound();
 		}
-	} else if (MatchItemWithName(CurItem->type, "Repair manual")) {
-		failed_usage = improve_program(get_program_index_with_name("Repair equipment"));
 	}
 
-	play_item_sound(CurItem->type, &Me.pos);
-
-	// In some cases the item concerned is a one-shot-device like a health potion, which should
-	// evaporize after the first application.  Therefore we delete the item from the inventory list.
-	//
-
 	if (!failed_usage) {
+
+		play_item_sound(CurItem->type, &Me.pos);
+
+		// Apply busy time and busy type
+		Me.busy_time = ItemMap[CurItem->type].right_use.busy_time;
+		Me.busy_type = ItemMap[CurItem->type].right_use.busy_type;
+
+		// In some cases the item concerned is a one-shot-device like a health potion, which should
+		// evaporize after the first application.  Therefore we delete the item from the inventory list.
+		//
 		if (CurItem->multiplicity > 1)
 			CurItem->multiplicity--;
 		else
@@ -974,9 +947,9 @@ static int FindFirstInventoryIndexWithItemType(int Itemtype)
 	// Severe error:  Item type NOT found in inventory!!!
 	//
 	fprintf(stderr, "\n\nItemType: '%d'.\n", Itemtype);
-	ErrorMessage(__FUNCTION__, "\
+	error_message(__FUNCTION__, "\
 There was an item code for an item to locate in inventory, but inventory\n\
-did not contain this item type at all!  This indicates a severe bug in FreedroidRPG.", PLEASE_INFORM, IS_FATAL);
+did not contain this item type at all!  This indicates a severe bug in FreedroidRPG.", PLEASE_INFORM | IS_FATAL);
 
 	return (-1);
 
@@ -1022,10 +995,10 @@ void DeleteOneInventoryItemsOfType(int Itemtype)
 	// This point must never be reached or a severe error has occurred...
 	//
 	fprintf(stderr, "\n\nItemType: '%d'.\n", Itemtype);
-	ErrorMessage(__FUNCTION__, "\
+	error_message(__FUNCTION__, "\
 One single item of all the items of a given type in the Tux inventory\n\
 should be removed, but there was not even one such item ever found in\n\
-Tux inventory.  Something must have gone awry...", PLEASE_INFORM, IS_FATAL);
+Tux inventory.  Something must have gone awry...", PLEASE_INFORM | IS_FATAL);
 
 };				// void DeleteOneInventoryItemsOfType( int Itemtype  )
 
@@ -1207,8 +1180,8 @@ static int find_free_floor_index(level* drop_level)
 	}
 
 	// We did not find a free index.
-	ErrorMessage(__FUNCTION__, "The item array for level %d was full.\n",
-	            PLEASE_INFORM, IS_WARNING_ONLY, drop_level->levelnum);
+	error_message(__FUNCTION__, "The item array for level %d was full.",
+	            PLEASE_INFORM, drop_level->levelnum);
 	return -1;
 }
 
@@ -1399,10 +1372,10 @@ int GetFreeInventoryIndex(void)
 	// occurred, an error message must be printed out and the program
 	// must be terminated.
 	//
-	ErrorMessage(__FUNCTION__, "\
+	error_message(__FUNCTION__, "\
 A FREE INVENTORY INDEX POSITION COULD NOT BE FOUND.\n\
 This is an internal error, that must never happen unless there are\n\
-severe bugs in the inventory system.", PLEASE_INFORM, IS_FATAL);
+severe bugs in the inventory system.", PLEASE_INFORM | IS_FATAL);
 	return (-1);		// just to make compilers happy.
 };				// int GetFreeInventoryIndex( void )
 
@@ -1642,7 +1615,8 @@ void HandleInventoryScreen(void)
 		int item_idx;
 		level *item_lvl = NULL;
 
-		if ((item_idx = get_floor_item_index_under_mouse_cursor(&item_lvl)) != -1) {
+		if (MouseCursorIsInUserRect(GetMousePos_x(), GetMousePos_y()) 
+		   && (item_idx = get_floor_item_index_under_mouse_cursor(&item_lvl)) != -1) {
 			// Try to auto-put or auto-equip the item. If it's not possible,
 			// the item will be 'put in hand'.
 			if (check_for_items_to_pickup(item_lvl, item_idx)) {
@@ -1677,7 +1651,7 @@ void HandleInventoryScreen(void)
 			|| MouseCursorIsOnButton(WEAPON_MODE_BUTTON, CurPos.x, CurPos.y)) {
 			
 			// Check if the item can be installed in the weapon slot
-			if (!ItemMap[item_held_in_hand->type].item_can_be_installed_in_weapon_slot) {
+			if (ItemMap[item_held_in_hand->type].slot != WEAPON_SLOT) {
 				append_new_game_message(_("You cannot fight with this!"));
 				return;
 			}
@@ -1692,7 +1666,7 @@ void HandleInventoryScreen(void)
 			// the stat requirements for usage are met.  But maybe this is a 2-handed weapon.
 			// In this case we need to do some extra check.  If it isn't a 2-handed weapon,
 			// then we can just go ahead and equip the item
-			if (!ItemMap[item_held_in_hand->type].item_gun_requires_both_hands) {
+			if (!ItemMap[item_held_in_hand->type].weapon_needs_two_hands) {
 				DropHeldItemToSlot(&(Me.weapon_item));
 				return;
 			}
@@ -1735,7 +1709,7 @@ void HandleInventoryScreen(void)
 		if (MouseCursorIsOnButton(SHIELD_RECT_BUTTON, CurPos.x, CurPos.y)) {
 			
 			// Check if the item can be installed in the shield slot
-			if (!ItemMap[item_held_in_hand->type].item_can_be_installed_in_shield_slot) {
+			if (ItemMap[item_held_in_hand->type].slot != SHIELD_SLOT) {
 				append_new_game_message(_("You cannot equip this!"));
 				return;
 			}
@@ -1756,7 +1730,7 @@ void HandleInventoryScreen(void)
 			// A shield, when equipped, will push out any 2-handed weapon currently
 			// equipped from it's weapon slot.... So first check if a 2-handed
 			// weapon is equipped.
-			if (!ItemMap[Me.weapon_item.type].item_gun_requires_both_hands) {
+			if (!ItemMap[Me.weapon_item.type].weapon_needs_two_hands) {
 				DropHeldItemToSlot(&(Me.shield_item));
 				return;
 			}
@@ -1786,9 +1760,9 @@ void HandleInventoryScreen(void)
 			char propcheck;
 			item *slot;
 		} dropslots[] = { {
-		DRIVE_RECT_BUTTON, tocheck->item_can_be_installed_in_drive_slot, &(Me.drive_item)}, {
-		ARMOUR_RECT_BUTTON, tocheck->item_can_be_installed_in_armour_slot, &(Me.armour_item)}, {
-		HELMET_RECT_BUTTON, tocheck->item_can_be_installed_in_special_slot, &(Me.special_item)},};
+		DRIVE_RECT_BUTTON, (tocheck->slot == BOOT_SLOT), &(Me.drive_item)}, {
+		ARMOUR_RECT_BUTTON, (tocheck->slot == ARMOR_SLOT), &(Me.armour_item)}, {
+		HELMET_RECT_BUTTON, (tocheck->slot == HELM_SLOT), &(Me.special_item)},};
 		int i;
 		for (i = 0; i < sizeof(dropslots) / sizeof(dropslots[0]); i++) {
 			if (MouseCursorIsOnButton(dropslots[i].btnidx, CurPos.x, CurPos.y) && dropslots[i].propcheck
@@ -1814,7 +1788,7 @@ void HandleInventoryScreen(void)
 	//
 	if (MouseRightClicked()) {
 
-		if (Me.readied_skill == get_program_index_with_name("Repair equipment")) {
+		if (Me.readied_skill == get_program_index_with_name("Repair equipment") && Me.busy_time <= 0) {
 			// Here we know, that the repair skill is selected, therefore we try to 
 			// repair the item currently under the mouse cursor.
 			//
@@ -1837,7 +1811,7 @@ void HandleInventoryScreen(void)
 					if (Me.Inventory[Grabbed_InvPos].max_durability != -1)
 						self_repair_item(&(Me.Inventory[Grabbed_InvPos]));
 					else
-						ApplyItem(&(Me.Inventory[Grabbed_InvPos]));
+						apply_item(&(Me.Inventory[Grabbed_InvPos]));
 				}
 			} else {
 				int i;
@@ -1859,10 +1833,9 @@ void HandleInventoryScreen(void)
 
 				Grabbed_InvPos = GetInventoryItemAt(Inv_GrabLoc.x, Inv_GrabLoc.y);
 
-				if (Grabbed_InvPos != -1) {
+				if ((Grabbed_InvPos != -1) && (&(Me.Inventory[Grabbed_InvPos]) != item_held_in_hand)) {
 					// At this point we know, that we have just applied something from the inventory
-					//
-					ApplyItem(&(Me.Inventory[Grabbed_InvPos]));
+					apply_item(&(Me.Inventory[Grabbed_InvPos]));
 				}
 			}
 		}
@@ -1947,7 +1920,7 @@ int try_give_item(item *ItemPointer)
 	// In the special case of money, we add the amount of money to our
 	// money counter and eliminate the item on the floor.
 
-	if (MatchItemWithName(ItemPointer->type, "Valuable Circuits")) {
+	if (item_spec_eq_id(ItemPointer->type, "Valuable Circuits")) {
 		play_item_sound(ItemPointer->type, &Me.pos);
 		Me.Gold += ItemPointer->multiplicity;
 		DeleteItem(ItemPointer);
@@ -1971,7 +1944,7 @@ int try_give_item(item *ItemPointer)
 	// Maybe the item is of a kind that can be equipped right now.  Then
 	// we decide to directly drop it to the corresponding slot.
 
-	if ((Me.weapon_item.type == (-1)) && (ItemMap[ItemPointer->type].item_can_be_installed_in_weapon_slot)) {
+	if ((Me.weapon_item.type == (-1)) && (ItemMap[ItemPointer->type].slot == WEAPON_SLOT)) {
 		if (ItemUsageRequirementsMet(ItemPointer, TRUE)) {
 			// Now we're picking up a weapon while no weapon is equipped.  But still
 			// it might be a 2-handed weapon while there is some shield equipped.  Well,
@@ -1985,14 +1958,14 @@ int try_give_item(item *ItemPointer)
 			// So now we know that some shield item is equipped.  Let's be careful:  2-handed
 			// weapons will be rejected from direct addition to the slot.
 			//
-			if (!ItemMap[ItemPointer->type].item_gun_requires_both_hands) {
+			if (!ItemMap[ItemPointer->type].weapon_needs_two_hands) {
 				raw_move_picked_up_item_to_entry(ItemPointer, &(Me.weapon_item), Inv_Loc);
 				return 1;
 			}
 		}
 	}
 
-	if ((Me.shield_item.type == (-1)) && (ItemMap[ItemPointer->type].item_can_be_installed_in_shield_slot)) {
+	if ((Me.shield_item.type == (-1)) && (ItemMap[ItemPointer->type].slot == SHIELD_SLOT)) {
 		if (ItemUsageRequirementsMet(ItemPointer, TRUE)) {
 			// Auto-equipping shields can be done.  But only if there isn't a 2-handed
 			// weapon equipped already.  Well, in case of no weapon present it's easy:
@@ -2004,28 +1977,28 @@ int try_give_item(item *ItemPointer)
 			// But now we know, that there is some weapon present.  We need to be careful:
 			// it might be a 2-handed weapon.
 			// 
-			if (!ItemMap[Me.weapon_item.type].item_gun_requires_both_hands) {
+			if (!ItemMap[Me.weapon_item.type].weapon_needs_two_hands) {
 				raw_move_picked_up_item_to_entry(ItemPointer, &(Me.shield_item), Inv_Loc);
 				return 1;
 			}
 		}
 	}
 
-	if ((Me.armour_item.type == (-1)) && (ItemMap[ItemPointer->type].item_can_be_installed_in_armour_slot)) {
+	if ((Me.armour_item.type == (-1)) && (ItemMap[ItemPointer->type].slot == ARMOR_SLOT)) {
 		if (ItemUsageRequirementsMet(ItemPointer, TRUE)) {
 			raw_move_picked_up_item_to_entry(ItemPointer, &(Me.armour_item), Inv_Loc);
 			return 1;
 		}
 	}
 
-	if ((Me.drive_item.type == (-1)) && (ItemMap[ItemPointer->type].item_can_be_installed_in_drive_slot)) {
+	if ((Me.drive_item.type == (-1)) && (ItemMap[ItemPointer->type].slot == BOOT_SLOT)) {
 		if (ItemUsageRequirementsMet(ItemPointer, TRUE)) {
 			raw_move_picked_up_item_to_entry(ItemPointer, &(Me.drive_item), Inv_Loc);
 			return 1;
 		}
 	}
 
-	if ((Me.special_item.type == (-1)) && (ItemMap[ItemPointer->type].item_can_be_installed_in_special_slot)) {
+	if ((Me.special_item.type == (-1)) && (ItemMap[ItemPointer->type].slot == HELM_SLOT)) {
 		if (ItemUsageRequirementsMet(ItemPointer, TRUE)) {
 			raw_move_picked_up_item_to_entry(ItemPointer, &(Me.special_item), Inv_Loc);
 			return 1;
@@ -2044,10 +2017,10 @@ int try_give_item(item *ItemPointer)
 	// inventory later.
 
 	if ((ItemMap[ItemPointer->type].inv_size.x == 1) &&
-	    (ItemMap[ItemPointer->type].inv_size.y == 1) && (ItemMap[ItemPointer->type].item_combat_use_description)) {
+	    (ItemMap[ItemPointer->type].inv_size.y == 1) && (ItemMap[ItemPointer->type].right_use.tooltip)) {
 		DebugPrintf(2, "\n\nTrying to place this item inside of the quick inventory first...");
 		Inv_Loc.y = INVENTORY_GRID_HEIGHT - 1;
-		for (Inv_Loc.x = 0; Inv_Loc.x < INVENTORY_GRID_HEIGHT - ItemMap[ItemPointer->type].inv_size.x + 1; Inv_Loc.x++) {
+		for (Inv_Loc.x = 0; Inv_Loc.x < INVENTORY_GRID_WIDTH - ItemMap[ItemPointer->type].inv_size.x + 1; Inv_Loc.x++) {
 			if (place_item_on_this_position_if_you_can(ItemPointer, Inv_Loc, InvPos))
 				return 1;
 		}
@@ -2072,29 +2045,6 @@ int try_give_item(item *ItemPointer)
 	return 1;
 }
 
-const char *ammo_desc_for_weapon(int type) {
-	const char *ammo_desc[] = {
-		"", // no ammunition
-		_("Laser power pack"),
-		_("Plasma energy container"),
-		_("2 mm Exterminator Ammunition"),
-		_(".22 LR Ammunition"),
-		_("Shotgun shells"),
-		_("9x19mm Ammunition"),
-		_("7.62x39mm Ammunition"),
-		_(".50 BMG (12.7x99mm) Ammunition"),
-	};
-
-	itemspec *weapon = &ItemMap[type];
-
-	if (weapon->item_gun_use_ammunition < 0 || weapon->item_gun_use_ammunition >= sizeof(ammo_desc)/sizeof(ammo_desc[0])) {
-		ErrorMessage(__FUNCTION__, "Unknown ammunition type %d for weapon %s.",
-					 PLEASE_INFORM, IS_FATAL, weapon->item_gun_use_ammunition, weapon->item_name);
-	}
-
-	return ammo_desc[weapon->item_gun_use_ammunition];
-}
-
 /**
  * \brief Places the item to the inventory of the player or to the floor.
  *
@@ -2133,4 +2083,50 @@ int item_is_currently_equipped(item * Item)
 	return 0;
 }
 
+enum slot_type get_slot_type_by_name(char *name)
+{
+	struct { 
+		const char *name;
+		enum slot_type slot;
+	} slots[] = {	{ "weapon", WEAPON_SLOT }, 
+					{"drive", 	BOOT_SLOT 	}, 
+					{"shield", 	SHIELD_SLOT }, 
+					{"armour", 	ARMOR_SLOT 	}, 
+					{"special", HELM_SLOT 	}};
+
+	int i;
+	for (i = 0; i < sizeof(slots)/sizeof(slots[0]); i++) {
+		if (!strcmp(name, slots[i].name))
+			return slots[i].slot;
+	}
+	return NO_SLOT;
+}
+
+/**
+ * \brief Get the name of a item specs by its type.
+ * If the item spec haven't a title, the name is used, and otherwise "UNNAMED ITEM".
+ * \param type A valid type of itemspec (different of -1).
+ * \return The title of the item.
+ */
+const char *item_specs_get_name(int type)
+{
+	if (ItemMap[type].name)
+		return ItemMap[type].name;
+	else if (ItemMap[type].id)
+		return ItemMap[type].id;
+	return "BUG - UNNAMED ITEM";
+}
+
+enum _busytype get_busy_type_by_name(char *name)
+{
+	if (!strcmp(name, "drinking")) {
+		return DRINKING_POTION;
+	} else if (!strcmp(name, "throwing")) {
+		return THROWING_GRENADE;
+	} else if (!strcmp(name, "pill")) {
+		return TAKING_PILL;
+	}
+	return NONE;
+}
+
 #undef _items_c
