@@ -41,53 +41,151 @@
 
 static int input_keyboard_locked; // Block the execution of the keybindings.
 
-/* name of each keybinding */
-const char *keybindNames[] = {
-	/* General */
-	"keychart", "fullscreen", "quit", "wall_transparency", "grab_input", "take_screenshot",
+/* Default keybindings
+ *
+ * If changing defaults, please make sure to update relevant parts in
+ * README
+ * map/titles/StartOfGame.title
+ * map/titles/level_editor_help.title
+ */
 
-	/* Ingame */
-	"inventory", "skill", "character", "quests",
-	"reload", "autorun",
-	"move_north", "move_south", "move_east", "move_west",
-	"quicksave", "quickload", "pause",
-	"automap",
-	"show_item_labels",
-	"activate_program0", "activate_program1", "activate_program2", "activate_program3",
-	"activate_program4", "activate_program5", "activate_program6", "activate_program7",
-	"quick_inventory0", "quick_inventory1", "quick_inventory2",
-	"quick_inventory3", "quick_inventory4", "quick_inventory5",
-	"quick_inventory6", "quick_inventory7", "quick_inventory8",
-	"quick_inventory9", "close_windows", "game_escape_menu",
+const keybind_t default_keybinds[] = {
+	{ N_("keychart"), SDLK_F1, KMOD_NONE },
+	{ N_("fullscreen"), SDLK_F2, KMOD_NONE },
+	{ N_("grab_input"), SDLK_g, KMOD_LCTRL },
+	{ N_("quit"), SDLK_q, KMOD_LCTRL },
+	{ N_("wall_transparency"), SDLK_t, KMOD_NONE },
+	{ N_("take_screenshot"), SDLK_PRINT, KMOD_NONE },
 
-	/* Leveleditor */
-	"drop_item",
-	"place_obstacle_kp1", "place_obstacle_kp2", "place_obstacle_kp3",
-	"place_obstacle_kp4", "place_obstacle_kp5", "place_obstacle_kp6",
-	"place_obstacle_kp7", "place_obstacle_kp8", "place_obstacle_kp9",
-	"change_obstacle_label", "change_map_label", "zoom_out",
-	"cycle_marked_object",
-	"cut", "copy", "paste",
-	"next_selection_type", "previous_selection_type",
-	"next_tab", "previous_tab", "undo", "redo", "beautify_grass", "beautify_water",
-	"toggle_waypoint", "toggle_waypoint_randomspawn", "connect_waypoint",
-	"toolbar_scroll_left", "toolbar_scroll_right", "toolbar_step_left", "toolbar_step_right",
-	"run_map_validator",
+	/* Game */
+	{ N_("inventory"), SDLK_i, KMOD_NONE },
+	{ N_("skill"), SDLK_s, KMOD_NONE },
+	{ N_("character"), SDLK_c, KMOD_NONE },
+	{ N_("quests"), SDLK_q, KMOD_NONE },
+	{ N_("reload"), SDLK_r, KMOD_NONE },
+	{ N_("autorun"), SDLK_u, KMOD_NONE },
 
-	/* Cheat keys */
-	"cheat_xp+_1k", "cheat_xp*_2",
-	"cheat_melee", "cheat_range", "cheat_programing", "cheat_melee_down", "cheat_range_down", "cheat_programing_down",
-	"cheat_drop_random_item", "cheat_drop_random_magical_item",
-	"cheat_respawn_level",
-	"cheat_menu", "cheat_level_editor",
-	"cheat_reload_graphics",
-	NULL
-};				/* must terminate with NULL */
+	{ N_("move_north"), SDLK_UP, KMOD_NONE },
+	{ N_("move_south"), SDLK_DOWN, KMOD_NONE },
+	{ N_("move_east"), SDLK_RIGHT, KMOD_NONE },
+	{ N_("move_west"), SDLK_LEFT, KMOD_NONE },
+
+	{ N_("quicksave"), SDLK_F3, KMOD_NONE },
+	{ N_("quickload"), SDLK_F4, KMOD_NONE },
+	{ N_("pause"), SDLK_p, KMOD_NONE },
+	{ N_("show_item_labels"), SDLK_z, KMOD_NONE },
+	{ N_("automap"), SDLK_TAB, KMOD_NONE },
+
+	{ N_("activate_program0"), SDLK_F5, KMOD_NONE },
+	{ N_("activate_program1"), SDLK_F6, KMOD_NONE },
+	{ N_("activate_program2"), SDLK_F7, KMOD_NONE },
+	{ N_("activate_program3"), SDLK_F8, KMOD_NONE },
+	{ N_("activate_program4"), SDLK_F9, KMOD_NONE },
+	{ N_("activate_program5"), SDLK_F10, KMOD_NONE },
+	{ N_("activate_program6"), SDLK_F11, KMOD_NONE },
+	{ N_("activate_program7"), SDLK_F12, KMOD_NONE },
+
+	{ N_("quick_inventory0"), SDLK_0, KMOD_NONE },
+	{ N_("quick_inventory1"), SDLK_1, KMOD_NONE },
+	{ N_("quick_inventory2"), SDLK_2, KMOD_NONE },
+	{ N_("quick_inventory3"), SDLK_3, KMOD_NONE },
+	{ N_("quick_inventory4"), SDLK_4, KMOD_NONE },
+	{ N_("quick_inventory5"), SDLK_5, KMOD_NONE },
+	{ N_("quick_inventory6"), SDLK_6, KMOD_NONE },
+	{ N_("quick_inventory7"), SDLK_7, KMOD_NONE },
+	{ N_("quick_inventory8"), SDLK_8, KMOD_NONE },
+	{ N_("quick_inventory9"), SDLK_9, KMOD_NONE },
+
+	{ N_("close_windows"), SDLK_SPACE, KMOD_NONE },
+	{ N_("game_escape_menu"), SDLK_ESCAPE, KMOD_NONE },
+
+	/* Level editor */
+	{ N_("drop_item"), SDLK_g, KMOD_NONE },
+	{ N_("place_obstacle_kp0"), SDLK_KP0, KMOD_NONE },
+	{ N_("place_obstacle_kp1"), SDLK_KP1, KMOD_NONE },
+	{ N_("place_obstacle_kp2"), SDLK_KP2, KMOD_NONE },
+	{ N_("place_obstacle_kp3"), SDLK_KP3, KMOD_NONE },
+	{ N_("place_obstacle_kp4"), SDLK_KP4, KMOD_NONE },
+	{ N_("place_obstacle_kp5"), SDLK_KP5, KMOD_NONE },
+	{ N_("place_obstacle_kp6"), SDLK_KP6, KMOD_NONE },
+	{ N_("place_obstacle_kp7"), SDLK_KP7, KMOD_NONE },
+	{ N_("place_obstacle_kp8"), SDLK_KP8, KMOD_NONE },
+	{ N_("place_obstacle_kp9"), SDLK_KP9, KMOD_NONE },
+	{ N_("change_obstacle_label"), SDLK_h, KMOD_NONE },
+	{ N_("change_map_label"), SDLK_m, KMOD_NONE },
+	{ N_("zoom_out"), SDLK_o, KMOD_NONE },
+	{ N_("cycle_marked_object"), SDLK_n, KMOD_NONE },
+	{ N_("next_tab"), SDLK_f, KMOD_NONE },
+	{ N_("previous_tab"), SDLK_f, KMOD_LSHIFT },
+	{ N_("undo"), SDLK_z, KMOD_NONE },
+	{ N_("redo"), SDLK_y, KMOD_NONE },
+	{ N_("beautify_grass"), SDLK_b, KMOD_LCTRL },
+	{ N_("beautify_water"), SDLK_w, KMOD_LCTRL },
+	{ N_("toggle_waypoint"), SDLK_w, KMOD_NONE },
+	{ N_("toggle_waypoint_randomspawn"), SDLK_w, KMOD_LSHIFT },
+	{ N_("connect_waypoint"), SDLK_c, KMOD_NONE },
+	{ N_("toolbar_scroll_left"), SDLK_PAGEUP, KMOD_NONE },
+	{ N_("toolbar_scroll_right"), SDLK_PAGEDOWN, KMOD_NONE },
+	{ N_("toolbar_step_left"), SDLK_PAGEUP, KMOD_LCTRL },
+	{ N_("toolbar_step_right"), SDLK_PAGEDOWN, KMOD_LCTRL },
+	{ N_("cut"), SDLK_x, KMOD_LCTRL },
+	{ N_("copy"), SDLK_c, KMOD_LCTRL },
+	{ N_("paste"), SDLK_v, KMOD_LCTRL },
+	{ N_("delete"), SDLK_BACKSPACE, KMOD_NONE },
+	{ N_("next_selection_type"), SDLK_TAB, KMOD_NONE },
+	{ N_("previous_selection_type"), SDLK_TAB, KMOD_LSHIFT },
+	{ N_("run_map_validator"), SDLK_e, KMOD_LCTRL },
+
+	/* Cheat */
+	{ N_("cheat_xp+_1k"), SDLK_KP1, KMOD_NONE },
+	{ N_("cheat_xp*_2"), SDLK_KP2, KMOD_NONE },
+	{ N_("cheat_melee"), SDLK_KP7, KMOD_NONE },
+	{ N_("cheat_range"), SDLK_KP8, KMOD_NONE },
+	{ N_("cheat_programing"), SDLK_KP9, KMOD_NONE },
+	{ N_("cheat_melee_down"), SDLK_KP4, KMOD_NONE },
+	{ N_("cheat_range_down"), SDLK_KP5, KMOD_NONE },
+	{ N_("cheat_programing_down"), SDLK_KP6, KMOD_NONE },
+	{ N_("cheat_drop_random_item"), SDLK_r, KMOD_LCTRL },
+	{ N_("cheat_drop_random_magical_item"), SDLK_r, KMOD_LCTRL | KMOD_LSHIFT },
+	{ N_("cheat_respawn_level"), SDLK_r, KMOD_LCTRL | KMOD_LALT | KMOD_LSHIFT },
+	{ N_("cheat_level_editor"), SDLK_e, KMOD_LCTRL | KMOD_LALT },
+	{ N_("cheat_menu"), SDLK_c, KMOD_LCTRL | KMOD_LALT | KMOD_LSHIFT },
+	{ N_("cheat_reload_graphics"), SDLK_g, KMOD_LCTRL | KMOD_LALT | KMOD_LSHIFT },
+	{ N_("cheat_inc_repair"), SDLK_r, KMOD_RCTRL  },
+	{ N_("cheat_dec_repair"), SDLK_r, KMOD_RCTRL | KMOD_RSHIFT  },
+
+#ifdef WITH_RTPROF
+	/* RT Profiler */
+	{ N_("rtprof_switch_activation"), SDLK_p, KMOD_LALT },
+	{ N_("rtprof_clear_probes"), SDLK_o, KMOD_LALT },
+#endif
+
+	{ NULL }
+};
+
+/*
+ * On some keyboards (AZERTY is an example), the shift modifier is needed to
+ * access the numeric keys, which are at the top of the alphanumeric keyboard.
+ * In this case, the standard 'toupper()' method used to get the actual value of
+ * the pressed key does not work.
+ * To circumvent the problem, we use the unicode value of the key, and transform
+ * it into a keypad sym value.
+ *
+ * TRUE is returned if the pressed key is a numkey.
+ */
+static int get_numkey(SDL_keysym *keysym, int *numkey)
+{
+	if (!ISKEYPAD((int)keysym->sym) && keysym->unicode >= '0' && keysym->unicode <= '9') {
+		*numkey = SDLK_0 + (keysym->unicode - '0');
+		return TRUE;
+	}
+	return FALSE;
+}
 
 /**
  * @fn void input_keyboard_init (void)
  *
- * @brief Initializes the input subsystem (does not set keys).
+ * @brief Initializes the input subsystem (using default keybindings).
  */
 void input_keyboard_init(void)
 {
@@ -95,18 +193,18 @@ void input_keyboard_init(void)
 
 	GameConfig.input_keybinds[0].name = NULL;
 
-	if (sizeof(keybindNames) / sizeof(keybindNames[0]) > sizeof(GameConfig.input_keybinds) / sizeof(GameConfig.input_keybinds[0]))
-		ErrorMessage(__FUNCTION__,
-			     "There are %zu keyboard commands defined in keyboard.c, but GameConfig structure only supports %zu.\n",
-			     PLEASE_INFORM, IS_FATAL,
-			     sizeof(keybindNames) / sizeof(keybindNames[0]),
+	if (sizeof(default_keybinds) / sizeof(default_keybinds[0]) > sizeof(GameConfig.input_keybinds) / sizeof(GameConfig.input_keybinds[0]))
+		error_message(__FUNCTION__,
+			     "There are " SIZE_T_F " keyboard commands defined in keyboard.c, but GameConfig structure only supports " SIZE_T_F ".",
+			     PLEASE_INFORM | IS_FATAL,
+			     sizeof(default_keybinds) / sizeof(default_keybinds[0]),
 			     sizeof(GameConfig.input_keybinds) / sizeof(GameConfig.input_keybinds[0]));
 
 	/* creates a null keybinding for each */
-	for (i = 0; keybindNames[i] != NULL; i++) {
-		GameConfig.input_keybinds[i].name = (char *)keybindNames[i];
-		GameConfig.input_keybinds[i].key = SDLK_UNKNOWN;
-		GameConfig.input_keybinds[i].mod = KMOD_NONE;
+	for (i = 0; default_keybinds[i].name != NULL; i++) {
+		GameConfig.input_keybinds[i].name = (char *)default_keybinds[i].name;
+		GameConfig.input_keybinds[i].key = default_keybinds[i].key;
+		GameConfig.input_keybinds[i].mod = default_keybinds[i].mod;
 	}
 
 	GameConfig.input_keybinds[i].name = NULL;
@@ -114,15 +212,6 @@ void input_keyboard_init(void)
 	input_keyboard_locked = 0;
 }
 
-/**
- * @fn void input_exit (void)
- *
- * @brief Exits the input subsystem.
- */
-void input_exit(void)
-{
-}
-
 /**
  *
  * @brief Binds key to action keybind.
@@ -134,20 +223,22 @@ void input_exit(void)
 void input_set_keybind(char *keybind, SDLKey key, SDLMod mod)
 {
 	int i;
-	for (i = 0; keybindNames[i] != NULL; i++)
-		if (strcmp(keybind, GameConfig.input_keybinds[i].name) == 0) {
+	for (i = 0; GameConfig.input_keybinds[i].name != NULL; i++)
+		if (!strcmp(keybind, GameConfig.input_keybinds[i].name)) {
 			GameConfig.input_keybinds[i].key = key;
 			GameConfig.input_keybinds[i].mod = mod;
 			return;
 		}
-	ErrorMessage(__FUNCTION__, "Unable to set keybinding '%s', that command doesn't exist.\n", NO_NEED_TO_INFORM, IS_WARNING_ONLY,
-		     keybind);
+	error_message(__FUNCTION__,
+	             "Unable to set keybinding '%s', that command doesn't exist.",
+	             NO_REPORT,
+	             keybind);
 }
 
 void input_get_keybind(const char *cmdname, SDLKey * key, SDLMod * mod)
 {
 	int i;
-	for (i = 0; keybindNames[i] != NULL; i++) {
+	for (i = 0; GameConfig.input_keybinds[i].name != NULL; i++) {
 		if (!strcmp(cmdname, GameConfig.input_keybinds[i].name)) {
 			if (key)
 				*key = GameConfig.input_keybinds[i].key;
@@ -156,8 +247,10 @@ void input_get_keybind(const char *cmdname, SDLKey * key, SDLMod * mod)
 			return;
 		}
 	}
-	ErrorMessage(__FUNCTION__, "Unable to get keybinding for command '%s', that command does not exist.\n", PLEASE_INFORM, IS_FATAL,
-		     cmdname);
+	error_message(__FUNCTION__,
+	             "Unable to get keybinding for command '%s', that command does not exist.",
+	             PLEASE_INFORM | IS_FATAL,
+	             cmdname);
 
 	/* This dead code writes "key" and "mod" to shut up GCC */
 	*key = GameConfig.input_keybinds[0].key;
@@ -169,149 +262,26 @@ void input_get_keybind(const char *cmdname, SDLKey * key, SDLMod * mod)
  * @param cmd name of the command
  * @param out pointer to a sufficiently large buffer to hold the key string
  */
-void input_get_keybind_string(const char *cmd, char *out)
+void input_get_keybind_string(const keybind_t *kb, char *out)
 {
-	SDLKey key;
-	SDLMod mod;
-
-	input_get_keybind(cmd, &key, &mod);
-
 	const char *ctrl_modstr = "";
 	const char *alt_modstr = "";
 	const char *shift_modstr = "";
 
-	if (mod & KMOD_LCTRL || mod & KMOD_RCTRL)
-		ctrl_modstr = "C-";
-	if (mod & KMOD_LALT || mod & KMOD_RALT)
-		alt_modstr = "A-";
-	if (mod & KMOD_LSHIFT || mod & KMOD_RSHIFT)
-		shift_modstr = "S-";
+	if ((kb->mod & KMOD_LCTRL) || (kb->mod & KMOD_RCTRL))
+		ctrl_modstr = "Ctrl+";
+	if ((kb->mod & KMOD_LALT)  || (kb->mod & KMOD_RALT))
+		alt_modstr = "Alt+";
+	if ((kb->mod & KMOD_LSHIFT) || (kb->mod & KMOD_RSHIFT))
+		shift_modstr = "Shift+";
 
-	sprintf(out, "%s%s%s%s", ctrl_modstr, alt_modstr, shift_modstr, SDL_GetKeyName(key));
+	sprintf(out, "%s%s%s%s", ctrl_modstr, alt_modstr, shift_modstr, SDL_GetKeyName(kb->key));
 }
 
-
-/**
- * @fn void input_setDefault (void)
- *
- * @brief Sets the default input keys.
- */
-void input_set_default(void)
-{
-	/*
-	   If changing defaults, please make sure to update relevant parts in
-	   README 
-	   map/titles/StartOfGame.title
-	   map/titles/level_editor_help.title
-	 */
-
-	input_set_keybind("keychart", SDLK_F1, KMOD_NONE);
-	input_set_keybind("fullscreen", SDLK_F2, KMOD_NONE);
-	input_set_keybind("grab_input", SDLK_g, KMOD_LCTRL);
-	input_set_keybind("quit", SDLK_q, KMOD_LCTRL);
-	input_set_keybind("wall_transparency", SDLK_t, KMOD_NONE);
-	input_set_keybind("take_screenshot", SDLK_PRINT, KMOD_NONE);
-
-	/* Game */
-	input_set_keybind("inventory", SDLK_i, KMOD_NONE);
-	input_set_keybind("skill", SDLK_s, KMOD_NONE);
-	input_set_keybind("character", SDLK_c, KMOD_NONE);
-	input_set_keybind("quests", SDLK_q, KMOD_NONE);
-	input_set_keybind("reload", SDLK_r, KMOD_NONE);
-	input_set_keybind("autorun", SDLK_u, KMOD_NONE);
-
-	input_set_keybind("move_north", SDLK_UP, KMOD_NONE);
-	input_set_keybind("move_south", SDLK_DOWN, KMOD_NONE);
-	input_set_keybind("move_east", SDLK_RIGHT, KMOD_NONE);
-	input_set_keybind("move_west", SDLK_LEFT, KMOD_NONE);
-
-	input_set_keybind("quicksave", SDLK_F3, KMOD_NONE);
-	input_set_keybind("quickload", SDLK_F4, KMOD_NONE);
-	input_set_keybind("pause", SDLK_p, KMOD_NONE);
-	input_set_keybind("show_item_labels", SDLK_z, KMOD_NONE);
-	input_set_keybind("automap", SDLK_TAB, KMOD_NONE);
-
-	input_set_keybind("activate_program0", SDLK_F5, KMOD_NONE);
-	input_set_keybind("activate_program1", SDLK_F6, KMOD_NONE);
-	input_set_keybind("activate_program2", SDLK_F7, KMOD_NONE);
-	input_set_keybind("activate_program3", SDLK_F8, KMOD_NONE);
-	input_set_keybind("activate_program4", SDLK_F9, KMOD_NONE);
-	input_set_keybind("activate_program5", SDLK_F10, KMOD_NONE);
-	input_set_keybind("activate_program6", SDLK_F11, KMOD_NONE);
-	input_set_keybind("activate_program7", SDLK_F12, KMOD_NONE);
-
-	input_set_keybind("quick_inventory0", SDLK_0, KMOD_NONE);
-	input_set_keybind("quick_inventory1", SDLK_1, KMOD_NONE);
-	input_set_keybind("quick_inventory2", SDLK_2, KMOD_NONE);
-	input_set_keybind("quick_inventory3", SDLK_3, KMOD_NONE);
-	input_set_keybind("quick_inventory4", SDLK_4, KMOD_NONE);
-	input_set_keybind("quick_inventory5", SDLK_5, KMOD_NONE);
-	input_set_keybind("quick_inventory6", SDLK_6, KMOD_NONE);
-	input_set_keybind("quick_inventory7", SDLK_7, KMOD_NONE);
-	input_set_keybind("quick_inventory8", SDLK_8, KMOD_NONE);
-	input_set_keybind("quick_inventory9", SDLK_9, KMOD_NONE);
-
-	input_set_keybind("close_windows", SDLK_SPACE, KMOD_NONE);
-	input_set_keybind("game_escape_menu", SDLK_ESCAPE, KMOD_NONE);
-
-	/* Level editor */
-	input_set_keybind("drop_item", SDLK_g, KMOD_NONE);
-	input_set_keybind("place_obstacle_kp1", SDLK_KP1, KMOD_NONE);
-	input_set_keybind("place_obstacle_kp2", SDLK_KP2, KMOD_NONE);
-	input_set_keybind("place_obstacle_kp3", SDLK_KP3, KMOD_NONE);
-	input_set_keybind("place_obstacle_kp4", SDLK_KP4, KMOD_NONE);
-	input_set_keybind("place_obstacle_kp5", SDLK_KP5, KMOD_NONE);
-	input_set_keybind("place_obstacle_kp6", SDLK_KP6, KMOD_NONE);
-	input_set_keybind("place_obstacle_kp7", SDLK_KP7, KMOD_NONE);
-	input_set_keybind("place_obstacle_kp8", SDLK_KP8, KMOD_NONE);
-	input_set_keybind("place_obstacle_kp9", SDLK_KP9, KMOD_NONE);
-	input_set_keybind("place_obstacle_kp9", SDLK_KP9, KMOD_NONE);
-	input_set_keybind("change_obstacle_label", SDLK_h, KMOD_NONE);
-	input_set_keybind("change_map_label", SDLK_m, KMOD_NONE);
-	input_set_keybind("zoom_out", SDLK_o, KMOD_NONE);
-	input_set_keybind("cycle_marked_object", SDLK_n, KMOD_NONE);
-	input_set_keybind("next_tab", SDLK_f, KMOD_NONE);
-	input_set_keybind("previous_tab", SDLK_f, KMOD_LSHIFT);
-	input_set_keybind("undo", SDLK_z, KMOD_NONE);
-	input_set_keybind("redo", SDLK_y, KMOD_NONE);
-	input_set_keybind("beautify_grass", SDLK_b, KMOD_LCTRL);
-	input_set_keybind("beautify_water", SDLK_w, KMOD_LCTRL);
-	input_set_keybind("toggle_waypoint", SDLK_w, KMOD_NONE);
-	input_set_keybind("toggle_waypoint_randomspawn", SDLK_w, KMOD_LSHIFT);
-	input_set_keybind("connect_waypoint", SDLK_c, KMOD_NONE);
-	input_set_keybind("toolbar_scroll_left", SDLK_PAGEUP, KMOD_NONE);
-	input_set_keybind("toolbar_scroll_right", SDLK_PAGEDOWN, KMOD_NONE);
-	input_set_keybind("toolbar_step_left", SDLK_PAGEUP, KMOD_LCTRL);
-	input_set_keybind("toolbar_step_right", SDLK_PAGEDOWN, KMOD_LCTRL);
-	input_set_keybind("cut", SDLK_x, KMOD_LCTRL);
-	input_set_keybind("copy", SDLK_c, KMOD_LCTRL);
-	input_set_keybind("paste", SDLK_v, KMOD_LCTRL);
-	input_set_keybind("next_selection_type", SDLK_TAB, KMOD_NONE);
-	input_set_keybind("previous_selection_type", SDLK_TAB, KMOD_LSHIFT);
-	input_set_keybind("run_map_validator", SDLK_e, KMOD_LCTRL);
-
-	/* Cheat */
-	input_set_keybind("cheat_xp+_1k", SDLK_KP1, KMOD_NONE);
-	input_set_keybind("cheat_xp*_2", SDLK_KP2, KMOD_NONE);
-	input_set_keybind("cheat_melee", SDLK_KP7, KMOD_NONE);
-	input_set_keybind("cheat_range", SDLK_KP8, KMOD_NONE);
-	input_set_keybind("cheat_programing", SDLK_KP9, KMOD_NONE);
-	input_set_keybind("cheat_melee_down", SDLK_KP4, KMOD_NONE);
-	input_set_keybind("cheat_range_down", SDLK_KP5, KMOD_NONE);
-	input_set_keybind("cheat_programing_down", SDLK_KP6, KMOD_NONE);
-	input_set_keybind("cheat_drop_random_item", SDLK_r, KMOD_LCTRL);
-	input_set_keybind("cheat_drop_random_magical_item", SDLK_r, KMOD_LCTRL | KMOD_LSHIFT);
-	input_set_keybind("cheat_respawn_level", SDLK_r, KMOD_LCTRL | KMOD_LALT | KMOD_LSHIFT);
-	input_set_keybind("cheat_level_editor", SDLK_e, KMOD_LCTRL | KMOD_LALT);
-	input_set_keybind("cheat_menu", SDLK_c, KMOD_LCTRL | KMOD_LALT | KMOD_LSHIFT);
-	input_set_keybind("cheat_reload_graphics", SDLK_g, KMOD_LCTRL | KMOD_LALT | KMOD_LSHIFT);
-
-}
-
-#define KEYCHART_RECT_X (GameConfig.screen_width / 250)
+#define KEYCHART_RECT_X (GameConfig.screen_width / 100)
 #define KEYCHART_RECT_Y (GameConfig.screen_height / 10)
-#define KEYCHART_RECT_W (0.99 * GameConfig.screen_width)
-#define KEYCHART_RECT_H (0.8 * GameConfig.screen_height)
+#define KEYCHART_RECT_W (GameConfig.screen_width - 2*KEYCHART_RECT_X)
+#define KEYCHART_RECT_H (GameConfig.screen_height - 2*KEYCHART_RECT_Y)
 
 /**
  * Display the keychart
@@ -330,12 +300,12 @@ static int display_keychart(unsigned int startidx, unsigned int cursor, int high
 
 	ShadowingRectangle(Screen, keychart_rect);
 
-	CenteredPutStringFont(Screen, Para_BFont, FontHeight(Para_BFont), "Key chart");
+	put_string_centered(Para_Font, get_font_height(Para_Font), _("Key chart"));
 
-	if (startidx >= sizeof(keybindNames) / sizeof(keybindNames[0]))
+	if (startidx >= sizeof(default_keybinds) / sizeof(default_keybinds[0]))
 		return -1;
 
-	for (i = startidx; keybindNames[i] != NULL; i++) {
+	for (i = startidx; GameConfig.input_keybinds[i].name != NULL; i++) {
 		char keystr[100] = "";
 		const char *font_str = font_switchto_neon;
 		
@@ -343,25 +313,25 @@ static int display_keychart(unsigned int startidx, unsigned int cursor, int high
 			font_str = font_switchto_red;
 		}
 
-		input_get_keybind_string(keybindNames[i], &keystr[0]);
+		input_get_keybind_string(&(GameConfig.input_keybinds[i]), &keystr[0]);
 
-		sprintf(txt, "%s%s%s - %s\n", font_str, (i == cursor) ? "** " : "   ",
-			GameConfig.input_keybinds[i].name, keystr);
-		PutString(Screen, xpos, ypos, txt);
+		sprintf(txt, "%s%s%s: %s", font_str, (i == cursor) ? "> " : "   ",
+			_(GameConfig.input_keybinds[i].name), keystr);
+		put_string(get_current_font(), xpos, ypos, txt);
 
-		ypos += FontHeight(GetCurrentFont());
+		ypos += get_font_height(get_current_font());
 
-		if (ypos > keychart_rect.y + keychart_rect.h - FontHeight(GetCurrentFont())) {
+		if (ypos > keychart_rect.y + keychart_rect.h - get_font_height(get_current_font())) {
 			if (xpos > keychart_rect.x)
 				break;
 			else {
-				xpos += 300;
+				xpos += keychart_rect.w / 2;
 				ypos = keychart_rect.y;
 			}
 		}
 	}
 
-	PutStringFont(Screen, FPS_Display_BFont, 100, GameConfig.screen_height - FontHeight(FPS_Display_BFont),
+	put_string(FPS_Display_Font, 100, GameConfig.screen_height - get_font_height(FPS_Display_Font),
 		      _("ARROWS to select, ENTER to remap, ESCAPE to exit"));
 	our_SDL_flip_wrapper();
 
@@ -375,13 +345,13 @@ static int display_keychart(unsigned int startidx, unsigned int cursor, int high
 static int get_nb_commands_per_page()
 {
 	int ypos = KEYCHART_RECT_Y;
-	BFont_Info *our_font = FPS_Display_BFont;
+	struct font *our_font = FPS_Display_Font;
 	int i = 0;
 
 	while (1) {
-		ypos += FontHeight(our_font);
+		ypos += get_font_height(our_font);
 		i++;
-		if (ypos > KEYCHART_RECT_Y + KEYCHART_RECT_H - FontHeight(our_font)) {
+		if (ypos > KEYCHART_RECT_Y + KEYCHART_RECT_H - get_font_height(our_font)) {
 			break;
 		}
 	}
@@ -399,17 +369,18 @@ void keychart()
 	int endpos;
 	int cursor = 0;
 	SDL_Event event;
-	const int maxcmds = sizeof(keybindNames) / sizeof(keybindNames[0]) - 2;
 	int per_page = get_nb_commands_per_page();
+	// The actual number of keybinds is the length of the default_keybinds array
+	const int maxcmds = sizeof(default_keybinds) / sizeof(default_keybinds[0]) - 2;
 
 	Activate_Conservative_Frame_Computation();
 
 	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
 	while (!done) {
-		while (SDL_PollEvent(&event)) {
+		if (SDL_PollEvent(&event)) {
 
 			if (event.type == SDL_QUIT) {
-				Terminate(EXIT_SUCCESS, TRUE);
+				Terminate(EXIT_SUCCESS);
 			}
 
 			endpos = display_keychart(startpos, cursor, FALSE);
@@ -466,7 +437,7 @@ void keychart()
 					newmod &= ~(KMOD_CAPS | KMOD_NUM | KMOD_MODE);	/* We want to ignore "global" modifiers. */
 					GameConfig.input_keybinds[cursor].mod = newmod;
 
-					for (i = 0; keybindNames[i] != NULL; i++) {
+					for (i = 0; GameConfig.input_keybinds[i].name != NULL; i++) {
 						if (i == cursor)
 							continue;
 
@@ -481,6 +452,8 @@ void keychart()
 
 				}
 			}
+		} else {
+			SDL_Delay(1);
 		}
 	}
 	SDL_EnableKeyRepeat(0, SDL_DEFAULT_REPEAT_INTERVAL);
@@ -522,15 +495,16 @@ void input_release_keyboard(void)
  *    @param keynum The index of the  keybind.
  *    @param value The value of the keypress (defined above).
  */
-#define KEYPRESS(s)    ((strcmp(GameConfig.input_keybinds[keynum].name,s)==0) && value==KEY_PRESS)
+#define KEYPRESS(s)    (!strcmp(GameConfig.input_keybinds[keynum].name,(s)) && value==KEY_PRESS)
 #define INGAME()	(game_status == INSIDE_GAME)
 #define INMENU()	(game_status == INSIDE_MENU)
 #define INLVLEDIT()	(game_status == INSIDE_LVLEDITOR)
+
 static int input_key(int keynum, int value)
 {
 	/* Non lockable keybindings */
  	if (KEYPRESS("quit")) {
-		Terminate(EXIT_SUCCESS, TRUE);
+		Terminate(EXIT_SUCCESS);
 		return 0;
 	}
 
@@ -570,6 +544,14 @@ static int input_key(int keynum, int value)
 			respawn_level(Me.pos.z);
 		} else if (KEYPRESS("cheat_reload_graphics")) {
 			reload_graphics();
+		} else if (KEYPRESS("cheat_inc_repair")) {
+			if(Me.skill_level[get_program_index_with_name("Repair equipment")] < NUMBER_OF_SKILL_LEVELS-1) {
+				Me.skill_level[get_program_index_with_name("Repair equipment")] += 1;
+			}
+		} else if (KEYPRESS("cheat_dec_repair")) {
+			if(Me.skill_level[get_program_index_with_name("Repair equipment")] > 1) {
+				Me.skill_level[get_program_index_with_name("Repair equipment")] -= 1;
+			}
 		}
 	}
 
@@ -609,7 +591,7 @@ static int input_key(int keynum, int value)
 			SaveGame();
 			return 0;
 		} else if (KEYPRESS("quickload")) {
-			LoadGame();
+			load_game();
 			return 0;
 		} else if (KEYPRESS("pause")) {
 			Pause();
@@ -621,9 +603,9 @@ static int input_key(int keynum, int value)
 			   && value == KEY_PRESS) {
 			int number = atoi(GameConfig.input_keybinds[keynum].name + strlen("activate_program"));
 			if (number >= 10 || number < 0) {
-				ErrorMessage(__FUNCTION__,
-					     "Tried to activate skill number %d - only shortcuts from F1 to F9 are supported\n",
-					     PLEASE_INFORM, IS_WARNING_ONLY, number);
+				error_message(__FUNCTION__,
+					     "Tried to activate skill number %d - only shortcuts from F1 to F9 are supported",
+					     PLEASE_INFORM, number);
 				return 0;
 			}
 
@@ -641,8 +623,8 @@ static int input_key(int keynum, int value)
 			   && value == KEY_PRESS) {
 			int number = atoi(GameConfig.input_keybinds[keynum].name + strlen("quick_inventory"));
 			if (number >= 10 || number < 0) {
-				ErrorMessage(__FUNCTION__, "Tried to use quick inventory item %d - only 1-9 + 0 are supported.\n",
-					     PLEASE_INFORM, IS_WARNING_ONLY, number);
+				error_message(__FUNCTION__, "Tried to use quick inventory item %d - only 1-9 + 0 are supported.",
+					     PLEASE_INFORM, number);
 				return 0;
 			}
 
@@ -679,6 +661,15 @@ static int input_key(int keynum, int value)
 			}
 			return 0;
 		}
+#ifdef WITH_RTPROF
+		else if (KEYPRESS("rtprof_switch_activation")) {
+			rtprof_switch_activation();
+			return 0;
+		} else if (KEYPRESS("rtprof_clear_probes")) {
+			rtprof_clear_probes();
+			return 0;
+		}
+#endif
 	} else if (INLVLEDIT()) {
 		if (KEYPRESS("drop_item")) {
 			ItemDropFromLevelEditor();
@@ -739,8 +730,8 @@ static int input_key(int keynum, int value)
 			   && value == KEY_PRESS) {
 			int number = atoi(GameConfig.input_keybinds[keynum].name + strlen("place_obstacle_kp"));
 			if (number >= 10 || number < 0) {
-				ErrorMessage(__FUNCTION__, "Tried to place obstacle using kp%d - only supported are kp1 to kp9\n",
-					     PLEASE_INFORM, IS_WARNING_ONLY, number);
+				error_message(__FUNCTION__, "Tried to place obstacle using kp%d - only kp1 to kp9 are supported",
+					     PLEASE_INFORM, number);
 				return 0;
 			}
 			level_editor_place_aligned_object(number);
@@ -763,6 +754,9 @@ static int input_key(int keynum, int value)
 		} else if (KEYPRESS("paste")) {
 			level_editor_paste_selection();
 			return 0;
+		} else if (KEYPRESS("delete")) {
+			level_editor_delete_selection();
+			return 0;
 		}
 	}
 
@@ -783,9 +777,10 @@ static int input_key(int keynum, int value)
 
 		SDL_WM_GrabInput(mode);
 	} else if (KEYPRESS("take_screenshot")) {
+		play_sound("effects/CameraTakesPicture.ogg");
 		char filename[1000];
 		char relative_filename[100];
-		sprintf(relative_filename, "%s.screenshot-%d.bmp", Me.character_name, SDL_GetTicks()/1000);
+		sprintf(relative_filename, "%s.screenshot-%d.png", Me.character_name, SDL_GetTicks()/1000);
 		sprintf(filename, "%s/%s", our_config_dir, relative_filename);
 		save_screenshot(filename, FALSE);
 		alert_window(_("Screenshot saved to \"%s\" in your .freedroid_rpg/ directory."), relative_filename);
@@ -806,7 +801,7 @@ static int input_key_event(SDLKey key, SDLMod mod, int value)
 	int noteaten = -1;
 	mod &= ~(KMOD_CAPS | KMOD_NUM | KMOD_MODE);	/* We want to ignore "global" modifiers. */
 
-	for (i = 0; keybindNames[i] != NULL; i++)
+	for (i = 0; GameConfig.input_keybinds[i].name != NULL; i++)
 		if ((GameConfig.input_keybinds[i].key == key) && (GameConfig.input_keybinds[i].mod == mod)) {
 			if (!(noteaten = input_key(i, value)))
 				break;
@@ -814,9 +809,18 @@ static int input_key_event(SDLKey key, SDLMod mod, int value)
 	return noteaten;
 }
 
-int input_key_press(SDL_Event * event)
+int input_key_press(SDL_Event *event)
 {
-	input_key_event(event->key.keysym.sym, event->key.keysym.mod, KEY_PRESS);
+	SDLKey sym = event->key.keysym.sym;
+	SDLMod mod = event->key.keysym.mod;
+
+	int numkey;
+	if (get_numkey(&event->key.keysym, &numkey)) {
+		sym = numkey;
+		mod = 0;
+	}
+
+	input_key_event(sym, mod, KEY_PRESS);
 
 	return 0;
 }
@@ -846,29 +850,35 @@ static int kptoprint(int key) {
 int getchar_raw(int *mod)
 {
 	SDL_Event event;
-	int Returnkey;
+	int Returnkey = 0;
 
 	while (1) {
 		SDL_WaitEvent(&event);	/* wait for next event */
 
 		if (event.type == SDL_QUIT) {
-			Terminate(EXIT_SUCCESS, TRUE);
+			Terminate(EXIT_SUCCESS);
 		}
 
 		if (event.type == SDL_KEYDOWN) {
+			if (get_numkey(&event.key.keysym, &Returnkey)) {
+				if (mod)
+					*mod = 0;
+				break;
+			}
+
 			Returnkey = (int)event.key.keysym.sym;
-			if (!mod && event.key.keysym.mod & KMOD_SHIFT)
+			if (!mod && (event.key.keysym.mod & KMOD_SHIFT))
 				Returnkey = toupper((int)event.key.keysym.sym);
 
 			if (mod) {
 				/* If we have a modifier, we assign it */
 				*mod = event.key.keysym.mod;
 
-				/* And we also discard the keypress if the sym is e.g. CTRL because we explicitely
+				/* And we also discard the keypress if the sym is e.g. CTRL because we explicitly
 				 * asked to get a modifier, which makes sense only if we ignore CTRL as a key.
 				 */
 				if (event.key.keysym.sym >= SDLK_NUMLOCK && event.key.keysym.sym <= SDLK_COMPOSE) {
-					goto next;
+					continue;
 				}
 			}
 			/*
@@ -876,12 +886,14 @@ int getchar_raw(int *mod)
 			 * "The keyboard syms have been cleverly chosen to map to ASCII"
 			 * ... I hope that this design feature is portable, and durable ;)
 			 */
-			if(ISKEYPAD(Returnkey)) {
+			if (ISKEYPAD(Returnkey)) {
 				Returnkey = kptoprint(Returnkey);
 			}
-			return (Returnkey);
-		}
- next:		;
 
-	}			/* while(1) */
-}				/* getchar_raw() */
+			// A key was pressed, stop the loop
+			break;
+		}
+	}
+
+	return (Returnkey);
+}
