@@ -32,32 +32,24 @@
 
 #define _light_c
 
-#include "system.h"
-
-#include "defs.h"
 #include "struct.h"
 #include "global.h"
-#include "map.h"
 #include "proto.h"
 
-//--------------------
-// 
-#define MAX_NUMBER_OF_LIGHT_SOURCES 200
-
 /* Position and strength of a light source */
 struct light_source {
 	gps pos;
 	gps vpos;
 	int strength;
-} light_sources[MAX_NUMBER_OF_LIGHT_SOURCES];
+};
 
+static struct dynarray light_sources;
 
 light_radius_config LightRadiusConfig = { -1, -1, -1, -1, 0.0 };
 
 int *light_strength_buffer = NULL;
 #define LIGHT_RADIUS_TEXTURE_DIVISOR 10	// Default number of screen pixels per cell
 #define LIGHT_RADIUS_TEXTURE_MAX_SIZE 64	// texture max size is 64x64
-#define LIGHT_STRENGTH_CELL_ADDR(x,y) (light_strength_buffer + (y)*LightRadiusConfig.cells_w + (x))
 #define LIGHT_STRENGTH_CELL(x,y) *(int*)(light_strength_buffer + (y)*LightRadiusConfig.cells_w + (x))
 
 #define UNIVERSAL_TUX_HEIGHT 100.0	// Empirical Tux height, in the universal coordinate system
@@ -201,10 +193,6 @@ enum interpolation_methods {
 	ALONG_X   = 1,	 // Interpolation with the neighbor along X axis is needed
 	ALONG_Y   = 2,	 // Interpolation with the neighbor along Y axis is needed
 	ALONG_XY  = 3, 	 // ALONG_X | ALONG_Y
-	ALONG_D   = 4,   // Interpolation with the neighbor on the diagonal is needed
-	ALONG_XD  = 5,   // ALONG_X | ALONG_D
-	ALONG_YD  = 6,   // ALONG_Y | ALONG_D
-	ALONG_XYD = 7,   // ALONG_X | ALONG_Y | ALONG_D
 };
 
 /* Definition of the Interpolation Data Storage for an interpolation area */
@@ -509,6 +497,17 @@ void LightRadiusClean()
 	}
 }
 
+static void add_light_source(gps pos, gps vpos, int strength)
+{
+	struct light_source src;
+
+	src.pos      = pos;
+	src.vpos     = vpos;
+	src.strength = strength;
+
+	dynarray_add(&light_sources, &src, sizeof(src));
+}
+
 /**
  * There might be some obstacles that emit some light.  Yet, we can't
  * go through all the obstacle list of a level every frame at sufficiently
@@ -517,76 +516,49 @@ void LightRadiusClean()
  */
 void update_light_list()
 {
-	int i;
 	Level light_level = curShip.AllLevels[Me.pos.z];
 	struct visible_level *visible_lvl, *next_lvl;
 	level *curr_lvl;
 	int curr_id;
 	int glue_index;
+	int light_strength;
 	int obs_index;
-	int next_light_emitter_index;
 	int map_x, map_y;	
 	obstacle *emitter;
-	int blast;
+	int blast_idx;
 	gps me_vpos;
 
-	// At first we fill out the light sources array with 'empty' information,
-	// i.e. such positions, that won't affect our location for sure.
-	//
-	for (i = 0; i < MAX_NUMBER_OF_LIGHT_SOURCES; i++) {
-		light_sources[i].pos.x = -200;
-		light_sources[i].pos.y = -200;
-		light_sources[i].pos.z = -1;
-		light_sources[i].vpos.x = -1;
-		light_sources[i].vpos.y = -1;
-		light_sources[i].vpos.z = -1;
-		light_sources[i].strength = 0;
-	}
-	next_light_emitter_index = 0;
+	dynarray_init(&light_sources, 10, sizeof(struct light_source));
 
 	// Now we fill in the Tux position as the very first light source, that will
 	// always be present.
 	//
-	light_sources[0].pos.x = Me.pos.x;
-	light_sources[0].pos.y = Me.pos.y;
-	light_sources[0].pos.z = Me.pos.z;
-	light_sources[0].strength = light_level->light_bonus + Me.light_bonus_from_tux;	
-	light_sources[0].vpos = light_sources[0].pos;
+	light_strength = light_level->light_bonus + Me.light_bonus_from_tux;
+
 	// We must not in any case tear a hole into the beginning of the list though...
-	if (light_sources[0].strength <= 0)
-		light_sources[0].strength = 1;
-	next_light_emitter_index = 1;
+	if (light_strength <= 0)
+		light_strength = 1;
+
+	add_light_source(Me.pos, Me.pos, light_strength);
 
 	// Now we can fill in any explosions, that are currently going on.
 	// These will typically emanate a lot of light.
 	//
-	for (blast = 0; blast < MAXBLASTS; blast++) {
-		if (!(AllBlasts[blast].type == DROIDBLAST))
+	for (blast_idx = 0; blast_idx < MAXBLASTS; blast_idx++) {
+		if (!(AllBlasts[blast_idx].type == DROIDBLAST))
 			continue;
 
 		// We add some light strength according to the phase of the blast
 		//
-		int light_strength = 10 + AllBlasts[blast].phase / 2;
+		int light_strength = 10 + AllBlasts[blast_idx].phase / 2;
 		if (light_strength < 0) continue;
 
-		light_sources[next_light_emitter_index].pos.x = AllBlasts[blast].pos.x;
-		light_sources[next_light_emitter_index].pos.y = AllBlasts[blast].pos.y;
-		light_sources[next_light_emitter_index].pos.z = AllBlasts[blast].pos.z;
-		update_virtual_position(&light_sources[next_light_emitter_index].vpos,
-				&light_sources[next_light_emitter_index].pos, Me.pos.z);
-		if (light_sources[next_light_emitter_index].vpos.x == -1)
+		gps vpos;
+		update_virtual_position(&vpos, &AllBlasts[blast_idx].pos, Me.pos.z);
+		if (vpos.x == -1)
 			continue;
 
-		light_sources[next_light_emitter_index].strength = light_strength;
-		next_light_emitter_index++;
-
-		// We must not write beyond the bounds of our light sources array!
-		//
-		if (next_light_emitter_index >= MAX_NUMBER_OF_LIGHT_SOURCES - 1) {
-			ErrorMessage(__FUNCTION__, "\
-WARNING!  End of light sources array reached!", NO_NEED_TO_INFORM, IS_WARNING_ONLY);
-			return;
-		}
+		add_light_source(AllBlasts[blast_idx].pos, vpos, light_strength);
 	}
 
 	// Now we can fill in the remaining light sources of this level.
@@ -640,26 +612,12 @@ WARNING!  End of light sources array reached!", NO_NEED_TO_INFORM, IS_WARNING_ON
 					if (!emitted_light_strength)
 						continue;
 
-					// Now we know that this one needs to be inserted!
-					//
-					light_sources[next_light_emitter_index].pos.x = emitter->pos.x;
-					light_sources[next_light_emitter_index].pos.y = emitter->pos.y;
-					light_sources[next_light_emitter_index].pos.z = emitter->pos.z;
-					update_virtual_position(&light_sources[next_light_emitter_index].vpos,
-							&light_sources[next_light_emitter_index].pos, Me.pos.z);
-					if (light_sources[next_light_emitter_index].vpos.x == -1)
+					gps vpos;
+					update_virtual_position(&vpos, &emitter->pos, Me.pos.z);
+					if (vpos.x == -1)
 						continue;
 
-					light_sources[next_light_emitter_index].strength = emitted_light_strength;
-					next_light_emitter_index++;
-
-					// We must not write beyond the bounds of our light sources array!
-					//
-					if (next_light_emitter_index >= MAX_NUMBER_OF_LIGHT_SOURCES - 1) {
-						ErrorMessage(__FUNCTION__, "\
-WARNING!  End of light sources array reached!", NO_NEED_TO_INFORM, IS_WARNING_ONLY);
-						return;
-					}
+					add_light_source(emitter->pos, vpos, emitted_light_strength);
 				}
 			}
 		}
@@ -681,23 +639,12 @@ WARNING!  End of light sources array reached!", NO_NEED_TO_INFORM, IS_WARNING_ON
 			if (fabsf(me_vpos.y - erot->pos.y) >= FLOOR_TILES_VISIBLE_AROUND_TUX)
 				continue;
 
-			// Now we know that this one needs to be inserted!
-			//
-			light_sources[next_light_emitter_index].pos.x = erot->pos.x;
-			light_sources[next_light_emitter_index].pos.y = erot->pos.y;
-			light_sources[next_light_emitter_index].pos.z = erot->pos.z;
-			update_virtual_position(&light_sources[next_light_emitter_index].vpos,
-					&light_sources[next_light_emitter_index].pos, Me.pos.z);
-			light_sources[next_light_emitter_index].strength = 5;
-			next_light_emitter_index++;
+			gps vpos;
+			update_virtual_position(&vpos, &erot->pos, Me.pos.z);
+			if (vpos.x == -1)
+				continue;
 
-			// We must not write beyond the bounds of our light sources array!
-			//
-			if (next_light_emitter_index >= MAX_NUMBER_OF_LIGHT_SOURCES - 1) {
-				ErrorMessage(__FUNCTION__, "\
-WARNING!  End of light sources array reached!", NO_NEED_TO_INFORM, IS_WARNING_ONLY);
-				return;
-			}
+			add_light_source(erot->pos, vpos, 5);
 		}
 	}
 }				// void update_light_list ( )
@@ -714,6 +661,7 @@ static int calculate_light_strength(gps *cell_vpos)
 	float squared_dist;
 	gps cell_rpos;
 	struct interpolation_data_cell ilights;
+	struct light_source *lights = light_sources.arr;
 
 	// 1. Light interpolation
 	//
@@ -728,7 +676,7 @@ static int calculate_light_strength(gps *cell_vpos)
 	int final_light_strength = max(0, ilights.minimum_light_value);
 
 	// Interpolated light emitted from Tux
-	light_sources[0].strength = ilights.light_bonus + Me.light_bonus_from_tux;
+	lights[0].strength = ilights.light_bonus + Me.light_bonus_from_tux;
 
 	// 2. Compute the light strength value at "cell_vpos"
 	// Nota: the following code being quite obscure, some explanations are quite
@@ -783,14 +731,8 @@ static int calculate_light_strength(gps *cell_vpos)
 
 	// Now, here is the final algorithm
 	//
-	for (i = 0; i < MAX_NUMBER_OF_LIGHT_SOURCES; i++) {
-		// If we've reached the end of the current list of light
-		// sources, then we can stop immediately.
-		//
-		if (light_sources[i].strength == 0)
-			break;
-
-		float absolute_intensity = ilights.minimum_light_value + light_sources[i].strength;
+	for (i = 0; i < light_sources.size; i++) {
+		float absolute_intensity = ilights.minimum_light_value + lights[i].strength;
 		
 		// Optimization 1:
 		// If absolute_intensity is lower than current intensity, we do not take
@@ -801,8 +743,8 @@ static int calculate_light_strength(gps *cell_vpos)
 
 		// Some pre-computations
 		// First transform light source's position into virtual position, related to Tux's current level
-		xdist = light_sources[i].vpos.x - cell_vpos->x;
-		ydist = light_sources[i].vpos.y - cell_vpos->y;
+		xdist = lights[i].vpos.x - cell_vpos->x;
+		ydist = lights[i].vpos.y - cell_vpos->y;
 		squared_dist = xdist * xdist + ydist * ydist;
 
 		// Comparison between current light strength and the light source's strength (line 2 of the pseudo-code)
@@ -813,7 +755,7 @@ static int calculate_light_strength(gps *cell_vpos)
 		// Visibility check (line 3 of pseudo_code)
 		// with a small optimization : no visibility check if the target is very closed to the light
 		if (squared_dist > (0.5*0.5)) {
-			if (!DirectLineColldet(light_sources[i].vpos.x, light_sources[i].vpos.y, cell_vpos->x, cell_vpos->y, cell_vpos->z, &VisiblePassFilter))
+			if (!DirectLineColldet(lights[i].vpos.x, lights[i].vpos.y, cell_vpos->x, cell_vpos->y, cell_vpos->z, &VisiblePassFilter))
 				continue;
 		}
 
@@ -1028,7 +970,7 @@ void blit_classic_SDL_light_radius(int decay_x, int decay_y)
 
 		for (i = 0; i < NUMBER_OF_SHADOW_IMAGES; i++) {
 			sprintf(constructed_file_name, "light_radius_chunks/iso_light_radius_darkness_%04d.png", i);
-			load_image(&light_radius_chunk[i], constructed_file_name, FALSE);
+			load_image(&light_radius_chunk[i], constructed_file_name, NO_MOD);
 		}
 
 		lrc_nb_columns = (int)ceilf((float)GameConfig.screen_width / (float)(LRC_ISO_WIDTH + LRC_ISO_GAP_X)) + 1;
@@ -1118,6 +1060,8 @@ void blit_light_radius(void)
 	} else {
 		blit_classic_SDL_light_radius(decay_x, decay_y);
 	}
+
+	dynarray_free(&light_sources);
 }	// void blit_light_radius ( void )
 
 #undef _light_c
