@@ -64,6 +64,8 @@ enum ActionType {
 	ACT_CHANGE_FLOOR_LAYER,
 	ACT_CREATE_MAP_LABEL,
 	ACT_REMOVE_MAP_LABEL,
+	ACT_CREATE_ENEMY,
+	ACT_REMOVE_ENEMY
 };
 
 typedef struct {
@@ -155,6 +157,9 @@ typedef struct {
 			int x;
 			int y;
 		} delete_map_label;
+
+		enemy *create_enemy;
+		enemy *delete_enemy;
 	} d;
 } action;
 
@@ -167,6 +172,7 @@ enum {
 EXTERN void LevelEditor(void);
 EXTERN item *ItemDropFromLevelEditor(void);
 EXTERN void TestMap(void);
+EXTERN void leveleditor_cleanup();
 
 enum lvledit_object_type {
 	OBJECT_OBSTACLE,
@@ -193,6 +199,6 @@ EXTERN int current_floor_layer;
 
 EXTERN void get_random_droids_from_user(void);
 
-EXTERN void place_special_force(gps, int);
+EXTERN enemy *create_special_force(gps, int);
 
 #endif
