@@ -38,6 +38,8 @@
 
 #include "lvledit/lvledit.h"
 #include "lvledit/lvledit_widgets.h"
+#include "lvledit/lvledit_tool_select.h"
+#include "lvledit/lvledit_tool_place.h"
 
 /* Undo/redo action lists */
 LIST_HEAD(to_undo);
@@ -56,6 +58,10 @@ static void clear_action(action * action)
 		free(action->d.change_obstacle_name.new_name);
 	else if (action->type == ACT_SET_MAP_LABEL && action->d.change_label_name.new_name != NULL)
 		free(action->d.change_label_name.new_name);
+	else if (action->type == ACT_CREATE_MAP_LABEL && action->d.create_map_label.label_name != NULL)
+		free(action->d.create_map_label.label_name);
+	else if (action->type == ACT_CREATE_ENEMY && action->d.create_enemy != NULL)
+		enemy_free(action->d.create_enemy);
 
 	list_del(&action->node);	//< removes an action from a list
 	free(action);		//< frees the action
@@ -174,8 +180,14 @@ static action *action_create(int type, va_list args)
 			act->d.delete_map_label.x = va_arg(args, int);
 			act->d.delete_map_label.y = va_arg(args, int);
 			break;
+		case ACT_CREATE_ENEMY:
+			act->d.create_enemy = va_arg(args, enemy *);
+			break;
+		case ACT_REMOVE_ENEMY:
+			act->d.delete_enemy = va_arg(args, enemy *);
+			break;
 		default:
-			ErrorMessage(__FUNCTION__, "Unknown action type %d\n", PLEASE_INFORM, IS_FATAL, type);
+			error_message(__FUNCTION__, "Unknown action type %d", PLEASE_INFORM | IS_FATAL, type);
 	}
 
 	return act;
@@ -193,8 +205,11 @@ void action_push(int type, ...)
 			list_add(&act->node, &to_redo);
 			break;
 		case REDO:
+			list_add(&act->node, &to_undo);
+			break;
 		case NORMAL:
 			list_add(&act->node, &to_undo);
+			clear_action_list(&to_redo);
 			break;
 	}
 
@@ -226,6 +241,7 @@ static int action_change_obstacle_label(level *EditLevel, obstacle *obstacle, ch
 	// If the obstacle already has a label, remove it
 	char *old_label = get_obstacle_extension(EditLevel, obstacle, OBSTACLE_EXTENSION_LABEL);
 	if (old_label) {
+		old_label = strdup(old_label);
 		del_obstacle_extension(EditLevel, obstacle, OBSTACLE_EXTENSION_LABEL);
 	}
 
@@ -233,17 +249,18 @@ static int action_change_obstacle_label(level *EditLevel, obstacle *obstacle, ch
 	if (undoable) {
 		action_push(ACT_SET_OBSTACLE_LABEL, obstacle, old_label);
 	} else {
-		free(old_label);
+		if (old_label) {
+			free(old_label);
+			old_label = NULL;
+		}
 	}
 
 	// If the new label is empty, we are done
 	if (!name || !strlen(name))
 		return 0;
 
-	name = strdup(name);
-
 	// Assign the new label
-	add_obstacle_extension(EditLevel, obstacle, OBSTACLE_EXTENSION_LABEL, name);
+	add_obstacle_extension(EditLevel, obstacle, OBSTACLE_EXTENSION_LABEL, strdup(name));
 
 	return undoable;
 }
@@ -274,6 +291,9 @@ void action_remove_obstacle_user(Level EditLevel, obstacle * our_obstacle)
 	posx = our_obstacle->pos.x;
 	posy = our_obstacle->pos.y;
 
+	// The obstacle can be in the selection list
+	remove_element_from_selection(our_obstacle);
+
 	// make an undoable removal
 	del_obstacle(our_obstacle);
 	action_push(ACT_CREATE_OBSTACLE, posx, posy, type);
@@ -295,7 +315,7 @@ item *action_create_item(level *EditLevel, float x, float y, int type)
 		// Display a popup window with a number selector in order to choose the
 		// multiplicity of the item
 		multiplicity =
-		    do_graphical_number_selection_in_range(1, (!MatchItemWithName(type, "Valuable Circuits")) ? 100 : 1000, 1, 0);
+		    do_graphical_number_selection_in_range(1, (!item_spec_eq_id(type, "Valuable Circuits")) ? 100 : 1000, 1, 0);
 
 		if (multiplicity == 0) {
 			// Do not drop an item on the floor when the user cancelled the action.
@@ -326,6 +346,9 @@ void action_remove_item(level *EditLevel, item *it)
 	x = it->pos.x;
 	y = it->pos.y;
 
+	// The item can be in the selection list
+	remove_element_from_selection(it);
+
 	// Remove the item on the map
 	DeleteItem(it);
 
@@ -402,9 +425,15 @@ void action_remove_waypoint(level *EditLevel, int x, int y)
 	// Save waypoint information for the undo action
 	int old_random_spawn = wpts[wpnum].suppress_random_spawn;
 
+	// The waypoint can be in the selection list
+	remove_element_from_selection(&wpts[wpnum]);
+
 	// Remove the waypoint on the map
 	del_waypoint(EditLevel, x, y);
 
+	// If a route was being traced from this waypoint, remove it
+	leveleditor_place_reset_waypoint_route(wpnum);
+
 	// Make an undoable action
 	action_push(ACT_CREATE_WAYPOINT, x, y, old_random_spawn);
 }
@@ -551,10 +580,10 @@ static void action_change_map_label(level *EditLevel, int i, char *name, int x,
 		map_label = &ACCESS_MAP_LABEL(EditLevel->map_labels, i);
 
 		// Get the old label for undoable actions
-		old_label = map_label->label_name;
+		old_label = strdup(map_label->label_name);
 
 		// Delete the map label
-		del_map_label(EditLevel, old_label);
+		del_map_label(EditLevel, map_label->label_name);
 	}
 
 	action_push(ACT_SET_MAP_LABEL, i, old_label, x, y);
@@ -563,10 +592,8 @@ static void action_change_map_label(level *EditLevel, int i, char *name, int x,
 	if (!name || !strlen(name))
 		return;
 
-	name = strdup(name);
-
 	// Create a new map label at the position of cursor
-	add_map_label(EditLevel, x, y, name);
+	add_map_label(EditLevel, x, y, strdup(name));
 }
 
 void level_editor_action_change_map_label_user(level *EditLevel, float x, float y)
@@ -577,23 +604,25 @@ void level_editor_action_change_map_label_user(level *EditLevel, float x, float
 	char suggested_label[200];
 	int i;
 
-	suggested_label[0] = 0;
+	suggested_label[0] = '\0';
 
 	// We check if a map label already exists for this spot
 	for (i = 0; i < EditLevel->map_labels.size; i++) {
 		map_label = &ACCESS_MAP_LABEL(EditLevel->map_labels, i);
 
-		if ((fabsf(map_label->pos.x + 0.5 - x) < 0.5) &&
-			 (fabsf(map_label->pos.y + 0.5 - y) < 0.5)) {
-			
+		if ((fabs(map_label->pos.x + 0.5 - x) < 0.5) &&
+			 (fabs(map_label->pos.y + 0.5 - y) < 0.5)) {
+
 			// Use the old label as a suggestion
 			old_name = map_label->label_name;
-			strcpy(suggested_label, old_name);
+			strncpy(suggested_label, old_name, sizeof(suggested_label) - 1);
+			suggested_label[sizeof(suggested_label) - 1] = '\0';
 			break;
 		}
 	}
-		
+
 	// Check if the name entered already exists
+
 	while (1) {
 		// Show popup window to enter a new map label
 		name = GetEditableStringInPopupWindow(sizeof(suggested_label) - 1, _("\nPlease enter map label: \n\n"), suggested_label);
@@ -615,7 +644,8 @@ void level_editor_action_change_map_label_user(level *EditLevel, float x, float
 		alert_window("%s", _("The new name of the map label already exists, please choose an other name."));
 
 		// Copy the name in order to have it in the input box
-		strcpy(suggested_label, name);
+		strncpy(suggested_label, name, sizeof(suggested_label) - 1);
+		suggested_label[sizeof(suggested_label) - 1] = '\0';
 		free(name);
 
 		// Restore the menu background in order to correctly draw the next popup window
@@ -624,6 +654,7 @@ void level_editor_action_change_map_label_user(level *EditLevel, float x, float
 
 	// Change a map label when the name enter by the user is valid
 	action_change_map_label(EditLevel, i, name, rintf(x - 0.5), rintf(y - 0.5));
+	free(name);
 }
 
 /**
@@ -636,7 +667,7 @@ void level_editor_action_change_map_label_user(level *EditLevel, float x, float
  */
 void action_create_map_label(level *lvl, int x, int y, char *label_name)
 {
-	add_map_label(lvl, x, y, label_name);
+	add_map_label(lvl, x, y, strdup(label_name));
 
 	action_push(ACT_REMOVE_MAP_LABEL, x, y);
 }
@@ -657,12 +688,45 @@ void action_remove_map_label(level *lvl, int x, int y)
 	if (m) {
 		old_label_name = strdup(m->label_name);
 
+		remove_element_from_selection(m);
+
 		del_map_label(lvl, m->label_name);
 
 		action_push(ACT_CREATE_MAP_LABEL, x, y, old_label_name);
 	}
 }
 
+enemy *action_create_enemy(level *lvl, enemy *en)
+{
+	enemy_insert_into_lists(en, TRUE);
+	action_push(ACT_REMOVE_ENEMY, en);
+	return en;
+}
+
+void action_remove_enemy(level *lvl, enemy *en)
+{
+	enemy *current_enemy, *nerot;
+
+	remove_element_from_selection(en);
+
+	BROWSE_ALIVE_BOTS_SAFE(current_enemy, nerot) {
+		if (current_enemy == en)
+			list_del(&current_enemy->global_list);
+	}
+
+	BROWSE_DEAD_BOTS_SAFE(current_enemy, nerot) {
+		if (current_enemy == en)
+			list_del(&current_enemy->global_list);
+	}
+
+	BROWSE_LEVEL_BOTS_SAFE(current_enemy, nerot, lvl->levelnum) {
+		if (current_enemy == en)
+			list_del(&current_enemy->level_list);
+	}
+
+	action_push(ACT_CREATE_ENEMY, en);
+}
+
 /**
  *  @fn void jump_to_level( int target_map, float x, float y)
  *
@@ -732,7 +796,7 @@ static void action_do(level * level, action * a)
 		action_set_floor_layer(level, a->d.change_floor.x, a->d.change_floor.y, a->d.change_floor.layer, a->d.change_floor.type);
 		break;
 	case ACT_MULTIPLE_ACTIONS:
-		ErrorMessage(__FUNCTION__, "Passed a multiple actions meta-action as parameter. A real action is needed.\n", PLEASE_INFORM, IS_WARNING_ONLY);
+		error_message(__FUNCTION__, "Passed a multiple actions meta-action as parameter. A real action is needed.", PLEASE_INFORM);
 		break;
 	case ACT_SET_OBSTACLE_LABEL:
 		action_change_obstacle_label(level, a->d.change_obstacle_name.obstacle, a->d.change_obstacle_name.new_name, 1);
@@ -752,6 +816,13 @@ static void action_do(level * level, action * a)
 	case ACT_REMOVE_MAP_LABEL:
 		action_remove_map_label(level, a->d.delete_map_label.x, a->d.delete_map_label.y);
 		break;
+	case ACT_CREATE_ENEMY:
+		action_create_enemy(level, a->d.create_enemy);
+		a->d.create_enemy = NULL;
+		break;
+	case ACT_REMOVE_ENEMY:
+		action_remove_enemy(level, a->d.delete_enemy);
+		break;
 	}
 }
 
@@ -847,7 +918,7 @@ void level_editor_place_aligned_object(int positionid)
 void CreateNewMapLevel(int level_num)
 {
 	level *NewLevel;
-	int i, k, l;
+	int i, k;
 
 	// Get the memory for one level 
 	//
@@ -878,10 +949,7 @@ void CreateNewMapLevel(int level_num)
 	for (i = 0; i < NewLevel->ylen; i++) {
 		NewLevel->map[i] = MyMalloc(NewLevel->xlen * sizeof(map_tile));
 		for (k = 0; k < NewLevel->xlen; k++) {
-			NewLevel->map[i][k].floor_values[0] = ISO_FLOOR_SAND;
-			for (l = 1; l < MAX_FLOOR_LAYERS; l++)
-				NewLevel->map[i][k].floor_values[l] = ISO_FLOOR_EMPTY;
-			dynarray_init(&NewLevel->map[i][k].glued_obstacles, 0, sizeof(int));
+			init_map_tile(&NewLevel->map[i][k]);
 		}
 	}
 	// Now we initialize the level jump interface variables with 'empty' values
@@ -924,58 +992,39 @@ void CreateNewMapLevel(int level_num)
 
 void delete_map_level(int lnum)
 {
-	int i;
-	level *l = curShip.AllLevels[lnum];
-
-	free(l->Levelname);
-	free(l->Background_Song_Name);
-
-	// Clear out all the existing glue information.
-	free_glued_obstacles(l);
-
-	// Delete floor tiles
-	for (i = 0; i < l->ylen; i++)
-		free(l->map[i]);
-
 	// Remove references to this level from others
-	for (i = 0; i < MAX_LEVELS; i++) {
-		if (!level_exists(i))
-			continue;
+	level *l = NULL;
 
-		if (curShip.AllLevels[i]->jump_target_north == lnum)
-			curShip.AllLevels[i]->jump_target_north = -1;
-		if (curShip.AllLevels[i]->jump_target_south == lnum)
-			curShip.AllLevels[i]->jump_target_south = -1;
-		if (curShip.AllLevels[i]->jump_target_west == lnum)
-			curShip.AllLevels[i]->jump_target_west = -1;
-		if (curShip.AllLevels[i]->jump_target_east == lnum)
-			curShip.AllLevels[i]->jump_target_east = -1;
+	BROWSE_LEVELS(l) {
+		if (l->jump_target_north == lnum)
+			l->jump_target_north = -1;
+		if (l->jump_target_south == lnum)
+			l->jump_target_south = -1;
+		if (l->jump_target_west == lnum)
+			l->jump_target_west = -1;
+		if (l->jump_target_east == lnum)
+			l->jump_target_east = -1;
 	}
 
-	// Free obstacle extensions.
-	dynarray_free(&l->obstacle_extensions);
-
-	// Free map labels.
-	dynarray_free(&l->map_labels);
-
-	// Free waypoints.
-	dynarray_free(&l->waypoints);
-
 	// Free memory
-	free(curShip.AllLevels[lnum]);
-
+	free_ship_level(curShip.AllLevels[lnum]);
 	curShip.AllLevels[lnum] = NULL;
 
 	if (lnum == curShip.num_levels - 1)
 		curShip.num_levels--;
 
+	// Removing a level cannot be undo, so we need to clear everything that could have
+	// a reference to an object of the deleted level.
+	action_freestack();
+	clear_selection(-1);
+	clear_clipboard(-1);
 }
 
 static int get_chest_contents(level *l, obstacle *o, item *items[MAX_ITEMS_IN_INVENTORY])
 {
 	struct dynarray *itemlist = get_obstacle_extension(l, o, OBSTACLE_EXTENSION_CHEST_ITEMS);
 
-	memset(items, 0, MAX_ITEMS_IN_INVENTORY);
+	memset(items, 0, MAX_ITEMS_IN_INVENTORY*sizeof(item *));
 
 	if (!itemlist) {
 		return 0;
@@ -983,10 +1032,8 @@ static int get_chest_contents(level *l, obstacle *o, item *items[MAX_ITEMS_IN_IN
 
 	int i;
 	int curitem = 0;
-	for (i = 0; i < itemlist->size; i++) {
+	for (i = 0; i < itemlist->size && i < MAX_ITEMS_IN_INVENTORY; i++) {
 		items[curitem++] = &((item *)itemlist->arr)[i];
-		if (curitem == MAX_ITEMS_IN_INVENTORY - 1)
-			break;
 	}
 
 	return curitem;
@@ -1012,19 +1059,10 @@ void level_editor_edit_chest(obstacle *o)
 	struct dynarray *itemlist = get_obstacle_extension(CURLEVEL(), o, OBSTACLE_EXTENSION_CHEST_ITEMS);
 
 	// Safety check
-	switch (o->type) {
-	case ISO_H_CHEST_CLOSED:
-	case ISO_H_CHEST_OPEN:
-	case ISO_V_CHEST_CLOSED:
-	case ISO_V_CHEST_OPEN:
-	case ISO_E_CHEST2_CLOSED:
-	case ISO_S_CHEST2_CLOSED:
-	case ISO_N_CHEST2_CLOSED:
-	case ISO_W_CHEST2_CLOSED:
-		break;
-	default:
-		ErrorMessage(__FUNCTION__, "Tried to edit the contents of a chest, but the obstacle is not a chest.\n", PLEASE_INFORM,
-			     IS_FATAL);
+	struct obstacle_spec *obs_spec = get_obstacle_spec(o->type);
+	if (!obs_spec->action || strncmp(obs_spec->action, "chest", 5)) {
+		error_message(__FUNCTION__, "Tried to edit the contents of a chest, but the obstacle is not a chest.",
+				     PLEASE_INFORM | IS_FATAL);
 	}
 
 	while (!done) {
