@@ -64,8 +64,9 @@ int lvledit_set_zoomfact(float zf)
  */
 static void gps_show() {
 	static char gps_text[200];
+	// TRANSLATORS: Used to display a GPS position (X=10.5 Y=34.3 L=12 layer=1)
 	snprintf(gps_text, sizeof(gps_text) - 1, _(" X=%3.1f Y=%3.1f L=%d layer=%d\n"), Me.pos.x, Me.pos.y, Me.pos.z, current_floor_layer);
-	display_text_using_line_height(gps_text, User_Rect.x + 1, GameConfig.screen_height - 1 * FontHeight(GetCurrentFont()), NULL /*&User_Rect */ , 1.0);
+	display_text(gps_text, User_Rect.x + 1, GameConfig.screen_height - 1 * get_font_height(get_current_font()), NULL /*&User_Rect */ , 1.0);
 }
 
 /**
@@ -75,7 +76,7 @@ void lvledit_display_fps(void) {
 	static char fps_text[200];
 	sprintf(fps_text, _(" FPS=%d\n"), get_current_fps());
 	display_text(fps_text, User_Rect.x + 1,	
-		GameConfig.screen_height - 2 * FontHeight(GetCurrentFont()), NULL);
+		GameConfig.screen_height - 2 * get_font_height(get_current_font()), NULL, 1.0);
 }
 
 /**
@@ -93,7 +94,7 @@ static void print_label_information(level *EditLevel)
 		sprintf(PanelText, _("\n Map label information: \n label_name=\"%s\"."), m->label_name);
 
 		// Display the map label information on the screen
-		display_text_using_line_height(PanelText, User_Rect.x, GameConfig.screen_height - 5 * FontHeight(GetCurrentFont()), NULL, 1.0);
+		display_text(PanelText, User_Rect.x, GameConfig.screen_height - 5 * get_font_height(get_current_font()), NULL, 1.0);
 
 		return;
 	}
@@ -109,7 +110,7 @@ static void show_cursor(int must_zoom)
 	// we need to load it.
 	//
 	if (!image_loaded(&level_editor_cursor)) {
-		load_image(&level_editor_cursor, "level_editor_floor_cursor.png", TRUE);
+		load_image(&level_editor_cursor, "level_editor_floor_cursor.png", USE_OFFSET);
 	}
 
 	float scale = must_zoom ? lvledit_zoomfact_inv() : 1.0;
@@ -136,7 +137,7 @@ void draw_connection_between_tiles(float x1, float y1, float x2, float y2, int m
 	// we need to load it.
 	//
 	if (!image_loaded(&level_editor_dot_cursor)) {
-		load_image(&level_editor_dot_cursor, "level_editor_waypoint_dot.png", TRUE);
+		load_image(&level_editor_dot_cursor, "level_editor_waypoint_dot.png", USE_OFFSET);
 	}
 
 	// So now that the dot cursor has been loaded, we can start to
@@ -188,11 +189,11 @@ static void show_waypoints(int mask)
 	// Maybe, if the level editor floor cursor has not yet been loaded,
 	// we need to load it.
 	if (!image_loaded(&level_editor_waypoint_cursor[0])) {
-		load_image(&level_editor_waypoint_cursor[0], "level_editor_waypoint_cursor.png", TRUE);
+		load_image(&level_editor_waypoint_cursor[0], "level_editor_waypoint_cursor.png", USE_OFFSET);
 	}
 
 	if (!image_loaded(&level_editor_waypoint_cursor[1])) {
-		load_image(&level_editor_waypoint_cursor[1], "level_editor_norand_waypoint_cursor.png", TRUE);
+		load_image(&level_editor_waypoint_cursor[1], "level_editor_norand_waypoint_cursor.png", USE_OFFSET);
 	}
 
 	for (i = 0; i < EditLevel()->waypoints.size; i++) {
@@ -216,7 +217,12 @@ static void show_waypoints(int mask)
 		int *connections = wpts[i].connections.arr;
 
 		for (j = 0; j < wpts[i].connections.size; j++) {
-			waypoint *to_wp = &wpts[connections[j]];
+			// Check validity of the connected waypoint
+			int connected_waypoint = connections[j];
+			if (connected_waypoint < 0 || connected_waypoint >= EditLevel()->waypoints.size)
+				continue;
+
+			waypoint *to_wp = &wpts[connected_waypoint];
 
 			if (((EditX() == wpts[i].x) && (EditY() == wpts[i].y)) || GameConfig.show_wp_connections) {
 				draw_connection_between_tiles(x, y, to_wp->x + 0.5, to_wp->y + 0.5, mask, wpts[i].suppress_random_spawn);
@@ -265,6 +271,12 @@ static void show_map_labels(int must_zoom)
 
 		struct image *img = get_map_label_image();
 		display_image_on_map(img, map_label->pos.x + 0.5, map_label->pos.y + 0.5, IMAGE_SCALE_RGB_TRANSFO(scale, r, g, b));
+
+                if (!GameConfig.omit_map_labels_in_level_editor) {
+                    show_backgrounded_label_at_map_position(map_label->label_name,
+                                                            0, map_label->pos.x,
+                                                            map_label->pos.y, must_zoom);
+                }
 	}
 }
 
@@ -303,7 +315,7 @@ void leveleditor_display()
 	if (GameConfig.Draw_Framerate)
 		lvledit_display_fps();
 
-	SetCurrentFont(FPS_Display_BFont);
+	set_current_font(FPS_Display_Font);
 
 	// Now we print out the current status directly onto the window:
 	//
@@ -311,12 +323,12 @@ void leveleditor_display()
 		waypoint *wpts = EditLevel()->waypoints.arr;
 
 		sprintf(linebuf, _(" Source waypoint selected : X=%d Y=%d. "), wpts[OriginWaypoint].x, wpts[OriginWaypoint].y);
-		LeftPutString(Screen, GameConfig.screen_height - 2 * FontHeight(GetCurrentFont()), linebuf);
+		put_string_left(FPS_Display_Font, GameConfig.screen_height - 2 * get_font_height(get_current_font()), linebuf);
 	}
 	// Now we print out the latest connection operation success or failure...
 	//
 	if (VanishingMessageEndDate > SDL_GetTicks()) {
-		display_text_using_line_height(VanishingMessage, 1, GameConfig.screen_height - 8 * FontHeight(GetCurrentFont()), NULL, 1.0);
+		display_text(VanishingMessage, 1, GameConfig.screen_height - 8 * get_font_height(get_current_font()), NULL, 1.0);
 	}
 
 	// Construct the linked list of visible levels.
@@ -325,7 +337,7 @@ void leveleditor_display()
 	display_widgets();
 
 	if (EditLevel()->random_dungeon) {
-		sprintf(VanishingMessage, " This level is automatically generated. \n Editing will have no effect.");
+		sprintf(VanishingMessage, _(" This level is automatically generated. \n Editing will have no effect."));
 		VanishingMessageEndDate = SDL_GetTicks() + 100;
 	}
 
