@@ -101,31 +101,19 @@ void EditLevelDimensions(void)
 		InitiateMenu("--EDITOR_BACKGROUND--");
 
 		i = 0;
-		sprintf(Options[i], _("North"));
-		strcat(Options[i], " ");
-		strcat(Options[i], _("Edge"));
-		strcat(Options[i], ": -/+  (<-/->)");
+		sprintf(Options[i], _("North Edge: -/+  (<-/->)"));
 		MenuTexts[i] = Options[i];
 		i++;
 
-		sprintf(Options[i], _("East"));
-		strcat(Options[i], " ");
-		strcat(Options[i], _("Edge"));
-		strcat(Options[i], ": -/+  (<-/->)");
+		sprintf(Options[i], _("East Edge: -/+  (<-/->)"));
 		MenuTexts[i] = Options[i];
 		i++;
 
-		sprintf(Options[i], _("South"));
-		strcat(Options[i], " ");
-		strcat(Options[i], _("Edge"));
-		strcat(Options[i], ": -/+  (<-/->)");
+		sprintf(Options[i], _("South Edge: -/+  (<-/->)"));
 		MenuTexts[i] = Options[i];
 		i++;
 
-		sprintf(Options[i], _("West"));
-		strcat(Options[i], " ");
-		strcat(Options[i], _("Edge"));
-		strcat(Options[i], ": -/+  (<-/->)");
+		sprintf(Options[i], _("West Edge: -/+  (<-/->)"));
 		MenuTexts[i] = Options[i];
 		i++;
 
@@ -138,7 +126,7 @@ void EditLevelDimensions(void)
 		MenuTexts[i++] = _("Back");
 		MenuTexts[i++] = "";
 
-		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_BFont);
+		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_Font);
 
 		while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 			SDL_Delay(1);
@@ -238,38 +226,22 @@ static void SetLevelInterfaces(void)
 
 		i = 0;
 
-		sprintf(Options[i], _("Jump target"));
-		strcat(Options[i], " ");
-		strcat(Options[i], _("North"));
-		sprintf(Options[i + 1], ": %d.  (<-/->)", EditLevel->jump_target_north);
-		strcat(Options[i], Options[i + 1]);
+		sprintf(Options[i], _("Neighbor level North: %d.  (<-/->)"), EditLevel->jump_target_north);
 		MenuTexts[i] = Options[i];
 		i++;
-		sprintf(Options[i], _("Jump target"));
-		strcat(Options[i], " ");
-		strcat(Options[i], _("East"));
-		sprintf(Options[i + 1], ": %d.  (<-/->)", EditLevel->jump_target_east);
-		strcat(Options[i], Options[i + 1]);
+		sprintf(Options[i], _("Neighbor level East: %d.  (<-/->)"), EditLevel->jump_target_east);
 		MenuTexts[i] = Options[i];
 		i++;
-		sprintf(Options[i], _("Jump target"));
-		strcat(Options[i], " ");
-		strcat(Options[i], _("South"));
-		sprintf(Options[i + 1], ": %d.  (<-/->)", EditLevel->jump_target_south);
-		strcat(Options[i], Options[i + 1]);
+		sprintf(Options[i], _("Neighbor level South: %d.  (<-/->)"), EditLevel->jump_target_south);
 		MenuTexts[i] = Options[i];
 		i++;
-		sprintf(Options[i], _("Jump target"));
-		strcat(Options[i], " ");
-		strcat(Options[i], _("West"));
-		sprintf(Options[i + 1], ": %d.  (<-/->)", EditLevel->jump_target_west);
-		strcat(Options[i], Options[i + 1]);
+		sprintf(Options[i], _("Neighbor level West: %d.  (<-/->)"), EditLevel->jump_target_west);
 		MenuTexts[i] = Options[i];
 		i++;
 		MenuTexts[i++] = _("Back");
 		MenuTexts[i++] = "";
 
-		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_BFont);
+		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_Font);
 
 		while (EnterPressed() || MouseLeftPressed())
 			SDL_Delay(1);
@@ -296,7 +268,7 @@ static void SetLevelInterfaces(void)
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 
-			tgt = get_number_popup("\n Please enter new level number: \n\n", "");
+			tgt = get_number_popup(_("\n Please enter new level number: \n\n"), "");
 
 			if (tgt >= -1 && tgt < curShip.num_levels) {
 				EditLevel->jump_target_north = (tgt);
@@ -311,7 +283,7 @@ static void SetLevelInterfaces(void)
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 
-			tgt = get_number_popup("\n Please enter new level number: \n\n", "");
+			tgt = get_number_popup(_("\n Please enter new level number: \n\n"), "");
 
 			if (tgt >= -1 && tgt < curShip.num_levels) {
 				EditLevel->jump_target_east = (tgt);
@@ -326,7 +298,7 @@ static void SetLevelInterfaces(void)
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 
-			tgt = get_number_popup("\n Please enter new level number: \n\n", "");
+			tgt = get_number_popup(_("\n Please enter new level number: \n\n"), "");
 
 			if (tgt >= -1 && tgt < curShip.num_levels) {
 				EditLevel->jump_target_south = (tgt);
@@ -341,7 +313,7 @@ static void SetLevelInterfaces(void)
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 
-			tgt = get_number_popup("\n Please enter new level number: \n\n", "");
+			tgt = get_number_popup(_("\n Please enter new level number: \n\n"), "");
 
 			if (tgt >= -1 && tgt < curShip.num_levels) {
 				EditLevel->jump_target_west = (tgt);
@@ -446,7 +418,7 @@ static void AddRemLevel(void)
 		while (EscapePressed())
 			SDL_Delay(1);
 
-		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_BFont);
+		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_Font);
 
 		while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 			SDL_Delay(1);
@@ -482,7 +454,7 @@ static void AddRemLevel(void)
 				action_jump_to_level_center(new_level_num);
 			}
 			else      //curShip.num_levels >= MAX_LEVELS
-				alert_window("%s", _("Reached the maximum number of levels."));
+				alert_window(_("Reached the maximum number of levels."));
 			break;
 		case REMOVE_CURRENT_LEVEL:
 			if (game_root_mode == ROOT_IS_GAME)
@@ -491,7 +463,7 @@ static void AddRemLevel(void)
 				SDL_Delay(1);
 
 			if (EditLevel()->levelnum == 0) {
-				alert_window("%s", _("Cannot remove level number 0.\n"));
+				alert_window(_("Cannot remove level number 0.\n"));
 				break;
 			}
 
@@ -529,6 +501,7 @@ static void LevelOptions(void)
 {
 	char *MenuTexts[100];
 	char Options[20][1000];
+	char class[16];
 	int proceed_now = FALSE;
 	int MenuPosition = 1;
 	int tgt = -1;
@@ -536,6 +509,8 @@ static void LevelOptions(void)
 	int l = 0;
 	int *droid_types;
 
+	class[15] = 0; // null-character to null terminated strncpy
+
 	enum {
 		CHANGE_LEVEL_POSITION = 1,
 		SET_LEVEL_NAME,
@@ -544,6 +519,9 @@ static void LevelOptions(void)
 		SET_LEVEL_INTERFACE_POSITION,
 		SET_RANDOM_LEVEL,
 		SET_RANDOM_DROIDS,
+		SET_NO_RESPAWN,
+		SET_DROP_CLASS,
+		SET_TELEPORT_BLOCKED,
 		SET_TELEPORT_PAIR,
 		CHANGE_LIGHT,
 		SET_BACKGROUND_SONG_NAME,
@@ -557,72 +535,81 @@ static void LevelOptions(void)
 		InitiateMenu("--EDITOR_BACKGROUND--");
 
 		i = 0;
-		sprintf(Options[i], _("Level"));
-		sprintf(Options[i + 1], ": %d.  (<-/->)", EditLevel()->levelnum);
-		strcat(Options[i], Options[i + 1]);
+		sprintf(Options[i], _("Level: %d.  (<-/->)"), EditLevel()->levelnum);
 		MenuTexts[i] = Options[i];
 		i++;
-		sprintf(Options[i], _("Name"));
-		sprintf(Options[i + 1], ": %s", EditLevel()->Levelname);
-		strcat(Options[i], Options[i + 1]);
+		sprintf(Options[i], _("Level name (untranslated): %s"), EditLevel()->Levelname);
 		MenuTexts[i] = Options[i];
 		i++;
-		sprintf(Options[i], _("Size"));
-		sprintf(Options[i + 1], ":  X %d  Y %d ", EditLevel()->xlen, EditLevel()->ylen);
-		strcat(Options[i], Options[i + 1]);
+		sprintf(Options[i], _("Size:  X %d  Y %d"), EditLevel()->xlen, EditLevel()->ylen);
 		MenuTexts[i] = Options[i];
 		i++;
-		sprintf(Options[i], _("Floor layers"));
-		sprintf(Options[i + 1], ":  %d", EditLevel()->floor_layers);
-		strcat(Options[i], Options[i + 1]);
+		sprintf(Options[i], _("Floor layers:  %d"), EditLevel()->floor_layers);
 		MenuTexts[i] = Options[i];
 		i++;
-		sprintf(Options[i], _("Edge"));
-		strcat(Options[i], " ");
-		sprintf(Options[i + 1], _("Interface"));
-		strcat(Options[i], Options[i + 1]);
-		strcat(Options[i], ":");
+		sprintf(Options[i], _("Edge Interface: "));
 		if (EditLevel()->jump_target_north == -1 && EditLevel()->jump_target_east == -1
 		    && EditLevel()->jump_target_south == -1 && EditLevel()->jump_target_west == -1) {
-			strcat(Options[i], " none");
+			strcat(Options[i], _("none"));
 		} else {
 			if (EditLevel()->jump_target_north != -1)
-				sprintf(Options[i] + strlen(Options[i]), "  N: %d", EditLevel()->jump_target_north);
+				sprintf(Options[i] + strlen(Options[i]), _(" N: %d"), EditLevel()->jump_target_north);
 			if (EditLevel()->jump_target_east != -1)
-				sprintf(Options[i] + strlen(Options[i]), "  E: %d", EditLevel()->jump_target_east);
+				sprintf(Options[i] + strlen(Options[i]), _(" E: %d"), EditLevel()->jump_target_east);
 			if (EditLevel()->jump_target_south != -1)
-				sprintf(Options[i] + strlen(Options[i]), "  S: %d", EditLevel()->jump_target_south);
+				sprintf(Options[i] + strlen(Options[i]), _(" S: %d"), EditLevel()->jump_target_south);
 			if (EditLevel()->jump_target_west != -1)
-				sprintf(Options[i] + strlen(Options[i]), "  W: %d", EditLevel()->jump_target_west);
+				sprintf(Options[i] + strlen(Options[i]), _(" W: %d"), EditLevel()->jump_target_west);
 
 		}
 		MenuTexts[i] = Options[i];
 		i++;
 
-		sprintf(Options[i], "%s: %s", _("Random dungeon"),
-			EditLevel()->random_dungeon == 2 ? _("2 connections") : EditLevel()->random_dungeon ==
-			1 ? _("1 connection") : _("no"));
+		if (EditLevel()->random_dungeon == 2) {
+			sprintf(Options[i], _("Random dungeon: 2 connections"));
+		} else if (EditLevel()->random_dungeon == 1) {
+			sprintf(Options[i], _("Random dungeon: 1 connection"));
+		} else {
+			sprintf(Options[i], _("Random dungeon: no"));
+		}
 		MenuTexts[i] = Options[i];
 		i++;
 
 		droid_types = EditLevel()->random_droids.types;
-		sprintf(Options[i], "Random droids: %d Types: ", EditLevel()->random_droids.nr);
+		sprintf(Options[i], _("Random droids: %d Types: "), EditLevel()->random_droids.nr);
 		for (j = 0; j < EditLevel()->random_droids.types_size; j++) {
 				if (j)
 					strcat(Options[i], ", ");
-				strcat(Options[i], Druidmap[droid_types[j]].druidname);
+				strcat(Options[i], Droidmap[droid_types[j]].droidname);
 		}
 		MenuTexts[i] = Options[i];
 		i++;
 
+		sprintf(Options[i], "%s: %s", _("Respawn"),
+			(EditLevel()->flags & NO_RESPAWN) ? _("no") : _("yes"));
+		MenuTexts[i] = Options[i];
+		i++;
+
+		if (EditLevel()->drop_class >= 0 && EditLevel()->drop_class <= MAX_DROP_CLASS)
+			snprintf(class, 16, "%d", EditLevel()->drop_class);
+		else
+			strncpy(class, _("None"), 15);
+
+		sprintf(Options[i], _("Item drop class for obstacles: %s.  (<-/->)"), class);
+		MenuTexts[i] = Options[i];
+		i++;
+
+		sprintf(Options[i], "%s: %s", _("Teleport blockade"),
+			(EditLevel()->flags & TELEPORT_BLOCKED) ? _("yes") : _("no"));
+		MenuTexts[i] = Options[i];
+		i++;
+
 		sprintf(Options[i], "%s: %s", _("Teleport pair"),
 				mapgen_teleport_pair_str(EditLevel()->teleport_pair));
 		MenuTexts[i] = Options[i];
 		i++;
 
-		sprintf(Options[i], _("Light"));
-		strcat(Options[i], ":  ");
-		strcat(Options[i], _("Ambient"));
+		sprintf(Options[i], _("Light:  Ambient"));
 		if (l == 0) {
 			sprintf(Options[i + 1], " [%d]  ", EditLevel()->minimum_light_value);
 			strcat(Options[i], Options[i + 1]);
@@ -634,7 +621,7 @@ static void LevelOptions(void)
 			strcat(Options[i], _("Bonus"));
 			sprintf(Options[i + 1], " [%d]  (<-/->)", EditLevel()->light_bonus);
 		} else
-			sprintf(Options[i + 1], "Im a bug");
+			sprintf(Options[i + 1], "I'm a bug");
 		strcat(Options[i], Options[i + 1]);
 		MenuTexts[i] = Options[i];
 		i++;
@@ -644,12 +631,11 @@ static void LevelOptions(void)
 		strcat(Options[i], Options[i + 1]);
 		MenuTexts[i] = Options[i];
 		i++;
-		sprintf(Options[i], _("Infinite Running Stamina"));
-		strcat(Options[i], ": ");
+		sprintf(Options[i], _("Infinite Running Stamina: "));
 		if (EditLevel()->infinite_running_on_this_level)
-			strcat(Options[i], _("YES"));
+			strcat(Options[i], _("yes"));
 		else
-			(strcat(Options[i], _("NO")));
+			(strcat(Options[i], _("no")));
 		MenuTexts[i] = Options[i];
 		i++;
 		MenuTexts[i++] = _("Add/Rem Level");
@@ -659,7 +645,7 @@ static void LevelOptions(void)
 		while (EscapePressed())
 			SDL_Delay(1);
 
-		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_BFont);
+		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_Font);
 
 		while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 			SDL_Delay(1);
@@ -676,7 +662,7 @@ static void LevelOptions(void)
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 
-			tgt = get_number_popup("\n Please enter new level number: \n\n", "");
+			tgt = get_number_popup(_("\n Please enter new level number: \n\n"), "");
 
 			if (tgt >= 0 && tgt < curShip.num_levels) {
 				if (level_exists(tgt)) {
@@ -693,6 +679,33 @@ static void LevelOptions(void)
 		case SET_RANDOM_DROIDS:
 			get_random_droids_from_user();
 			break;
+		case SET_NO_RESPAWN:
+			EditLevel()->flags ^= NO_RESPAWN;
+			break;
+		case SET_DROP_CLASS:
+			if (LeftPressed() || RightPressed()) {	//left or right arrow ? handled below 
+				break;
+			}
+			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
+				SDL_Delay(1);
+
+			if (EditLevel()->drop_class >= 0 && EditLevel()->drop_class <= MAX_DROP_CLASS)
+				snprintf(class, 16, "%d", EditLevel()->drop_class);
+			else
+				strcpy(class, "");
+
+			tgt = get_number_popup(_("\n Please enter new drop class: \n\n"), class);
+			if (tgt < -1)
+				EditLevel()->drop_class = -1;
+			else if (tgt > MAX_DROP_CLASS)
+				EditLevel()->drop_class = MAX_DROP_CLASS;
+			else
+				EditLevel()->drop_class = tgt;
+				
+			break;
+		case SET_TELEPORT_BLOCKED:
+			EditLevel()->flags ^= TELEPORT_BLOCKED;
+			break;
 		case SET_TELEPORT_PAIR:
 			EditLevel()->teleport_pair = mapgen_cycle_teleport_pair(EditLevel()->teleport_pair);
 			break;
@@ -720,7 +733,7 @@ static void LevelOptions(void)
 		case SET_LEVEL_NAME:
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
-			char *newLevelname = GetEditableStringInPopupWindow(1000, _("\n Please enter new level name: \n\n"), EditLevel()->Levelname);
+			char *newLevelname = GetEditableStringInPopupWindow(1000, _("\n Please enter new level name (in English): \n\n"), EditLevel()->Levelname);
 			if (newLevelname)
 				EditLevel()->Levelname = newLevelname;
 			break;
@@ -838,6 +851,20 @@ static void LevelOptions(void)
 					while (RightPressed()) ;
 				}
 				break;
+			case SET_DROP_CLASS:
+				if (LeftPressed()) {
+					EditLevel()->drop_class--;
+					if (EditLevel()->drop_class < -1)
+						EditLevel()->drop_class = -1;
+					while (LeftPressed());
+				}
+				if (RightPressed()) {
+					EditLevel()->drop_class++;
+					if (EditLevel()->drop_class > MAX_DROP_CLASS)
+						EditLevel()->drop_class = MAX_DROP_CLASS;
+					while (RightPressed());
+				}
+				break;
 
 			case CHANGE_LIGHT:
 				if (l == 0) {
@@ -878,63 +905,6 @@ static void LevelOptions(void)
 	return;
 };				// void LevelOptions ( void );
 
-static void AdvancedOptions(void)
-{
-	char *MenuTexts[100];
-	int proceed_now = FALSE;
-	int MenuPosition = 1;
-	int i;
-
-	enum {
-		RUN_MAP_VALIDATION = 1,
-//		RUN_LUA_VALIDATION,
-		LEAVE_OPTIONS_MENU,
-	};
-
-	game_status = INSIDE_MENU;
-
-	while (!proceed_now) {
-
-		InitiateMenu("--EDITOR_BACKGROUND--");
-
-		i = 0;
-		MenuTexts[i++] = _("Run Map Level Validator");
-//		MenuTexts[i++] = _("Run Dialog Lua Validator");
-		MenuTexts[i++] = _("Back");
-		MenuTexts[i++] = "";
-
-		while (EscapePressed())
-			SDL_Delay(1);
-
-		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_BFont);
-
-		while (EnterPressed() || SpacePressed() || MouseLeftPressed())
-			SDL_Delay(1);
-
-		switch (MenuPosition) {
-		case (-1):
-			while (EscapePressed()) ;
-			proceed_now = !proceed_now;
-			break;
-		case RUN_MAP_VALIDATION:
-			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
-				SDL_Delay(1);
-			level_validation();
-			break;
-		case LEAVE_OPTIONS_MENU:
-			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
-				SDL_Delay(1);
-			proceed_now = !proceed_now;
-			break;
-		default:
-			break;
-
-		}		// switch
-	}
-
-	game_status = INSIDE_LVLEDITOR;
-	return;
-};				// void AdvancedOptions ( void );
 
 /**
  *
@@ -958,7 +928,7 @@ int DoLevelEditorMainMenu()
 	enum {
 		ENTER_LEVEL_POSITION = 1,
 		LEVEL_OPTIONS_POSITION,
-		ADVANCED_OPTIONS_POSITION,
+		RUN_MAP_VALIDATION,
 		TEST_MAP_POSITION,
 		SAVE_LEVEL_POSITION,
 //              MANAGE_LEVEL_POSITION,
@@ -968,18 +938,18 @@ int DoLevelEditorMainMenu()
 		QUIT_POSITION,
 	};
 
+	MenuItemSelectedSound();
+
 	while (!proceed_now) {
 
 		InitiateMenu("--EDITOR_BACKGROUND--");
 
 		i = 0;
-		sprintf(Options[i], _("Level"));
-		sprintf(Options[i + 1], ": %d - %s", EditLevel()->levelnum, EditLevel()->Levelname);
-		strcat(Options[i], Options[i + 1]);
+		sprintf(Options[i], _("Level name (untranslated): %d - %s"), EditLevel()->levelnum, EditLevel()->Levelname);
 		MenuTexts[i] = Options[i];
 		i++;
 		MenuTexts[i++] = _("Level Options");
-		MenuTexts[i++] = _("Advanced Options");
+		MenuTexts[i++] = _("Run Map Level Validator");
 		if (game_root_mode == ROOT_IS_LVLEDIT) {
 			MenuTexts[i++] = _("Playtest Mapfile");
 			MenuTexts[i++] = _("Save Mapfile");
@@ -990,7 +960,7 @@ int DoLevelEditorMainMenu()
 			//              MenuTexts[i++] = " ";
 		}
 		MenuTexts[i++] = _("Continue Editing");
-                MenuTexts[i++] = _("Show Help");
+		MenuTexts[i++] = _("Show Help");
 		MenuTexts[i++] = _("Quit to Main Menu");
 		MenuTexts[i++] = _("Exit FreedroidRPG");
 		MenuTexts[i++] = "";
@@ -998,7 +968,7 @@ int DoLevelEditorMainMenu()
 		while (EscapePressed())
 			SDL_Delay(1);
 
-		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_BFont);
+		MenuPosition = DoMenuSelection("", MenuTexts, -1, "--EDITOR_BACKGROUND--", FPS_Display_Font);
 
 		while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 			SDL_Delay(1);
@@ -1019,7 +989,7 @@ int DoLevelEditorMainMenu()
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 
-			tgt = get_number_popup("\n Please enter new level number: \n\n", "");
+			tgt = get_number_popup(_("\n Please enter new level number: \n\n"), "");
 
 			if (tgt >= 0 && tgt < curShip.num_levels) {
 				if (level_exists(tgt)) {
@@ -1034,10 +1004,10 @@ int DoLevelEditorMainMenu()
 				SDL_Delay(1);
 			LevelOptions();
 			break;
-		case ADVANCED_OPTIONS_POSITION:
+		case RUN_MAP_VALIDATION:
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
-			AdvancedOptions();
+			level_validation();
 			break;
 		case TEST_MAP_POSITION:
 			TestMap();
@@ -1057,7 +1027,7 @@ int DoLevelEditorMainMenu()
                 case SHOW_HELP:
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
-			PlayATitleFile("level_editor_help.title");
+			PlayATitleFile("level_editor_help.lua");
 			break;
 		case QUIT_TO_MAIN_POSITION:
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
@@ -1074,7 +1044,7 @@ int DoLevelEditorMainMenu()
 			break;
 		case QUIT_POSITION:
 			DebugPrintf(2, "\nvoid EscapeMenu( void ): Quit Requested by user.  Terminating...");
-			Terminate(EXIT_SUCCESS, TRUE);
+			Terminate(EXIT_SUCCESS);
 			break;
 		default:
 			break;
