@@ -3,12 +3,13 @@ extern int *wall_tiles_list;
 extern int *machinery_tiles_list;
 extern int *furniture_tiles_list;
 extern int *container_tiles_list;
-extern int *plant_tiles_list;
+extern int *nature_tiles_list;
 extern int *misc_tiles_list;
 extern int *waypoint_list;
 extern int *sidewalk_floor_list;
 extern int *water_floor_list;
 extern int *grass_floor_list;
+extern int *sand_floor_list;
 extern int *square_floor_list;
 extern int *other_floor_list;
 extern int *melee_items_list;
