@@ -234,13 +234,9 @@ static void end_rectangle_floor(int commit)
  */
 void place_enemy(int droid_type)
 {
-	gps pos;
-
-	pos.x = mouse_mapcoord.x;
-	pos.y = mouse_mapcoord.y;
-	pos.z = EditLevel()->levelnum;
-
-	place_special_force(pos, droid_type);
+	gps pos = { mouse_mapcoord.x, mouse_mapcoord.y, EditLevel()->levelnum };
+	enemy *en = create_special_force(pos, droid_type);
+	action_create_enemy(EditLevel(), en);
 }
 
 /**
@@ -347,13 +343,17 @@ static void add_wall(float x, float y, int type, enum _level_editor_directions d
 	wall->position.y = y;
 
 	// Create the new wall on the map
-	wall->address = action_create_obstacle_user(EditLevel(), x, y, type);
+	wall->address = add_obstacle(EditLevel(), x, y, type);
 
-	// Add the new wall in the linked list
-	list_add_tail(&(wall->list), &(state.l_elements_head));
-
-	// Define the new direction of the line of walls
-	state.l_direction = direction;
+	if (wall->address) {
+		// Add the new wall in the linked list
+		list_add_tail(&(wall->list), &(state.l_elements_head));
+		// Define the new direction of the line of walls
+		state.l_direction = direction;
+	} else {
+		free(wall);
+	}
+	// cppcheck-suppress memleak
 }
 
 /**
@@ -401,7 +401,7 @@ static void line_moving_backwards(void)
 	last_pos = last_wall->position;
 
 	// Remove the last wall
-	action_remove_obstacle_user(EditLevel(), last_wall->address);
+	del_obstacle(last_wall->address);
 	list_del(&last_wall->list);
 	free(last_wall);
 
@@ -618,15 +618,19 @@ static void end_wall_line(int commit)
 
 	// Remove the linked list
 	list_for_each_entry_safe(e, ne, &state.l_elements_head, list) {
-		list_del(&e->list);
-		free(e);
-
 		if (!commit) {
 			// When the user wants cancel the line of walls, we must remove all
 			// the walls
 			del_obstacle(e->address);
+		} else {
+			/* Add all the actions needed to be able to delete
+			 * the entire line of walls. */
+			action_push(ACT_REMOVE_OBSTACLE, e->address);
 		}
 		nb_actions++;
+
+		list_del(&e->list);
+		free(e);
 	}
 
 	if (commit)
@@ -640,6 +644,10 @@ int leveleditor_place_input(SDL_Event *event)
 
 	if (!mouse_in_level) {
 		// We must not place objects outside of the level
+		// And we stop drawing walls if mouse leaved the level
+		if (our_mode == LINE_WALLS) {
+			end_wall_line(1);
+		}
 		return 0;
 	}
 
@@ -682,7 +690,7 @@ int leveleditor_place_input(SDL_Event *event)
 				return 1;
 				break;
 			default:
-				alert_window("%s", _("Place tool does not support this type of object."));
+				alert_window(_("Place tool does not support this type of object."));
 			}
 		}
 	} else if (our_mode == RECTANGLE_FLOOR) {
@@ -737,3 +745,11 @@ void leveleditor_place_reset()
 	}
 	OriginWaypoint = -1;
 }
+
+void leveleditor_place_reset_waypoint_route(int wpnum)
+{
+	if (our_mode == CONNECT_WAYPOINT && state.startwp == wpnum) {
+		end_waypoint_route();
+	}
+	OriginWaypoint = -1;
+}
