@@ -137,6 +137,69 @@ int element_in_selection(void *data)
 	return 0;
 }
 
+/**
+ * Remove an element from the selection list.
+ * Return 1 if the element was actually removed (it was in the selection list), or 0 if not.
+ */
+int remove_element_from_selection(void *data)
+{
+	struct selected_element *e, *next;
+	list_for_each_entry_safe(e, next, &selected_elements, node) {
+		switch(e->type) {
+		case OBJECT_FLOOR:
+			{
+				// We must compare the floor tiles in the wrapper structure
+				struct lvledit_map_tile *t = e->data;
+				if (t->tile == data) {
+					free(t->tile);
+					t->tile = NULL;
+					free(e->data);
+					e->data = NULL;
+					list_del(&e->node);
+					free(e);
+					e = NULL;
+					return 1;
+				}
+			}
+			break;
+		case OBJECT_WAYPOINT:
+			// We must compare the coordinates of the two waypoints
+			if (((waypoint *)e->data)->x == ((waypoint *)data)->x &&
+				((waypoint *)e->data)->y == ((waypoint *)data)->y) {
+				free(e->data);
+				e->data = NULL;
+				list_del(&e->node);
+				free(e);
+				e = NULL;
+				return 1;
+			}
+			break;
+		case OBJECT_MAP_LABEL:
+			// We must compare the coordinates of the two map labels
+			if (((map_label *)e->data)->pos.x == ((map_label *)data)->pos.x &&
+				((map_label *)e->data)->pos.y == ((map_label *)data)->pos.y) {
+				free(e->data);
+				e->data = NULL;
+				list_del(&e->node);
+				free(e);
+				e = NULL;
+				return 1;
+			}
+			break;
+		default:
+			if (e->data == data) {
+				list_del(&e->node);
+				free(e);
+				e = NULL;
+				return 1;
+			}
+			break;
+		}
+	}
+
+	return 0;
+}
+
 static void __calc_min_max(float x, float y, moderately_finepoint *cmin, moderately_finepoint *cmax)
 {
 	if (x < cmin->x)
@@ -245,7 +308,7 @@ static void add_object_to_list(struct list_head *list, void *data, enum lvledit_
 	list_add(&e->node, list);
 }
 
-static void __clear_selected_list(struct list_head *lst, int nbelem)
+static void __clear_selected_list(struct list_head *lst, int nbelem, int is_clipboard)
 {
 	struct selected_element *e, *ne;
 	list_for_each_entry_safe(e, ne, lst, node) {
@@ -253,6 +316,14 @@ static void __clear_selected_list(struct list_head *lst, int nbelem)
 			return;
 		if (e->type == OBJECT_FLOOR || e->type == OBJECT_WAYPOINT || e->type == OBJECT_MAP_LABEL) {
 			/* Unselecting floor tiles, waypoints or map labels requires freeing the duplicated data */
+			if (e->type == OBJECT_FLOOR && is_clipboard) {
+				/* Clearing the clipboard requires freeing the wrapped floor tiles */
+				struct lvledit_map_tile *t = e->data;
+				free(t->tile);
+			} else if (e->type == OBJECT_MAP_LABEL && is_clipboard) {
+				struct map_label *m = e->data;
+				free(m->label_name);
+			}
 			free(e->data);
 		}
 		list_del(&e->node);
@@ -315,7 +386,7 @@ static void select_item_on_tile(int x, int y)
 		if (it->type == -1)
 			continue;
 
-		if ((fabsf(x - it->pos.x) <= 0.5) && (fabsf(y - it->pos.y) <= 0.5)) {
+		if ((it->pos.x - x) >= 0 && (it->pos.x - x) <= 1 && (it->pos.y - y) >= 0 && (it->pos.y - y) <= 1) {
 			if (!element_in_selection(it)) {
 				add_object_to_list(&selected_elements, it, OBJECT_ITEM);
 				state.rect_nbelem_selected++;
@@ -381,8 +452,8 @@ static void select_object_on_tile(int x, int y)
 		select_special_forces_on_tile(x, y);
 		break;
 	default:
-		ErrorMessage(__FUNCTION__,
-				"Abstract object type %d for leveleditor not supported.\n", PLEASE_INFORM, IS_WARNING_ONLY, selection_type());
+		error_message(__FUNCTION__,
+				"Abstract object type %d for leveleditor not supported.", PLEASE_INFORM, selection_type());
 		break;
 	}
 }
@@ -395,12 +466,12 @@ static void select_object_on_tile(int x, int y)
  */
 void clear_selection(int nbelem)
 {
-	__clear_selected_list(&selected_elements, nbelem);
+	__clear_selected_list(&selected_elements, nbelem, 0);
 }
 
 void clear_clipboard(int nbelem)
 {
-	__clear_selected_list(&clipboard_elements, nbelem);
+	__clear_selected_list(&clipboard_elements, nbelem, 1);
 }
 
 static void start_rect_select()
@@ -824,7 +895,7 @@ void level_editor_cycle_marked_object()
 			level_editor_cycle_marked_item();
 			break;
 		default:
-			ErrorMessage(__FUNCTION__, "Unhandled selection type. \n", IS_WARNING_ONLY, NO_NEED_TO_INFORM);
+			error_message(__FUNCTION__, "Unhandled selection type.", NO_REPORT);
 			break;
 	}
 	
@@ -832,6 +903,67 @@ void level_editor_cycle_marked_object()
 	state.rect_nbelem_selected = 1;
 }
 
+static int clear_current_floor_layers(level *lvl, int coord_x, int coord_y)
+{
+	int i;
+
+	if (GameConfig.show_all_floor_layers) {
+		for (i = 0; i < lvl->floor_layers; i++)
+			action_set_floor_layer(lvl, coord_x, coord_y, i, ISO_FLOOR_EMPTY);
+		return i;
+	} else {
+		action_set_floor_layer(lvl, coord_x, coord_y, current_floor_layer, ISO_FLOOR_EMPTY);
+		return 1;
+	}
+}
+
+void level_editor_delete_selection()
+{
+	struct selected_element *e, *next;
+	int nbelem = 0;
+	if (mode != FD_RECTDONE) {
+		// We get called from the outside so check mode coherency first
+		return;
+	}
+
+	list_for_each_entry_safe(e, next, &selected_elements, node) {
+		switch (e->type) {
+		case OBJECT_OBSTACLE:
+			action_remove_obstacle_user(EditLevel(), e->data);
+			nbelem++;
+			break;
+		case OBJECT_FLOOR:
+			// We replace the tile we cut by a black tile
+			nbelem += clear_current_floor_layers(EditLevel(),
+				((struct lvledit_map_tile *) (e->data))->coord.x,
+				((struct lvledit_map_tile *) (e->data))->coord.y);
+			break;
+		case OBJECT_ITEM:
+			action_remove_item(EditLevel(), e->data);
+			nbelem++;
+			break;
+		case OBJECT_WAYPOINT:
+			action_remove_waypoint(EditLevel(), ((waypoint *)(e->data))->x, ((waypoint *)(e->data))->y);
+			nbelem++;
+			break;
+		case OBJECT_MAP_LABEL:
+			action_remove_map_label(EditLevel(), ((map_label *)(e->data))->pos.x, ((map_label *)(e->data))->pos.y);
+			nbelem++;
+			break;
+		case OBJECT_ENEMY:
+			action_remove_enemy(EditLevel(), e->data);
+			nbelem++;
+			break;
+		default:
+			break;
+		}
+	}
+	
+	action_push(ACT_MULTIPLE_ACTIONS, nbelem);
+
+	clear_selection(-1);
+}
+
 void level_editor_copy_selection()
 {
 	struct selected_element *e;
@@ -888,7 +1020,7 @@ void level_editor_copy_selection()
 		case OBJECT_MAP_LABEL:
 			m = MyMalloc(sizeof(map_label));
 			memcpy(m, e->data, sizeof(map_label));
-
+			m->label_name = strdup(((map_label*) e->data)->label_name);
 			add_object_to_list(&clipboard_elements, m, OBJECT_MAP_LABEL);
 			break;
 		default:
@@ -897,65 +1029,6 @@ void level_editor_copy_selection()
 	}
 }
 
-static int clear_current_floor_layers(level *lvl, int coord_x, int coord_y)
-{
-	int i;
-
-	if (GameConfig.show_all_floor_layers) {
-		for (i = 0; i < lvl->floor_layers; i++)
-			action_set_floor_layer(lvl, coord_x, coord_y, i, ISO_FLOOR_EMPTY);
-		return i;
-	} else {
-		action_set_floor_layer(lvl, coord_x, coord_y, current_floor_layer, ISO_FLOOR_EMPTY);
-		return 1;
-	}
-}
-
-void level_editor_cut_selection()
-{
-	struct selected_element *e;
-	int nbelem = 0;
-	if (mode != FD_RECTDONE) {
-		// We get called from the outside so check mode coherency first
-		return;
-	}
-
-	level_editor_copy_selection();
-
-	list_for_each_entry(e, &selected_elements, node) {
-		switch (e->type) {
-		case OBJECT_OBSTACLE:
-			action_remove_obstacle_user(EditLevel(), e->data);
-			nbelem++;
-			break;
-		case OBJECT_FLOOR:
-			// We replace the tile we cut by a black tile
-			nbelem += clear_current_floor_layers(EditLevel(),
-				((struct lvledit_map_tile *) (e->data))->coord.x,
-				((struct lvledit_map_tile *) (e->data))->coord.y);
-			break;
-		case OBJECT_ITEM:
-			action_remove_item(EditLevel(), e->data);
-			nbelem++;
-			break;
-		case OBJECT_WAYPOINT:
-			action_remove_waypoint(EditLevel(), ((waypoint *)(e->data))->x, ((waypoint *)(e->data))->y);
-			nbelem++;
-			break;
-		case OBJECT_MAP_LABEL:
-			action_remove_map_label(EditLevel(), ((map_label *)(e->data))->pos.x, ((map_label *)(e->data))->pos.y);
-			nbelem++;
-			break;
-		default:
-			break;
-		}
-	}
-	
-	action_push(ACT_MULTIPLE_ACTIONS, nbelem);
-
-	clear_selection(-1);
-}
-
 void level_editor_paste_selection()
 {		
 	struct selected_element *e;
@@ -964,7 +1037,7 @@ void level_editor_paste_selection()
 	waypoint *w;
 	obstacle *o;
 	item *it;
-	int nbact = 0;
+	int counter, nbact = 0;
 	moderately_finepoint cmin, cmax, center;
 
 	if (mode == FD_RECT || mode == DRAGDROP) {
@@ -1005,9 +1078,11 @@ void level_editor_paste_selection()
 			}
 			
 			// Add and select
-			add_object_to_list(&selected_elements, action_create_obstacle_user(EditLevel(), o->pos.x, o->pos.y, o->type), OBJECT_OBSTACLE);
-			
-			nbact++;
+			struct obstacle *newo = action_create_obstacle_user(EditLevel(), o->pos.x, o->pos.y, o->type);
+			if (newo) {
+				add_object_to_list(&selected_elements, newo, OBJECT_OBSTACLE);
+				nbact++;
+			}
 			break;
 		case OBJECT_FLOOR:
 			t = e->data;
@@ -1084,10 +1159,21 @@ void level_editor_paste_selection()
 				break;
 			}
 
+			// Create the autostr to manipulate string
+			struct auto_string *str = alloc_autostr(32);
+			autostr_printf(str, "%s", m->label_name);
+			
+			// Modify the label until possible value
+			counter = 1;
+			while (map_label_exists(str->value)) {
+				autostr_printf(str, "%s-%d", m->label_name, counter++);
+			}
+
 			// Add and select
-			action_create_map_label(EditLevel(), m->pos.x, m->pos.y, m->label_name);
+			action_create_map_label(EditLevel(), m->pos.x, m->pos.y, str->value);
 			select_map_label_on_tile(m->pos.x, m->pos.y);
 
+			free_autostr(str);
 			nbact++;
 			break;
 		default:
@@ -1098,6 +1184,12 @@ void level_editor_paste_selection()
 	action_push(ACT_MULTIPLE_ACTIONS, nbact);
 }
 
+void level_editor_cut_selection()
+{
+	level_editor_copy_selection();
+	level_editor_delete_selection();
+}
+
 int leveleditor_select_input(SDL_Event * event)
 {
 	if(!mouse_in_level)
