@@ -29,35 +29,37 @@
 
 #define _leveleditor_validator_c
 
+#include "proto.h"
 #include "lvledit/lvledit_validator.h"
 #include "lang.h"
 
-#define IS_CHEST(t)  ( (t) >= ISO_H_CHEST_CLOSED && (t) <= ISO_V_CHEST_OPEN )
-#define IS_BARREL(t) ( (t) >= ISO_BARREL_1       && (t) <= ISO_BARREL_4     )
-#define IS_SIGN(t)   ( (t) >= ISO_SIGN_1         && (t) <= ISO_SIGN_3       )
+#define IS_CHEST(t)    ( get_obstacle_spec(t)->action && !strncmp(get_obstacle_spec(t)->action, "chest", 5) )
+#define IS_BARREL(t)   ( (t) >= ISO_BARREL_1       && (t) <= ISO_BARREL_4     )
+#define IS_TERMINAL(t) ( get_obstacle_spec(t)->action && !strncmp(get_obstacle_spec(t)->action, "terminal", 8) )
+#define IS_SIGN(t)     ( get_obstacle_spec(t)->action && !strncmp(get_obstacle_spec(t)->action, "sign", 4) )
 
 static char *bigline = "====================================================================";
 static char *line = "--------------------------------------------------------------------";
 static char *sepline = "+------------------------------";
 
 static void lvlval_chest_execute(struct level_validator *this, struct lvlval_ctx *validator_ctx);
-static void *lvlval_chest_parse_excpt(char *string);
+static void *lvlval_chest_parse_excpt(char *str);
 static int lvlval_chest_cmp_data(void *opaque_data1, void *opaque_data2);
 
 static void lvlval_waypoint_execute(struct level_validator *this, struct lvlval_ctx *validator_ctx);
-static void *lvlval_waypoint_parse_excpt(char *string);
+static void *lvlval_waypoint_parse_excpt(char *str);
 static int lvlval_waypoint_cmp_data(void *opaque_data1, void *opaque_data2);
 
 static void lvlval_neighborhood_execute(struct level_validator *this, struct lvlval_ctx *validator_ctx);
-static void *lvlval_neighborhood_parse_excpt(char *string);
+static void *lvlval_neighborhood_parse_excpt(char *str);
 static int lvlval_neighborhood_cmp_data(void *opaque_data1, void *opaque_data2);
 
 static void lvlval_obstacles_execute(struct level_validator *this, struct lvlval_ctx *validator_ctx);
-static void *lvlval_obstacles_parse_excpt(char *string);
+static void *lvlval_obstacles_parse_excpt(char *str);
 static int lvlval_obstacles_cmp_data(void *opaque_data1, void *opaque_data2);
 
 static void lvlval_extensions_execute(struct level_validator *this, struct lvlval_ctx *validator_ctx);
-static void *lvlval_extensions_parse_excpt(char *string);
+static void *lvlval_extensions_parse_excpt(char *str);
 static int lvlval_extensions_cmp_data(void *opaque_data1, void *opaque_data2);
 
 static void lvlval_map_labels_execute(struct level_validator *this, struct lvlval_ctx *validator_ctx);
@@ -173,6 +175,7 @@ static void validator_print_error(struct lvlval_ctx *validator_ctx, struct lvlva
 	
 	vprintf(validator_error->format, args);
 	printf("\n");
+	va_end(args);
 }
 
 static void validator_print_separator(struct lvlval_ctx *validator_ctx)
@@ -242,7 +245,7 @@ static void get_excpt_list(char *section_pointer)
 
 		char *validator_type = ReadAndMallocStringFromData(ptr, "Type=\"", "\"");
 		if (!validator_type) {
-			ErrorMessage(__FUNCTION__, "The Validator's Type of an exception was not found!\n", PLEASE_INFORM, IS_FATAL);
+			error_message(__FUNCTION__, "The Validator's Type of an exception was not found!", PLEASE_INFORM | IS_FATAL);
 			return;
 		}
 
@@ -275,8 +278,8 @@ static void get_excpt_list(char *section_pointer)
 		free(validator_type);
 
 		if (one_validator->initial == '\0') {	// Sub-validator was not found
-			ErrorMessage(__FUNCTION__, "The Validator's Type specified in an exception does not exist!\n", PLEASE_INFORM,
-				     IS_FATAL);
+			error_message(__FUNCTION__, "The Validator's Type specified in an exception does not exist!",
+					PLEASE_INFORM | IS_FATAL);
 			return;
 		}
 		
@@ -296,27 +299,28 @@ static void get_excpt_list(char *section_pointer)
 
 static void load_excpt_lists(char *filename)
 {
-	char fpath[2048];
+	char fpath[PATH_MAX];
 	char *main_file_pointer;
 	char *section_pointer;
 
 #	define START_OF_DATA_STRING   "*** Beginning of LevelValidator Exceptions List ***"
 #	define END_OF_DATA_STRING     "*** End of LevelValidator Exceptions List ***"
 
-	// Read whole file in memory    
-	find_file(filename, MAP_DIR, fpath, 0);
-	main_file_pointer = ReadAndMallocAndTerminateFile(fpath, END_OF_DATA_STRING);
+	// Read whole file in memory
+	if (find_file(filename, MAP_DIR, fpath, NO_REPORT)) {
+		main_file_pointer = ReadAndMallocAndTerminateFile(fpath, END_OF_DATA_STRING);
 
-	// Search beginning of list 
-	section_pointer = strstr(main_file_pointer, START_OF_DATA_STRING);
-	if (section_pointer == NULL) {
-		ErrorMessage(__FUNCTION__, "Start of exceptions list not found!\n", PLEASE_INFORM, IS_FATAL);
-		return;
+		// Search beginning of list
+		section_pointer = strstr(main_file_pointer, START_OF_DATA_STRING);
+		if (section_pointer == NULL) {
+			error_message(__FUNCTION__, "Start of exceptions list not found!", PLEASE_INFORM);
+			return;
+		}
+		// Parse the list
+		get_excpt_list(section_pointer);
+
+		free(main_file_pointer);
 	}
-	// Parse the list
-	get_excpt_list(section_pointer);
-
-	free(main_file_pointer);
 
 #	undef START_OF_DATA_STRING
 #	undef END_OF_DATA_STRING
@@ -422,14 +426,14 @@ struct chest_excpt_data {
  * Parse "chest" exception
  */
 
-static void *lvlval_chest_parse_excpt(char *string)
+static void *lvlval_chest_parse_excpt(char *str)
 {
-	struct chest_excpt_data *data = (struct chest_excpt_data *)malloc(sizeof(struct chest_excpt_data));
+	struct chest_excpt_data *data = (struct chest_excpt_data *)MyMalloc(sizeof(struct chest_excpt_data));
 
-	ReadValueFromString(string, "Idx=", "%d", &(data->obj_id), NULL);
-	ReadValueFromString(string, "X=", "%f", &(data->obj_pos.x), NULL);
-	ReadValueFromString(string, "Y=", "%f", &(data->obj_pos.y), NULL);
-	ReadValueFromString(string, "L=", "%d", &(data->obj_pos.z), NULL);
+	ReadValueFromString(str, "Idx=", "%d", &(data->obj_id), NULL);
+	ReadValueFromString(str, "X=", "%f", &(data->obj_pos.x), NULL);
+	ReadValueFromString(str, "Y=", "%f", &(data->obj_pos.y), NULL);
+	ReadValueFromString(str, "L=", "%d", &(data->obj_pos.z), NULL);
 
 	return (data);
 }
@@ -519,15 +523,15 @@ struct waypoint_excpt_data {
  * Parse a 'waypoint' exception
  */
 
-static void *lvlval_waypoint_parse_excpt(char *string)
+static void *lvlval_waypoint_parse_excpt(char *str)
 {
-	char *validator_type = ReadAndMallocStringFromData(string, "Type=\"", "\"");
+	char *validator_type = ReadAndMallocStringFromData(str, "Type=\"", "\"");
 	if (!validator_type || strlen(validator_type) != 2) {
-		ErrorMessage(__FUNCTION__, "The Subtest name of an exception is not valid!\n", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "The Subtest name of an exception is not valid!", PLEASE_INFORM | IS_FATAL);
 		return NULL;
 	}
 
-	struct waypoint_excpt_data *data = (struct waypoint_excpt_data *)malloc(sizeof(struct waypoint_excpt_data));
+	struct waypoint_excpt_data *data = (struct waypoint_excpt_data *)MyMalloc(sizeof(struct waypoint_excpt_data));
 
 	data->subtest = validator_type[1];
 	free(validator_type);
@@ -536,9 +540,9 @@ static void *lvlval_waypoint_parse_excpt(char *string)
 	case 'P':
 	case 'O':
 	case 'S':
-		ReadValueFromString(string, "X=", "%f", &(data->wp_pos[0].x), NULL);
-		ReadValueFromString(string, "Y=", "%f", &(data->wp_pos[0].y), NULL);
-		ReadValueFromString(string, "L=", "%d", &(data->wp_pos[0].z), NULL);
+		ReadValueFromString(str, "X=", "%f", &(data->wp_pos[0].x), NULL);
+		ReadValueFromString(str, "Y=", "%f", &(data->wp_pos[0].y), NULL);
+		ReadValueFromString(str, "L=", "%d", &(data->wp_pos[0].z), NULL);
 		data->wp_pos[1].x = data->wp_pos[1].y = 0.0;
 		data->wp_pos[1].z = 0;
 		break;
@@ -546,16 +550,16 @@ static void *lvlval_waypoint_parse_excpt(char *string)
 	case 'D':
 	case 'W':
 	case 'Q':
-		ReadValueFromString(string, "X1=", "%f", &(data->wp_pos[0].x), NULL);
-		ReadValueFromString(string, "Y1=", "%f", &(data->wp_pos[0].y), NULL);
-		ReadValueFromString(string, "L1=", "%d", &(data->wp_pos[0].z), NULL);
-		ReadValueFromString(string, "X2=", "%f", &(data->wp_pos[1].x), NULL);
-		ReadValueFromString(string, "Y2=", "%f", &(data->wp_pos[1].y), NULL);
-		ReadValueFromString(string, "L2=", "%d", &(data->wp_pos[1].z), NULL);
+		ReadValueFromString(str, "X1=", "%f", &(data->wp_pos[0].x), NULL);
+		ReadValueFromString(str, "Y1=", "%f", &(data->wp_pos[0].y), NULL);
+		ReadValueFromString(str, "L1=", "%d", &(data->wp_pos[0].z), NULL);
+		ReadValueFromString(str, "X2=", "%f", &(data->wp_pos[1].x), NULL);
+		ReadValueFromString(str, "Y2=", "%f", &(data->wp_pos[1].y), NULL);
+		ReadValueFromString(str, "L2=", "%d", &(data->wp_pos[1].z), NULL);
 		break;
 
 	default:
-		ErrorMessage(__FUNCTION__, "The Subtest name of an exception is invalid!\n", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "The Subtest name of an exception is invalid!", PLEASE_INFORM | IS_FATAL);
 		free(data);
 		return NULL;
 	}
@@ -909,21 +913,21 @@ struct neighborhood_excpt_data {
  * Parse a 'neighborhood' exception
  */
 
-static void *lvlval_neighborhood_parse_excpt(char *string)
+static void *lvlval_neighborhood_parse_excpt(char *str)
 {
-	struct neighborhood_excpt_data *data = (struct neighborhood_excpt_data *)malloc(sizeof(struct neighborhood_excpt_data));
+	struct neighborhood_excpt_data *data = (struct neighborhood_excpt_data *)MyMalloc(sizeof(struct neighborhood_excpt_data));
 
-	char *direction_name = ReadAndMallocStringFromData(string, "Interface:", " ");
+	char *direction_name = ReadAndMallocStringFromData(str, "Interface:", " ");
 	if (!direction_name) {
-		ErrorMessage(__FUNCTION__, "The Direction of an exception was not found!\n", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "The Direction of an exception was not found!", PLEASE_INFORM | IS_FATAL);
 		free(data);
 		return NULL;
 	}
 	data->jumptarget = direction_name[0];
 	free(direction_name);
 
-	ReadValueFromString(string, "of Level:", "%d", &(data->from_level), NULL);
-	ReadValueFromString(string, "to Level:", "%d", &(data->to_level), NULL);
+	ReadValueFromString(str, "of Level:", "%d", &(data->from_level), NULL);
+	ReadValueFromString(str, "to Level:", "%d", &(data->to_level), NULL);
 
 	return (data);
 }
@@ -1114,15 +1118,15 @@ struct obstacle_excpt_data {
 	float border;
 };
 
-static void *lvlval_obstacles_parse_excpt(char *string)
+static void *lvlval_obstacles_parse_excpt(char *str)
 {
-	char *validator_type = ReadAndMallocStringFromData(string, "Type=\"", "\"");
+	char *validator_type = ReadAndMallocStringFromData(str, "Type=\"", "\"");
 	if (!validator_type || strlen(validator_type) != 2) {
-		ErrorMessage(__FUNCTION__, "The Subtest name of an exception is not valid!\n", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "The Subtest name of an exception is not valid!", PLEASE_INFORM | IS_FATAL);
 		return NULL;
 	}
 
-	struct obstacle_excpt_data *data = (struct obstacle_excpt_data *)malloc(sizeof(struct obstacle_excpt_data));
+	struct obstacle_excpt_data *data = (struct obstacle_excpt_data *)MyMalloc(sizeof(struct obstacle_excpt_data));
 
 	data->subtest = validator_type[1];
 	free(validator_type);
@@ -1132,15 +1136,15 @@ static void *lvlval_obstacles_parse_excpt(char *string)
 	case 'S':
 	case 'W':
 	case 'E':
-		ReadValueFromString(string, "X=", "%f", &(data->obs_pos.x), NULL);
-		ReadValueFromString(string, "Y=", "%f", &(data->obs_pos.y), NULL);
-		ReadValueFromString(string, "L=", "%d", &(data->obs_pos.z), NULL);
-		ReadValueFromString(string, "T=", "%d", &(data->type), NULL);
-		ReadValueFromString(string, "border=", "%f", &(data->border), NULL);
+		ReadValueFromString(str, "X=", "%f", &(data->obs_pos.x), NULL);
+		ReadValueFromString(str, "Y=", "%f", &(data->obs_pos.y), NULL);
+		ReadValueFromString(str, "L=", "%d", &(data->obs_pos.z), NULL);
+		ReadValueFromString(str, "T=", "%d", &(data->type), NULL);
+		ReadValueFromString(str, "border=", "%f", &(data->border), NULL);
 		break;
 
 	default:
-		ErrorMessage(__FUNCTION__, "The Subtest name of an exception is invalid!\n", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "The Subtest name of an exception is invalid!", PLEASE_INFORM | IS_FATAL);
 		free(data);
 		return NULL;
 	}
@@ -1204,7 +1208,7 @@ static void lvlval_obstacles_execute(struct level_validator *this, struct lvlval
 			continue;
 
 		if (o->pos.x < 0 || o->pos.x > max_x || o->pos.y < 0 || o->pos.y > max_y) {
-			obs_error.format = "[ERROR] Obstacle n.%d on level %d, type %d, has position %f %f. Allowed X position ranges from 0 to %f. Allowed Y position ranges from 0 to %f.";
+			obs_error.format = "[ERROR] Obstacle #%d on level %d, type %d, has position %f %f. Allowed X position ranges from 0 to %f. Allowed Y position ranges from 0 to %f.";
 			validator_print_error(validator_ctx, &obs_error, i, l->levelnum, o->type, o->pos.x, o->pos.y, max_x, max_y);
 			continue;
 		}
@@ -1274,33 +1278,50 @@ struct extension_excpt_data {
 	char subtest;
 	gps obs_pos;
 	int obs_index;
+	int obs_type;
 	int ext_type;
 };
 
-static void *lvlval_extensions_parse_excpt(char *string)
+static void *lvlval_extensions_parse_excpt(char *str)
 {
-	char *validator_type = ReadAndMallocStringFromData(string, "Type=\"", "\"");
+	char *validator_type = ReadAndMallocStringFromData(str, "Type=\"", "\"");
 	if (!validator_type || strlen(validator_type) != 2) {
-		ErrorMessage(__FUNCTION__, "The Subtest name of an exception is not valid!\n", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "The Subtest name of an exception is not valid!", PLEASE_INFORM | IS_FATAL);
 		return NULL;
 	}
 
-	struct extension_excpt_data *data = (struct extension_excpt_data *)malloc(sizeof(struct extension_excpt_data));
+	struct extension_excpt_data *data = (struct extension_excpt_data *)MyMalloc(sizeof(struct extension_excpt_data));
 
 	data->subtest = validator_type[1];
 	free(validator_type);
 
 	switch (data->subtest) {
 	case 'S':
-		ReadValueFromString(string, "X=", "%f", &(data->obs_pos.x), NULL);
-		ReadValueFromString(string, "Y=", "%f", &(data->obs_pos.y), NULL);
-		ReadValueFromString(string, "L=", "%d", &(data->obs_pos.z), NULL);
-		ReadValueFromString(string, "ObsIdx=", "%d", &(data->obs_index), NULL);
-		ReadValueFromString(string, "ExtType=", "%d", &(data->ext_type), NULL);
+		ReadValueFromString(str, "X=", "%f", &(data->obs_pos.x), NULL);
+		ReadValueFromString(str, "Y=", "%f", &(data->obs_pos.y), NULL);
+		ReadValueFromString(str, "L=", "%d", &(data->obs_pos.z), NULL);
+		ReadValueFromString(str, "ObsIdx=", "%d", &(data->obs_index), NULL);
+		ReadValueFromString(str, "ObsType=", "%d", &(data->obs_type), NULL);
+		data->ext_type = -1;
+		break;
+	case 'V':
+		ReadValueFromString(str, "X=", "%f", &(data->obs_pos.x), NULL);
+		ReadValueFromString(str, "Y=", "%f", &(data->obs_pos.y), NULL);
+		ReadValueFromString(str, "L=", "%d", &(data->obs_pos.z), NULL);
+		ReadValueFromString(str, "ObsIdx=", "%d", &(data->obs_index), NULL);
+		ReadValueFromString(str, "ObsType=", "%d", &(data->obs_type), NULL);
+		ReadValueFromString(str, "ExtType=", "%d", &(data->ext_type), NULL);
+		break;
+	case 'T':
+		ReadValueFromString(str, "X=", "%f", &(data->obs_pos.x), NULL);
+		ReadValueFromString(str, "Y=", "%f", &(data->obs_pos.y), NULL);
+		ReadValueFromString(str, "L=", "%d", &(data->obs_pos.z), NULL);
+		ReadValueFromString(str, "ObsIdx=", "%d", &(data->obs_index), NULL);
+		ReadValueFromString(str, "ObsType=", "%d", &(data->obs_type), NULL);
+		ReadValueFromString(str, "ExtType=", "%d", &(data->ext_type), NULL);
 		break;
-
 	default:
-		ErrorMessage(__FUNCTION__, "The Subtest name of an exception is invalid!\n", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "The Subtest name of an exception is invalid!", PLEASE_INFORM | IS_FATAL);
 		free(data);
 		return NULL;
 	}
@@ -1324,6 +1345,9 @@ static int lvlval_extensions_cmp_data(void *opaque_data1, void *opaque_data2)
 	if (data1->obs_index != data2->obs_index)
 		return FALSE;
 
+	if (data1->obs_type != data2->obs_type)
+		return FALSE;
+
 	if (data1->ext_type != data2->ext_type)
 		return FALSE;
 
@@ -1342,16 +1366,35 @@ static int lvlval_extensions_cmp_data(void *opaque_data1, void *opaque_data2)
 
 static void lvlval_extensions_execute(struct level_validator *this, struct lvlval_ctx *validator_ctx)
 {
-	struct lvlval_error extension_error = {
+	struct lvlval_error empty_error = {
 		.title = "Invalid obstacle extension",
 		.comment = "The extension data of an obstacle is missing, or the data is wrong",
+		.format = "[Type=\"ES\"] X=%f:Y=%f:L=%d ObsIdx=%d ObsType=%d (Obstacle image: %s) -> SIGNMESSAGE missing or empty (warning)",
 		.caught = FALSE,
 		.code = VALIDATION_WARNING
 	};
 
+	struct lvlval_error passive_error = {
+		.title = "Invalid obstacle extension",
+		.comment = "The extension is associated to a non-interactive obstacle",
+		.format = "[Type=\"EV\"] X=%f:Y=%f:L=%d ObsIdx=%d ObsType=%d (Obstacle image: %s) ExtType=%d (data: \"%s\") - (warning)",
+		.caught = FALSE,
+		.code = VALIDATION_WARNING
+	};
+
+	struct lvlval_error action_error = {
+		.title = "Invalid obstacle extension",
+		.comment = "The extension type is incoherent with the obstacle's action",
+		.format = "[Type=\"ET\"] X=%f:Y=%f:L=%d ObsIdx=%d ObsType=%d (Obstacle image: %s) ExtType=%d (data: \"%s\") -> Not compatible with the obstacle's action \"%s\"",
+		.caught = FALSE,
+		.code = VALIDATION_ERROR
+	};
+
 	int i;
 	level *l = validator_ctx->this_level;
 
+	// Check that all signs have an extension
+
 	for (i = 0; i < MAX_OBSTACLES_ON_MAP; i++) {
 		obstacle *o = &l->obstacle_list[i];
 
@@ -1360,18 +1403,78 @@ static void lvlval_extensions_execute(struct level_validator *this, struct lvlva
 
 		if (IS_SIGN(o->type)) {
 			struct extension_excpt_data to_check =
-				{ 'S', {o->pos.x, o->pos.y, l->levelnum}, get_obstacle_index(l, o), o->type };
+				{ 'S', {o->pos.x, o->pos.y, l->levelnum}, get_obstacle_index(l, o), o->type, -1 };
 
 			if (!lookup_exception(this, &to_check)) {
-				if (!get_obstacle_extension(l, o, OBSTACLE_EXTENSION_SIGNMESSAGE)) {
-					extension_error.format = "[Type=\"ES\"] X=%f:Y=%f:L=%d ObsIdx=%d ExtType=%d -> SIGNMESSAGE missing (warning)";
-					validator_print_error(validator_ctx, &extension_error, o->pos.x, o->pos.y, l->levelnum, get_obstacle_index(l, o), o->type);
+				const char *msg = (const char *)get_obstacle_extension(l, o, OBSTACLE_EXTENSION_SIGNMESSAGE);
+				if (!msg || *msg == '\0') {
+					const char *img = ((char **)get_obstacle_spec(o->type)->filenames.arr)[0];
+					validator_print_error(validator_ctx, &empty_error, o->pos.x, o->pos.y, l->levelnum, get_obstacle_index(l, o), o->type, img);
 				}
 			}
 		}
 	}
 
 	validator_print_separator(validator_ctx);
+
+	// Check that all dialog/sign extensions are associated to an interactive obstacle
+
+	for (i = 0; i < l->obstacle_extensions.size; i++) {
+		struct obstacle_extension *ext = &ACCESS_OBSTACLE_EXTENSION(l->obstacle_extensions, i);
+
+		if (ext->type != OBSTACLE_EXTENSION_DIALOGFILE && ext->type != OBSTACLE_EXTENSION_SIGNMESSAGE)
+			continue;
+
+		struct obstacle *o = ext->obs;
+		struct extension_excpt_data to_check =
+			{ 'V', {o->pos.x, o->pos.y, l->levelnum}, get_obstacle_index(l, o), o->type, ext->type };
+
+		if (lookup_exception(this, &to_check))
+			continue;
+
+		char *action = get_obstacle_spec(o->type)->action;
+		if (!action || *action == '\0') {
+			const char *img = ((char **)get_obstacle_spec(o->type)->filenames.arr)[0];
+			validator_print_error(validator_ctx, &passive_error, o->pos.x, o->pos.y, l->levelnum, get_obstacle_index(l, o), o->type, img, ext->type, (char *)ext->data );
+		}
+	}
+
+	validator_print_separator(validator_ctx);
+
+	// Check that all dialog/sign extensions are associated with the right interactive obstacle
+
+	for (i = 0; i < l->obstacle_extensions.size; i++) {
+		struct obstacle_extension *ext = &ACCESS_OBSTACLE_EXTENSION(l->obstacle_extensions, i);
+
+		if (ext->type != OBSTACLE_EXTENSION_DIALOGFILE && ext->type != OBSTACLE_EXTENSION_SIGNMESSAGE)
+			continue;
+
+		struct obstacle *o = ext->obs;
+		struct extension_excpt_data to_check =
+			{ 'T', {o->pos.x, o->pos.y, l->levelnum}, get_obstacle_index(l, o), o->type, ext->type };
+
+		if (lookup_exception(this, &to_check))
+			continue;
+
+		char *action = get_obstacle_spec(o->type)->action;
+		if (!action || *action == '\0') {
+			continue; // Handled by previous check
+		}
+
+		const char *img = ((char **)get_obstacle_spec(o->type)->filenames.arr)[0];
+
+		if (ext->type == OBSTACLE_EXTENSION_DIALOGFILE && !IS_TERMINAL(o->type)) {
+			validator_print_error(validator_ctx, &action_error, o->pos.x, o->pos.y, l->levelnum,
+					              get_obstacle_index(l, o), o->type, img, get_obstacle_spec(o->type)->action, (char *)ext->data);
+		}
+		if (ext->type == OBSTACLE_EXTENSION_SIGNMESSAGE && !IS_SIGN(o->type)) {
+			validator_print_error(validator_ctx, &action_error, o->pos.x, o->pos.y, l->levelnum,
+					              get_obstacle_index(l, o), o->type, img, get_obstacle_spec(o->type)->action, (char *)ext->data);
+		}
+	}
+
+	validator_print_separator(validator_ctx);
+
 }
 
 //===========================================================
@@ -1430,30 +1533,32 @@ static void lvlval_map_labels_execute(struct level_validator *this, struct lvlva
 
 int level_validation()
 {
+	game_status = INSIDE_MENU;
+
 	enum validator_return_code final_rc = VALIDATION_PASS;
 	int uncaught_excpt = FALSE;
 
 	SDL_Rect background_rect = { UNIVERSAL_COORD_W(20), UNIVERSAL_COORD_H(20), UNIVERSAL_COORD_W(600), UNIVERSAL_COORD_H(440) };
 	SDL_Rect report_rect = { UNIVERSAL_COORD_W(30), UNIVERSAL_COORD_H(30), UNIVERSAL_COORD_W(600), UNIVERSAL_COORD_H(430) };
 
-	BFont_Info *current_font = GetCurrentFont();
-	int row_height = FontHeight(current_font);
+	struct font *current_font = get_current_font();
+	int row_height = get_font_height(current_font);
 	int max_rows = (report_rect.h / row_height) - 4;	// 4 lines are reserved for header and footer
-	int column_width = TextWidth("000: empty");
+	int column_width = text_width(get_current_font(), "000: empty");
 	AssembleCombatPicture(ONLY_SHOW_MAP_AND_TEXT | SHOW_ITEMS | OMIT_TUX | GameConfig.omit_obstacles_in_level_editor *
 				OMIT_OBSTACLES | GameConfig.omit_enemies_in_level_editor * OMIT_ENEMIES | OMIT_BLASTS | SKIP_LIGHT_RADIUS |
 				NO_CURSOR | OMIT_ITEMS_LABEL);
-	SetCurrentFont(current_font);	// Reset font, in case it was modified by AssembleCombatPicture()
+	set_current_font(current_font);	// Reset font, in case it was modified by AssembleCombatPicture()
 
 	ShadowingRectangle(Screen, background_rect);
 
 	// Title
 
-	CenteredPutString(Screen, report_rect.y, "Level Validation tests - Summary\n");
+	put_string_centered(FPS_Display_Font, report_rect.y, "Level Validation tests - Summary\n");
 
 	// Load exceptions rules
 
-	load_excpt_lists("lvleditor_exceptions.dat");
+	load_excpt_lists("lvledit_exceptions.dat");
 
 	// Init map labels validator data
 	dynarray_init(&map_labels, 1024, sizeof(struct lvlval_map_label));
@@ -1476,10 +1581,10 @@ int level_validation()
 		if (!level_exists(l)) {
 			// Empty level
 			char txt[40];
- 			sprintf(txt, "%03d: \2empty", l);
- 			int lines = display_text_using_line_height(txt, col_pos, row_pos, &report_rect, 1.0);
+			sprintf(txt, "%03d: [w]empty", l);
+			int lines = display_text(txt, col_pos, row_pos, &report_rect, 1.0);
  			row_pos += lines * row_height;
-			SetCurrentFont(current_font);	// Reset font
+			set_current_font(current_font);	// Reset font
 		} else {
 			// Loop on each validation function
 			int v = 0;
@@ -1492,19 +1597,19 @@ int level_validation()
 			char txt[40];
 			switch (validator_ctx.return_code) {
 			case VALIDATION_ERROR:
-				sprintf(txt, "\2%03d: \1fail", l);
+				sprintf(txt, "[w]%03d: [r]fail", l);
 				break;
 			case VALIDATION_WARNING:
-				sprintf(txt, "\2%03d: \3warn", l);
+				sprintf(txt, "[w]%03d: [n]warn", l);
 				break;
 			case VALIDATION_PASS:
 			default:
-				sprintf(txt, "\2%03d: \2pass", l);
+				sprintf(txt, "[w]%03d: [w]pass", l);
 				break;
 			}
-			int lines = display_text_using_line_height(txt, col_pos, row_pos, &report_rect, 1.0);
+			int lines = display_text(txt, col_pos, row_pos, &report_rect, 1.0);
 			row_pos += lines * row_height;
-			SetCurrentFont(current_font);	// Reset font in case of the red "fail" was displayed
+			set_current_font(current_font);	// Reset font in case of the red "fail" was displayed
 
 			// Set final return code
 			compose_return_code(&final_rc, validator_ctx.return_code);
@@ -1524,16 +1629,16 @@ int level_validation()
 
 	int posy = report_rect.y + report_rect.h - row_height;
 
-	CenteredPutString(Screen, posy, "--- End of List --- Press Space to return to leveleditor ---");
+	put_string_centered(FPS_Display_Font, posy, "--- End of List --- Press Space to return to leveleditor ---");
 
 	if (final_rc != VALIDATION_PASS) {
 		posy -= row_height;
-		CenteredPutString(Screen, posy, "\1Some tests were invalid. See the report in the console\3");
+		put_string_centered(FPS_Display_Font, posy, "[r]Some tests were invalid. See the report in the console[n]");
 	}
 
 	if (uncaught_excpt) {
 		posy -= row_height;
-		CenteredPutString(Screen, posy, "\1Some exceptions were not caught. See the report in the console\3");
+		put_string_centered(FPS_Display_Font, posy, "[r]Some exceptions were not caught. See the report in the console[n]");
 	}
 
 	our_SDL_flip_wrapper();
@@ -1543,6 +1648,8 @@ int level_validation()
 	// Hack: eat all pending events.
 	input_handle();
 
+	game_status = INSIDE_LVLEDITOR;
+
 	return (final_rc != VALIDATION_PASS);
 }
 
@@ -1556,7 +1663,7 @@ int level_validation_on_console_only()
 	
 	// Load exceptions rules
 
-	load_excpt_lists("lvleditor_exceptions.dat");
+	load_excpt_lists("lvledit_exceptions.dat");
 
 	// Init map labels validator data
 	dynarray_init(&map_labels, 1024, sizeof(struct lvlval_map_label));
