@@ -49,13 +49,13 @@ static void categoryselect_display(struct widget *w)
 {
 	struct widget_lvledit_categoryselect *cs = w->ext;
 	SDL_Rect tr, hr;
-	int tab_width = 80;
+	int tab_width = 70;
 
-	draw_rectangle(&w->rect, 80, 100, 100, 150);
+	draw_rectangle(&w->rect, 70, 100, 100, 150);
 
-	BFont_Info *PreviousFont;
-	PreviousFont = GetCurrentFont();
-	SetCurrentFont(Messagevar_BFont);
+	struct font *PreviousFont;
+	PreviousFont = get_current_font();
+	set_current_font(Messagevar_Font);
 
 	tr.y = w->rect.y;
 	tr.w = 2;
@@ -67,12 +67,12 @@ static void categoryselect_display(struct widget *w)
 	hr.x = w->rect.x;
 
 	if (cs == currently_selected_category)
-		draw_rectangle(&hr, 100, 220, 220, 350);
+		draw_rectangle(&hr, 90, 220, 220, 350);
 
-	display_text(cs->title, hr.x + 2, hr.y, &hr);
+	display_text(cs->title, hr.x + 2, hr.y, &hr, 1.0);
 	tr.x = hr.x + tab_width - 2;
 	draw_rectangle(&tr, 0, 0, 0, 136);
-	SetCurrentFont(PreviousFont);
+	set_current_font(PreviousFont);
 }
 
 struct widget_lvledit_categoryselect *get_current_object_type()
@@ -88,11 +88,11 @@ void widget_lvledit_categoryselect_activate(struct widget_lvledit_categoryselect
 struct widget *widget_lvledit_categoryselector_create(int x, char *text, enum lvledit_object_type type, int *olist)
 {
 	struct widget *a = MyMalloc(sizeof(struct widget));
+	widget_init(a);
 	a->type = WIDGET_CATEGORY_SELECTOR;
-	widget_set_rect(a, x * 80, 73, 80, 17);
+	widget_set_rect(a, x * 70, 73, 70, 17);
 	a->display = categoryselect_display;
 	a->handle_event = categoryselect_handle_event;
-	a->enabled = 1;
 
 	struct widget_lvledit_categoryselect *cs = MyMalloc(sizeof(struct widget_lvledit_categoryselect));
 	cs->type = type;
