@@ -38,11 +38,11 @@
  * current level.
 */
 
-static float minimap_scale = 175.0;
+static float minimap_scale = 240.0;
 
 // The center position of the minimap on the screen
 #define MINIMAP_CENTER_X (GameConfig.screen_width - (WIDGET_MINIMAP_WIDTH / 2))
-#define MINIMAP_CENTER_Y (GameConfig.screen_height - (WIDGET_MINIMAP_HEIGHT -70))
+#define MINIMAP_CENTER_Y (GameConfig.screen_height - (WIDGET_MINIMAP_HEIGHT / 2))
 	
 /**
  * Translate coordinates screen space (relative to the minimap) to map space.
@@ -76,7 +76,7 @@ static void draw_line_at_minimap_position(float x1, float y1, float x2, float y2
 	int r1, c1, r2, c2;
 	minimap_to_screen(x1, y1, &r1, &c1);
 	minimap_to_screen(x2, y2, &r2, &c2);
-	draw_line(r1, c1, r2, c2, SDL_MapRGB(Screen->format, 0xFF, 0xFF, 0xFF), 1);
+	draw_line(r1, c1, r2, c2, 0xFF, 0xFF, 0xFF, 1);
 }
 
 static int minimap_handle_event(struct widget *w, SDL_Event *event)
@@ -137,9 +137,9 @@ static void minimap_display(struct widget *w)
 			SDL_Rect tr;
 			tr.w = 90.0;
 			tr.h = 90.0;
-			tr.x = r - TextWidth(text) / 2;
-			tr.y = c - FontHeight(GetCurrentFont()) / 2;
-			display_text(text, tr.x, tr.y, &tr);
+			tr.x = r - text_width(get_current_font(), text) / 2;
+			tr.y = c - get_font_height(get_current_font()) / 2;
+			display_text(text, tr.x, tr.y, &tr, 1.0);
 		}
 	}
 
@@ -153,11 +153,11 @@ static void minimap_display(struct widget *w)
 struct widget *widget_lvledit_minimap_create()
 {
 	struct widget *a = MyMalloc(sizeof(struct widget));
+	widget_init(a);
 	a->type = WIDGET_MINIMAP;
 	widget_set_rect(a, GameConfig.screen_width - WIDGET_MINIMAP_WIDTH, GameConfig.screen_height - WIDGET_MINIMAP_HEIGHT, WIDGET_MINIMAP_WIDTH, WIDGET_MINIMAP_HEIGHT); 
 	a->display = minimap_display;
 	a->handle_event = minimap_handle_event;
-	a->enabled = 1;
 
 	struct widget_lvledit_minimap *n = MyMalloc(sizeof(struct widget_lvledit_minimap));
 	a->ext = n;
