@@ -37,24 +37,30 @@
 static struct widget_lvledit_categoryselect *previous_category = NULL;
 static int num_blocks_per_line = 0;
 
-static int display_info = 0;
+static int display_info_idx = 0;
 
-static void toolbar_mousepress(struct widget *vt, SDL_Event *event)
+static int toolbar_mousepress(struct widget *vt, SDL_Event *event, int select)
 {
 	struct widget_lvledit_categoryselect *cs = get_current_object_type();
 	int i;
 	int x;
+	int idx = -1;
 
 	for (i = 0; i < num_blocks_per_line; i++) {
 		x = INITIAL_BLOCK_WIDTH / 2 + INITIAL_BLOCK_WIDTH * i;
 
 		if (event->button.x > x && event->button.x < x + INITIAL_BLOCK_WIDTH)
-			cs->selected_tile_nb = cs->toolbar_first_block + i;
+			idx = cs->toolbar_first_block + i;
 	}
 
 	for (i = 0; cs->indices[i] != -1; i++) ;
-	if (cs->selected_tile_nb >= i - 1)
-		cs->selected_tile_nb = i - 1;
+	if (idx >= i - 1)
+		idx = i - 1;
+
+	if (select && idx != -1)
+		cs->selected_tile_nb = idx;
+
+	return idx;
 }
 
 static int toolbar_handle_event(struct widget *w, SDL_Event *event)
@@ -63,9 +69,11 @@ static int toolbar_handle_event(struct widget *w, SDL_Event *event)
 		case SDL_MOUSEBUTTONDOWN:
 			switch (event->button.button) {
 				case MOUSE_BUTTON_3:
-					display_info = 1;
+					display_info_idx = toolbar_mousepress(w, event, FALSE);
+					break;
+
 				case MOUSE_BUTTON_1:
-					toolbar_mousepress(w, event);
+					toolbar_mousepress(w, event, TRUE);
 					break;	
 
 				case SDL_BUTTON_WHEELUP:
@@ -79,17 +87,19 @@ static int toolbar_handle_event(struct widget *w, SDL_Event *event)
 				default:
 					return 0;
 			}
+			return 1;
+
 		case SDL_MOUSEMOTION:
 			return 1;
 
 		case SDL_MOUSEBUTTONUP:
 			if (event->button.button == 3)
-				display_info = 0;
+				display_info_idx = -1;
 			return 1;
 
 		case SDL_USEREVENT:
 			if (event->user.code == EVENT_MOUSE_LEAVE)
-				display_info = 0;
+				display_info_idx = -1;
 			break;
 
 		default:
@@ -102,8 +112,9 @@ static void print_obstacle_info(char *str, int obs_idx)
 {
 	int flags = get_obstacle_spec(obs_idx)->flags;
 
-	sprintf(str, "Obs. number %d, %s\n", obs_idx, ((char **)get_obstacle_spec(obs_idx)->filenames.arr)[0]);
-
+	// ;TRANSLATORS: "Obs." is for "obstacle" please keep it abbreviated in your language respectively
+	sprintf(str, _("Obs. number %d, %s\n"), obs_idx, ((char **)get_obstacle_spec(obs_idx)->filenames.arr)[0]);
+	// these refer to the internal flags, so let's not translate them for now...
 	if (flags & IS_HORIZONTAL)
 		strcat(str, "- IS_HORIZONTAL\n");
 	if (flags & IS_VERTICAL)
@@ -120,52 +131,52 @@ static void print_obstacle_info(char *str, int obs_idx)
 
 static void print_enemy_info(char *str, int en_idx)
 {
-	sprintf(str, 	"%s\n\
+	sprintf(str, 	_("%s\n\
 			class: %d, xp_reward: %hd\n\
 			heal: %.2f /s, max_energy: %.2f, max_speed: %.2f\n\
-			aggression_dist: %.2f, eyeing_tux_for: %.2f s",
-				Druidmap[en_idx].druidname,
-				Druidmap[en_idx].class, Druidmap[en_idx].experience_reward,
-				Druidmap[en_idx].healing_friendly, Druidmap[en_idx].maxenergy, Druidmap[en_idx].maxspeed,
-				Druidmap[en_idx].aggression_distance, Druidmap[en_idx].time_spent_eyeing_tux);
+			aggression_dist: %.2f, eyeing_tux_for: %.2f s"),
+				Droidmap[en_idx].droidname,
+				Droidmap[en_idx].class, Droidmap[en_idx].experience_reward,
+				Droidmap[en_idx].healing_friendly, Droidmap[en_idx].maxenergy, Droidmap[en_idx].maxspeed,
+				Droidmap[en_idx].aggression_distance, Droidmap[en_idx].time_spent_eyeing_tux);
 }
 
 static void print_item_info(char *str, int item_idx)
 {
 	struct itemspec *item = &ItemMap[item_idx];
-	if (item->item_gun_ammo_clip_size) { //Mostly Guns
-		sprintf(str, 	"%s (%d/%s)\n\
-			Damage: %d-%d, Recharge: %.2f, Reload: %.2f,\n\
-			STR: >%d\n%s",
-				item->item_name, item->item_gun_ammo_clip_size, ammo_desc_for_weapon(item_idx),
-				item->base_item_gun_damage, item->base_item_gun_damage + item->item_gun_damage_modifier,
-				item->item_gun_recharging_time, item->item_gun_reloading_time,
+	if (item->weapon_ammo_type) { // Weapons with ammunition: mostly guns, or melee weapons needing a power pack
+		sprintf(str, 	_("%s (%s)\n\
+			Damage: %d-%d, Attack Time: %.2fs, Reload Time: %.2fs, Ammo: (%d/%s)\n\
+			STR: >%d\n%s"),
+				item_specs_get_name(item_idx), item->id,
+				item->weapon_base_damage, item->weapon_base_damage + item->weapon_damage_modifier,
+				item->weapon_attack_time, item->weapon_reloading_time,
+				item->weapon_ammo_clip_size, item->weapon_ammo_type,
 				item->item_require_strength,
-				item->item_gun_requires_both_hands ? "Requires two hands\n" : "" );
+				item->weapon_needs_two_hands ? _("Requires two hands\n") : "" );
 
-	} else if (item->item_can_be_installed_in_weapon_slot) { //Most Melee Weapons
-		sprintf(str, 	"%s\n\
-			Damage: %d-%d, Recharge: %.2f,\n\
-			STR: >%d, DEX: >%d, COOL: >%d\n%s",
-				item->item_name,
-				item->base_item_gun_damage, item->base_item_gun_damage + item->item_gun_damage_modifier,
-				item->item_gun_recharging_time,
+	} else if (item->slot == WEAPON_SLOT) { // Other weapons
+		sprintf(str, 	_("%s (%s)\n\
+			Damage: %d-%d, Attack Time: %.2fs,\n\
+			STR: >%d, DEX: >%d, COOL: >%d\n%s"),
+				item_specs_get_name(item_idx), item->id,
+				item->weapon_base_damage, item->weapon_base_damage + item->weapon_damage_modifier,
+				item->weapon_attack_time,
 				item->item_require_strength, item->item_require_dexterity, item->item_require_cooling,
-				item->item_gun_requires_both_hands ? "Requires two hands\n" : "" );
+				item->weapon_needs_two_hands ? _("Requires two hands\n") : "" );
 
-	} else if (item->item_can_be_installed_in_drive_slot || item->item_can_be_installed_in_armour_slot || 
-				item->item_can_be_installed_in_shield_slot || item->item_can_be_installed_in_special_slot) {
-		sprintf(str, 	"%s\n\
+	} else if (item->slot & (SHIELD_SLOT | HELM_SLOT | ARMOR_SLOT | BOOT_SLOT)) {
+		sprintf(str, 	_("%s (%s)\n\
 			Armor: %d-%d, Durability:  %d-%d,\n\
-			STR: >%d, DEX: >%d, COOL: >%d\n",
-				item->item_name,
+			STR: >%d, DEX: >%d, COOL: >%d\n"),
+				item_specs_get_name(item_idx), item->id,
 				item->base_armor_class, item->base_armor_class + item->armor_class_modifier,
 				item->base_item_durability, item->base_item_durability + item->item_durability_modifier,
 				item->item_require_strength, item->item_require_dexterity, item->item_require_cooling);
 
 	} else {
-		sprintf(str, 	"%s\n%s",
-			item->item_name, item->item_description);
+		sprintf(str, "%s (%s)\n%s",
+			item_specs_get_name(item_idx), item->id, item->item_description);
 	}
 }
 
@@ -175,18 +186,18 @@ static void leveleditor_print_object_info(enum lvledit_object_type type, int *ar
 	case OBJECT_FLOOR:
 			if (array[idx] >= MAX_UNDERLAY_FLOOR_TILES) {
 				int index = array[idx] - MAX_UNDERLAY_FLOOR_TILES;
-				char **filename = dynarray_member(&overlay_floor_tile_filenames, index, sizeof(char *));
-				sprintf(str, "Overlay floor tile %d, filename %s\n", index, *filename); 
+				struct floor_tile_spec *floor_tile = dynarray_member(&overlay_floor_tiles, index, sizeof(struct floor_tile_spec));
+				sprintf(str, _("Overlay floor tile number %d, filename %s\n"), index, ((char **)floor_tile->filenames.arr)[0]);
 			} else {
-				char **filename = dynarray_member(&underlay_floor_tile_filenames, array[idx], sizeof(char *));
-				sprintf(str, "Underlay floor tile number %d, filename %s\n", array[idx], *filename);
+				struct floor_tile_spec *floor_tile = dynarray_member(&underlay_floor_tiles, array[idx], sizeof(struct floor_tile_spec));
+				sprintf(str, _("Underlay floor tile number %d, filename %s\n"), array[idx], ((char **)floor_tile->filenames.arr)[0]);
 			}
 			break;
 	case OBJECT_OBSTACLE:
 			print_obstacle_info(str, array[idx]);
 			break;
 	case OBJECT_WAYPOINT:
-			sprintf(str, "Waypt %s connection, %s for random spawn\n", "two way", array[idx] ? "NOK" : "OK");
+			sprintf(str, _("Waypoint %s connection, %sused for random spawn\n"), _("two way"), array[idx] ? _("not ") : "");
 			break;
 	case OBJECT_ITEM:
 			print_item_info(str, array[idx]);
@@ -194,6 +205,9 @@ static void leveleditor_print_object_info(enum lvledit_object_type type, int *ar
 	case OBJECT_ENEMY:
 			print_enemy_info(str, array[idx]);
 			break;
+	case OBJECT_MAP_LABEL:
+			sprintf(str, _("There currently is only one type of map label"));
+			break;
 	default:
 			*str = 0;
 	}
@@ -216,7 +230,7 @@ static struct image *leveleditor_get_object_image(enum lvledit_object_type type,
 	case OBJECT_ENEMY:
 			return get_droid_portrait_image(array[idx]);
 	default:
-			ErrorMessage(__FUNCTION__, "Abstract object type %d for leveleditor not supported.\n", PLEASE_INFORM, IS_FATAL, type);
+			error_message(__FUNCTION__, "Abstract object type %d for leveleditor not supported.", PLEASE_INFORM | IS_FATAL, type);
 			break;
 	}
 
@@ -277,9 +291,9 @@ static void toolbar_display(struct widget *vt)
 
 		if (cindex == cs->selected_tile_nb) {
 			HighlightRectangle(Screen, TargetRectangle);
-			if (display_info) {
+			if (display_info_idx != -1) {
 				// Display information about the currently selected object
-				leveleditor_print_object_info(cs->type, cs->indices, cindex, VanishingMessage);
+				leveleditor_print_object_info(cs->type, cs->indices, display_info_idx, VanishingMessage);
 				VanishingMessageEndDate = SDL_GetTicks() + 100;
 			}
 		}
@@ -291,11 +305,11 @@ static void toolbar_display(struct widget *vt)
 struct widget *widget_lvledit_toolbar_create()
 {
 	struct widget *a = MyMalloc(sizeof(struct widget));
+	widget_init(a);
 	a->type = WIDGET_TOOLBAR;
 	widget_set_rect(a, 0, 0, GameConfig.screen_width, 73);
 	a->display = toolbar_display;
 	a->handle_event = toolbar_handle_event;
-	a->enabled = 1;
 
 	struct widget_lvledit_toolbar *t = MyMalloc(sizeof(struct widget_lvledit_toolbar));
 	a->ext = t;
