@@ -51,45 +51,46 @@ typedef struct {
 } object_category;
 
 static object_category obstacle_category_list[] = {
-	{ "WALL", &wall_tiles_list },
-	{ "FURNITURE", &furniture_tiles_list },
-	{ "MACHINERY", &machinery_tiles_list },
-	{ "CONTAINER", &container_tiles_list },
-	{ "PLANT", &plant_tiles_list },
-	{ "OTHER", &misc_tiles_list },
-	{ "ALL", &all_obstacles_list }
+	{ N_("wall"), &wall_tiles_list },
+	{ N_("furniture"), &furniture_tiles_list },
+	{ N_("machinery"), &machinery_tiles_list },
+	{ N_("container"), &container_tiles_list },
+	{ N_("nature"), &nature_tiles_list },
+	{ N_("other"), &misc_tiles_list },
+	{ N_("all"), &all_obstacles_list }
 };
 
 static object_category floor_category_list[] = {
-	{ "SIDEWALK", &sidewalk_floor_list },
-	{ "WATER", &water_floor_list },
-	{ "GRASS", &grass_floor_list },
-	{ "SQUARE", &square_floor_list },
-	{ "OTHER", &other_floor_list },
-	{ "ALL", &floor_tiles_list }
+	{ N_("sidewalk"), &sidewalk_floor_list },
+	{ N_("water"), &water_floor_list },
+	{ N_("grass"), &grass_floor_list },
+	{ N_("square"), &square_floor_list },
+	{ N_("sand"), &sand_floor_list },
+	{ N_("other"), &other_floor_list },
+	{ N_("all"), &floor_tiles_list }
 };
 
 static object_category item_category_list[] = {
-	{ "MELEE", &melee_items_list },
-	{ "GUN", &gun_items_list },
-	{ "DEFENSIVE", &defense_items_list },
-	{ "USEABLE", &spell_items_list },
-	{ "OTHER", &other_items_list },
-	{ "ALL", &all_items_list }
+	{ N_("melee"), &melee_items_list },
+	{ N_("gun"), &gun_items_list },
+	{ N_("defensive"), &defense_items_list },
+	{ N_("usable"), &spell_items_list },
+	{ N_("other"), &other_items_list },
+	{ N_("all"), &all_items_list }
 };
 
 static object_category waypoint_category_list[] = {
-	{ "ALL", &waypoint_list }
+	{ N_("all"), &waypoint_list }
 };
 
 static object_category map_label_category_list[] = {
-	{ "ALL", &map_label_list }
+	{ N_("all"), &map_label_list }
 };
 
 static object_category enemy_category_list[] = {
-	{ "DROIDS", &droid_enemies_list },
-	{ "HUMANS", &human_enemies_list },
-	{ "ALL", &all_enemies_list },
+	{ N_("droids"), &droid_enemies_list },
+	{ N_("humans"), &human_enemies_list },
+	{ N_("all"), &all_enemies_list },
 };
 
 static struct {
@@ -102,10 +103,13 @@ static struct {
 	{ OBJECT_ITEM, item_category_list, sizeof(item_category_list) / sizeof(item_category_list[0]) },
 	{ OBJECT_ENEMY, enemy_category_list, sizeof(enemy_category_list) / sizeof(enemy_category_list[0]) },
 	{ OBJECT_WAYPOINT, waypoint_category_list, sizeof(waypoint_category_list) / sizeof(waypoint_category_list[0]) },
-	{ OBJECT_MAP_LABEL, map_label_category_list, sizeof(map_label_category_list) / sizeof(waypoint_category_list[0]) },
+	{ OBJECT_MAP_LABEL, map_label_category_list, sizeof(map_label_category_list) / sizeof(map_label_category_list[0]) },
 };
 
+//
 // Level editor buttons' primary actions
+//
+
 static void undo_button_click(struct widget_button *wb)
 {
 	level_editor_action_undo();
@@ -179,6 +183,11 @@ static void toggle_obstacles_button_click(struct widget_button *wb)
 	GameConfig.omit_obstacles_in_level_editor = !GameConfig.omit_obstacles_in_level_editor;
 }
 
+static void toggle_map_labels_button_click(struct widget_button *wb)
+{
+	GameConfig.omit_map_labels_in_level_editor = !GameConfig.omit_map_labels_in_level_editor;
+}
+
 static void zoom_in_button_click(struct widget_button *wb)
 {
 	GameConfig.zoom_is_on = !GameConfig.zoom_is_on;
@@ -250,121 +259,10 @@ static void zoom_in_button_right_click(struct widget_button *wb)
 	VanishingMessageEndDate = SDL_GetTicks() + 1000;
 }
 
-// Level editor buttons' update callbacks
-static void toggle_enemies_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = GameConfig.omit_enemies_in_level_editor;
-}
-
-static void toggle_tooltips_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = !GameConfig.show_lvledit_tooltips;
-}
-
-static void toggle_collisions_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = !draw_collision_rectangles;
-}
-
-static void toggle_waypoints_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = !GameConfig.show_wp_connections;
-}
-
-static void zoom_in_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = !GameConfig.zoom_is_on;
-}
-
-static void toggle_obstacles_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = GameConfig.omit_obstacles_in_level_editor;
-}
-
-static void toggle_grid_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = GameConfig.show_grid;
-}
-
-static void next_object_button_update(struct widget *w)
-{
-	w->enabled = level_editor_can_cycle_marked_object();
-}
-
-static void delete_obstacle_button_update(struct widget *w)
-{
-	w->enabled = !selection_empty();
-}
-
-static void edit_chest_button_update(struct widget *w)
-{
-	obstacle *o = single_tile_selection(OBJECT_OBSTACLE);
-	if (o) {
-		switch (o->type) {
-			case ISO_H_CHEST_CLOSED:
-			case ISO_V_CHEST_CLOSED:
-			case ISO_E_CHEST2_CLOSED:
-			case ISO_S_CHEST2_CLOSED:
-			case ISO_N_CHEST2_CLOSED:
-			case ISO_W_CHEST2_CLOSED:
-				w->enabled = 1;
-				break;
-
-			default:
-				w->enabled = 0;
-		} 
-	}
-	else {
-		w->enabled = 0;
-	}
-}
-
-static void new_obstacle_label_button_update(struct widget *w)
-{
-	w->enabled = single_tile_selection(OBJECT_OBSTACLE) ? 1 : 0;
-}
-
-static void undo_button_update(struct widget *w)
-{
-	w->enabled = !list_empty(&to_undo);
-}
-
-static void redo_button_update(struct widget *w)
-{
-	w->enabled = !list_empty(&to_redo);
-}
-
-static void typeselect_obstacle_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = (selection_type() == OBJECT_OBSTACLE);
-}
-
-static void typeselect_floor_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = (selection_type() == OBJECT_FLOOR);
-}
-
-static void typeselect_item_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = (selection_type() == OBJECT_ITEM);
-}
-
-static void typeselect_enemy_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = (selection_type() == OBJECT_ENEMY);
-}
-
-static void typeselect_waypoint_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = (selection_type() == OBJECT_WAYPOINT);
-}
-
-static void typeselect_map_label_button_update(struct widget *w)
-{
-	WIDGET_BUTTON(w)->active = (selection_type() == OBJECT_MAP_LABEL);
-}
-
+//
 // Leveleditor floor layers button callbacks
+//
+
 static void floor_layers_button_click(struct widget_button *wb)
 {
 	GameConfig.show_all_floor_layers = !GameConfig.show_all_floor_layers;
@@ -378,15 +276,127 @@ static void floor_layers_button_right_click(struct widget_button *wb)
 	action_change_floor_layer(EditLevel(), next_layer);
 }
 
-static void floor_layers_button_update(struct widget *w)
+//
+// Widgets update callbacks
+//
+
+static void _enable_if_undo_list_not_empty(struct widget *w)
+{
+	w->enabled = !list_empty(&to_undo);
+}
+
+static void _enable_if_redo_list_not_empty(struct widget *w)
+{
+	w->enabled = !list_empty(&to_redo);
+}
+
+static void _enable_is_selection_not_empty(struct widget *w)
+{
+	w->enabled = !selection_empty();
+}
+
+static void _enable_if_can_cycle_marked_object(struct widget *w)
+{
+	w->enabled = level_editor_can_cycle_marked_object();
+}
+
+static void _activate_if_show_wp_connections(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = !GameConfig.show_wp_connections;
+}
+
+static void _enable_if_single_obstacle_selected(struct widget *w)
+{
+	w->enabled = single_tile_selection(OBJECT_OBSTACLE) ? 1 : 0;
+}
+
+static void _enable_if_single_chest_selected(struct widget *w)
+{
+	w->enabled = 0;
+
+	obstacle *o = single_tile_selection(OBJECT_OBSTACLE);
+	if (o) {
+		struct obstacle_spec *obs_spec = get_obstacle_spec(o->type);
+		if (obs_spec->action && !strncmp(obs_spec->action, "chest", 5))
+			w->enabled = 1;
+	}
+}
+
+static void _activate_if_enemies_not_shown(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = GameConfig.omit_enemies_in_level_editor;
+}
+
+static void _activate_if_tooltips_not_shown(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = !GameConfig.show_lvledit_tooltips;
+}
+
+static void _activate_if_collrects_not_shown(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = !draw_collision_rectangles;
+}
+
+static void _activate_if_grid_shown(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = GameConfig.show_grid;
+}
+
+static void _activate_if_obstacles_not_shown(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = GameConfig.omit_obstacles_in_level_editor;
+}
+
+static void _activate_if_map_labels_not_shown(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = GameConfig.omit_map_labels_in_level_editor;
+}
+
+static void _activate_if_zoom_off(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = !GameConfig.zoom_is_on;
+}
+
+static void _activate_if_all_floors_not_shown(struct widget *w)
 {
 	w->enabled = selection_type() == OBJECT_FLOOR && EditLevel()->floor_layers > 1;
 	WIDGET_BUTTON(w)->active = !GameConfig.show_all_floor_layers;
 }
 
+static void _activate_if_selection_of_obstacles(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = (selection_type() == OBJECT_OBSTACLE);
+}
+
+static void _activate_if_selection_of_tiles(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = (selection_type() == OBJECT_FLOOR);
+}
+
+static void _activate_if_selection_of_items(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = (selection_type() == OBJECT_ITEM);
+}
+
+static void _activate_if_selection_of_enemies(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = (selection_type() == OBJECT_ENEMY);
+}
+
+static void _activate_if_selection_of_waypoints(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = (selection_type() == OBJECT_WAYPOINT);
+}
+
+static void _activate_if_selection_of_map_labels(struct widget *w)
+{
+	WIDGET_BUTTON(w)->active = (selection_type() == OBJECT_MAP_LABEL);
+}
+
 /**
  * This function returns the editor top level widget and creates it if necessary.
  */
+
 struct widget_group *get_lvledit_ui()
 {
 	if (level_editor_widget_group)
@@ -414,77 +424,193 @@ struct widget_group *get_lvledit_ui()
 		char *text;
 		string tooltip_text;
 		int  number_of_toggle_states;
-		void (*activate_button) (struct widget_button *);
-		void (*activate_button_secondary) (struct widget_button *);
-		void (*update) (struct widget *);
+		void (*activate_button)(struct widget_button *);
+		void (*activate_button_secondary)(struct widget_button *);
+		void (*update)(struct widget *);
 	} b[] = {
 		{LEVEL_EDITOR_UNDO_BUTTON, NULL,
 			_("Undo\n\nUse this button to undo your last actions."),
-			1, undo_button_click, NULL, undo_button_update},
+			1,
+			undo_button_click,
+			NULL,
+			_enable_if_undo_list_not_empty
+		},
 		{LEVEL_EDITOR_REDO_BUTTON, NULL,
 			_("Redo\n\nUse this button to redo an action."),
-			1, redo_button_click, NULL, redo_button_update},
+			1,
+			redo_button_click,
+			NULL,
+			_enable_if_redo_list_not_empty
+		},
 		{LEVEL_EDITOR_SAVE_SHIP_BUTTON, NULL,
 			_("Save Map\n\nThis button will save your current map over the file '../map/levels.dat' from your current working directory."),
-			2, save_ship_button_click, NULL, NULL},
+			2,
+			save_ship_button_click,
+			NULL,
+			NULL
+		},
 		{LEVEL_EDITOR_BEAUTIFY_GRASS_BUTTON, NULL,
 			_("Beautify grass button\n\nUse this button to automatically 'beautify' rough edges of the grass-sand tiles. It will apply to the selected floor or, if not applicable, to the entire level.\n\nYou can also use Ctrl-b for this."),
-			1, beautify_grass_button_click, NULL, NULL},
+			1,
+			beautify_grass_button_click,
+			NULL,
+			NULL
+		},
 		{LEVEL_EDITOR_DELETE_OBSTACLE_BUTTON, NULL,
-			_("Delete selected obstacle\n\nUse this button to delete the currently marked obstacle.\n\nYou can also use Ctrl-X for this."),
-			1, delete_obstacle_button_click, NULL, delete_obstacle_button_update},
+			_("Delete selected objects\n\nUse this button to delete the currently marked objects.\n\nYou can also use Ctrl-X for this."),
+			1,
+			delete_obstacle_button_click,
+			NULL,
+			_enable_is_selection_not_empty
+		},
 		{LEVEL_EDITOR_NEXT_OBJECT_BUTTON, NULL,
 			_("Next object on currently selected tile\n\nUse this button to cycle the currently marked object on this tile.\n\nYou can also use the N key for this."),
-			1, next_object_button_click, NULL, next_object_button_update},
+			1,
+			next_object_button_click,
+			NULL,
+			_enable_if_can_cycle_marked_object
+		},
 		{LEVEL_EDITOR_TOGGLE_WAYPOINT_CONNECTIONS_BUTTON, NULL,
 			_("Toggle display waypoint connections\n\nUse this button to toggle between waypoint connections displayed on and off."),
-			2, toggle_waypoints_button_click, NULL, toggle_waypoints_button_update},
+			2,
+			toggle_waypoints_button_click,
+			NULL,
+			_activate_if_show_wp_connections
+		},
 		{LEVEL_EDITOR_NEW_OBSTACLE_LABEL_BUTTON, NULL,
 			_("New obstacle label\n\nUse this button to attach a label to the currently marked obstacle.  These obstacle labels can be used to define obstacles to be modified by events.\n"),
-			1, new_obstacle_label_button_click, NULL, new_obstacle_label_button_update},
+			1,
+			new_obstacle_label_button_click,
+			NULL,
+			_enable_if_single_obstacle_selected
+		},
 		{LEVEL_EDITOR_EDIT_CHEST_BUTTON, NULL,
 			_("Edit chests contents\n\nUse this button to change the contents of the chest. If it is empty the game may generate random items at run time."),
-			1, edit_chest_button_click, NULL, edit_chest_button_update},
+			1,
+			edit_chest_button_click,
+			NULL,
+			_enable_if_single_chest_selected
+		},
 		{LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON, NULL,
 			_("Toggle display enemies\n\nUse this button to toggle between enemies displayed in level editor or enemies hidden in level editor."),
-			2, toggle_enemies_button_click, NULL, toggle_enemies_button_update},
+			2,
+			toggle_enemies_button_click,
+			NULL,
+			_activate_if_enemies_not_shown
+		},
 		{LEVEL_EDITOR_TOGGLE_TOOLTIPS_BUTTON, NULL,
 			_("Toggle display tooltips\n\nUse this button to toggle these annoying help windows on and off."),
-			2, toggle_tooltips_button_click, NULL, toggle_tooltips_button_update},
+			2,
+			toggle_tooltips_button_click,
+			NULL,
+			_activate_if_tooltips_not_shown
+		},
 		{LEVEL_EDITOR_TOGGLE_COLLISION_RECTS_BUTTON, NULL,
 			_("Toggle display collision rectangles\n\nUse this button to toggle the visible collision rectangles on and off."),
-			2, toggle_collisions_button_click, NULL, toggle_collisions_button_update},
+			2,
+			toggle_collisions_button_click,
+			NULL,
+			_activate_if_collrects_not_shown
+		},
 		{LEVEL_EDITOR_TOGGLE_GRID_BUTTON_OFF, NULL,
 			_("Toggle grid mode.\n\nUse this button to toggle grid displaying on and off.\n\nUse right click to change the grid mode."),
-			2, toggle_grid_button_click, toggle_grid_button_right_click, toggle_grid_button_update},
+			2,
+			toggle_grid_button_click,
+			toggle_grid_button_right_click,
+			_activate_if_grid_shown
+		},
 		{LEVEL_EDITOR_TOGGLE_OBSTACLES_BUTTON, NULL,
 			_("Toggle display obstacles\n\nUse this button to toggle between obstacles displayed in level editor or obstacles hidden in level editor."),
-			2, toggle_obstacles_button_click, NULL, toggle_obstacles_button_update},
+			2,
+			toggle_obstacles_button_click,
+			NULL,
+			_activate_if_obstacles_not_shown
+		},
+		{LEVEL_EDITOR_TOGGLE_MAP_LABELS_BUTTON, NULL,
+			_("Toggle display map labels name\n\nUse this button to toggle between map labels name displayed in level editor or map labels name hidden in level editor."),
+			2,
+			toggle_map_labels_button_click,
+			NULL,
+			_activate_if_map_labels_not_shown
+		},
 		{LEVEL_EDITOR_ZOOM_IN_BUTTON, NULL,
 			_("Zoom in/out\n\nUse this button to zoom INTO or OUT of the level.\n\nUse right click to change the zoom ratio.\n"),
-			2, zoom_in_button_click, zoom_in_button_right_click, zoom_in_button_update},
+			2,
+			zoom_in_button_click,
+			zoom_in_button_right_click,
+			_activate_if_zoom_off
+		},
 		{LEVEL_EDITOR_ALL_FLOOR_LAYERS_BUTTON, NULL,
 			_("Toggle floor layers\n\nUse this button to toggle between all floor layers displayed or single floor layer displayed.\n\nUse right click to change the current floor layer.\n"),
-			2, floor_layers_button_click, floor_layers_button_right_click, floor_layers_button_update},
+			2,
+			floor_layers_button_click,
+			floor_layers_button_right_click,
+			_activate_if_all_floors_not_shown
+		},
 		{LEVEL_EDITOR_QUIT_BUTTON, NULL,
 			_("Test Map\n\nThis will save your map and reload it after you finish testing, avoiding saving an unclean world state."),
-			1, quit_button_click, NULL, NULL},
-		{RIGHT_LEVEL_EDITOR_BUTTON, NULL, NULL,
-			1, editor_right_button_click, NULL},
-		{LEFT_LEVEL_EDITOR_BUTTON, NULL, NULL,
-			1, editor_left_button_click, NULL, NULL},
-		{LEVEL_EDITOR_TYPESELECT_OBSTACLE_BUTTON, "OBSTACLE", NULL,
-			2, typeselect_obstacle_button_click, NULL, typeselect_obstacle_button_update},
-		{LEVEL_EDITOR_TYPESELECT_FLOOR_BUTTON, "FLOOR", NULL,
-			2, typeselect_floor_button_click, NULL, typeselect_floor_button_update},
-		{LEVEL_EDITOR_TYPESELECT_ITEM_BUTTON, "ITEM", NULL,
-			2, typeselect_item_button_click, NULL, typeselect_item_button_update},
-		{LEVEL_EDITOR_TYPESELECT_ENEMY_BUTTON, "ENEMY", NULL,
-			2, typeselect_enemy_button_click, NULL, typeselect_enemy_button_update},
-		{LEVEL_EDITOR_TYPESELECT_WAYPOINT_BUTTON, "WP", NULL,
-			2, typeselect_waypoint_button_click, NULL, typeselect_waypoint_button_update},
-		{LEVEL_EDITOR_TYPESELECT_MAP_LABEL_BUTTON, "LABEL", NULL,
-			2, typeselect_map_label_button_click, NULL, typeselect_map_label_button_update}
+			1,
+			quit_button_click,
+			NULL,
+			NULL
+		},
+		{RIGHT_LEVEL_EDITOR_BUTTON, NULL,
+			NULL,
+			1,
+			editor_right_button_click,
+			NULL,
+			NULL
+		},
+		{LEFT_LEVEL_EDITOR_BUTTON, NULL,
+			NULL,
+			1,
+			editor_left_button_click,
+			NULL,
+			NULL
+		},
+		{LEVEL_EDITOR_TYPESELECT_OBSTACLE_BUTTON, _("Obstacle"),
+			NULL,
+			2,
+			typeselect_obstacle_button_click,
+			NULL,
+			_activate_if_selection_of_obstacles
+		},
+		{LEVEL_EDITOR_TYPESELECT_FLOOR_BUTTON, _("Floor"),
+			NULL,
+			2,
+			typeselect_floor_button_click,
+			NULL,
+			_activate_if_selection_of_tiles
+		},
+		{LEVEL_EDITOR_TYPESELECT_ITEM_BUTTON, _("Item"),
+			NULL,
+			2,
+			typeselect_item_button_click,
+			NULL,
+			_activate_if_selection_of_items
+		},
+		{LEVEL_EDITOR_TYPESELECT_ENEMY_BUTTON, _("Enemy"),
+			NULL,
+			2,
+			typeselect_enemy_button_click,
+			NULL,
+			_activate_if_selection_of_enemies
+		},
+		//; TRANSLATORS: Waypoint
+		{LEVEL_EDITOR_TYPESELECT_WAYPOINT_BUTTON, _("Wayp."),
+			NULL,
+			2,
+			typeselect_waypoint_button_click,
+			NULL,
+			_activate_if_selection_of_waypoints
+		},
+		{LEVEL_EDITOR_TYPESELECT_MAP_LABEL_BUTTON, _("Label"),
+			NULL,
+			2,
+			typeselect_map_label_button_click,
+			NULL,
+			_activate_if_selection_of_map_labels
+		}
 	};
 
 	int i, j;
@@ -507,9 +633,9 @@ struct widget_group *get_lvledit_ui()
 		// Buttons use 3 images for each toggle state. Radio buttons have two toggle states.
 		for (j = 0; j < b[i].number_of_toggle_states; j++) {
 			// AllMousePressButtons have only two images: normal and pressed. Hovered state will be ignored.
-			button->image[j][0] = widget_load_image_resource(AllMousePressButtons[b[i].btn_index + j * 2].button_image_file_name, 0);		// Normal state
+			button->image[j][0] = widget_load_image_resource(AllMousePressButtons[b[i].btn_index + j * 2].button_image_file_name, NO_MOD);		// Normal state
 			button->image[j][1] = NULL;												// Hovered state
-			button->image[j][2] = widget_load_image_resource(AllMousePressButtons[b[i].btn_index + j * 2 + 1].button_image_file_name, 0);	// Pressed state
+			button->image[j][2] = widget_load_image_resource(AllMousePressButtons[b[i].btn_index + j * 2 + 1].button_image_file_name, NO_MOD);	// Pressed state
 		}
 		
 		// Set button size.
@@ -527,8 +653,8 @@ struct widget_group *get_lvledit_ui()
 	lvledit_build_tile_lists();
 
 	// Load categories for obstacles
-	char fpath[4096];
-	find_file("leveleditor_obstacle_categories.lua", MAP_DIR, fpath, 0);
+	char fpath[PATH_MAX];
+	find_file("lvledit_obstacle_categories.lua", MAP_DIR, fpath, PLEASE_INFORM | IS_FATAL);
 	run_lua_file(LUA_CONFIG, fpath);
 
 	// Build list of all obstacles
@@ -565,6 +691,20 @@ struct widget_group *get_lvledit_ui()
 	return level_editor_widget_group;
 }
 
+void free_lvledit_ui()
+{
+	if (level_editor_widget_group)
+	{
+		struct widget *w = WIDGET(level_editor_widget_group);
+		w->free(w);
+		free(level_editor_widget_group);
+	}
+
+	if (all_obstacles_list) {
+		free(all_obstacles_list);
+	}
+}
+
 struct widget *get_active_widget(int x, int y)
 {
 	struct widget *w;
@@ -619,6 +759,7 @@ void lvledit_categoryselect_switch(int direction)
 			break;
 		}
 	}
+	if (!categories || length == 0) return;
 
 	// Find the current category
 	for (i = 0; i < length; i++) {
@@ -630,7 +771,6 @@ void lvledit_categoryselect_switch(int direction)
 	int index = i + direction;
 	if (index < 0)
 		index = length - 1;
-	if (length)
-		index %= length;
+	index %= length;
 	widget_lvledit_categoryselect_activate(categories[index].cs);
 }
