@@ -39,9 +39,9 @@
 #include "widgets/widgets.h"
 
 #ifdef __OpenBSD__
-#include "ieeefp.h"
+#include <ieeefp.h>
 #else
-#include "fenv.h"
+#include <fenv.h>
 #endif
 
 float LastGotIntoBlastSound = 2;
@@ -55,7 +55,7 @@ void UpdateCountersForThisFrame();
  */
 void DoAllMovementAndAnimations(void)
 {
-	animate_obstacles();	// this is a pure client issue.  Not dependent upon the players.
+	animate_scenery();	// this is a pure client issue.  Not dependent upon the players.
 
 	animate_blasts();	// move blasts to the right current "phase" of the blast
 
@@ -123,12 +123,10 @@ void Game()
  * ----------------------------------------------------------------- */
 int main(int argc, char *argv[])
 {
+	prepare_execution(argc, argv);
 
-#if defined HAVE_UNISTD_H && defined HAVE_DIRNAME
-	// change working directory to the executable's directory
-	if (chdir(dirname(argv[0])))
-		fprintf(stderr, "Couldn't change working directory to %s.\n", dirname(argv[0]));
-#endif
+	init_data_dirs_path();
+	lang_init();
 
 	QuitProgram = FALSE;
 	draw_collision_rectangles = FALSE;
@@ -151,7 +149,7 @@ int main(int argc, char *argv[])
 	if (do_benchmark) {
 		/* Benchmark mode? Do not run the regular game from here. */
 		int failed = benchmark();
-		Terminate(failed ? EXIT_FAILURE : EXIT_SUCCESS, FALSE);
+		Terminate(failed ? EXIT_FAILURE : EXIT_SUCCESS);
 	}
 
 	int skip_menu = FALSE;
@@ -185,15 +183,15 @@ int main(int argc, char *argv[])
 			LevelEditor();
 			break;
 		case ROOT_IS_UNKNOWN:
-			ErrorMessage(__FUNCTION__, "Game root mode is unknown - was not properly set to GAME or LVLEDIT.\n", PLEASE_INFORM,
-				     IS_FATAL);
+			error_message(__FUNCTION__, "Game root mode is unknown - was not properly set to GAME or LVLEDIT.",
+					PLEASE_INFORM | IS_FATAL);
 			break;
 		}
 	}			// while !QuitProgram 
 
 	LightRadiusClean();
 
-	Terminate(EXIT_SUCCESS, TRUE);
+	Terminate(EXIT_SUCCESS);
 	return (0);
 };				// int main ( void )
 
@@ -287,14 +285,6 @@ void UpdateCountersForThisFrame()
 			timeout_from_item_drop = 0;
 	}
 
-	for (i = 0; i < MAXBULLETS; i++) {
-		if (AllBullets[i].time_to_hide_still > 0) {
-			AllBullets[i].time_to_hide_still -= latest_frame_time;
-			if (AllBullets[i].time_to_hide_still < 0)
-				AllBullets[i].time_to_hide_still = 0;
-		}
-	}
-
 	// Maybe some items are just thrown in the air and still in the air.
 	// We need to keep track of the time the item has spent in the air so far.
 	//
@@ -367,12 +357,13 @@ void UpdateCountersForThisFrame()
 		if (Me.energy - lose_life < 1 && Me.paralyze_duration == 0) {
 			DoSkill(get_program_index_with_name("Emergency shutdown"),
 				calculate_program_heat_cost(get_program_index_with_name("Emergency shutdown")));
+			// TRANSLATORS: Screen message, displayed below 'Automatically shutting down'
 			SetNewBigScreenMessage(_("due to near lethal overheating"));
 			SetNewBigScreenMessage(_("Automatically shutting down"));
 			lose_life = (Me.temperature - Me.max_temperature) * latest_frame_time / 10;
 		}
 		if (lose_life > 0) {
-			hit_tux(lose_life, -1);
+			hit_tux(lose_life);
 		}
 	}
 
@@ -425,10 +416,7 @@ void UpdateCountersForThisFrame()
 			// Now maybe it's time to respawn?  If we really have multiple
 			// players of course, this check would have to look a bit different...
 			//
-			if (Me.time_since_last_visit_or_respawn[i] > 600) {
-				/* Do not respawn the town. At all. */
-				if (!strcmp(curShip.AllLevels[i]->Levelname, "Town"))
-					continue;
+			if (Me.time_since_last_visit_or_respawn[i] > 600 && !(curShip.AllLevels[i]->flags & NO_RESPAWN)) {
 
 				DebugPrintf(-10, "\nNow respawning all bots on level : %d. ", i);
 				Me.time_since_last_visit_or_respawn[i] = 0;
