@@ -47,7 +47,7 @@ static void new_level(int w, int h)
 		}
 	}
 
-	rooms = malloc(100 * sizeof(struct roominfo));
+	rooms = MyMalloc(100 * sizeof(struct roominfo));
 	max_rooms = 100;
 	total_rooms = 0;
 }
@@ -355,29 +355,6 @@ static void place_doors()
 	}
 }
 
-void room_to_console(int w, int h) {
-	int x, y;
-	char c;
-	printf("\n");
-	for (x = 0; x < w + 2; x++)
-		putchar('#');
-	putchar('\n');
-	for (y = 0; y < h; y++) {
-		putchar('#');
-		for (x = 0; x < w; x++) {
-			switch(mapgen_get_tile(x, y)) {
-				case TILE_FLOOR: c = '.'; break;
-				default: c = ' ';
-			}
-			putchar(c);
-		}
-		putchar('#');
-		putchar('\n');
-	}
-	for (x = 0; x < w + 2; x++)
-		putchar('#');
-}
-
 // Turn the given room into corridor and return whether
 // the transformation was successful
 static int make_corridor(int room)
@@ -489,7 +466,7 @@ void mapgen_convert(struct dungeon_info *di, int w, int h, unsigned char *tiles)
 	qsort(idx, di->num_rooms, sizeof(int), cmp_room_surface);
 
 	i = 0;
-	while(tries && n) {
+	while(tries && n && (i < di->num_rooms)) {
 		if (idx[i] != di->enter && idx[i] != di->exit) {
 			if (make_corridor(idx[i]))
 				n--;
@@ -502,20 +479,20 @@ void mapgen_convert(struct dungeon_info *di, int w, int h, unsigned char *tiles)
 	place_doors();
 
 	qsort(idx, di->num_rooms, sizeof(int), cmp_room_surface);
-	di->sorted_square = idx;
-	mapgen_place_obstacles(di, w, h, tiles);
+
+	mapgen_place_obstacles(di, w, h, tiles, idx);
 }
 
 static void add_teleport(int telnum, int x, int y, int tpair)
 {
 	const int helpers[2][4] = {
-		{ ISO_REFRESH_1,  ISO_REFRESH_1,  ISO_REFRESH_1,  ISO_REFRESH_1 },
+		{ ISO_DROID_NEST_GREEN, ISO_DROID_NEST_GREEN, ISO_DROID_NEST_GREEN, ISO_DROID_NEST_GREEN },
 		{ ISO_ENHANCER_RU, ISO_ENHANCER_LU, ISO_ENHANCER_RD, ISO_ENHANCER_LD }
 	};
 
 	char *warp, *fromwarp;
 	char tmp[500];
-	int obstacle, helper;
+	int obs_type, helper;
 
 	sprintf(tmp, "%dtoX%d", target_level->levelnum, telnum);
 	warp = strdup(tmp);
@@ -530,11 +507,11 @@ static void add_teleport(int telnum, int x, int y, int tpair)
 	// That 0.5 translation is added to the other obstacles around the labels
 	// to have a coherent position.
 
-	obstacle = telnum ? teleport_pairs[tpair].exit : teleport_pairs[tpair].enter;
-	mapgen_add_obstacle(x + 0.5, y + 0.5, obstacle);
+	obs_type = telnum ? teleport_pairs[tpair].exit : teleport_pairs[tpair].enter;
+	mapgen_add_obstacle(x + 0.5, y + 0.5, obs_type);
 
 	// Decorate room with teleport if the obstacle is the cloud
-	if (obstacle == ISO_TELEPORTER_1) {
+	if (obs_type == ISO_TELEPORTER_1) {
 		helper = MyRandom(1);
 		mapgen_add_obstacle(x + 1 + 0.5, y - 1 + 0.5, helpers[helper][0]);
 		mapgen_add_obstacle(x - 1 + 0.5, y - 1 + 0.5, helpers[helper][1]);
@@ -590,8 +567,14 @@ int mapgen_add_room(int x, int y, int w, int h)
 	int newid = total_rooms;
 
 	if (total_rooms == max_rooms) {
-		max_rooms++;
-		rooms = realloc(rooms, max_rooms * sizeof(struct roominfo));
+		// Add 10 more slots
+		max_rooms += 10;
+		struct roominfo *buffer = realloc(rooms, max_rooms * sizeof(struct roominfo));
+		if (!buffer) {
+			error_message(__FUNCTION__, "Not enough memory to reallocate the 'rooms' datastruct (requested size: " SIZE_T_F ").", IS_FATAL, max_rooms * sizeof(struct roominfo));
+			return -1;
+		}
+		rooms = buffer;
 	}
 
 	total_rooms++;
@@ -603,7 +586,7 @@ int mapgen_add_room(int x, int y, int w, int h)
 	rooms[newid].h = h;
 	rooms[newid].num_neighbors = 0;
 	rooms[newid].max_neighbors = 8;
-	rooms[newid].neighbors = malloc(rooms[newid].max_neighbors * sizeof(int));
+	rooms[newid].neighbors = MyMalloc(rooms[newid].max_neighbors * sizeof(int));
 	rooms[newid].num_doors = 0;
 
 	return newid;
@@ -792,7 +775,7 @@ void mapgen_add_door(int x, int y, int from, int to)
 {
 	int num = rooms[from].num_doors;
 	if (num == MAX_DOORS) {
-			ErrorMessage(__FUNCTION__, "Maximal number of doors for a room exceeded\n", PLEASE_INFORM, IS_FATAL);
+			error_message(__FUNCTION__, "Maximal number of doors for a room exceeded", PLEASE_INFORM | IS_FATAL);
 			return;
 	}
 	rooms[from].doors[num].x = x;
@@ -802,20 +785,6 @@ void mapgen_add_door(int x, int y, int from, int to)
 	rooms[from].num_doors++;
 }
 
-void mapgen_delete_door(int from, int to)
-{
-	struct roominfo *ri = &rooms[from];
-	int i;
-	for (i = 0; i < ri->num_doors; i++)
-		if (ri->doors[i].room == to)
-			break;
-	if (i < ri->num_doors) {
-		ri->num_doors--;
-		for (; i < ri->num_doors; i++)
-			ri->doors[i] = ri->doors[i + 1];
-	}
-}
-
 static void add_neighbor(struct roominfo *r, int neigh)
 {
 	int newid = r->num_neighbors;
@@ -857,7 +826,7 @@ void MakeConnect(int x, int y, enum connection_type type)
 			wp_x = x - 1;
 			break;
 		default:
-			ErrorMessage(__FUNCTION__, "Unknown connection type %d\n", PLEASE_INFORM, IS_FATAL, type);
+			error_message(__FUNCTION__, "Unknown connection type %d", PLEASE_INFORM | IS_FATAL, type);
 			break;
 
 	} 
