@@ -85,30 +85,30 @@ static int set_generic_wall(int x, int y, int wall, int theme)
 	// A value '1' of 'processed' means that data was processed, otherwise
 	// the caller should process them itself.
 	int processed = 1;
-	int obstacle;
+	int obs_type;
 	int room = mapgen_get_room(x, y);
 	int period = room != -1 ? rooms[room].period : 0;
 	if (wall & WALL_PART) {
 		if (wall & WALL_N) {
-			obstacle = theme_data[theme].wall_n;
+			obs_type = theme_data[theme].wall_n;
 			if (period && !(x % period))
-				obstacle = theme_data[theme].window_wall_h;
-			mapgen_add_obstacle(x + 0.5, y, obstacle);
+				obs_type = theme_data[theme].window_wall_h;
+			mapgen_add_obstacle(x + 0.5, y, obs_type);
 		} else if (wall & WALL_S) {
-			obstacle = theme_data[theme].wall_n;
+			obs_type = theme_data[theme].wall_n;
 			if (period && !(x % period))
-				obstacle = theme_data[theme].window_wall_h;
-			mapgen_add_obstacle(x + 0.5, y + 1, obstacle);
+				obs_type = theme_data[theme].window_wall_h;
+			mapgen_add_obstacle(x + 0.5, y + 1, obs_type);
 		} else if (wall & WALL_W) {
-			obstacle = theme_data[theme].wall_w;
+			obs_type = theme_data[theme].wall_w;
 			if (period && !(y % period))
-				obstacle = theme_data[theme].window_wall_v;
-			mapgen_add_obstacle(x, y + 0.5, obstacle);
+				obs_type = theme_data[theme].window_wall_v;
+			mapgen_add_obstacle(x, y + 0.5, obs_type);
 		} else if (wall & WALL_E) {
-			obstacle = theme_data[theme].wall_w;
+			obs_type = theme_data[theme].wall_w;
 			if (period && !(y % period))
-				obstacle = theme_data[theme].window_wall_v;
-			mapgen_add_obstacle(x + 1, y + 0.5, obstacle);
+				obs_type = theme_data[theme].window_wall_v;
+			mapgen_add_obstacle(x + 1, y + 0.5, obs_type);
 		} else {
 			processed = 0;
 		}
@@ -525,7 +525,7 @@ static int set_living_theme_recursive(int room, int depth, int *vis)
 	return count;
 }
 
-void mapgen_place_obstacles(struct dungeon_info *di, int w, int h, unsigned char *tiles)
+void mapgen_place_obstacles(struct dungeon_info *di, int w, int h, unsigned char *tiles, int *sorted_square)
 {
 #define MIN_LIVING_ROOMS	6
 
@@ -553,9 +553,9 @@ void mapgen_place_obstacles(struct dungeon_info *di, int w, int h, unsigned char
 	vis[di->exit] = SPECIAL_ROOM;
 	// Place main room to the biggest among visited
 	for (i = 0; i < total_rooms; i++)
-		if (vis[di->sorted_square[i]] == 1) {
-			place_main_room(di->sorted_square[i]);
-			vis[di->sorted_square[i]] = MAIN_ROOM;
+		if (vis[sorted_square[i]] == 1) {
+			place_main_room(sorted_square[i]);
+			vis[sorted_square[i]] = MAIN_ROOM;
 			break;
 		}
 
@@ -582,6 +582,7 @@ void mapgen_place_obstacles(struct dungeon_info *di, int w, int h, unsigned char
 					if (mapgen_get_room(x, y - 1) != room && mapgen_get_tile(x, y - 1) == TILE_FLOOR)
 						wall |= WALL_N;
 					themes[rooms[room2].theme](x, y, wall);
+					// no break (really ?)
 				case TILE_FLOOR:
 					wall = 0;
 					if (tiles[y * w + x - 1] == TILE_WALL)
