@@ -43,17 +43,16 @@ void mission_diary_add(const char *mis_name, const char *diarytext)
 	int mis_num = GetMissionIndexByName(mis_name);
 	int idx = 0;
 
-	while (Me.AllMissions[mis_num].mission_diary_texts[idx] && idx < MAX_MISSION_DESCRIPTION_TEXTS)
+	while (idx < MAX_MISSION_DESCRIPTION_TEXTS && Me.AllMissions[mis_num].mission_diary_texts[idx])
 		idx++;
 
 	if (idx >= MAX_MISSION_DESCRIPTION_TEXTS) {
-		ErrorMessage(__FUNCTION__,
-			     "There is no more room to append mission diary text \"%s\" to mission name \"%s\". Doing nothing.\n",
-			     PLEASE_INFORM, IS_WARNING_ONLY, diarytext, mis_name);
+		error_message(__FUNCTION__,
+			     "There is no more room to append mission diary text \"%s\" to mission name \"%s\". Doing nothing.",
+			     PLEASE_INFORM, diarytext, mis_name);
 		return;
 	}
 
-	Mission_Status_Change_Sound();
 
 	Me.AllMissions[mis_num].mission_description_time[idx] = Me.current_game_date;
 
@@ -69,7 +68,6 @@ void mission_diary_add(const char *mis_name, const char *diarytext)
  ----------------------------------------------------------------------*/
 void CheckIfMissionIsComplete(void)
 {
-	int ItemCounter;
 	int mis_num;
 	static int CheckMissionGrid;
 	int this_mission_seems_completed;
@@ -103,12 +101,12 @@ void CheckIfMissionIsComplete(void)
 		this_mission_seems_completed = TRUE;
 		checked_one_criterion = FALSE;
 
-		// Continue if the Mission target KillOne is given but not fulfilled
+		// Continue if the Mission target KillMarker is given but not fulfilled
 		//
-		if (Me.AllMissions[mis_num].KillOne != (-1)) {
+		if (Me.AllMissions[mis_num].KillMarker != (-1)) {
 			enemy *erot;
 			BROWSE_ALIVE_BOTS(erot) {
-				if ((erot->marker == Me.AllMissions[mis_num].KillOne)) {
+				if (erot->marker == Me.AllMissions[mis_num].KillMarker) {
 					DebugPrintf(MIS_COMPLETE_DEBUG,
 						    "\nOne of the marked droids is still alive... (%p at %f:%f on %d)\n", erot,
 						    erot->pos.x, erot->pos.y, erot->pos.z);
@@ -119,39 +117,11 @@ void CheckIfMissionIsComplete(void)
 			}
 			checked_one_criterion = TRUE;
 		}
-		// Continue if the Mission target fetch_item is given but not fulfilled
+		// Continue if the Mission target must_clear_level is given but not fulfilled
 		//
-		if (Me.AllMissions[mis_num].fetch_item != (-1)) {
-
-			for (ItemCounter = 0; ItemCounter < MAX_ITEMS_IN_INVENTORY; ItemCounter++) {
-				if (Me.Inventory[ItemCounter].type == Me.AllMissions[mis_num].fetch_item) {
-					DebugPrintf(MIS_COMPLETE_DEBUG, "\nDesired item IS PRESENT!!");
-					break;
-				}
-			}
-			if (ItemCounter >= MAX_ITEMS_IN_INVENTORY) {
-				// goto CheckNextMission;
-				this_mission_seems_completed = FALSE;
-			}
-			checked_one_criterion = TRUE;
-		}
-		// Continue if the Mission target must_clear_first_level is given but not fulfilled
-		//
-		if (Me.AllMissions[mis_num].must_clear_first_level != (-1)) {
+		if (Me.AllMissions[mis_num].must_clear_level != (-1)) {
 			enemy *erot;
-			BROWSE_LEVEL_BOTS(erot, Me.AllMissions[mis_num].must_clear_first_level) {
-				if (!is_friendly(erot->faction, FACTION_SELF)) {
-					this_mission_seems_completed = FALSE;
-					break;
-				}
-			}
-			checked_one_criterion = TRUE;
-		}
-		// Continue if the Mission target must_clear_second_level is given but not fulfilled
-		//
-		if (Me.AllMissions[mis_num].must_clear_second_level != (-1)) {
-			enemy *erot;
-			BROWSE_LEVEL_BOTS(erot, Me.AllMissions[mis_num].must_clear_second_level) {
+			BROWSE_LEVEL_BOTS(erot, Me.AllMissions[mis_num].must_clear_level) {
 				if (!is_friendly(erot->faction, FACTION_SELF)) {
 					this_mission_seems_completed = FALSE;
 					break;
@@ -160,51 +130,6 @@ void CheckIfMissionIsComplete(void)
 			checked_one_criterion = TRUE;
 		}
 
-		// Continue if the Mission target MustReachLevel is given but not fulfilled
-		//
-		if (Me.AllMissions[mis_num].MustReachLevel != (-1)) {
-			if (Me.pos.z != Me.AllMissions[mis_num].MustReachLevel) {
-				DebugPrintf(MIS_COMPLETE_DEBUG, "\nLevel number does not match...");
-				continue;
-			}
-			checked_one_criterion = TRUE;
-		}
-		// Continue if the Mission target MustReachPoint.x is given but not fulfilled
-		//
-		if (Me.AllMissions[mis_num].MustReachPoint.x != (-1)) {
-			if (Me.pos.x != Me.AllMissions[mis_num].MustReachPoint.x) {
-				DebugPrintf(MIS_COMPLETE_DEBUG, "\nX coordinate does not match...");
-				continue;
-			}
-			checked_one_criterion = TRUE;
-		}
-		// Continue if the Mission target MustReachPoint.y is given but not fulfilled
-		//
-		if (Me.AllMissions[mis_num].MustReachPoint.y != (-1)) {
-			if (Me.pos.y != Me.AllMissions[mis_num].MustReachPoint.y) {
-				DebugPrintf(MIS_COMPLETE_DEBUG, "\nY coordinate does not match...");
-				continue;
-			}
-			checked_one_criterion = TRUE;
-		}
-		// Continue if the Mission target MustLiveTime is given but not fulfilled
-		//
-		if (Me.AllMissions[mis_num].MustLiveTime != (-1)) {
-			if (Me.MissionTimeElapsed < Me.AllMissions[mis_num].MustLiveTime) {
-				DebugPrintf(MIS_COMPLETE_DEBUG, "\nTime Limit not yet reached...");
-				continue;
-			}
-			checked_one_criterion = TRUE;
-		}
-		// Continue if the Mission target MustBeOne is given but not fulfilled
-		//
-		if (Me.AllMissions[mis_num].MustBeOne != (-1)) {
-			if (Me.marker != Me.AllMissions[mis_num].MustBeOne) {
-				DebugPrintf(MIS_COMPLETE_DEBUG, "\nYou're not yet one of the marked ones...");
-				continue;
-			}
-			checked_one_criterion = TRUE;
-		}
 
 		// If the mission actually had criteria we checked for, and those criteria
 		// are OK, the mission is finished.
@@ -223,7 +148,6 @@ void AssignMission(const char *name)
 {
 	int MissNum = GetMissionIndexByName(name);
 
-	Mission_Status_Change_Sound();
 	Me.AllMissions[MissNum].MissionWasAssigned = TRUE;
 
 	if (Me.AllMissions[MissNum].assignment_lua_code)
@@ -239,7 +163,6 @@ void CompleteMission(const char *name)
 {
 	int MissNum = GetMissionIndexByName(name);
 
-	Mission_Status_Change_Sound();
 	Me.AllMissions[MissNum].MissionIsComplete = TRUE;
 
 	if (Me.AllMissions[MissNum].completion_lua_code)
@@ -299,7 +222,7 @@ void GetQuestList(char *QuestListFilename)
 	char *EndOfMissionTargetPointer;
 	int MissionTargetIndex = 0;
 	char *MissionTargetPointer;
-	char fpath[2048];
+	char fpath[PATH_MAX];
 	char *MissionFileContents;
 	char InnerPreservedLetter = 0;
 	int diary_entry_nr;
@@ -309,22 +232,15 @@ void GetQuestList(char *QuestListFilename)
 
 #define MISSION_TARGET_NAME_INITIALIZER "Mission Name=_\""
 
-#define MISSION_TARGET_FETCH_ITEM_STRING "Mission target is to fetch item : \""
-#define MISSION_TARGET_KILL_ONE_STRING "Mission target is to kill droids with marker : "
-#define MISSION_TARGET_MUST_CLEAR_FIRST_LEVEL "Mission target is to kill all hostile droids this first level : "
-#define MISSION_TARGET_MUST_CLEAR_SECOND_LEVEL "Mission target is to also kill all hostile droids on second level : "
-#define MISSION_TARGET_MUST_REACH_LEVEL_STRING "Mission target is to reach level : "
-#define MISSION_TARGET_MUST_REACH_POINT_X_STRING "Mission target is to reach X-Pos : "
-#define MISSION_TARGET_MUST_REACH_POINT_Y_STRING "Mission target is to reach Y-Pos : "
-#define MISSION_TARGET_MUST_LIVE_TIME_STRING "Mission target is to live for how many seconds : "
-#define MISSION_TARGET_MUST_BE_ONE_STRING "Mission target is to overtake a droid with marker : "
+#define MISSION_TARGET_KILL_MARKER "Mission target is to kill droids with marker : "
+#define MISSION_TARGET_MUST_CLEAR_LEVEL "Mission target is to kill all hostile droids on this level : "
 
 #define MISSION_ASSIGNMENT_LUACODE_STRING "Assignment LuaCode={"
 #define MISSION_COMPLETION_LUACODE_STRING "Completion LuaCode={"
 
 	// At first we must load the quest list file given...
 	//
-	find_file(QuestListFilename, MAP_DIR, fpath, 0);
+	find_file(QuestListFilename, MAP_DIR, fpath, PLEASE_INFORM | IS_FATAL);
 	MissionFileContents = ReadAndMallocAndTerminateFile(fpath, "*** END OF QUEST LIST *** LEAVE THIS TERMINATOR IN HERE ***");
 	MissionTargetPointer = MissionFileContents;
 
@@ -337,8 +253,8 @@ void GetQuestList(char *QuestListFilename)
 		EndOfMissionTargetPointer = LocateStringInData(MissionTargetPointer, MISSION_TARGET_SUBSECTION_END_STRING);
 
 		if (MissionTargetIndex >= MAX_MISSIONS_IN_GAME)
-			ErrorMessage(__FUNCTION__, "The number of quests specified in %s exceeds MAX_MISSIONS_IN_GAME (%d).\n",
-				     PLEASE_INFORM, IS_FATAL, QuestListFilename, MAX_MISSIONS_IN_GAME);
+			error_message(__FUNCTION__, "The number of quests specified in %s exceeds MAX_MISSIONS_IN_GAME (%d).",
+				     PLEASE_INFORM | IS_FATAL, QuestListFilename, MAX_MISSIONS_IN_GAME);
 
 		// We need to add an inner terminator here, so that the strstr operation
 		// below will know where to stop within this subsection.
@@ -353,36 +269,12 @@ void GetQuestList(char *QuestListFilename)
 		// From here on we read the details of the mission target, i.e. what the
 		// influencer has to do, so that the mission can be thought of as completed
 		//
-		if (strstr(MissionTargetPointer, MISSION_TARGET_FETCH_ITEM_STRING)) {
-			char *iname = ReadAndMallocStringFromData(MissionTargetPointer, MISSION_TARGET_FETCH_ITEM_STRING, "\"");
-			Me.AllMissions[MissionTargetIndex].fetch_item = GetItemIndexByName(iname);
-			free(iname);
-		} else
-			Me.AllMissions[MissionTargetIndex].fetch_item = -1;
 
-		ReadValueFromStringWithDefault(MissionTargetPointer, MISSION_TARGET_KILL_ONE_STRING, "%d", "-1",
-				    &Me.AllMissions[MissionTargetIndex].KillOne, EndOfMissionTargetPointer);
+		ReadValueFromStringWithDefault(MissionTargetPointer, MISSION_TARGET_KILL_MARKER, "%d", "-1",
+				    &Me.AllMissions[MissionTargetIndex].KillMarker, EndOfMissionTargetPointer);
 
-		ReadValueFromStringWithDefault(MissionTargetPointer, MISSION_TARGET_MUST_CLEAR_FIRST_LEVEL, "%d", "-1", 
-				    &Me.AllMissions[MissionTargetIndex].must_clear_first_level, EndOfMissionTargetPointer);
-
-		ReadValueFromStringWithDefault(MissionTargetPointer, MISSION_TARGET_MUST_CLEAR_SECOND_LEVEL, "%d", "-1",
-				    &Me.AllMissions[MissionTargetIndex].must_clear_second_level, EndOfMissionTargetPointer);
-
-		ReadValueFromStringWithDefault(MissionTargetPointer, MISSION_TARGET_MUST_BE_ONE_STRING, "%d", "-1", 
-				    &Me.AllMissions[MissionTargetIndex].MustBeOne, EndOfMissionTargetPointer);
-
-		ReadValueFromStringWithDefault(MissionTargetPointer, MISSION_TARGET_MUST_REACH_POINT_X_STRING, "%d", "-1",
-				    &Me.AllMissions[MissionTargetIndex].MustReachPoint.x, EndOfMissionTargetPointer);
-
-		ReadValueFromStringWithDefault(MissionTargetPointer, MISSION_TARGET_MUST_REACH_POINT_Y_STRING, "%d", "-1", 
-				    &Me.AllMissions[MissionTargetIndex].MustReachPoint.y, EndOfMissionTargetPointer);
-
-		ReadValueFromStringWithDefault(MissionTargetPointer, MISSION_TARGET_MUST_REACH_LEVEL_STRING, "%d", "-1", 
-				    &Me.AllMissions[MissionTargetIndex].MustReachLevel, EndOfMissionTargetPointer);
-
-		ReadValueFromStringWithDefault(MissionTargetPointer, MISSION_TARGET_MUST_LIVE_TIME_STRING, "%lf", "-1",
-				    &Me.AllMissions[MissionTargetIndex].MustLiveTime, EndOfMissionTargetPointer);
+		ReadValueFromStringWithDefault(MissionTargetPointer, MISSION_TARGET_MUST_CLEAR_LEVEL, "%d", "-1", 
+				    &Me.AllMissions[MissionTargetIndex].must_clear_level, EndOfMissionTargetPointer);
 
 		if (strstr(MissionTargetPointer, MISSION_COMPLETION_LUACODE_STRING)) {
 			Me.AllMissions[MissionTargetIndex].completion_lua_code =
@@ -442,7 +334,7 @@ int GetMissionIndexByName(const char *name)
 			return cidx;
 	}
 
-	ErrorMessage(__FUNCTION__, "Unable to find mission named %s\n", PLEASE_INFORM, IS_FATAL, name);
+	error_message(__FUNCTION__, "Unable to find mission named \"%s\"", PLEASE_INFORM | IS_FATAL, name);
 	return -1;
 }
 
