@@ -75,18 +75,24 @@ void defrag_obstacle_array(level *lvl)
 
 			// Fill in this spot with the last obstacle
 			memcpy(&lvl->obstacle_list[i], &lvl->obstacle_list[array_end], sizeof(obstacle));
-			lvl->obstacle_list[array_end].type = -1;
-			array_end--;
 
 			// Re-address obstacle extension pointing to the obstacle we've moved
-			change_extensions(lvl, &lvl->obstacle_list[array_end+1], &lvl->obstacle_list[i]);
+			change_extensions(lvl, &lvl->obstacle_list[array_end], &lvl->obstacle_list[i]);
 
 			// Glue the moved obstacle.
 			glue_obstacle(lvl, &lvl->obstacle_list[i]);
+
+			// Mark the old entry as unused
+			lvl->obstacle_list[array_end].type = -1;
+
+			// Reverse scan the array to find the new array's end
+			do {
+				array_end--;
+			} while (lvl->obstacle_list[array_end].type == -1 && array_end > i);
 		}
 	}
 
-	dirty_animated_obstacle_lists(lvl->levelnum);
+	dirty_animated_obstacle_list(lvl->levelnum);
 }
 
 /**
@@ -128,6 +134,9 @@ void del_obstacle_extension(level *lvl, obstacle *obs, enum obstacle_extension_t
 		if (ext->type != type)
 			continue;
 
+		free(ext->data);
+		ext->data = NULL;
+
 		dynarray_del(&lvl->obstacle_extensions, i, sizeof(struct obstacle_extension));
 	}
 }
@@ -143,6 +152,9 @@ void del_obstacle_extensions(level *lvl, obstacle *obs)
 		if (ext->obs != obs)
 			continue;
 
+		free(ext->data);
+		ext->data = NULL;
+
 		dynarray_del(&lvl->obstacle_extensions, i, sizeof(struct obstacle_extension));
 	}
 }
@@ -160,7 +172,6 @@ void add_obstacle_extension(level *lvl, obstacle *obs, enum obstacle_extension_t
 	ext.obs = obs;
 	ext.data = data;
 
-
 	dynarray_add(&lvl->obstacle_extensions, &ext, sizeof(struct obstacle_extension));
 }
 
@@ -175,9 +186,10 @@ void free_obstacle_extensions(level *lvl)
 		switch (ext->type) {
 			case OBSTACLE_EXTENSION_CHEST_ITEMS:
 				dynarray_free(ext->data);
-				/* Fall-through */
+				// no break
 			case OBSTACLE_EXTENSION_LABEL:
 			case OBSTACLE_EXTENSION_DIALOGFILE:
+			case OBSTACLE_EXTENSION_SIGNMESSAGE:
 				free(ext->data);
 				ext->data = NULL;
 				break;
