@@ -50,6 +50,10 @@ int gl_max_texture_size;
  */
 int our_SDL_flip_wrapper()
 {
+#ifdef DEBUG_QUAD_BORDER
+	debug_quad_border_seed = 0;
+#endif
+
 	if (use_open_gl)
 		SDL_GL_SwapBuffers();
 	else
@@ -186,18 +190,15 @@ SDL_Surface *our_IMG_load_wrapper(const char *file)
 	surf = IMG_Load(file);
 
 	if (surf == NULL) {
-		ErrorMessage(__FUNCTION__, "IMG_Load returned NULL. IMG_GetError() : %s.\n", PLEASE_INFORM, IS_WARNING_ONLY, IMG_GetError());
+		error_message(__FUNCTION__, "IMG_Load returned NULL. IMG_GetError() : %s.", PLEASE_INFORM, IMG_GetError());
 		return (NULL);
 	}
 
 	if (use_open_gl) {
-
 //		flip_image_vertically(surf);
-
-		return surf;;
-	} else {
-		return surf;
 	}
+
+	return surf;
 }
 
 /**
@@ -208,6 +209,7 @@ SDL_Surface *our_IMG_load_wrapper(const char *file)
  * the surface, so that the dimensions will reach the next biggest power
  * of two in both directions, width and length.
  */
+#ifdef HAVE_LIBGL
 static SDL_Surface *pad_image_for_texture(SDL_Surface * our_surface)
 {
 	int x = 1;
@@ -232,18 +234,15 @@ static SDL_Surface *pad_image_for_texture(SDL_Surface * our_surface)
 
 	return padded_surf;
 }
+#endif
 
 /**
  * If OpenGL is in use, we need to make textured quads out of our normal
  * SDL surfaces.
- * 
- * make_texture_out_of_surface and make_texture_out_of_prepadded_image are
- * the entry points for this function.
  */
+#ifdef HAVE_LIBGL
 static void do_make_texture_out_of_surface(struct image * our_image, int txw, int txh, void *data)
 {
-
-#ifdef HAVE_LIBGL
 	// Stop any image batch being constructed, if relevant
 	end_image_batch();
 
@@ -253,7 +252,7 @@ static void do_make_texture_out_of_surface(struct image * our_image, int txw, in
 		glGenTextures(1, &our_image->texture);
 	}
 
-	our_image->texture_has_been_created = TRUE;
+	our_image->texture_type = TEXTURE_CREATED;
 	DebugPrintf(1, "Using texture %d\n", our_image->texture);
 
 	glBindTexture(GL_TEXTURE_2D, (our_image->texture));
@@ -272,20 +271,18 @@ static void do_make_texture_out_of_surface(struct image * our_image, int txw, in
 	our_image->tex_y1 = 1.0;
 
 	open_gl_check_error_status(__FUNCTION__);
-
+}
 #endif
 
-}
 
 void make_texture_out_of_surface(struct image * our_image)
 {
-
 #ifdef HAVE_LIBGL
 
 	SDL_Surface *right_sized_image;
 
-	// This fills up the image with transparent material, so that 
-	// it will have powers of 2 as the dimensions, which is a requirement
+	// This fills up the image with transparent material, and makes
+	// it have powers of 2 as the dimensions, which is a requirement
 	// for textures on most OpenGL capable cards.
 	//
 	right_sized_image = pad_image_for_texture(our_image->surface);
@@ -306,61 +303,7 @@ void make_texture_out_of_surface(struct image * our_image)
 	our_image->surface = NULL;
 
 #endif
-
-};				// void make_texture_out_of_surface ( iso_image* our_image )
-
-void make_texture_out_of_prepadded_image(struct image * our_image)
-{
-	do_make_texture_out_of_surface(our_image, our_image->surface->w, our_image->surface->h, our_image->surface->pixels);
-
-};				// void make_texture_out_of_prepadded_image (...)
-
-/**
- * This function checks the error status of the OpenGL driver.  An error
- * will produce at least a warning message, maybe even program termination
- * if the errors are really severe.
- */
-void open_gl_check_error_status(const char *name_of_calling_function)
-{
-
-#ifdef HAVE_LIBGL
-
-	switch (glGetError()) {
-	case GL_NO_ERROR:
-		// All is well.  No messages need to be generated...
-		break;
-	case GL_INVALID_ENUM:
-		fprintf(stderr, "\nCheck occurred in function: %s.", name_of_calling_function);
-		ErrorMessage(__FUNCTION__, "Error code GL_INVALID_ENUM received!", PLEASE_INFORM, IS_WARNING_ONLY);
-		break;
-	case GL_INVALID_VALUE:
-		fprintf(stderr, "\nCheck occurred in function: %s.", name_of_calling_function);
-		ErrorMessage(__FUNCTION__, "Error code GL_INVALID_VALUE received!", PLEASE_INFORM, IS_WARNING_ONLY);
-		break;
-	case GL_INVALID_OPERATION:
-		fprintf(stderr, "\nCheck occurred in function: %s.", name_of_calling_function);
-		ErrorMessage(__FUNCTION__, "Error code GL_INVALID_OPERATION received!", PLEASE_INFORM, IS_WARNING_ONLY);
-		break;
-	case GL_STACK_OVERFLOW:
-		fprintf(stderr, "\nCheck occurred in function: %s.", name_of_calling_function);
-		ErrorMessage(__FUNCTION__, "Error code GL_STACK_OVERFLOW received!", PLEASE_INFORM, IS_FATAL);
-		break;
-	case GL_STACK_UNDERFLOW:
-		fprintf(stderr, "\nCheck occurred in function: %s.", name_of_calling_function);
-		ErrorMessage(__FUNCTION__, "Error code GL_STACK_UNDERFLOW received!", PLEASE_INFORM, IS_FATAL);
-		break;
-	case GL_OUT_OF_MEMORY:
-		fprintf(stderr, "\nCheck occurred in function: %s.", name_of_calling_function);
-		ErrorMessage(__FUNCTION__, "Error code GL_OUT_OF_MEMORY received!", PLEASE_INFORM, IS_FATAL);
-		// raise ( SIGSEGV );
-		break;
-	default:
-		fprintf(stderr, "\nCheck occurred in function: %s.", name_of_calling_function);
-		ErrorMessage(__FUNCTION__, "Unhandled error code received!", PLEASE_INFORM, IS_FATAL);
-		break;
-	}
-#endif
-};				// void open_gl_check_error_status ( char* name_of_calling_function  )
+}
 
 /**
  * This function does the first part of the OpenGL parameter 
@@ -370,7 +313,6 @@ void open_gl_check_error_status(const char *name_of_calling_function)
  */
 void safely_set_open_gl_viewport_and_matrix_mode(void)
 {
-
 #ifdef HAVE_LIBGL
 
 	glViewport(0, 0, GameConfig.screen_width, GameConfig.screen_height);
@@ -382,8 +324,7 @@ void safely_set_open_gl_viewport_and_matrix_mode(void)
 	open_gl_check_error_status(__FUNCTION__);
 
 #endif
-
-};				// void safely_set_open_gl_viewport_and_matrix_mode ( void )
+}
 
 /**
  * This function does the second part of the OpenGL parameter 
@@ -393,7 +334,6 @@ void safely_set_open_gl_viewport_and_matrix_mode(void)
  */
 void safely_set_some_open_gl_flags_and_shade_model(void)
 {
-
 #ifdef HAVE_LIBGL
 
 	glClearColor(0.0, 0.0, 0.0, 1.0);
@@ -406,15 +346,14 @@ void safely_set_some_open_gl_flags_and_shade_model(void)
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
 	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-    
+
 	glEnableClientState(GL_VERTEX_ARRAY);
 	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 
 	open_gl_check_error_status(__FUNCTION__);
 
 #endif
-
-};				// void safely_set_some_open_gl_flags_and_shade_model ( void )
+}
 
 /**
  * Initialize the OpenGL interface.
@@ -422,21 +361,16 @@ void safely_set_some_open_gl_flags_and_shade_model(void)
 int safely_initialize_our_default_open_gl_parameters(void)
 {
 #ifdef HAVE_LIBGL
-	// Set up the screen, viewport matrix, coordinate system and all that...
-	//
+	init_opengl_debug();
 	safely_set_open_gl_viewport_and_matrix_mode();
-
 	safely_set_some_open_gl_flags_and_shade_model();
-
 	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &gl_max_texture_size);
-
 	open_gl_check_error_status(__FUNCTION__);
 
 #endif
 
-	return (TRUE);
-
-};				// int safely_initialize_our_default_open_gl_parameters ( void )
+	return TRUE;
+}
 
 /**
  * This function restores the menu background, that must have been stored
@@ -662,7 +596,7 @@ void blit_open_gl_stretched_texture_light_radius(int decay_x, int decay_y)
 	local_iso_image.tex_h = LightRadiusConfig.texture_h;
 	local_iso_image.w = LightRadiusConfig.texture_w;
 	local_iso_image.h = LightRadiusConfig.texture_h;
-	local_iso_image.texture_has_been_created = TRUE;
+	local_iso_image.texture_type = TEXTURE_CREATED | IS_SUBTEXTURE;
 	local_iso_image.offset_x = 0;
 	local_iso_image.offset_y = 0;
 	local_iso_image.tex_x0 = 0.0;
@@ -727,6 +661,7 @@ static struct background {
 		{"number_selector.png", EMPTY_IMAGE, 0, 0,  TRUE },
 		{"item_upgrade.png", EMPTY_IMAGE, ITEM_UPGRADE_RECT_X, ITEM_UPGRADE_RECT_Y,  FALSE },
 		{"item_upgrade_crafting.png", EMPTY_IMAGE, ADDON_CRAFTING_RECT_X, ADDON_CRAFTING_RECT_Y, FALSE },
+        {"console_bg1.jpg", EMPTY_IMAGE, 0, 0, TRUE },
 };
 
 /**
@@ -747,7 +682,7 @@ void blit_background(const char *background)
 	}
 
 	if (!bg) {
-		ErrorMessage(__FUNCTION__, "Received a request to display background %s which is unknown. Doing nothing.", PLEASE_INFORM, IS_WARNING_ONLY, background);
+		error_message(__FUNCTION__, "Received a request to display background %s which is unknown. Doing nothing.", PLEASE_INFORM, background);
 		return;
 	}
 
@@ -755,7 +690,7 @@ void blit_background(const char *background)
 	if (!image_loaded(&bg->img)) {
 		char path[2048];
 		sprintf(path, "backgrounds/%s", bg->filename);
-		load_image(&bg->img, path, FALSE);
+		load_image(&bg->img, path, USE_WIDE);
 	}
 
 
@@ -766,7 +701,7 @@ void blit_background(const char *background)
 	float scalex = 1.0, scaley = 1.0;
 	if (bg->must_scale) {
 		scalex = (float)GameConfig.screen_width / bg->img.w;
-		scaley = (float)GameConfig.screen_height / (bg->img.w * 3 / 4);
+		scaley = (float)GameConfig.screen_height / bg->img.h;
 	}
 
 	x *= scalex;
