@@ -30,6 +30,8 @@
 #include "savestruct_internal.h"
 #include "lauxlib.h"
 
+#include "build_flags.h"
+
 /**
  * Macro used to define the write function of a numeric data type.
  * \ingroup simplerw
@@ -75,28 +77,11 @@ define_read_xxx(uint8_t);
   * \param strout The auto_string to be filled
   * \param data   Pointer to the data to write
   */
+#if __WIN32__
 define_write_xxx(uint8_t, "%hu");
-
-/**
- * \fn void read_char(lua_State *L, int index, char *data)
- * Read a char data from the Lua stack.
- * \ingroup simplerw
- *
- * \param L     Current Lua State
- * \param index Lua stack index of the data
- * \param data  Pointer to the resulting data storage
- */
-define_read_xxx(char);
-
-/**
- * \fn void write_char(struct auto_string *strout, char *data)
- * Write a char data into an auto_string.
- * \ingroup simplerw
- *
- * \param strout The auto_string to be filled
- * \param data   Pointer to the data to write
- */
-define_write_xxx(char, "%hd");
+#else
+define_write_xxx(uint8_t, "%hhu");
+#endif
 
 /**
  * \fn void read_uint16_t(lua_State *L, int index, uint16_t *data)
@@ -252,7 +237,7 @@ void read_string(lua_State *L, int index, string *data)
  */
 void write_string(struct auto_string *strout, string *data)
 {
-	autostr_append(strout, "\"%s\"", (*data) ? *data : "");
+	autostr_append(strout, "[=[%s]=]", (*data) ? *data : "");
 }
 
 /**
@@ -435,31 +420,7 @@ void write_list_head_t(struct auto_string *strout, list_head_t *data)
  * \param data  Pointer to the resulting data storage
  * \param size  Size of the array (max number of value to read)
  */
-void read_char_array(lua_State *L, int index, char *data, int size)
-{
-	*data = '\0';
-	lua_is_of_type_or_abort(L, index, LUA_TSTRING);
-	char *ptr = (char *)lua_tostring(L, index);
-	if (ptr) {
-		memcpy(data, ptr, size);
-		// ensure the string is null-terminated
-		*(data+size-1) = '\0';
-	}
-}
 
-/**
- * Write a char array.
- * \ingroup overloadrw
- *
- * In order to improve read performances, a char array is saved as a Lua string.
- * \param strout The auto_string to be filled
- * \param data   Pointer to the data to write
- * \param size   Size of the array
- */
-void write_char_array(struct auto_string *strout, char *data, int size)
-{
-	autostr_append(strout, "\"%.*s\"", size, data);
-}
 
 /**
  * Read a keybind_t array.
@@ -476,7 +437,7 @@ void read_keybind_t_array(lua_State *L, int index, keybind_t *data, int size)
 	lua_is_of_type_or_abort(L, index, LUA_TTABLE);
 	keybind_t keybind;
 	int i;
-	for (i = 0; i < lua_objlen(L, -1); i++) {
+	for (i = 0; i < lua_rawlen(L, -1); i++) {
 		lua_rawgeti(L, index, i+1);
 		read_keybind_t(L, -1, &keybind);
 		input_set_keybind(keybind.name, keybind.key, keybind.mod);
@@ -521,19 +482,28 @@ void save_game_data(struct auto_string *strout)
 	npc *n;
 	int i;
 
+	autostr_append(strout, "--[[\n");
+	autostr_append(strout,
+		"SAVEGAME: %s %s %s;sizeof(tux_t)=%d;sizeof(enemy)=%d;sizeof(bullet)=%d;MAXBULLETS=%d\n",
+		SAVEGAME_VERSION, SAVEGAME_REVISION, VERSION, (int)sizeof(tux_t), (int)sizeof(enemy), (int)sizeof(bullet), (int)MAXBULLETS);
+	autostr_append(strout, "BUILD_CFLAGS: %s\n", BUILD_CFLAGS);
+	autostr_append(strout, "BUILD_LDFLAGS: %s\n", BUILD_LDFLAGS);
+	autostr_append(strout, "VERSION: %s\n", freedroid_version);
+	autostr_append(strout, "--]]\n");
+
 	write_lua_variables(strout);
 
 	autostr_append(strout, "tux_t");
 	write_tux_t(strout, &Me);
 	autostr_append(strout, "\n");
 
-	list_for_each_entry(erot, &dead_bots_head, global_list) {
+	list_for_each_entry_reverse(erot, &dead_bots_head, global_list) {
 		autostr_append(strout, "dead_enemy");
 		write_enemy(strout, erot);
 		autostr_append(strout, "\n");
 	}
 
-	list_for_each_entry(erot, &alive_bots_head, global_list) {
+	list_for_each_entry_reverse(erot, &alive_bots_head, global_list) {
 		autostr_append(strout, "alive_enemy");
 		write_enemy(strout, erot);
 		autostr_append(strout, "\n");
@@ -565,6 +535,23 @@ void save_game_data(struct auto_string *strout)
 	for (i = 0; i < FACTION_NUMBER_OF_FACTIONS; i++)
 		write_faction(strout, &i);
 	autostr_append(strout, "}\n");
+
+	for (i = 0; i < MAX_LEVELS; i++) {
+		if (level_exists(i)) {
+			struct level *lvl = curShip.AllLevels[i];
+			int x, y;
+			for (y = 0; y < lvl->ylen; y++) {
+				for (x = 0; x < lvl->xlen; x++) {
+					struct volatile_obstacle *volatile_obs;
+					list_for_each_entry(volatile_obs, &lvl->map[y][x].volatile_obstacles, volatile_list) {
+						autostr_append(strout, "volatile_obstacle");
+						write_volatile_obstacle(strout, volatile_obs);
+						autostr_append(strout, "\n");
+					}
+				}
+			}
+		}
+	}
 }
 
 /*
@@ -609,7 +596,7 @@ static int npc_ctor(lua_State *L)
 static int bullet_array_ctor(lua_State *L)
 {
 	int i;
-	for (i = 0; i < lua_objlen(L, 1) && i < MAXBULLETS; i++) {
+	for (i = 0; i < lua_rawlen(L, 1) && i < MAXBULLETS; i++) {
 		lua_rawgeti(L, 1, i+1);
 		read_bullet(L, -1, &AllBullets[i]);
 		lua_pop(L, 1);
@@ -621,7 +608,7 @@ static int bullet_array_ctor(lua_State *L)
 static int blast_array_ctor(lua_State *L)
 {
 	int i;
-	for (i = 0; i < lua_objlen(L, 1) && i < MAXBLASTS; i++) {
+	for (i = 0; i < lua_rawlen(L, 1) && i < MAXBLASTS; i++) {
 		lua_rawgeti(L, 1, i+1);
 		read_blast(L, -1, &AllBlasts[i]);
 		lua_pop(L, 1);
@@ -633,7 +620,7 @@ static int blast_array_ctor(lua_State *L)
 static int spell_active_array_ctor(lua_State *L)
 {
 	int i;
-	for (i = 0; i < lua_objlen(L, 1) && i < MAX_ACTIVE_SPELLS; i++) {
+	for (i = 0; i < lua_rawlen(L, 1) && i < MAX_ACTIVE_SPELLS; i++) {
 		lua_rawgeti(L, 1, i+1);
 		read_spell_active(L, -1, &AllActiveSpells[i]);
 		lua_pop(L, 1);
@@ -646,7 +633,7 @@ static int spell_active_array_ctor(lua_State *L)
 static int melee_shot_array_ctor(lua_State *L)
 {
 	int i;
-	for (i = 0; i < lua_objlen(L, 1) && i < MAX_MELEE_SHOTS; i++) {
+	for (i = 0; i < lua_rawlen(L, 1) && i < MAX_MELEE_SHOTS; i++) {
 		lua_rawgeti(L, 1, i+1);
 		read_melee_shot(L, -1, &AllMeleeShots[i]);
 		lua_pop(L, 1);
@@ -665,6 +652,21 @@ static int factions_ctor(lua_State *L)
 	return 0;
 }
 
+static int volatile_obstacle_ctor(lua_State *L)
+{
+	struct volatile_obstacle *volatile_obs = (struct volatile_obstacle *)MyMalloc(sizeof(struct volatile_obstacle));
+	read_volatile_obstacle(L, -1, volatile_obs);
+	if (!level_exists(volatile_obs->obstacle.pos.z)) {
+		error_message(__FUNCTION__, "Can not add the obstacle: unknown level %d.",
+				PLEASE_INFORM, volatile_obs->obstacle.pos.z);
+	}
+	struct level *lvl = curShip.AllLevels[volatile_obs->obstacle.pos.z];
+	add_volatile_obstacle(lvl, volatile_obs->obstacle.pos.x, volatile_obs->obstacle.pos.y,
+	                      volatile_obs->obstacle.type, volatile_obs->vanish_timeout);
+	free(volatile_obs);
+	return 0;
+}
+
 /**
  * Load game data.
  * \ingroup toprw
@@ -677,7 +679,7 @@ void load_game_data(char *strin)
 {
 	// Add the table constructors called by Lua when parsing a savegame
 
-	luaL_reg lfuncs[] = {
+	luaL_Reg lfuncs[] = {
 		{"tux_t", tux_ctor},
 		{"dead_enemy", dead_enemy_ctor},
 		{"alive_enemy", alive_enemy_ctor},
@@ -687,6 +689,7 @@ void load_game_data(char *strin)
 		{"spell_active_array", spell_active_array_ctor},
 		{"melee_shot_array", melee_shot_array_ctor},
 		{"factions", factions_ctor},
+		{"volatile_obstacle", volatile_obstacle_ctor},
 		{NULL, NULL}
 	};
 
@@ -699,7 +702,7 @@ void load_game_data(char *strin)
 	}
 
 	// Parse the configuration file, calling table constructors to create the
-        // associated C data structures
+	// associated C data structures
 
 	run_lua(LUA_DIALOG, strin);
 }
@@ -740,7 +743,7 @@ void load_freedroid_configuration(char *strin)
 {
 	// Add the table constructors called by Lua when parsing the configuration file
 
-	luaL_reg lfuncs[] = {
+	luaL_Reg lfuncs[] = {
 		{"configuration", configuration_ctor},
 		{NULL, NULL}
 	};
