@@ -99,7 +99,7 @@ int AssemblePointerListForItemShow(item ** ItemPointerListPointer, int IncludeWo
 	// pure unequipped inventory
 	//
 	for (i = 0; i < MAX_ITEMS_IN_INVENTORY; i++) {
-		if (Me.Inventory[i].type == (-1) || Me.Inventory[i].type < 0)
+		if (Me.Inventory[i].type < 0)
 			continue;
 		else {
 			*CurrentItemPointer = &(Me.Inventory[i]);
@@ -167,7 +167,7 @@ void ShowRescaledItem(int position, int TuxItemRow, item * ShowItem)
 	static struct image equipped_icon;
 
 	if (!image_loaded(&equipped_icon)) {
-		load_image(&equipped_icon, "cursors/mouse_cursor_0003.png", FALSE);
+		load_image(&equipped_icon, "cursors/mouse_cursor_0003.png", NO_MOD);
 	}
 
 	TuxItemRowRect.x = 55 * GameConfig.screen_width / 640;
@@ -234,18 +234,18 @@ void ShowItemPicture(int PosX, int PosY, int Number)
 
 		// Load new images
 		for (i = 0; i < MAX_NUMBER_OF_IMAGES_IN_ITEM_ROTATION; i++) {
-			char ConstructedFileName[2048];
-			char fpath[2048];
+			char ConstructedFileName[PATH_MAX];
+			char fpath[PATH_MAX];
 			sprintf(ConstructedFileName, "items/%s/portrait_%04d.jpg", ItemMap[Number].item_rotation_series_prefix, i + 1);
 
 			// Look for the next file
-			if (!find_file(ConstructedFileName, GRAPHICS_DIR, fpath, 1)) {
-				load_image(&item_rotation_img[i], ConstructedFileName, FALSE);
+			if (find_file(ConstructedFileName, GRAPHICS_DIR, fpath, SILENT)) {
+				load_image(&item_rotation_img[i], ConstructedFileName, NO_MOD);
 			} else {
 				NumberOfImagesInThisRotation = i;
 
 				if (!NumberOfImagesInThisRotation)
-					ErrorMessage(__FUNCTION__, "Unable to load any item rotation image for item \"%s\". File \"%s\" was not found.", PLEASE_INFORM, IS_WARNING_ONLY, ItemMap[Number].item_name, ConstructedFileName);
+					error_message(__FUNCTION__, "Unable to load any item rotation image for item \"%s\". File \"%s\" was not found.", PLEASE_INFORM, ItemMap[Number].id, ConstructedFileName);
 
 				break;
 			}
@@ -256,7 +256,7 @@ void ShowItemPicture(int PosX, int PosY, int Number)
 		strcpy(LastImageSeriesPrefix, ItemMap[Number].item_rotation_series_prefix);
 	}
 
-	RotationIndex = (SDL_GetTicks() / 70);
+	RotationIndex = (SDL_GetTicks() / 45);
 
 	RotationIndex = RotationIndex - (RotationIndex / NumberOfImagesInThisRotation) * NumberOfImagesInThisRotation;
 
@@ -299,8 +299,9 @@ static void fill_item_description(struct widget_text *desc, item *show_item, int
 		}
 
 		if (ItemMap[show_item->type].item_group_together_in_inventory) {
-			autostr_append(desc->text, _("Price per unit: %s%d%s\n"), buyone_highlight, price, font_switchto_neon);
-			autostr_append(desc->text, _("%s all: %s%d%s\n"), action, buyall_highlight, price * show_item->multiplicity, font_switchto_neon);
+			autostr_append(desc->text, _("Price per unit: %s%d%s\n%s all: %s%d%s\n"),
+			               buyone_highlight, price, font_switchto_neon,
+			               action, buyall_highlight, price * show_item->multiplicity, font_switchto_neon);
 		} else {
 			autostr_append(desc->text, _("%s Price: %s%d%s\n"), action, buyone_highlight, price, font_switchto_neon);
 		}
@@ -322,19 +323,20 @@ static void fill_item_description(struct widget_text *desc, item *show_item, int
 	}
 
 	/* If the item is a weapon, then we print out some weapon stats. */
-	if (info->item_gun_recharging_time > 0)
-		autostr_append(desc->text, _("Recharge time: %3.2f\n"),
-					   info->item_gun_recharging_time);
+	if (info->weapon_attack_time > 0)
+		autostr_append(desc->text, _("Time between two attacks: %3.2fs\n"),
+					   info->weapon_attack_time);
 
-	if (info->item_gun_reloading_time > 0)
-		autostr_append(desc->text, _("Time to reload ammo clip: %3.2f\n"),
-					   info->item_gun_reloading_time);
+	if (info->weapon_reloading_time > 0)
+		autostr_append(desc->text, _("Time to reload ammo clip: %3.2fs\n"),
+					   info->weapon_reloading_time);
 
 	autostr_append(desc->text, _("Notes: %s"), D_(info->item_description));
 
-	if (info->item_gun_use_ammunition)
-		autostr_append(desc->text, _("\nThis weapon requires %s."),
-					   ammo_desc_for_weapon(show_item->type));
+	if (info->weapon_ammo_type) {
+		const char *ammo = _(item_specs_get_name(get_item_type_by_id(info->weapon_ammo_type)));
+		autostr_append(desc->text, _("\nThis weapon requires %s."), ammo);
+	}
 }
 
 /**
@@ -403,8 +405,8 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 	/* Initialize the text widget. */
 	widget_text_init(&item_description, "");
 	widget_set_rect(WIDGET(&item_description), UNIVERSAL_COORD_W(258), UNIVERSAL_COORD_H(108), UNIVERSAL_COORD_W(346), UNIVERSAL_COORD_H(255));
-	item_description.font = FPS_Display_BFont;
-	item_description.line_height_factor = LINE_HEIGHT_FACTOR;
+	item_description.font = FPS_Display_Font;
+	item_description.line_height_factor = 1.0;
 	item_description.scroll_offset = scroll_to_top;
 
 	if (ItemIndex >= 0) {
@@ -488,7 +490,7 @@ int GreatShopInterface(int NumberOfItems, item * ShowPointerList[MAX_ITEMS_IN_IN
 
 		/* Show the amount of 'Valuable Circuits' Tux has. */
 		sprintf(GoldString, "%6d", (int)Me.Gold);
-		PutStringFont(Screen, FPS_Display_BFont, 40 * GameConfig.screen_width / 640 - 15,
+		put_string(FPS_Display_Font, 40 * GameConfig.screen_width / 640 - 15,
 			      370 * GameConfig.screen_height / 480, GoldString);
 
 		blit_mouse_cursor();
@@ -652,7 +654,7 @@ static void repair_item(item * RepairItem)
 		SDL_Delay(1);
 
 	if (calculate_item_repair_price(RepairItem) > Me.Gold) {
-		alert_window("%s\n\n%s", ItemMap[RepairItem->type].item_name, _("You can not afford to have this item repaired."));
+		alert_window("%s\n\n%s", D_(item_specs_get_name(RepairItem->type)), _("You can not afford to have this item repaired."));
 		return;
 	}
 
@@ -679,14 +681,14 @@ static void TryToSellItem(item * SellItem, int AmountToSellAtMost)
 	// items, so some extra care will be taken here...
 	//
 	if (SellItem->multiplicity < 1) {
-		ErrorMessage(__FUNCTION__, "\
+		error_message(__FUNCTION__, "\
 		An item sold seemed to have multiplicity < 1.  This might be due to some\n\
 		fatal errors in the engine OR it might be due to some items dropped on the\n\
 		maps somewhere long ago still had multiplicity=0 setting, which should not\n\
 		normally occur with 'freshly' generated items.  Well, that's some dust from\n\
 		the past, but now it should be fixed and not occur in future releases (0.9.10\n\
 		    or later) of the game.  If you encounter this message after release 0.9.10,\n\
-		please inform the developers...", PLEASE_INFORM, IS_WARNING_ONLY);
+		please inform the developers...", PLEASE_INFORM);
 	}
 
 	if (AmountToSellAtMost > SellItem->multiplicity)
@@ -728,7 +730,7 @@ static int buy_item(item *BuyItem, int amount)
 
 	// If the item is too expensive, bail out
 	if (item_price > Me.Gold) {
-		alert_window("%s\n\n%s", ItemMap[BuyItem->type].item_name, _("You can not afford this item."));
+		alert_window("%s\n\n%s", D_(item_specs_get_name(BuyItem->type)), _("You can not afford this item."));
 		return -1;
 	}
 
