@@ -123,7 +123,7 @@ void downgrade_program(int prog_id)
 			}
 		}
 
-		ErrorMessage(__FUNCTION__, "No skills are available for selection.", PLEASE_INFORM, IS_WARNING_ONLY);
+		error_message(__FUNCTION__, "No skills are available for selection.", PLEASE_INFORM);
 	}
 }
 
@@ -201,32 +201,36 @@ int get_program_index_with_name(const char *pname)
 
 	fprintf(stderr, "%s\n", pname);
 
-	ErrorMessage(__FUNCTION__, "\
-FreedroidRPG could not find the program name above in the program spec array!", PLEASE_INFORM, IS_FATAL);
+	error_message(__FUNCTION__, "\
+FreedroidRPG could not find the program name above in the program spec array!", PLEASE_INFORM | IS_FATAL);
 	return -1;
 }
 
 /**
  * This function creates a teleporter portal to the home location.
+ * @return 1 on success, 0 on failure
  */
-static void TeleportHome(void)
+int TeleportHome(void)
 {
-	location HomeSpot;
+	// Check if this level permits teleport
+	if (curShip.AllLevels[Me.pos.z]->flags & TELEPORT_BLOCKED) {
+		append_new_game_message(_("You cannot teleport here!"));
+		return 0;
+	}
 
 	// Find homespot position.
-
-	ResolveMapLabelOnShip("TeleportHomeTarget", &(HomeSpot));
+	gps home_spot = get_map_label_center("TeleportHomeTarget");
 
 	// If homespot was not found, a fatal error has been generated, but we
 	// however check it once again, to prevent any bug.
 
-	if (HomeSpot.x == -1)
-		return;
+	if (home_spot.z == -1)
+		return 1;
 
 	// Case 1 : Tux is in homespot's level, and there is a teleport anchor
 	//          -> teleport back to previous position
 	//
-	if (Me.pos.z == HomeSpot.level && ((Me.teleport_anchor.x != 0) || (Me.teleport_anchor.y != 0))) {
+	if (Me.pos.z == home_spot.z && (Me.teleport_anchor.z != -1)) {
 
 		// Teleport
 		teleport_arrival_sound();
@@ -237,9 +241,9 @@ static void TeleportHome(void)
 		// Reset teleport anchor
 		Me.teleport_anchor.x = 0;
 		Me.teleport_anchor.y = 0;
-		Me.teleport_anchor.z = 0;
+		Me.teleport_anchor.z = -1;
 
-		return;
+		return 1;
 	}
 
 	// Any other cases : Store current position and teleport Tux to homespot
@@ -250,9 +254,10 @@ static void TeleportHome(void)
 
 	teleport_arrival_sound();
 	reset_visible_levels();
-	Teleport(HomeSpot.level, HomeSpot.x, HomeSpot.y, TRUE, TRUE);
+	Teleport(home_spot.z, home_spot.x, home_spot.y, TRUE, TRUE);
 	clear_active_bullets();
 
+	return 1;
 }				// void TeleportHome ( void )
 
 /**
@@ -305,6 +310,12 @@ void HandleCurrentlyActivatedSkill()
 	return;
 };				// void HandleCurrentlyActivatedSkill( void )
 
+/**
+ * \brief Perform a skill action.
+ * \param skill_index The index of the skill.
+ * \param SpellCost The cost of spell calculated before.
+ * \return FALSE if the skill has failed, TRUE otherwise.
+ */
 int DoSkill(int skill_index, int SpellCost)
 {
 	enemy *droid_below_mouse_cursor = NULL;
@@ -317,7 +328,7 @@ int DoSkill(int skill_index, int SpellCost)
 	case PROGRAM_FORM_INSTANT:
 		droid_below_mouse_cursor = GetLivingDroidBelowMouseCursor();
 		if (droid_below_mouse_cursor == NULL)
-			goto done_handling_instant_hits;
+			return 0;
 		if (!DirectLineColldet(Me.pos.x,
 				       Me.pos.y,
 				       translate_pixel_to_map_location((float)input_axis.x,
@@ -325,11 +336,11 @@ int DoSkill(int skill_index, int SpellCost)
 								       TRUE),
 				       translate_pixel_to_map_location((float)input_axis.x,
 								       (float)input_axis.y, FALSE), Me.pos.z, &FlyablePassFilter))
-			goto done_handling_instant_hits;
+			return 0;
 
-		if ((Druidmap[droid_below_mouse_cursor->type].is_human && !SpellSkillMap[skill_index].hurt_humans)
-		    || (!Druidmap[droid_below_mouse_cursor->type].is_human && !SpellSkillMap[skill_index].hurt_bots))
-			return 1;
+		if ((Droidmap[droid_below_mouse_cursor->type].is_human && !SpellSkillMap[skill_index].hurt_humans)
+		    || (!Droidmap[droid_below_mouse_cursor->type].is_human && !SpellSkillMap[skill_index].hurt_bots))
+			return 0;
 
 		if (hitdmg > 0)
 			hit_enemy(droid_below_mouse_cursor, hitdmg, 1, -1, 1);
@@ -343,10 +354,34 @@ int DoSkill(int skill_index, int SpellCost)
 			droid_below_mouse_cursor->poison_damage_per_sec += hitdmg;
 		}
 
+		if (!strcmp(SpellSkillMap[skill_index].effect, "takeover")) {
+			if (!is_friendly(droid_below_mouse_cursor->faction, FACTION_SELF)) {
+				// Only hostile droids can be hacked.
+
+				float used_capsules_ratio = 1;
+				if (droid_takeover(droid_below_mouse_cursor, &used_capsules_ratio)) {
+					// Only capsules that were used generate heat - hard fights are more exhausting that easy ones
+					Me.temperature += used_capsules_ratio * SpellCost;
+
+					// upon successful takeover
+					// go directly to chat to choose droid program
+					if (GameConfig.talk_to_bots_after_takeover)
+						chat_with_droid(droid_below_mouse_cursor);
+					return 1;
+				}
+			} else {
+				return 0;
+			}
+		}
 		Me.temperature += SpellCost;
-		break;
+		return 1;
 
 	case PROGRAM_FORM_SELF:
+	
+		if (!strcmp(SpellSkillMap[skill_index].effect, "teleport_home"))
+			if (!TeleportHome())
+				return 0;
+		
 		Me.energy -= hitdmg;
 		Me.temperature += SpellCost;
 		Me.slowdown_duration += strcmp(SpellSkillMap[skill_index].effect, "slowdown") ? 0 : effdur;
@@ -354,7 +389,7 @@ int DoSkill(int skill_index, int SpellCost)
 		Me.invisible_duration += strcmp(SpellSkillMap[skill_index].effect, "invisibility") ? 0 : effdur;
 		Me.nmap_duration += strcmp(SpellSkillMap[skill_index].effect, "nmap") ? 0 : effdur;
 		Me.light_bonus_end_date = Me.current_game_date + (strcmp(SpellSkillMap[skill_index].effect, "light") ? 0 : effdur);
-		break;
+		return 1;
 
 	case PROGRAM_FORM_BULLET:
 		Me.temperature += SpellCost;
@@ -366,9 +401,9 @@ int DoSkill(int skill_index, int SpellCost)
 		bullet bul_parms;
 		/*XXX hardcoded laser pistol type */
 		if (SpellSkillMap[skill_index].graphics_code != -1)
-			FillInDefaultBulletStruct(&bul_parms, SpellSkillMap[skill_index].graphics_code, GetItemIndexByName("Laser pistol"));
+			bullet_init_for_player(&bul_parms, SpellSkillMap[skill_index].graphics_code, get_item_type_by_id("Laser pistol"));
 		else
-			FillInDefaultBulletStruct(&bul_parms, GetBulletByName("half_magenta"), GetItemIndexByName("Laser pistol"));
+			bullet_init_for_player(&bul_parms, GetBulletByName("half_magenta"), get_item_type_by_id("Laser pistol"));
 
 		bul_parms.freezing_level = strcmp(SpellSkillMap[skill_index].effect, "slowdown") ? 0 : effdur;
 		bul_parms.poison_duration = strcmp(SpellSkillMap[skill_index].effect, "poison") ? 0 : effdur;
@@ -382,7 +417,7 @@ int DoSkill(int skill_index, int SpellCost)
 		else
 			bul_parms.hit_type = ATTACK_HIT_BOTS;
 
-		FireTuxRangedWeaponRaw(GetItemIndexByName("Laser pistol"), -1, &bul_parms, target_location);
+		perform_tux_ranged_attack(get_item_type_by_id("Laser pistol"), &bul_parms, target_location);
 
 		return 1;	//no extra effects
 
@@ -393,71 +428,34 @@ int DoSkill(int skill_index, int SpellCost)
 
 		return 1;
 
-	}
+	case PROGRAM_FORM_SPECIAL:
 
- done_handling_instant_hits:
-
-	/*handle the special extra effects of the skill */
-	if (!strcmp(SpellSkillMap[skill_index].effect, "none")) {
-		goto out;
-	}
-
-	if (!strcmp(SpellSkillMap[skill_index].effect, "takeover")) {
-		if (!MouseCursorIsInUserRect(GetMousePos_x(), GetMousePos_y()))
-			goto out;
-		droid_below_mouse_cursor = GetLivingDroidBelowMouseCursor();
-		if (droid_below_mouse_cursor == NULL)
-			goto out;
-		if (!DirectLineColldet(Me.pos.x,
-				       Me.pos.y,
-				       translate_pixel_to_map_location((float)input_axis.x,
-								       (float)input_axis.y, TRUE),
-				       translate_pixel_to_map_location((float)input_axis.x,
-								       (float)input_axis.y, FALSE), Me.pos.z, &FlyablePassFilter))
-			goto out;
-
-		if (!is_friendly(droid_below_mouse_cursor->faction, FACTION_SELF)) {
-			// Only hostile droids can be hacked. 
-			//
-			if (droid_takeover(droid_below_mouse_cursor)) {
-				// upon successful takeover
-				// go directly to chat to choose droid program
-				if (GameConfig.talk_to_bots_after_takeover)
-					ChatWithFriendlyDroid(droid_below_mouse_cursor);
-			}
-		} else {
-			Me.temperature -= SpellCost;	//pretend nothing happened
+		/*handle the special extra effects of the skill */
+		if (!strcmp(SpellSkillMap[skill_index].effect, "none")) {
+			return 1;
 		}
-		goto out;
-	}
 
-	if (!strcmp(SpellSkillMap[skill_index].effect, "weapon")) {
-		if (!MouseCursorIsInUserRect(GetMousePos_x(), GetMousePos_y()))
-			goto out;
-		if (MouseCursorIsInUserRect(GetMousePos_x(), GetMousePos_y()))
+		if (!strcmp(SpellSkillMap[skill_index].effect, "weapon")) {
+			if (!MouseCursorIsInUserRect(GetMousePos_x(), GetMousePos_y()))
+				return 0;
+				
 			tux_wants_to_attack_now(TRUE);
-		goto out;
-	}
-
-	if (!strcmp(SpellSkillMap[skill_index].effect, "repair")) {
-		if (!MouseCursorIsInInvRect(GetMousePos_x(), GetMousePos_y())
-		    || (!GameConfig.Inventory_Visible)) {
-			// Do nothing here.  The right mouse click while in inventory screen
-			// will be handled in the inventory screen management function.
-			//
-			play_sound("effects/tux_ingame_comments/CantRepairThat.ogg");
+			return 1;
+		}
+		
+		if (!strcmp(SpellSkillMap[skill_index].effect, "repair")) {
+			if (!MouseCursorIsInInvRect(GetMousePos_x(), GetMousePos_y())
+				|| (!GameConfig.Inventory_Visible)) {
+				// Do nothing here.  The right mouse click while in inventory screen
+				// will be handled in the inventory screen management function.
+				//
+				play_sound("effects/tux_ingame_comments/CantRepairThat.ogg");
+			}
+			return 1;
 		}
-		goto out;
 	}
 
-	if (!strcmp(SpellSkillMap[skill_index].effect, "teleport_home")) {
-		TeleportHome();
-		goto out;
-	}
-
- out:
-
-	return 1;
+	return 0;
 };				// void HandleCurrentlyActivatedSkill( void )
 
 
@@ -614,8 +612,8 @@ static void ShowSkillsExplanationScreen(void)
 	TargetSkillRect.x = TEXT_OFFSET_X;
 	TargetSkillRect.w = 320 - 2 * TEXT_OFFSET_X;
 	TargetSkillRect.h = 480 - 15;
-	SetCurrentFont(FPS_Display_BFont);
-	display_text(D_(spec->description), 16, 16 + 64 + 16, &TargetSkillRect);
+	set_current_font(FPS_Display_Font);
+	display_text(D_(spec->description), 16, 16 + 64 + 16, &TargetSkillRect, 1.0);
 }
 
 /** 
@@ -647,7 +645,7 @@ static void establish_skill_subset_map(int *SkillSubsetMap)
 
 /** Activate nth skill from all skills.
  * @param skill_num is a index into all skills array. 
- *        It must points an aquired skill.
+ *        It must points an acquired skill.
  *
  */
 void activate_nth_skill(int skill_num)
@@ -659,9 +657,9 @@ void activate_nth_skill(int skill_num)
 	if (Me.skill_level[skill_num] > 0) {
 		Me.readied_skill = skill_num;
 	} else {
-		ErrorMessage(__FUNCTION__,
-			"Tried to activate skill number %d which was not acquired yet.\n",
-				PLEASE_INFORM, IS_WARNING_ONLY, skill_num);
+		error_message(__FUNCTION__,
+			"Tried to activate skill number %d which was not acquired yet.",
+				PLEASE_INFORM, skill_num);
 	}
 
 };				// void activate_nth_skill ( int skill_num )
@@ -676,7 +674,7 @@ void set_nth_quick_skill(int quick_skill)
 					       GetMousePos_y()) +
 		    NUMBER_OF_SKILLS_PER_SKILL_PAGE * GameConfig.spell_level_visible;
 		
-		// Variable number is an index into already aquired skills.
+		// Variable number is an index into already acquired skills.
 		// We change it into index into array of all skills.
 		int SkillSubsetMap[number_of_skills];
 		establish_skill_subset_map(&(SkillSubsetMap[0]));
@@ -684,9 +682,9 @@ void set_nth_quick_skill(int quick_skill)
 			
 		if (Me.skill_level[ski] <=  0) {
 			// Invalid skill was selected
-			ErrorMessage(__FUNCTION__,
-				"Tried to set skill number %d in quick skills. Skill was not acquired yet.\n",
-					PLEASE_INFORM, IS_WARNING_ONLY, ski);
+			error_message(__FUNCTION__,
+				"Tried to set skill number %d in quick skills. Skill was not acquired yet.",
+					PLEASE_INFORM, ski);
 			return;
 		}
 		int i;
@@ -705,7 +703,7 @@ void load_skill_icon_if_needed(spell_skill_spec *spec)
 	if (!image_loaded(&spec->icon_surface)) {
 		char filename[1000];
 		sprintf(filename, "skill_icons/%s", spec->icon_name);
-		load_image(&spec->icon_surface, filename, FALSE); 
+		load_image(&spec->icon_surface, filename, NO_MOD);
 	}
 }
 
@@ -724,7 +722,7 @@ static void load_skill_level_images_if_needed(void)
 		return;
 
 	// Load the image
-	load_image(&img, SKILL_LEVEL_BUTTON_FILE, FALSE);
+	load_image(&img, SKILL_LEVEL_BUTTON_FILE, NO_MOD);
 
 	// Create the subimages
 	for (i = 0; i < NUMBER_OF_SKILL_PAGES; i++) {
@@ -743,7 +741,7 @@ static void load_skill_level_images_if_needed(void)
 /**
  * This function displays the SKILLS SCREEN.  This is NOT the same as the
  * CHARACTER SCREEN.  In the skills screen you can see what skills/spells
- * you currenlty have availabe and you can select a new readied skill by
+ * you currenlty have available and you can select a new readied skill by
  * clicking on it with the mouse.
  */
 void ShowSkillsScreen(void)
@@ -782,7 +780,7 @@ void ShowSkillsScreen(void)
 	// We will use the FPS display font, cause the small one isn't 
 	// very well readable on the silver background
 	//
-	SetCurrentFont(FPS_Display_BFont);
+	set_current_font(FPS_Display_Font);
 
 	load_skill_level_images_if_needed();
 
@@ -833,7 +831,7 @@ void ShowSkillsScreen(void)
 
 		display_image_on_screen(&SpellSkillMap[SkillOfThisSlot].icon_surface, ButtonRect.x, ButtonRect.y, IMAGE_NO_TRANSFO);
 
-		SetCurrentFont(FPS_Display_BFont);
+		set_current_font(FPS_Display_Font);
 
 		// Program shortcut
 		int sci;
@@ -846,24 +844,24 @@ void ShowSkillsScreen(void)
 			// print the quick key number
 			char str[10];
 			sprintf(str, "F%d\n", 5 + sci);
-			display_text(str, ButtonRect.x + ButtonRect.w - 2 - TextWidth(str), ButtonRect.y, &SkillScreenRect);
+			display_text(str, ButtonRect.x + ButtonRect.w - 2 - text_width(get_current_font(), str), ButtonRect.y, &SkillScreenRect, 1.0);
 		}
 		// Name of the skill
 
 		display_text(D_(SpellSkillMap[SkillOfThisSlot].name),
 			    16 + 64 + 16 + SkillScreenRect.x,
-			    FIRST_SKILLRECT_Y - 6 + i * (64 + INTER_SKILLRECT_DIST) + SkillScreenRect.y, &SkillScreenRect);
+			    FIRST_SKILLRECT_Y - 6 + i * (64 + INTER_SKILLRECT_DIST) + SkillScreenRect.y, &SkillScreenRect, 1.0);
 
-		SetCurrentFont(Messagestat_BFont);
+		set_current_font(Messagestat_Font);
 		int tmp, tmp2;
 		int nextypos =
-		    FIRST_SKILLRECT_Y - 8 + i * (64 + INTER_SKILLRECT_DIST) + SkillScreenRect.y + 2 * FontHeight(GetCurrentFont());
+		    FIRST_SKILLRECT_Y - 8 + i * (64 + INTER_SKILLRECT_DIST) + SkillScreenRect.y + 2 * get_font_height(get_current_font());
 
 		// Program revision
 		sprintf(CharText, _("Program revision: %c%d%c "), font_switchto_msgvar[0], Me.skill_level[SkillOfThisSlot],
 			font_switchto_msgstat[0]);
-		display_text(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect);
-		nextypos += FontHeight(GetCurrentFont());
+		display_text(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect, 1.0);
+		nextypos += get_font_height(get_current_font());
 
 		// Heat cost/cooling
 		tmp = calculate_program_heat_cost(SkillOfThisSlot);
@@ -872,8 +870,8 @@ void ShowSkillsScreen(void)
 				sprintf(CharText, _("Heat produced: %c%d%c "), font_switchto_msgvar[0], tmp, font_switchto_msgstat[0]);
 			else
 				sprintf(CharText, _("Cooling: %c%d%c "), font_switchto_msgvar[0], -tmp, font_switchto_msgstat[0]);
-			display_text(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect);
-			nextypos += FontHeight(GetCurrentFont());
+			display_text(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect, 1.0);
+			nextypos += get_font_height(get_current_font());
 		}
 		// Damage/healing
 		tmp = calculate_program_hit_damage_low(SkillOfThisSlot);
@@ -892,8 +890,8 @@ void ShowSkillsScreen(void)
 					sprintf(CharText, _("Healing: %c%d-%d%c "), font_switchto_msgvar[0], -tmp, -tmp2,
 						font_switchto_msgstat[0]);
 			}
-			display_text(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect);
-			nextypos += FontHeight(GetCurrentFont());
+			display_text(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect, 1.0);
+			nextypos += get_font_height(get_current_font());
 		}
 		// Special effect and duration
 		if (strcmp(SpellSkillMap[SkillOfThisSlot].effect, "none")) {
@@ -917,9 +915,9 @@ void ShowSkillsScreen(void)
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "weapon"))
 				sprintf(CharText, " ");
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "repair"))
-				sprintf(CharText, "Repair items, degrading them a bit");
-			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "satellite"))
-				sprintf(CharText, " ");
+				sprintf(CharText, _("Repair items, degrading them a bit"));
+			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "nmap"))
+				sprintf(CharText, _("Detect enemies"));
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "light"))
 				sprintf(CharText, _("Lighten area"));
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "burnup"))
@@ -930,8 +928,8 @@ void ShowSkillsScreen(void)
 				sprintf(CharText + strlen(CharText), _(" for %c%.1f%c seconds"), font_switchto_msgvar[0], dur,
 					font_switchto_msgstat[0]);
 
-			display_text(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect);
-			nextypos += FontHeight(GetCurrentFont());
+			display_text(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect, 1.0);
+			nextypos += get_font_height(get_current_font());
 		}
 	}
 
