@@ -36,7 +36,7 @@ void play_open_chest_sound(void)
 
 void Play_Spell_ForceToEnergy_Sound()
 {
-	play_sound_cached("effects/Spell_ForceToEnergy_Sound_0.ogg");
+	play_sound("effects/Spell_ForceToEnergy_Sound_0.ogg");
 }
 
 /**
@@ -54,7 +54,7 @@ void No_Ammo_Sound(void)
 
 	now = SDL_GetTicks();
 	if (SDL_GetTicks() - PreviousSound >= 0.25 * 1000) {
-		play_sound_cached("effects/No_Ammo_Sound_0.ogg");
+		play_sound("effects/No_Ammo_Sound_0.ogg");
 		PreviousSound = now;
 	}
 }
@@ -74,7 +74,7 @@ void Not_Enough_Power_Sound(void)
 
 	now = SDL_GetTicks();
 	if (SDL_GetTicks() - PreviousSound >= 1.15 * 1000) {
-		play_sound_cached("effects/tux_ingame_comments/Not_Enough_Power_Sound_0.ogg");
+		play_sound("effects/tux_ingame_comments/Not_Enough_Power_Sound_0.ogg");
 		PreviousSound = now;
 	}
 }
@@ -94,7 +94,7 @@ void Not_Enough_Dist_Sound(void)
 
 	now = SDL_GetTicks();
 	if (SDL_GetTicks() - PreviousSound >= 1.15 * 1000) {
-		play_sound_cached("effects/tux_ingame_comments/Not_Enough_Dist_Sound_0.ogg");
+		play_sound("effects/tux_ingame_comments/Not_Enough_Dist_Sound_0.ogg");
 		PreviousSound = now;
 	}
 }
@@ -109,7 +109,7 @@ void Not_Enough_Dist_Sound(void)
  */
 void play_greeting_sound(enemy *ThisRobot)
 {
-	int SoundCode = Druidmap[ThisRobot->type].greeting_sound_type;
+	int SoundCode = Droidmap[ThisRobot->type].greeting_sound_type;
 	const char *sounds[] = {
 		"effects/bot_sounds/First_Contact_Sound_0.ogg",
 		"effects/bot_sounds/First_Contact_Sound_1.ogg",
@@ -134,7 +134,7 @@ void play_greeting_sound(enemy *ThisRobot)
 
 	// Ensure that SoundCode will index a sound in the sounds[] c string array.
 	if ((SoundCode < 0) || (SoundCode >= (sizeof(sounds) / sizeof(sounds[0])))) {
-		ErrorMessage(__FUNCTION__, "\nUnknown Greeting sound!!!", PLEASE_INFORM, IS_WARNING_ONLY);
+		error_message(__FUNCTION__, "Unknown Greeting sound!!!", PLEASE_INFORM);
 		return;
 	}
 
@@ -153,14 +153,14 @@ void play_death_sound_for_bot(enemy * ThisRobot)
 	// If the keyword 'none' for the death sound file name is encountered,
 	// nothing will be done...
 	//.
-	if (!strcmp(Druidmap[ThisRobot->type].droid_death_sound_file_name, "none"))
+	if (!strcmp(Droidmap[ThisRobot->type].droid_death_sound_file_name, "none"))
 		return;
 
 	// Now we play the given death sound, looking for the file in the
 	// appropriate sound folder.
 	//
 	strcpy(filename, "effects/bot_sounds/");
-	strcat(filename, Druidmap[ThisRobot->type].droid_death_sound_file_name);
+	strcat(filename, Droidmap[ThisRobot->type].droid_death_sound_file_name);
 	play_sound_at_position(filename, &Me.pos, &ThisRobot->pos);
 }
 
@@ -189,7 +189,7 @@ void play_enter_attack_run_state_sound(enemy *ThisRobot)
 
 	if (MyRandom(5)) {
 		// get sound code
-		SoundCode = Druidmap[ThisRobot->type].greeting_sound_type;
+		SoundCode = Droidmap[ThisRobot->type].greeting_sound_type;
 
 		// sound codes are [0, 2]U[9, 18]
 		// this makes it so the index will be [0, number_of_sounds - 1]
@@ -227,9 +227,8 @@ void play_item_sound(int item_type, struct gps *item_pos)
 	// First some sanity check...
 	//
 	if (item_type < 0) {
-		fprintf(stderr, "\n\nitem_type %d \n", item_type);
-		ErrorMessage(__FUNCTION__, "\
-		                           negative item type received!\n", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "\
+		                           negative item type received (%d)!", PLEASE_INFORM | IS_FATAL, item_type);
 	}
 	// Now we can proceed and just play the sound associated with that item...
 	//
@@ -253,13 +252,13 @@ void CantCarrySound(void)
 		CurrentTicks = SDL_GetTicks();
 		switch (MyRandom(2)) {
 		case 0:
-			play_sound_cached("effects/tux_ingame_comments/ICantCarryAnyMore_Sound_0.ogg");
+			play_sound("effects/tux_ingame_comments/ICantCarryAnyMore_Sound_0.ogg");
 			break;
 		case 1:
-			play_sound_cached("effects/tux_ingame_comments/ICantCarryAnyMore_Sound_1.ogg");
+			play_sound("effects/tux_ingame_comments/ICantCarryAnyMore_Sound_1.ogg");
 			break;
 		case 2:
-			play_sound_cached("effects/tux_ingame_comments/ICantCarryAnyMore_Sound_2.ogg");
+			play_sound("effects/tux_ingame_comments/ICantCarryAnyMore_Sound_2.ogg");
 			break;
 		default:
 			break;
@@ -267,18 +266,13 @@ void CantCarrySound(void)
 	}
 }
 
-void Mission_Status_Change_Sound(void)
-{
-	play_sound_cached("effects/Mission_Status_Change_Sound_0.ogg");
-}
 
 /**
  * When the Tux uses the 'teleport home' spell, this sound will be played.
  */
 void teleport_arrival_sound(void)
 {
-	// play_sound_cached("effects/LeaveElevator_Sound_0.ogg");
-	play_sound_cached("effects/new_teleporter_sound.ogg");
+	play_sound("effects/new_teleporter_sound.ogg");
 }
 
 /**
@@ -291,19 +285,19 @@ void tux_scream_sound(void)
 {
 	switch (MyRandom(4)) {
 	case 0:
-		play_sound_cached("effects/Influencer_Scream_Sound_0.ogg");
+		play_sound("effects/Influencer_Scream_Sound_0.ogg");
 		break;
 	case 1:
-		play_sound_cached("effects/Influencer_Scream_Sound_1.ogg");
+		play_sound("effects/Influencer_Scream_Sound_1.ogg");
 		break;
 	case 2:
-		play_sound_cached("effects/Influencer_Scream_Sound_2.ogg");
+		play_sound("effects/Influencer_Scream_Sound_2.ogg");
 		break;
 	case 3:
-		play_sound_cached("effects/Influencer_Scream_Sound_3.ogg");
+		play_sound("effects/Influencer_Scream_Sound_3.ogg");
 		break;
 	case 4:
-		play_sound_cached("effects/Influencer_Scream_Sound_4.ogg");
+		play_sound("effects/Influencer_Scream_Sound_4.ogg");
 		break;
 	default:
 		break;
@@ -316,17 +310,17 @@ void tux_scream_sound(void)
  */
 void MenuItemSelectedSound(void)
 {
-	play_sound_cached("effects/Menu_Item_Selected_Sound_1.ogg");
+	play_sound_v("effects/Menu_Item_Selected_Sound_1.ogg", 0.5);
 }
 
 void MenuItemDeselectedSound(void)
 {
-	play_sound_cached("effects/Menu_Item_Deselected_Sound_0.ogg");
+	play_sound_v("effects/Menu_Item_Deselected_Sound_0.ogg", 0.5);
 }
 
 void MoveMenuPositionSound(void)
 {
-	play_sound_cached("effects/Move_Menu_Position_Sound_0.ogg");
+	play_sound_v("effects/MenuClick.ogg", 0.5);
 }
 
 void ThouArtDefeatedSound(void)
@@ -344,22 +338,17 @@ void ThouArtDefeatedSound(void)
  */
 void play_melee_weapon_hit_something_sound(void)
 {
-	switch (MyRandom(3)) {
-	case 0:
-		play_sound_cached("effects/swing_then_hit_1.ogg");
-		break;
-	case 1:
-		play_sound_cached("effects/swing_then_hit_2.ogg");
-		break;
-	case 2:
-		play_sound_cached("effects/swing_then_hit_3.ogg");
-		break;
-	case 3:
-		play_sound_cached("effects/swing_then_hit_4.ogg");
-		break;
-	default:
-		break;
-	}
+	const char *sounds[] = {
+		"effects/swing_then_hit_1.ogg",
+		"effects/swing_then_hit_2.ogg",
+		"effects/swing_then_hit_3.ogg",
+		"effects/swing_then_hit_4.ogg",
+	};
+	int sound_index = MyRandom(sizeof(sounds) / sizeof(sounds[0]) - 1);
+
+	// The target of the attack is very near Tux, so no need to play
+	// a positional sound.
+	play_sound(sounds[sound_index]);
 }
 
 void play_melee_weapon_missed_sound(struct gps *attacker_pos)
@@ -388,13 +377,15 @@ void fire_bullet_sound(int BulletType, struct gps *shooter_pos)
 	if (!sound_on)
 		return;
 
-	if (!Bulletmap[BulletType].sound) {
+	struct bulletspec *bullet_spec = dynarray_member(&bullet_specs, BulletType, sizeof(struct bulletspec));
+
+	if (!bullet_spec->sound) {
 		play_melee_weapon_missed_sound(shooter_pos);
 		return;
 	}
 
 	char sound_file[100] = "effects/bullets/";
-	strcat(sound_file, Bulletmap[BulletType].sound);
+	strcat(sound_file, bullet_spec->sound);
 
 	play_sound_at_position(sound_file, &Me.pos, shooter_pos);
 }
@@ -406,60 +397,41 @@ void fire_bullet_sound(int BulletType, struct gps *shooter_pos)
  */
 void Takeover_Set_Capsule_Sound(void)
 {
-	play_sound_cached("effects/TakeoverSetCapsule_Sound_0.ogg");
+	play_sound("effects/TakeoverSetCapsule_Sound_0.ogg");
 }
 
 void Takeover_Game_Won_Sound(void)
 {
-	play_sound_cached("effects/Takeover_Game_Won_Sound_0.ogg");
+	play_sound("effects/Takeover_Game_Won_Sound_0.ogg");
 }
 
 void Takeover_Game_Deadlock_Sound(void)
 {
-	play_sound_cached("effects/Takeover_Game_Deadlock_Sound_0.ogg");
+	play_sound("effects/Takeover_Game_Deadlock_Sound_0.ogg");
 }
 
 void Takeover_Game_Lost_Sound(void)
 {
-	play_sound_cached("effects/Takeover_Game_Lost_Sound_0.ogg");
+	play_sound("effects/Takeover_Game_Lost_Sound_0.ogg");
 }
 
-void play_blast_sound(int blast_type, struct gps *blast_pos)
+void play_blast_sound(char *blast_sound, struct gps *blast_pos)
 {
 	char fpath[2048];
 
-	if (!Blastmap[blast_type].sound_file)
+	if (!blast_sound)
 		return;
 
 	strcpy(fpath, "effects/");
-	strcat(fpath, Blastmap[blast_type].sound_file);
+	strcat(fpath, blast_sound);
 	play_sound_at_position(fpath, &Me.pos, blast_pos);
 }
 
-/**
- * Whenever an enemy is hit by the tux with a melee weapon, then the
- * following sound is played...
- */
-void PlayEnemyGotHitSound(int enemytype)
-{
-	switch (enemytype) {
-	case -1:
-		// Don't play anything at all...
-		break;
-	case 0:
-		// Play a grunting enemy got hit sound...
-		play_sound_cached("effects/Enemy_Got_Hit_Sound_0.ogg");
-		break;
-	default:
-		break;
-	};
-}
-
 /**
  * This function plays a sound of a bullet being reflected. It is only
  * used, when a bullets is compensated by the tux armor.
  */
 void BulletReflectedSound(void)
 {
-	play_sound_cached("effects/Bullet_Reflected_Sound_0.ogg");
+	play_sound("effects/Bullet_Reflected_Sound_0.ogg");
 }
