@@ -1,8 +1,8 @@
-/* 
+/*
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
- *   Copyright (c) 2004-2010 Arthur Huillet 
+ *   Copyright (c) 2004-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -18,8 +18,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with Freedroid; see the file COPYING. If not, write to the 
- *  Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+ *  along with Freedroid; see the file COPYING. If not, write to the
+ *  Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  *  MA  02111-1307  USA
  *
  */
@@ -29,6 +29,7 @@
 
 #include "system.h"
 #include "defs.h"
+#include "lua.h"
 
 typedef struct {
 	Uint8 r;
@@ -55,6 +56,7 @@ struct dynarray {
 };
 
 typedef struct dynarray item_dynarray;
+typedef struct dynarray string_dynarray;
 
 typedef struct upgrade_socket_dynarray {
 	struct upgrade_socket *arr;
@@ -62,15 +64,23 @@ typedef struct upgrade_socket_dynarray {
 	int capacity;
 } upgrade_socket_dynarray;
 
-/* prototype for BFont_Info from BFont.h */
-struct BFont_Info;
+struct font {
+	int height;
+	int letter_spacing;
+	struct BFont_Info *bfont;
+};
+
+enum image_transformation_mode {
+	HIGHLIGHTED = 1 << 1,
+	REPEATED    = 1 << 2
+};
 
 struct image_transformation {
-	SDL_Surface *surface;		
-	float scale_x;
-	float scale_y;
-	float c[4]; /**< color transformation, r g b a in that order */
-	int highlight;
+	SDL_Surface *surface;
+	float scale_x; /**< zoom factor or repeat factor, depending on transformation mode */
+	float scale_y; /**< zoom factor or repeat factor, depending on transformation mode */
+	float c[4];    /**< color transformation, r g b a in that order */
+	enum image_transformation_mode mode;
 };
 
 /**
@@ -83,7 +93,7 @@ struct image {
 	short offset_y;
 	short w;
 	short h;
-	int texture_has_been_created;
+	int texture_type;
 #ifdef HAVE_LIBGL
 	GLuint texture;
 #else
@@ -98,7 +108,7 @@ struct image {
 
 	struct image_transformation cached_transformation;
 };
-#define EMPTY_IMAGE { .surface = NULL , .offset_x = 0 , .offset_y = 0 , .texture_has_been_created = 0 , .cached_transformation = { NULL, 0.0, 0.0, { 0.0, 0.0, 0.0, 0.0}, 0 } }
+#define EMPTY_IMAGE { .surface = NULL , .offset_x = 0 , .offset_y = 0 , .texture_type = NO_TEXTURE , .cached_transformation = { NULL, 0.0, 0.0, { 0.0, 0.0, 0.0, 0.0}, 0 } }
 
 typedef struct mouse_press_button {
 	struct image button_image;
@@ -107,6 +117,7 @@ typedef struct mouse_press_button {
 	char scale_this_button;
 } mouse_press_button, *Mouse_press_button;
 
+/* Type definitions needed to create specific save/load functions */
 typedef char *string;
 
 typedef struct keybind {
@@ -122,6 +133,7 @@ typedef struct configuration_for_freedroid {
 	int All_Texts_Switch;
 	float Current_BG_Music_Volume;
 	float Current_Sound_FX_Volume;
+	int Current_Sound_Output_Fmt;
 	float current_gamma_correction;
 
 	int Inventory_Visible;
@@ -136,8 +148,10 @@ typedef struct configuration_for_freedroid {
 	int skip_light_radius;
 	int skill_explanation_screen_visible;
 	int enemy_energy_bars_visible;
+	int effect_countdowns_visible;
 	int limit_framerate;
 	int omit_obstacles_in_level_editor;
+	int omit_map_labels_in_level_editor;
 	int omit_enemies_in_level_editor;
 	int zoom_is_on;
 	int show_blood;		// this can be used to make the game more friendly for children...
@@ -167,6 +181,8 @@ typedef struct configuration_for_freedroid {
 
 	int difficulty_level;
 
+	string locale;
+
 	// This must be the last element of the structure, because the
 	// input keybind parsing code uses strtok which messes with the
 	// string.
@@ -194,12 +210,6 @@ typedef struct gps {
 	int z;
 } gps;
 
-typedef struct location {
-	unsigned char level;
-	int x;
-	int y;
-} location;
-
 typedef struct map_label {
 	char *label_name;
 	point pos;		// how many blocks does this big map insert cover?
@@ -213,14 +223,8 @@ typedef struct mission {
 	int MissionWasFailed;	// has the mission been failed?
 	int MissionExistsAtAll;	// is this mission entry used at all, or is it just unused memory?
 
-	int fetch_item;
-	int KillOne;
-	int must_clear_first_level;
-	int must_clear_second_level;
-	int MustReachLevel;
-	point MustReachPoint;
-	double MustLiveTime;
-	int MustBeOne;
+	int KillMarker;
+	int must_clear_level;
 
 	luacode completion_lua_code;
 	luacode assignment_lua_code;
@@ -250,43 +254,40 @@ struct addon_spec {
 };
 
 typedef struct itemspec {
-	char *item_name;
+	char *id;
+	char *name;
 	char *item_rotation_series_prefix;
 	char *item_description;
 	char *item_drop_sound_file_name;
 	char *item_inv_file_name;
 
-	char *item_combat_use_description;
-	char item_can_be_installed_in_weapon_slot;
-	char item_can_be_installed_in_drive_slot;
-	char item_can_be_installed_in_armour_slot;
-	char item_can_be_installed_in_shield_slot;
-	char item_can_be_installed_in_special_slot;
+	enum slot_type slot;
+
 	char *tux_part_instance;
 
 	char item_group_together_in_inventory;
 
-	// How good is the item as weapon???
-	float item_gun_recharging_time;	// time until the next shot can be made, measures in seconds
-	float item_gun_reloading_time;	// time needed to put a new charger
-	short item_gun_bullet_image_type;	// which type of image to use for displaying this bullet
-	float item_gun_speed;	// how fast should a bullet move straightforward?
-	short base_item_gun_damage;	//   damage done by this bullettype 
-	short item_gun_damage_modifier;	// modifier to the damage done by this bullettype 
-	float item_gun_bullet_lifetime;	// how long does a 'bullet' from this gun type live?
-	char item_gun_bullet_pass_through_hit_bodies;	// does this bullet go through hit bodies (e.g. like a laser sword)
-	short item_gun_ammo_clip_size;	//max. number of bullets in the charger
+	// General characteristics of a weapon item (ranged or melee)
+	char weapon_is_melee;                           // 1: melee weapon - 0: ranged weapon
+	char weapon_needs_two_hands;	                // 1: this a (strictly) 2-handed weapon
+	short int weapon_motion_class;                  // Tux's motion class to use
+	float weapon_attack_time;                       // duration of an attack: time until the next attack can be made, measured in seconds
+	float weapon_reloading_time;                    // time needed to put a new charger (if needed), added to attack_time on last ammunition
+	char* weapon_reloading_sound;                   // sound to play when reloading weapon
+	int weapon_base_damage;                         // damage done by this weapon
+	int weapon_damage_modifier;                     // modifier to the damage done by this weapon
+	char *weapon_ammo_type;                         // ammunition needed by the weapon (it can be anything that needs to be loaded in the weapon)
+	short weapon_ammo_clip_size;                    // max. number of ammunition in the charger
 
-	// the following values have only relevance in case of a melee weapon
-	short int item_weapon_is_melee;
-	float item_gun_start_angle_modifier;	// where to start with a melee weapon swing
-	int item_gun_use_ammunition;        // which ammunition does this gun use? see ammo_desc_for_weapon()
-	char item_gun_requires_both_hands;	// is this a (strictly) 2-handed weapon?
-	short int motion_class;				// Tux's motion class to use
+	// Characteristics of a weapon using bullets (ranged weapon)
+	short weapon_bullet_type;                       // bullet's type used by this weapon (index into bullet_specs), to get bullet's image and sound
+	float weapon_bullet_speed;                      // how fast should a bullet move straightforward?
+	float weapon_bullet_lifetime;	                // how long does a 'bullet' from this gun type live?
+	char weapon_bullet_pass_through_hit_bodies;     // does this bullet go through hit bodies (e.g. like a laser sword)
 
 	// how good is the item as armour or shield or other protection???
-	short base_armor_class;
-	short armor_class_modifier;
+	int base_armor_class;
+	int armor_class_modifier;
 
 	// which requirement for strength, dexterity and cooling does the item have?
 	short int item_require_strength;
@@ -294,8 +295,16 @@ typedef struct itemspec {
 	short int item_require_cooling;
 
 	// what durability does the item have?
-	short int base_item_durability;
-	short int item_durability_modifier;
+	int base_item_durability;
+	int item_durability_modifier;
+
+	struct {
+		char *tooltip;
+		char *skill;
+		char *add_skill;
+		enum _busytype busy_type;
+		int busy_time;
+	} right_use;
 
 	point inv_size;
 	struct image inventory_image;
@@ -304,8 +313,8 @@ typedef struct itemspec {
 
 	short int base_list_price;	// the base price of this item at the shop
 
-	short int min_drop_class;
-	short int max_drop_class;
+	int min_drop_class;
+	int max_drop_class;
 	int drop_amount;		// minimum number of items to drop at once
 	int drop_amount_max;	// maximum of items to drop at once
 
@@ -353,21 +362,22 @@ typedef struct item {
 	int quality;
 } item;
 
-typedef struct druidspec {
-	char *druidname;
+typedef struct droidspec {
+	char *droidname;
 	char *default_short_description;
 	char *droid_portrait_rotation_series_prefix;
 	char *droid_death_sound_file_name;
 	char *droid_attack_animation_sound_file_name;
 	int class;
-	float maxenergy;	// the maximum energy the batteries can carry 
+	float maxenergy;	// the maximum energy the batteries can carry
+	int sensor_id;		// What sensor is the most common to this droid?
 
 	float healing_friendly; // the energy/second the droid heals as a friendly towards Tux
 	float healing_hostile;  // the energy/second the droid heals as a hostile towards Tux
 
 	float maxspeed;
 
-	short experience_reward;	// experience_reward for the elimination of one droid of this type 
+	short experience_reward;	// experience_reward for the elimination of one droid of this type
 
 	float aggression_distance;
 	float time_spent_eyeing_tux;
@@ -384,13 +394,12 @@ typedef struct druidspec {
 	short int amount_of_tachyon_condensators;
 
 	short greeting_sound_type;	// which sample to play in order to 'greet' the influencer?
-	short got_hit_sound_type;	// which sample to play in order to 'greet' the influencer?
 	short to_hit;		// chance that this droid hits an unarmoured target
 	float recover_time_after_getting_hit;
-	char *notes;		// notes on the druid of this type 
+	char *notes;		// notes on the droid of this type
 	short int is_human;
 	short individual_shape_nr;
-} druidspec;
+} droidspec;
 
 typedef char s_char; // Used for pointer to static string which are not to be saved
 
@@ -420,18 +429,21 @@ typedef struct enemy {
 	// 1st set (identity)
 	//
 	short int id;                       // unique id of the droid. start at 1.
-	short int type;                     // the number of the droid specifications in Druidmap
+	short int type;                     // the number of the droid specifications in Droidmap
 	uint8_t SpecialForce;               // this flag will exclude the droid from initial shuffling of droids
 	int marker;                         // this provides a marker for special mission targets
 	short int max_distance_to_home;     // how far this robot will go before returning to it's home waypoint
 	string dialog_section_name;
 	string short_description_text;
 	short int on_death_drop_item_code;  // drop a pre-determined item when dying
+	int sensor_id;			// What is the sensor of this droid? Some droids have different sensors.
+
 
 	//--------------------
 	// 2nd set ('global state')
 	//
 	int faction;
+	uint8_t will_respawn;		// will this robot be revived by respawn_level()?
 	uint8_t will_rush_tux;          // will this robot approach the Tux on sight and open communication?
 	int combat_state;               // current state of the bot
 	float state_timeout;            // time spent in this state (used by "timeout" states such as STOP_AND_EYE_TARGET only)
@@ -483,15 +495,16 @@ typedef struct enemy {
 typedef struct npc {
 	string dialog_basename;
 	uint8_t chat_character_initialized;
-	uint8_t chat_flags[MAX_ANSWERS_PER_PERSON];
 
-	string shoplist[MAX_ITEMS_IN_INVENTORY]; //list of items that can be put in the inventory of the NPC
-	int shoplistweight[MAX_ITEMS_IN_INVENTORY]; //weight of each item: relative probability of appearance in inventory
+	string_dynarray enabled_nodes;
+
+	string shoplist[MAX_ITEMS_IN_NPC_SHOPLIST]; //list of items that can be put in the inventory of the NPC
+	int shoplistweight[MAX_ITEMS_IN_NPC_SHOPLIST]; //weight of each item: relative probability of appearance in inventory
 	item_dynarray npc_inventory;
 
 	float last_trading_date; // when did we trade with this NPC the last time?
 
-	list_head_t node; 
+	list_head_t node;
 } npc;
 
 typedef char automap_data_t[100][100];
@@ -504,12 +517,12 @@ typedef struct tux {
 	float dexterity_bonus_end_date;
 	float light_bonus_end_date;
 
-	finepoint speed;	// the current speed of the druid 
-	gps pos;		// current position in the whole ship 
+	finepoint speed;	// the current speed of the droid
+	gps pos;		// current position in the whole ship
 	gps teleport_anchor;	// where from have you last teleported home
 	gps mouse_move_target;	// where the tux is going automatically by virtue of mouse move
 
-	short int current_enemy_target_n;	//which enemy has been targeted 
+	short int current_enemy_target_n;	//which enemy has been targeted
 	uint8_t god_mode;
 	enemy *current_enemy_target_addr;	// which enemy has been targeted, address
 
@@ -520,7 +533,7 @@ typedef struct tux {
 	int map_maker_is_present;
 
 	float maxenergy;	// current top limit for the influencers energy
-	float energy;		// current energy level 
+	float energy;		// current energy level
 	float max_temperature;	// current top limit for temperature (highest is better)
 	float temperature;	// current temperature
 	float old_temperature;	// current temperature
@@ -534,14 +547,12 @@ typedef struct tux {
 
 	double busy_time;	// time remaining, until the weapon is ready to fire again...
 	int busy_type;		// reason why tux is busy (enum)
-	double phase;		// the current phase of animation 
+	double phase;		// the current phase of animation
 	float angle;
-	float walk_cycle_phase;	// 
-	float weapon_swing_time;	// How long is the current weapon swing in progress (in seconds of course) 
+	float walk_cycle_phase;	//
+	float weapon_swing_time;	// How long is the current weapon swing in progress (in seconds of course)
 	float MissionTimeElapsed;
-	float got_hit_time;	// how long stunned now since the last time tux got hit 
-
-	string savegame_version_string;	// a string to identify games from older FreedroidRPG versions
+	float got_hit_time;	// how long stunned now since the last time tux got hit
 
 	int base_cooling;
 	int base_dexterity;
@@ -561,12 +572,12 @@ typedef struct tux {
 	int paralyzing_melee_targets;	// duration for how long hit enemies are paralyzed
 	float experience_factor; // multiplier for the experience gained from bots
 
-	unsigned int Experience;	
-	int exp_level;			
+	unsigned int Experience;
+	int exp_level;
 
 	unsigned int Gold;
 	string character_name;
-	mission AllMissions[MAX_MISSIONS_IN_GAME];	// What must be done to fullfill this mission?
+	mission AllMissions[MAX_MISSIONS_IN_GAME];	// What must be done to fulfill this mission?
 	int marker;		// In case you've taken over a marked droid, this will contain the marker
 	float LastCrysoundTime;
 	float TextVisibleTime;
@@ -634,7 +645,7 @@ typedef struct tux {
 typedef struct bulletspec {
 	char *name;		// what is the name of this bullet type
 	char *sound;		// what sound to play
-	int phases;		// how many phases in motion to show 
+	int phases;		// how many phases in motion to show
 	double phase_changes_per_second;	// how many different phases to display every second
 	int blast_type;
 	struct image image[BULLET_DIRECTIONS][MAX_PHASES_IN_A_BULLET];
@@ -650,7 +661,6 @@ typedef struct bullet {
 	short int damage;	// damage done by this particular bullet
 	float time_in_seconds;	// how long does the bullet exist in seconds
 	float bullet_lifetime;	// how long can this bullet exist at most
-	float time_to_hide_still;	// countdown to when the bullet will actually appear
 	short int owner;
 	float angle;
 
@@ -668,7 +678,7 @@ typedef struct melee_shot	// this is a melee shot
 {
 	uint8_t attack_target_type;	//type of attack
 	uint8_t mine;		//is it mine?
-	short int bot_target_n;	//which enemy has been targeted 
+	short int bot_target_n;	//which enemy has been targeted
 	enemy *bot_target_addr;	// which enemy has been targeted, address
 	short int to_hit;	//chance to hit, percent
 	short int damage;
@@ -714,13 +724,13 @@ typedef struct spell_skill_spec {
 	struct image icon_surface;
 	short heat_cost;
 	short heat_cost_per_level;
-	short damage_base;
-	short damage_mod;
+	int damage_base;
+	int damage_mod;
 	short damage_per_level;
 	short hurt_bots;
 	short hurt_humans;
 	short form;
-	short present_at_startup;
+	char present_at_startup;
 	char *description;
 	char *effect;
 	float effect_duration;
@@ -742,9 +752,17 @@ typedef struct obstacle {
 	int frame_index;
 } obstacle;
 
+typedef struct volatile_obstacle {
+	obstacle obstacle;
+	float vanish_timeout;
+	list_head_t volatile_list;
+} volatile_obstacle;
+
 typedef struct map_tile {
 	Uint16 floor_values[MAX_FLOOR_LAYERS];
 	struct dynarray glued_obstacles;
+	list_head_t volatile_obstacles;
+	int timestamp;
 } map_tile;
 
 struct obstacle_extension {
@@ -765,6 +783,8 @@ typedef struct level {
 	char *Levelname;
 	char *Background_Song_Name;
 
+	short int drop_class;
+
 	int floor_layers;
 	map_tile *map[MAX_MAP_LINES];	// this is a vector of pointers
 	int jump_target_north;
@@ -785,10 +805,11 @@ typedef struct level {
 	} random_droids;
 
 	int teleport_pair;
+	int flags;
 } level, *Level;
 
 typedef void (*action_fptr) (level *obst_lvl, int obstacle_idx);
-typedef int (*animation_fptr) (level *obst_lvl, int obstacle_idx);
+typedef int (*animation_fptr) (level *obst_lvl, void *);
 
 struct obstacle_graphics {
 	int count;
@@ -797,7 +818,9 @@ struct obstacle_graphics {
 };
 
 typedef struct obstacle_spec {
-	char *label; 
+	char *name;
+
+	char *label;
 
 	//--------------------
 	// Some obstacles will block the Tux from walking through them.
@@ -816,6 +839,7 @@ typedef struct obstacle_spec {
 
 	char block_area_type;
 	int result_type_after_smashing_once;
+	int result_type_after_looting;
 
 	unsigned int flags;
 
@@ -827,6 +851,7 @@ typedef struct obstacle_spec {
 	char transparent;
 
 	struct dynarray filenames;
+	char *action;
 	action_fptr action_fn;
 
 	//--------------------
@@ -834,6 +859,18 @@ typedef struct obstacle_spec {
 	// This property defines the function to call to animate them
 	animation_fptr animation_fn;
 	float animation_fps;
+
+	//-----------------------
+	// Some obstacles have a different sounds / blast animations
+	// these properties allow us to define a specific sound or blast for
+	// a given obstacle
+	unsigned int blast_type;
+	char *smashed_sound;
+
+	//-----------------------
+	// Some obstacles are volatiles, and vanishes after some seconds
+	float vanish_delay;     // Start to vanish after 'delay' seconds
+	float vanish_duration;  // Number of seconds needed to disappear (after the 'delay')
 } obstacle_spec;
 
 struct obstacle_group {
@@ -841,6 +878,17 @@ struct obstacle_group {
 	struct dynarray members;
 };
 
+struct floor_tile_spec {
+	int frames;	// More than 1 for animated floor tiles
+	struct dynarray filenames;
+	struct image *images;
+	struct image *current_image;
+
+	// Properties for animated floor tiles
+	animation_fptr animation_fn;
+	float animation_fps;
+};
+
 struct visible_level {
 	int valid;
 	level *lvl_pointer;
@@ -855,14 +903,6 @@ typedef struct ship {
 	level *AllLevels[MAX_LEVELS];
 } ship;
 
-typedef struct dialogue_option {
-	char *topic;
-	char *option_text;
-	int no_text;
-	luacode lua_code;
-	int exists;
-} dialogue_option;
-
 typedef struct colldet_filter {
 	int (*callback) (struct colldet_filter *filter, obstacle *obs, int obs_idx);
 	void *data;
@@ -888,7 +928,7 @@ typedef struct screen_resolution {
 
 /*
  * [way|location]_free_of_droids's execution context
- * 
+ *
  * Note : '2' excepted bots are needed when the pathfinder is called
  *         to let a bot reach an other one (used during attack, for example).
  */
@@ -903,6 +943,7 @@ typedef struct freeway_context {
 typedef struct pathfinder_context {
 	colldet_filter *dlc_filter;	// DLC filter to use
 	freeway_context *frw_ctx;	// [way|location]_free_of_droids's execution context to use
+	int timestamp;
 } pathfinder_context;
 
 typedef struct {
@@ -991,4 +1032,56 @@ struct tux_motion_class_images {
 	char part_names[ALL_PART_GROUPS][64];
 };
 
+/**
+ * Holds all variables needed to run a chat
+ */
+struct lua_coroutine {
+	lua_State *thread;
+	int nargs;
+};
+
+struct chat_context {
+	enum chat_context_state state;  // current state of the chat engine.
+	int wait_user_click;            // TRUE if the chat engine is waiting for a user click.
+	enemy *partner;                 // The bot we are talking with.
+	struct npc *npc;                // The NPC containing the specifications of the dialog to run.
+	int partner_started;            // TRUE if the dialog was started by the partner.
+	int end_dialog;                 // TRUE if a dialog lua script asked to end the dialog.
+
+	int current_option;             // Current dialog node to run (-1 if none is selected)
+	struct lua_coroutine *script_coroutine;	// Handle to the lua co-routine running the current node script
+
+	struct list_head stack_node;    // Used to create a stack of chat_context.
+};
+
+struct langspec {
+	char *name;
+	char *locale;
+};
+
+struct codeset {
+	char *language;
+	char *encoding;
+};
+
+struct title_screen {
+	char *background;
+	char *song;
+	char *text;
+};
+
+struct data_dir {
+	const char *name;
+	char path[PATH_MAX];
+};
+
+struct difficulty {
+	float droid_max_speed;
+	float droid_hpmax;
+	float droid_hostile_healing;
+	float droid_friendly_healing;
+	float droid_experience_reward;
+	float droid_aggression_distance;
+};
+
 #endif
