@@ -51,7 +51,7 @@ static struct image ToColumnBlock;
 static struct image ToLeaderBlock;
 
 //--------------------
-// Class seperation of the blocks 
+// Class separation of the blocks 
 //
 int BlockClass[TO_BLOCKS] = {
 	CONNECTOR,		// cable
@@ -100,7 +100,7 @@ point PlaygroundStart[TO_COLORS] = {
 	{PURPLE_PLAYGROUND_X, PURPLE_PLAYGROUND_Y}
 };
 
-point DruidStart[TO_COLORS] = {
+point DroidStart[TO_COLORS] = {
 	{YELLOW_DROID_X, YELLOW_DROID_Y},
 	{PURPLE_DROID_X, PURPLE_DROID_Y}
 };
@@ -108,9 +108,10 @@ point DruidStart[TO_COLORS] = {
 int CapsuleCurRow[TO_COLORS] = { 0, 0 };
 
 int LeaderColor = YELLOW;		/* momentary leading color */
+int LeaderCapsuleCount = 6;		/* capsule count of currently leading color */
 int YourColor = YELLOW;
 int OpponentColor = PURPLE;
-int OpponentType;		/* The druid-type of your opponent */
+int OpponentType;		/* The droid-type of your opponent */
 enemy *cdroid;
 
 /* the display  column */
@@ -134,7 +135,7 @@ static void ShowPlayground(enemy *target);
 
 static void display_takeover_help()
 {
-	PlayATitleFile("TakeoverInstructions.title");
+	PlayATitleFile("TakeoverInstructions.lua");
 }
 
 /** 
@@ -148,22 +149,22 @@ static void show_droid_picture(int PosX, int PosY, int type)
 	static struct image droid_images[NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION];
 	int RotationIndex;
 
-	if (!strcmp(Druidmap[type].droid_portrait_rotation_series_prefix, "NONE_AVAILABLE_YET"))
+	if (!strcmp(Droidmap[type].droid_portrait_rotation_series_prefix, "NONE_AVAILABLE_YET"))
 		return;
 
 	// Maybe we have to reload the whole image series
 	//
-	if (strcmp(LastImageSeriesPrefix, Druidmap[type].droid_portrait_rotation_series_prefix)) {
+	if (strcmp(LastImageSeriesPrefix, Droidmap[type].droid_portrait_rotation_series_prefix)) {
 		int i;
 		for (i = 0; i < NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION; i++) {
 			delete_image(&droid_images[i]);
 
-			sprintf(filename, "droids/%s/portrait_%04d.jpg", Druidmap[type].droid_portrait_rotation_series_prefix, i + 1);
+			sprintf(filename, "droids/%s/portrait_%04d.jpg", Droidmap[type].droid_portrait_rotation_series_prefix, i + 1);
 
-			load_image(&droid_images[i], filename, FALSE);
+			load_image(&droid_images[i], filename, NO_MOD);
 		}
 			
-		strcpy(LastImageSeriesPrefix, Druidmap[type].droid_portrait_rotation_series_prefix);
+		strcpy(LastImageSeriesPrefix, Droidmap[type].droid_portrait_rotation_series_prefix);
 	}
 
 	RotationIndex = (SDL_GetTicks() / 50);
@@ -191,19 +192,19 @@ static void show_droid_info(int droidtype)
 	SDL_SetClipRect(Screen, NULL);
 
 	// Show background
-	blit_background("title.jpg");
+    blit_background("console_bg1.jpg");
 	blit_background("takeover_browser.png");
 
 	// Show droid portrait
 	show_droid_picture(UNIVERSAL_COORD_W(45), UNIVERSAL_COORD_H(213), droidtype);
 
 	// Show the droid name
-	SetCurrentFont(Menu_BFont);
+	set_current_font(Menu_Font);
 	clip.x = UNIVERSAL_COORD_W(330);
 	clip.y = UNIVERSAL_COORD_H(57);
 	clip.w = UNIVERSAL_COORD_W(200);
 	clip.h = UNIVERSAL_COORD_H(30);
-	display_text_using_line_height(Druidmap[droidtype].default_short_description, clip.x, clip.y, &clip, 1.0);
+	display_text(D_(Droidmap[droidtype].default_short_description), clip.x, clip.y, &clip, 1.0);
 }
 
 /**
@@ -211,19 +212,19 @@ static void show_droid_info(int droidtype)
  */
 static void init_droid_description(struct widget_text *w, int droidtype)
 {
-	char *item_name;
+	const char *item_name;
 	int weapon_type;
 
-	autostr_append(w->text, _("Unit Type %s\n"), Druidmap[droidtype].druidname);
+	autostr_append(w->text, _("Unit Type %s\n"), Droidmap[droidtype].droidname);
 	autostr_append(w->text, _("Entry : %d\n"), droidtype + 1);
 
-	if ((weapon_type = Druidmap[droidtype].weapon_item.type) >= 0)	// make sure item != -1 
-		item_name = D_(ItemMap[weapon_type].item_name);	// does not segfault
+	if ((weapon_type = Droidmap[droidtype].weapon_item.type) >= 0)	// make sure item != -1 
+		item_name = D_(item_specs_get_name(weapon_type));	// does not segfault
 	else
 		item_name = _("none");
 
 	autostr_append(w->text, _("\nArmament : %s\n"), item_name);
-	autostr_append(w->text, _("\nCores : %d\n"), 2 + Druidmap[droidtype].class);
+	autostr_append(w->text, _("\nCores : %d\n"), 2 + Droidmap[droidtype].class);
 
 
 	if (Me.TakeoverSuccesses[droidtype]+Me.TakeoverFailures[droidtype]) {
@@ -234,7 +235,7 @@ static void init_droid_description(struct widget_text *w, int droidtype)
 		autostr_append(w->text, _("\nTakeover Success : %2d%%\n"), success_ratio);
 	}
 
-	autostr_append(w->text, _("\nNotes: %s\n"), D_(Druidmap[droidtype].notes));
+	autostr_append(w->text, _("\nNotes: %s\n"), D_(Droidmap[droidtype].notes));
 }
 
 /**
@@ -264,7 +265,7 @@ static void ChooseColor(enemy *target)
 		while (SDL_PollEvent(&event)) {
 
 			if (event.type == SDL_QUIT) {
-				Terminate(EXIT_SUCCESS, TRUE);
+				Terminate(EXIT_SUCCESS);
 			}
 
 			if (event.type == SDL_MOUSEBUTTONDOWN) {
@@ -340,7 +341,7 @@ static void PlayGame(int countdown, enemy *target)
 
 	SDL_Event event;
 
-	sprintf(count_text, _("Subliminal"));	/* Make sure a value gets assigned to count_text */
+	sprintf(count_text, " ");	/* Make sure a value gets assigned to count_text */
 	count_tick_len = 100;	/* countdown in 1/10 second steps */
 	move_tick_len = 60;	/* allow motion at this tick-speed in ms */
 
@@ -382,7 +383,7 @@ static void PlayGame(int countdown, enemy *target)
 					return;	/* leave now, to avoid changing of LeaderColor! */
 				}
 			} else if (event.type == SDL_QUIT) {
-				Terminate(EXIT_SUCCESS, TRUE);
+				Terminate(EXIT_SUCCESS);
 			}
 		}
 
@@ -514,8 +515,11 @@ static void show_info_down_button() {
  *
  * The return value is TRUE/FALSE depending on whether the game was
  * finally won/lost.
+ * The function also writes the ratio of capsules the player needed to win (out of all of his capsules) into needed_capsules_ratio
+ * - Every capsule that has not been used within the game (numCapsules) is considered as not needed
+ * - For every conquered row that exceeds 7 (the minimal number of rows to win) one capsule is considered as not needed
  */
-int droid_takeover(enemy *target)
+int droid_takeover(enemy *target, float *needed_capsules_ratio)
 {
 	int menu_finished = FALSE;
 	int reward = 0;
@@ -526,7 +530,7 @@ int droid_takeover(enemy *target)
 	widget_text_init(&droid_info, "");
 	init_droid_description(&droid_info, target->type);
 	widget_set_rect(WIDGET(&droid_info), UNIVERSAL_COORD_W(258), UNIVERSAL_COORD_H(107), UNIVERSAL_COORD_W(346), UNIVERSAL_COORD_H(282));
-	droid_info.font = FPS_Display_BFont;
+	droid_info.font = FPS_Display_Font;
 	droid_info.content_above_func = show_info_up_button;
 	droid_info.content_below_func = show_info_down_button;
 
@@ -543,21 +547,21 @@ int droid_takeover(enemy *target)
 
 	while (SpacePressed() || MouseLeftPressed()) ;	// make sure space is release before proceed 
 
-	SwitchBackgroundMusicTo(TAKEOVER_BACKGROUND_MUSIC_SOUND);
+	switch_background_music("Bleostrada.ogg");
 
 	while (!menu_finished) {
 		show_droid_info(target->type);
 		widget_text_display(WIDGET(&droid_info));
 		ShowGenericButtonFromList(TAKEOVER_HELP_BUTTON);
-		SetCurrentFont(Para_BFont);
-		CenteredPutString(Screen, GameConfig.screen_height - 30, "For more information, click the help button.");
+		set_current_font(Para_Font);
+		put_string_centered(Para_Font, GameConfig.screen_height - 30, _("For more information, click the help button."));
 		blit_mouse_cursor();
 		our_SDL_flip_wrapper();
 
 		while (SDL_PollEvent(&event)) {
 
 			if (event.type == SDL_QUIT) {
-				Terminate(EXIT_SUCCESS, TRUE);
+				Terminate(EXIT_SUCCESS);
 			}
 
 			if (event.type == SDL_MOUSEBUTTONDOWN && event.button.button == SDL_BUTTON_LEFT) {
@@ -581,15 +585,17 @@ int droid_takeover(enemy *target)
 	while (!(!SpacePressed() && !EscapePressed() && !MouseLeftPressed())) ;
 
 	int player_capsules = 2 + Me.skill_level[get_program_index_with_name("Hacking")];
-	max_opponent_capsules = 2 + Druidmap[target->type].class;
+	max_opponent_capsules = 2 + Droidmap[target->type].class;
 
 	if (do_takeover(player_capsules, max_opponent_capsules, 100, target)) {
 		/* Won takeover */
 		Me.marker = target->marker;
 
-		reward = Druidmap[target->type].experience_reward * Me.experience_factor;
+		reward = Droidmap[target->type].experience_reward * Me.experience_factor;
 		Me.Experience += reward;
-		append_new_game_message(_("For taking control of your enemy, \4%s\5, you receive %d experience."), target->short_description_text, reward);
+		append_new_game_message(_("For taking control of your enemy, [s]%s[v], you receive %d experience."), D_(target->short_description_text), reward);
+
+		event_enemy_hacked(target);
 
 		// Maybe the enemy in question was a kind of 'boss monster' or it had
 		// some special item, that is relevant to a mission or quest.  In that
@@ -616,7 +622,8 @@ int droid_takeover(enemy *target)
 		// same marker) is attacked by the Tux.
 		//
 		target->marker = 0;
-                Me.TakeoverSuccesses[target->type]++;
+		Me.TakeoverSuccesses[target->type]++;
+
 	} else {
 		Me.energy *= 0.5;
                 Me.TakeoverFailures[target->type]++;
@@ -624,13 +631,18 @@ int droid_takeover(enemy *target)
 
 	clear_screen();
 
-	SwitchBackgroundMusicTo(CURLEVEL()->Background_Song_Name);
+	switch_background_music(CURLEVEL()->Background_Song_Name);
+
+	if (LeaderColor == YourColor) {
+		// set the ratio of needed capsules out of all (player) capsules
+		// - Every capsule that has not been used within the game (numCapsules) is considered as not needed
+		// - For every conquered row that exceeds 7 (the minimal number of rows to win) one capsule is considered as not needed
+		*needed_capsules_ratio = 1 - (float)(NumCapsules[YOU] + LeaderCapsuleCount - 7) / (float)player_capsules;
 
-	if (LeaderColor == YourColor)
 		return TRUE;
-	else
-		return FALSE;
-
+	} else {
+ 		return FALSE;
+	}
 };				// int Takeover( int enemynum ) 
 
 /*-----------------------------------------------------------------
@@ -759,7 +771,7 @@ static void GetTakeoverGraphics(void)
 	if (TakeoverGraphicsAreAlreadyLoaded)
 		return;
 
-	load_image(&img, TO_BLOCK_FILE, FALSE);
+	load_image(&img, TO_BLOCK_FILE, NO_MOD);
 
 	// Get the fill-blocks 
 	for (i = 0; i < NUM_FILL_BLOCKS; i++, curx += FILL_BLOCK_LEN + 2) {
@@ -912,35 +924,36 @@ static void ShowPlayground(enemy * target)
 	int i, j;
 	int color, player;
 	int block;
-	int xoffs, yoffs;
 	SDL_Rect Target_Rect;
-	int phase, dheight;
-	xoffs = User_Rect.x + (User_Rect.w - 2 * 290) / 2;
-	yoffs = User_Rect.y + (User_Rect.h - 2 * 140) / 2 + 30;
+	int xoffs = User_Rect.x + (User_Rect.w - 2 * 290) / 2;
+	int yoffs = User_Rect.y + (User_Rect.h - 2 * 140) / 2 + 40;
 
-	phase = (int) Me.phase;
+	blit_background("console_bg1.jpg");
+    
+	static struct image bg;
+	if (!image_loaded(&bg)) {
+		load_image(&bg, "backgrounds/takeover_console.png", NO_MOD);
+	}
 
-	blit_background("transfer.jpg");
+	display_image_on_screen(&bg, GameConfig.screen_width / 2 - 340, GameConfig.screen_height / 2 - 294, IMAGE_NO_TRANSFO);
 
 	if (target) {
-		// Find the difference of the droid's height and Tux's height.
-		// Tux's height is measured from the top of his head to the bottom of his feet
-		// This also accounts for the droid's blitting offset
-		struct tux_motion_class_images *current_motion_class_images = &tux_images[get_motion_class_id()];
-		dheight = current_motion_class_images->part_images[PART_GROUP_FEET][0][phase].offset_y +
-			current_motion_class_images->part_images[PART_GROUP_FEET][0][phase].h -
-			current_motion_class_images->part_images[PART_GROUP_HEAD][0][phase].offset_y -
-			enemy_images[target->type][0][0].h -
-			enemy_images[target->type][0][0].offset_y;
-
-		// Offset the droid's drawing rectangle by dheight minus 30 units.
-		Set_Rect(Target_Rect, xoffs + DruidStart[!YourColor].x, yoffs + dheight - 30, User_Rect.w, User_Rect.h);
+		int rotation_model = set_rotation_model_for_this_robot(target);
+		int offsetx = -enemy_images[rotation_model][0][0].offset_x;
+		int offsety = -enemy_images[rotation_model][0][0].offset_y;
+		Set_Rect(Target_Rect, xoffs + DroidStart[!YourColor].x + 20 + offsetx, (yoffs - 80) + offsety, User_Rect.w, User_Rect.h);
 		PutIndividuallyShapedDroidBody(target, Target_Rect, FALSE, FALSE);
 	}
 	//  SDL_SetColorKey (Screen, 0, 0);
 	SDL_SetClipRect(Screen, &User_Rect);
 
-	blit_tux(xoffs + DruidStart[YourColor].x, yoffs);
+	struct tux_motion_class_images *current_motion_class_images = &tux_images[get_motion_class_id()];
+	int phase = tux_anim.standing_keyframe;
+	struct image *tux_feet_image = &current_motion_class_images->part_images[PART_GROUP_FEET][phase][0];
+	struct image *tux_head_image = &current_motion_class_images->part_images[PART_GROUP_HEAD][phase][0];
+	int tux_height = (tux_feet_image->offset_y + tux_feet_image->h) - (tux_head_image->offset_y);
+
+	blit_tux(xoffs + DroidStart[YourColor].x + 20, (yoffs - 80) + (tux_height / 2) + tux_feet_image->offset_y);
 
 	Set_Rect(Target_Rect, xoffs + LEFT_OFFS_X, yoffs + LEFT_OFFS_Y, User_Rect.w, User_Rect.h);
 
@@ -1046,6 +1059,7 @@ void ClearPlayground(void)
 		for (layer = 0; layer < NUM_LAYERS; layer++)
 			for (row = 0; row < NUM_LINES; row++) {
 				ActivationMap[color][layer][row] = INACTIVE;
+				// cppcheck-suppress duplicateBranch
 				if (layer < TO_COLORS - 1)
 					ToPlayground[color][layer][row] = CABLE;
 				else
@@ -1077,7 +1091,7 @@ void InventPlayground(void)
 				if (ToPlayground[color][layer][row] != CABLE)
 					continue;
 
-				newElement = MyRandom(TO_ELEMENTS);
+				newElement = MyRandom(TO_ELEMENTS - 1);
 				if (MyRandom(MAX_PROB) > ElementProb[newElement]) {
 					row--;
 					continue;
@@ -1554,7 +1568,7 @@ void ProcessDisplayColumn(void)
 	static int CLayer = 3;	/* the connection-layer to the Column */
 	static int flicker_color = 0;
 	int row;
-	int GelbCounter, ViolettCounter;
+	int YellowCounter, PurpleCounter;
 
 	flicker_color = !flicker_color;
 
@@ -1599,17 +1613,18 @@ void ProcessDisplayColumn(void)
 	}			/* for */
 
 	// evaluate the winning color
-	GelbCounter = 0;
-	ViolettCounter = 0;
+	YellowCounter = 0;
+	PurpleCounter = 0;
 	for (row = 0; row < NUM_LINES; row++)
 		if (DisplayColumn[row] == YELLOW)
-			GelbCounter++;
+			YellowCounter++;
 		else
-			ViolettCounter++;
+			PurpleCounter++;
 
-	if (ViolettCounter < GelbCounter)
+	LeaderCapsuleCount = (PurpleCounter > YellowCounter) ? PurpleCounter : YellowCounter;
+	if (PurpleCounter < YellowCounter)
 		LeaderColor = YELLOW;
-	else if (ViolettCounter > GelbCounter)
+	else if (PurpleCounter > YellowCounter)
 		LeaderColor = PURPLE;
 	else
 		LeaderColor = DRAW;
@@ -1701,12 +1716,12 @@ void to_show_banner(const char *left, const char *right)
 	left_len = strlen(left);
 	if (left_len > LEFT_TEXT_LEN) {
 		printf("\nWarning: String %s too long for Left Infoline!!", left);
-		Terminate(EXIT_FAILURE, TRUE);
+		Terminate(EXIT_FAILURE);
 	}
 	right_len = strlen(right);
 	if (right_len > RIGHT_TEXT_LEN) {
 		printf("\nWarning: String %s too long for Right Infoline!!", right);
-		Terminate(EXIT_FAILURE, TRUE);
+		Terminate(EXIT_FAILURE);
 	}
 	// Now prepare the left/right text-boxes 
 	memset(left_box, ' ', LEFT_TEXT_LEN);	// pad with spaces 
@@ -1721,9 +1736,9 @@ void to_show_banner(const char *left, const char *right)
 	// Now the text should be ready and its
 	// time to display it...
 	DebugPrintf(2, "Takeover said: %s -- %s\n", left_box, right_box);
-	SetCurrentFont(Para_BFont);
-	display_text(left_box, LEFT_INFO_X, LEFT_INFO_Y, NULL);
-	display_text(right_box, RIGHT_INFO_X, RIGHT_INFO_Y, NULL);
+	set_current_font(Para_Font);
+	display_text(left_box, LEFT_INFO_X, LEFT_INFO_Y, NULL, 1.0);
+	display_text(right_box, RIGHT_INFO_X, RIGHT_INFO_Y, NULL, 1.0);
 
 };				// void to_show_banner (const char* left, const char* right)
 
