@@ -57,7 +57,7 @@ int get_lines_needed(const char *text, SDL_Rect t_rect, float line_height_factor
 {
 	t_rect.h = 32767; // arbitrary large number
 	display_char_disabled = TRUE;
-	int lines = display_text_using_line_height(text, t_rect.x, t_rect.y, &t_rect, line_height_factor);
+	int lines = display_text(text, t_rect.x, t_rect.y, &t_rect, line_height_factor);
 	display_char_disabled = FALSE;
 	return lines;
 }
@@ -72,13 +72,13 @@ void show_backgrounded_label_at_pixel_position(char *LabelText, int pos_x, int p
 
 	background_rect.x = pos_x - 1;
 	background_rect.y = pos_y;
-	background_rect.w = TextWidth(LabelText) + 2;
+	background_rect.w = text_width(get_current_font(), LabelText) + 2;
 	background_rect.h = 20;
 
 	draw_rectangle(&background_rect, 0, 0, 0, 255);
 
-	SetCurrentFont(FPS_Display_BFont);
-	PutString(Screen, pos_x, pos_y, LabelText);
+	set_current_font(FPS_Display_Font);
+	put_string(get_current_font(), pos_x, pos_y, LabelText);
 
 };				// void show_backgrounded_label_at_pixel_position ( char* LabelText , float fill_status , int pos_x , int pos_y )
 
@@ -100,12 +100,12 @@ void show_backgrounded_label_at_map_position(char *LabelText, float fill_status,
 /**
  * Show text inside a filled rectangle. The specified height is taken as a
  * minimum value, and it will be expanded vertically to fit the given text.
- * @Ret: The height of the rectangle
+ * @return The height of the rectangle
  */
-int show_backgrounded_text_rectangle(const char *text, struct BFont_Info *font, int x, int y, int w, int h)
+int show_backgrounded_text_rectangle(const char *text, struct font *font, int x, int y, int w, int h)
 {
-	BFont_Info *old_font = GetCurrentFont();
-	SetCurrentFont(font);
+	struct font *old_font = get_current_font();
+	set_current_font(font);
 
 	SDL_Rect t_rect;
 	t_rect.x = x;
@@ -118,7 +118,7 @@ int show_backgrounded_text_rectangle(const char *text, struct BFont_Info *font,
 	int lines = get_lines_needed(text, t_rect, 1.0);
 
 	// Calculate the rectangle height
-	int f_height = FontHeight(GetCurrentFont());
+	int f_height = get_font_height(get_current_font());
 	int r_height = (lines * f_height) + (IN_WINDOW_TEXT_OFFSET * 2);
 
 	// Set up and fill the rectangle.
@@ -131,69 +131,12 @@ int show_backgrounded_text_rectangle(const char *text, struct BFont_Info *font,
 	t_rect.h -= IN_WINDOW_TEXT_OFFSET;
 	t_rect.x += IN_WINDOW_TEXT_OFFSET;
 	t_rect.y += IN_WINDOW_TEXT_OFFSET;
-	display_text_using_line_height(text, t_rect.x, t_rect.y, &t_rect, 1.0);
+	display_text(text, t_rect.x, t_rect.y, &t_rect, 1.0);
 
-	SetCurrentFont(old_font);
+	set_current_font(old_font);
 	return r_height;
 }
 
-/**
- * In some cases it will be necessary to inform the user of something in
- * a big important style.  Then a popup window is suitable, with a mouse
- * click to confirm and make it go away again.
- */
-void alert_window(const char *text, ...)
-{
-	if (!SDL_WasInit(SDL_INIT_VIDEO))
-		return;
-
-	va_list args;
-	struct auto_string *buffer = alloc_autostr(256);
-	int w = 440;   // arbitrary
-	int h = 60;    // arbitrary
-	int x = (GameConfig.screen_width  - w) / 2;	// center of screen
-	int y = (GameConfig.screen_height - h) / 5 * 2; // 2/5 of screen from top
-	va_start(args, text);
-	autostr_vappend(buffer, text, args);
-	va_end(args);
-
-	Activate_Conservative_Frame_Computation();
-	StoreMenuBackground(1);
-
-	SDL_Event e;
-	while (1) {
-		SDL_Delay(1);
-		RestoreMenuBackground(1);
-
-		int r_height = show_backgrounded_text_rectangle(buffer->value, FPS_Display_BFont, x, y, w, h);
-		show_backgrounded_text_rectangle(_("Click to continue..."), Red_BFont, x, y + r_height, w, 10);
-
-		blit_mouse_cursor();
-		our_SDL_flip_wrapper();
-		save_mouse_state();
-		
-		SDL_WaitEvent(&e);
-		
-		switch (e.type) {
-		case SDL_QUIT:
-			Terminate(EXIT_SUCCESS, TRUE);
-			break;
-		case SDL_KEYDOWN:
-			if (e.key.keysym.sym == SDLK_SPACE || e.key.keysym.sym == SDLK_RETURN || e.key.keysym.sym == SDLK_ESCAPE)
-				goto wait_click_and_out;
-			break;
-		case SDL_MOUSEBUTTONDOWN:
-			if (e.button.button == 1)
-				goto wait_click_and_out;
-			break;
-		}
-	}
-
-wait_click_and_out:
-	while (SpacePressed() || EnterPressed() || EscapePressed() || MouseLeftPressed());
-	free_autostr(buffer);
-}
-
 /**
  * 
  */
@@ -202,10 +145,10 @@ int CutDownStringToMaximalSize(char *StringToCut, int LengthInPixels)
 	int StringIndex = 0;
 	int i;
 
-	if (TextWidth(StringToCut) <= LengthInPixels)
+	if (text_width(get_current_font(), StringToCut) <= LengthInPixels)
 		return FALSE;
 
-	StringIndex = LimitTextWidth(StringToCut, LengthInPixels);
+	StringIndex = limit_text_width(get_current_font(), StringToCut, LengthInPixels);
 	if (StringIndex < 1)
 		return FALSE;
 
@@ -238,7 +181,7 @@ int ScrollText(char *Text, const char *background_name)
 	Activate_Conservative_Frame_Computation();
 
 	SDL_Rect ScrollRect = { User_Rect.x + 10, User_Rect.y, User_Rect.w - 20, User_Rect.h };
-	SetCurrentFont(Para_BFont);
+	set_current_font(Para_Font);
 	StartInsertLine = ScrollRect.y + ScrollRect.h;
 	InsertLine = StartInsertLine;
 
@@ -246,7 +189,7 @@ int ScrollText(char *Text, const char *background_name)
 		
 		if (background_name)
 			blit_background(background_name);
-		Number_Of_Line_Feeds = display_text(Text, ScrollRect.x, InsertLine, &ScrollRect);
+		Number_Of_Line_Feeds = display_text(Text, ScrollRect.x, InsertLine, &ScrollRect, 1.0);
 		// We might add some buttons to be displayed here, so that, if you don't have
 		// a mouse wheel and don't know about cursor keys, you can still click on these
 		// buttons to control the scrolling speed of the text.
@@ -311,9 +254,9 @@ int ScrollText(char *Text, const char *background_name)
 			InsertLine = StartInsertLine;
 			speed = 0;
 		}
-		if (InsertLine + (Number_Of_Line_Feeds + 1) * (int)(FontHeight(GetCurrentFont()) * LINE_HEIGHT_FACTOR) < ScrollRect.y
+		if (InsertLine + (Number_Of_Line_Feeds + 1) * get_font_height(get_current_font()) < ScrollRect.y
 		    && (speed > 0)) {
-			InsertLine = ScrollRect.y - (Number_Of_Line_Feeds + 1) * (int)(FontHeight(GetCurrentFont()) * LINE_HEIGHT_FACTOR);
+			InsertLine = ScrollRect.y - (Number_Of_Line_Feeds + 1) * get_font_height(get_current_font());
 			speed = 0;
 		}
 
@@ -371,20 +314,72 @@ void DisplayBigScreenMessage(void)
 		if (Me.BigScreenMessageDuration[i] < GameConfig.delay_for_big_screen_messages) {
 			SDL_SetClipRect(Screen, NULL);
 
-			SetCurrentFont(Menu_BFont);
-			int x_pos = GameConfig.screen_width/2 - TextWidthFont(GetCurrentFont(), text)/2;
-			display_text_using_line_height(text, x_pos, y_pos, NULL /* clip */, 1.0);
+			set_current_font(Menu_Font);
+			int x_pos = GameConfig.screen_width/2 - text_width(get_current_font(), text)/2;
+			display_text(text, x_pos, y_pos, NULL /* clip */, 1.0);
 
 			if (!GameConfig.Inventory_Visible && !GameConfig.SkillScreen_Visible && !GameConfig.CharacterScreen_Visible)
 				Me.BigScreenMessageDuration[i] += Frame_Time();
 
-			y_pos += FontHeight(Menu_BFont);
+			y_pos += get_font_height(Menu_Font);
 		}
 	}
 
 };				// void DisplayBigScreenMessage( void )
 
-static int display_text_using_line_height_with_cursor(const char *text, int startx, int starty, const SDL_Rect *clip, float line_height_factor, int curpos)
+static void display_countdown(int pixel_x, int pixel_y, const char* message, float duration)
+{
+	char text[100];
+	int minutes = ((int) duration) / 60;
+	int seconds = ((int) duration) - (minutes * 60);
+
+	if (minutes >= 60) {
+		sprintf(text, "%s", message);
+	} else if (minutes) {
+		sprintf(text, "%s %dm%ds", message, minutes, seconds);
+	} else {
+		sprintf(text, "%s %.1fs", message, duration);
+	}
+
+	pixel_x -= text_width(get_current_font(), text) / 2;
+
+	display_text(text, pixel_x, pixel_y, NULL /* clip */, 1.0);
+}
+
+/**
+ * This function displays the remaining seconds of effect on Tux above him.
+ */
+void display_effect_countdowns(void)
+{
+	int pixel_x, pixel_y;
+	translate_map_point_to_screen_pixel(Me.pos.x, Me.pos.y, &pixel_x, &pixel_y);
+	pixel_y -= 128; /* tux height... */
+
+	if (Me.slowdown_duration > 0.0) {
+		set_current_font(Blue_Font);
+		display_countdown(pixel_x, pixel_y, _("slowed"), Me.slowdown_duration);
+		pixel_y -= get_font_height(Blue_Font);
+	}
+
+	if (Me.paralyze_duration > 0.0) {
+		set_current_font(Red_Font);
+		display_countdown(pixel_x, pixel_y, _("paralyzed"), Me.paralyze_duration);
+		pixel_y -= get_font_height(Red_Font);
+	}
+
+	if (Me.invisible_duration > 0.0) {
+		set_current_font(FPS_Display_Font);
+		display_countdown(pixel_x, pixel_y, _("invisible"), Me.invisible_duration);
+		pixel_y -= get_font_height(FPS_Display_Font);
+	}
+
+	if (Me.nmap_duration > 0.0) {
+		set_current_font(FPS_Display_Font);
+		display_countdown(pixel_x, pixel_y, _("scanning"), Me.nmap_duration);
+	}
+};		// void display_effect_countdowns( void )
+
+static int display_text_with_cursor(const char *text, int startx, int starty, const SDL_Rect *clip, float line_height_factor, int curpos)
 {
 	char *tmp;		// mobile pointer to the current position in the string to be printed
 	SDL_Rect Temp_Clipping_Rect;	// adding this to prevent segfault in case of NULL as parameter
@@ -393,10 +388,13 @@ static int display_text_using_line_height_with_cursor(const char *text, int star
 	int empty_lines_started = 0;
 	int current_curpos = 0;
 
-	int letter_spacing = get_letter_spacing(GetCurrentFont());
-	int tab_width = TABWIDTH * (CharWidth(GetCurrentFont(), TABCHAR) + letter_spacing);
+	if (text == NULL)
+		return 0;
 
-	if (!*text)
+	int letter_spacing = get_letter_spacing(get_current_font());
+	int tab_width = TABWIDTH * (font_char_width(get_current_font(), TABCHAR) + letter_spacing);
+
+	if (text[0]=='\0')
 		nblines = 0;
 
 	// We position the internal text cursor on the right spot for
@@ -442,8 +440,7 @@ static int display_text_using_line_height_with_cursor(const char *text, int star
 			cursor_y = MyCursorY;
 		}
 
-		if (handle_switch_font_char(*tmp)) {
-			tmp++;
+		if (handle_switch_font_char(&tmp)) {
 			current_curpos++;
 			continue;
 		}
@@ -458,28 +455,28 @@ static int display_text_using_line_height_with_cursor(const char *text, int star
 		}
 
 		// carriage return in the middle of a word if it is too big to fit on one line
-		if (isgraph(*tmp) && (MyCursorX + CharWidth(GetCurrentFont(), *tmp) + letter_spacing) > (clip->x + clip->w)) {
+		if (isgraph(*tmp) && (MyCursorX + font_char_width(get_current_font(), *tmp) + letter_spacing) > (clip->x + clip->w)) {
 			MyCursorX = clip->x;
-			MyCursorY += (int)(FontHeight(GetCurrentFont()) * line_height_factor);
+			MyCursorY += (int)(get_font_height(get_current_font()) * line_height_factor);
 			empty_lines_started++;
 		}
 
 		switch (*tmp) {
 		case '\n':
 			MyCursorX = clip->x;
-			MyCursorY += (int)(FontHeight(GetCurrentFont()) * line_height_factor);
+			MyCursorY += (int)(get_font_height(get_current_font()) * line_height_factor);
 			empty_lines_started++;
 			break;
 		case '\t':
 			MyCursorX = (int)ceilf((float)MyCursorX / (float)(tab_width)) * (tab_width);
 			break;
 		default:
-			if (MyCursorY > clip->y - (int)(FontHeight(GetCurrentFont()) * line_height_factor)
+			if (MyCursorY > clip->y - (int)(get_font_height(get_current_font()) * line_height_factor)
 				&& !display_char_disabled)
-				PutCharFont(Screen, GetCurrentFont(), MyCursorX, MyCursorY, *tmp);
+				put_char(get_current_font(), MyCursorX, MyCursorY, *tmp);
 
-			MyCursorX += CharWidth(GetCurrentFont(), *tmp)
-				+ get_letter_spacing(GetCurrentFont());
+			MyCursorX += font_char_width(get_current_font(), *tmp)
+				+ get_letter_spacing(get_current_font());
 
 			// At least one visible character must follow a line break or else
 			// the line is a trailing empty line not visible to the user. Such
@@ -506,7 +503,7 @@ static int display_text_using_line_height_with_cursor(const char *text, int star
 		SDL_Rect cursor_rect;
 		cursor_rect.x = cursor_x;
 		cursor_rect.y = cursor_y;
-		cursor_rect.h = FontHeight(GetCurrentFont());
+		cursor_rect.h = get_font_height(get_current_font());
 		cursor_rect.w = 8;
 		HighlightRectangle(Screen, cursor_rect);
 	}
@@ -524,17 +521,9 @@ static int display_text_using_line_height_with_cursor(const char *text, int star
  * @return number of lines written (from the first text line up to the
  *         last displayed line)
  */
-int display_text_using_line_height(const char *text, int startx, int starty, const SDL_Rect *clip, float line_height_factor)
+int display_text(const char *text, int startx, int starty, const SDL_Rect *clip, float line_height_factor)
 {
-	return display_text_using_line_height_with_cursor(text, startx, starty, clip, line_height_factor, -1);
-}
-
-/**
- * Display text using the default line height.
- */
-int display_text(const char *text, int x, int y, const SDL_Rect* clip_rect)
-{
-	return display_text_using_line_height(text, x, y, clip_rect, LINE_HEIGHT_FACTOR);
+	return display_text_with_cursor(text, startx, starty, clip, line_height_factor, -1);
 }
 
 /**
@@ -552,22 +541,22 @@ int ImprovedCheckLineBreak(char *Resttext, const SDL_Rect * clip, float line_hei
 {
 	int NeededSpace = 0;
 
-	int letter_spacing = get_letter_spacing(GetCurrentFont());
+	int letter_spacing = get_letter_spacing(get_current_font());
 
 	if (*Resttext == ' ')
-		NeededSpace = CharWidth(GetCurrentFont(), ' ') + letter_spacing;
+		NeededSpace = font_char_width(get_current_font(), ' ') + letter_spacing;
 	else if (*Resttext == '\t')
-		NeededSpace = TABWIDTH * (CharWidth(GetCurrentFont(), TABCHAR) + letter_spacing);
+		NeededSpace = TABWIDTH * (font_char_width(get_current_font(), TABCHAR) + letter_spacing);
 
 	Resttext++;
 	while ((*Resttext != ' ') && (*Resttext != '\t') && (*Resttext != '\n') && (*Resttext != 0)) {
-		NeededSpace += CharWidth(GetCurrentFont(), *Resttext) + letter_spacing;
+		NeededSpace += font_char_width(get_current_font(), *Resttext) + letter_spacing;
 		Resttext++;
 	}
 
 	if ((MyCursorX + NeededSpace) > (clip->x + clip->w)) {
 		MyCursorX = clip->x;
-		MyCursorY += (int)(FontHeight(GetCurrentFont()) * line_height_factor);
+		MyCursorY += (int)(get_font_height(get_current_font()) * line_height_factor);
 		return 1;
 	}
 
@@ -585,7 +574,7 @@ char *get_string(int MaxLen, const char *background_name, const char *text_for_o
 	int finished;
 	int x0, y0;
 
-	display_text(text_for_overhead_promt, 50, 50, NULL);
+	display_text(text_for_overhead_promt, 50, 50, NULL, 1.0);
 
 	// allocate memory for the users input
 	input = MyMalloc(MaxLen + 5);
@@ -598,12 +587,12 @@ char *get_string(int MaxLen, const char *background_name, const char *text_for_o
 
 	while (!finished) {
 		blit_background(background_name);
-		display_text(text_for_overhead_promt, 50, 50, NULL);
+		display_text(text_for_overhead_promt, 50, 50, NULL, 1.0);
 
 		x0 = MyCursorX;
 		y0 = MyCursorY;
 
-		PutString(Screen, x0, y0, input);
+		put_string(get_current_font(), x0, y0, input);
 		our_SDL_flip_wrapper();
 
 		key = getchar_raw(NULL);
@@ -613,10 +602,10 @@ char *get_string(int MaxLen, const char *background_name, const char *text_for_o
 			// useful for GL drivers that do true pageflipping (win32, nvidia 173.x, ...)
 			if (use_open_gl) {
 				blit_background(background_name);
-				display_text(text_for_overhead_promt, 50, 50, NULL);
+				display_text(text_for_overhead_promt, 50, 50, NULL, 1.0);
 				x0 = MyCursorX;
 				y0 = MyCursorY;
-				PutString(Screen, x0, y0, input);
+				put_string(get_current_font(), x0, y0, input);
 				our_SDL_flip_wrapper();
 			}
 			input[curpos] = 0;
@@ -682,7 +671,7 @@ char *GetEditableStringInPopupWindow(int MaxLen, const char *PopupWindowTitle, c
 	// Now we find the right position for the new string to start by writing
 	// out the title text once, just to get the cursor positioned right...
 	//
-	display_text(PopupWindowTitle, TargetRect.x, TargetRect.y, &TargetRect);
+	display_text(PopupWindowTitle, TargetRect.x, TargetRect.y, &TargetRect, 1.0);
 	x0 = MyCursorX;
 	y0 = MyCursorY;
 
@@ -703,16 +692,16 @@ char *GetEditableStringInPopupWindow(int MaxLen, const char *PopupWindowTitle, c
 		TargetRect.x += EDIT_WINDOW_TEXT_OFFSET;
 		TargetRect.y += EDIT_WINDOW_TEXT_OFFSET;
 
-		SetCurrentFont(FPS_Display_BFont);
+		set_current_font(FPS_Display_Font);
 
-		display_text(PopupWindowTitle, TargetRect.x, TargetRect.y, &TargetRect);
+		display_text(PopupWindowTitle, TargetRect.x, TargetRect.y, &TargetRect, 1.0);
 
 		if (PopupWindowTitle[strlen(PopupWindowTitle) - 1] != '\n')
-			display_text("\n\n", x0, y0, &TargetRect);
+			display_text("\n\n", x0, y0, &TargetRect, 1.0);
 
 		TargetRect.x = MyCursorX;
 		TargetRect.y = MyCursorY;
-		display_text_using_line_height_with_cursor(input, TargetRect.x, TargetRect.y, &TargetRect, LINE_HEIGHT_FACTOR, curpos);
+		display_text_with_cursor(input, TargetRect.x, TargetRect.y, &TargetRect, 1.0, curpos);
 
 		our_SDL_flip_wrapper();
 
@@ -818,16 +807,16 @@ void printf_SDL(SDL_Surface * screen, int x, int y, const char *fmt, ...)
 
 	tmp = (char *)MyMalloc(10000 + 1);
 	vsprintf(tmp, fmt, args);
-	PutString(screen, x, y, tmp);
+	put_string(get_current_font(), x, y, tmp);
 
 	// our_SDL_flip_wrapper (screen);
 
 	if (tmp[strlen(tmp) - 1] == '\n') {
 		MyCursorX = x;
-		MyCursorY = y + 1.1 * (GetCurrentFont()->h);
+		MyCursorY = y + 1.1 * (get_font_height(get_current_font()));
 	} else {
 		for (i = 0; i < ((int)strlen(tmp)); i++)
-			MyCursorX += CharWidth(GetCurrentFont(), tmp[i]);
+			MyCursorX += font_char_width(get_current_font(), tmp[i]);
 		MyCursorY = y;
 	}
 
@@ -854,7 +843,7 @@ int longest_line_width(char *text)
 		// Get the width of the current line.
 		char tmp = *text;
 		*text = '\0';
-		int line_width = TextWidth(line_start);
+		int line_width = text_width(get_current_font(), line_start);
 		*text = tmp;
 
 		// Update the width of the longest line.
@@ -866,9 +855,11 @@ int longest_line_width(char *text)
 		if (*text != '\0') {
 			line_start = text + 1;
 		} else {
-			return width;
+			break;
 		}
 	}
+
+	return width;
 }
 
 #undef _text_c
