@@ -39,9 +39,19 @@
 
 #include <zlib.h>
 
-extern int Number_Of_Item_Types;
 extern int debug_level;
 
+// A store for alert window or console error messages.
+// Used in alert_once_window() and error_once_window() to avoid to display
+// several the same message.
+
+struct msg_store_item {
+	int when;                 // ONCE_PER_RUN / ONCE_PER_GAME - see defs.h
+	struct auto_string *msg;  // The message already displayed.
+};
+
+static struct dynarray *msg_store = NULL;
+
 /** 
  * This function works a malloc, except that it also checks for
  * success and terminates in case of "out of memory", so we don't
@@ -58,13 +68,33 @@ void *MyMalloc(long Mamount)
 	if ((Mptr = calloc(1, (size_t) Mamount)) == NULL) {
 		fprintf(stderr, " MyMalloc(%ld) did not succeed!\n", Mamount);
 		fflush(stderr);
-		Terminate(EXIT_FAILURE, TRUE);
+		Terminate(EXIT_FAILURE);
 	}
 
 	return Mptr;
 
 };				// void* MyMalloc ( long Mamount )
 
+/**
+ * This function works as strdup, except that it also checks for
+ * success and terminates in case of "out of memory", so we don't
+ * need to do this always in the code.
+ */
+char *my_strdup(const char *src)
+{
+	if (src == NULL)
+		return NULL;
+
+	char *dst = strdup(src);
+	if (dst == NULL) {
+		fprintf(stderr, " my_strdup() did not succeed!\n");
+		fflush(stderr);
+		Terminate(EXIT_FAILURE);
+	}
+
+	return dst;
+}
+
 /**
  * This function is used for debugging purposes.  It writes the
  * given string on the screen, or simply does
@@ -82,49 +112,256 @@ void DebugPrintf(int db_level, const char *fmt, ...)
 	va_end(args);
 };				// void DebugPrintf ( int db_level, char *fmt, ...)
 
+/*
+ * Check if a msg is already in the error msg store. If not, register it.
+ * Return TRUE if the msg was not yet in the store, FALSE otherwise.
+ */
+static int _first_use_of_msg(int when, struct auto_string *msg)
+{
+	// A dynarray is used to store past alerts/errors, to remember which message was
+	// already displayed.
+
+	// On first use, initialize the msg store.
+
+	if (!msg_store)
+		msg_store = dynarray_alloc(10, sizeof(struct msg_store_item));
+
+	// If the message is already in the store, immediately return FALSE
+
+	int i;
+	for (i = 0; i < msg_store->size; i++) {
+		struct msg_store_item *stored_item = (struct msg_store_item *)dynarray_member(msg_store, i, sizeof(struct msg_store_item));
+		if (!strcmp(msg->value, stored_item->msg->value)) {
+			return FALSE;
+		}
+	}
+
+	// The message was not yet registered. Store it and return TRUE
+
+	struct msg_store_item *stored_item = (struct msg_store_item *)MyMalloc(sizeof(struct msg_store_item));
+	stored_item->when = when;
+	stored_item->msg = msg;
+	dynarray_add(msg_store, stored_item, sizeof(struct msg_store_item));
+	free(stored_item);
+
+	return TRUE;
+}
+
+/**
+ * Clean the alert/error message store, removing messages which are flagged as ONCE_PER_GAME
+ * so that they can be redisplayed
+ */
+void clean_error_msg_store()
+{
+	if (!msg_store) // msg store not yet used
+		return;
+
+	// We have to remove the item flagged as ONCE_PER_GAME, but we have to keep
+	// the others. We can not delete a dynarray member inside a loop over the
+	// dynarray. We thus create a new dynarray to replace the current one,
+	// containing the kept items.
+
+	struct dynarray *new_store = dynarray_alloc(10, sizeof(struct msg_store_item));
+
+	int i;
+	for (i = 0; i < msg_store->size; i++) {
+		struct msg_store_item *stored_item = (struct msg_store_item *)dynarray_member(msg_store, i, sizeof(struct msg_store_item));
+		if (stored_item->when == ONCE_PER_GAME) {
+			// free memory
+			free_autostr(stored_item->msg);
+			stored_item->msg = NULL;
+		} else {
+			// copy the item in the new store
+			dynarray_add(new_store, stored_item, sizeof(struct msg_store_item));
+		}
+	}
+	dynarray_free(msg_store);
+	free(msg_store);
+	msg_store = new_store;
+}
+
+/**
+ * Free all the memory slots used by the msg_store
+ */
+void free_error_msg_store()
+{
+	if (!msg_store) // msg store was not used
+		return;
+
+	int i;
+	for (i = 0; i < msg_store->size; i++) {
+		struct msg_store_item *stored_item = (struct msg_store_item *)dynarray_member(msg_store, i, sizeof(struct msg_store_item));
+		free_autostr(stored_item->msg);
+	}
+	dynarray_free(msg_store);
+	free(msg_store);
+}
+
 /**
  * This function should help to simplify and standardize the many error
  * messages possible in FreedroidRPG.
  */
-void ErrorMessage(const char *FunctionName, const char *fmt, int InformDevelopers, int IsFatal, ...)
+void error_message(const char *fn, const char *fmt, int error_type, ...)
 {
+	if (error_type & SILENT)
+		return;
+
 	va_list args;
-	va_start(args, IsFatal);
+	va_start(args, error_type);
 
-	fprintf(stderr, "\n---------------------------------------------------------------------------------\n\
-FreedroidRPG %s encountered a problem ", VERSION);
-	fprintf(stderr, "in function:  %s \n", FunctionName);
+	fprintf(stderr, "\n---------------------------------------------------------------------------------\n"
+	                "FreedroidRPG %s encountered a problem in function: %s\n", freedroid_version, fn);
 	vfprintf(stderr, fmt, args);
-
-	if (InformDevelopers) {
-		fprintf(stderr, "\
-\n\
-\n\
-If you encounter this message, please inform the FreedroidRPG developers about it\n\
-You can \n\
-\n\
-  send an e-mail to                    freedroid-discussion AT lists.sourceforge.net\n\
-  mention it on our IRC channel        #freedroid on irc.freenode.net\n\
-  post on our forum at                 https://sourceforge.net/apps/phpbb/freedroid\n\
-  or report the bug on our tracker at  http://bugs.freedroid.org/\n\
-\n\
-Thank you!\n\n");
-	}
-
-	if (IsFatal) {
-		fprintf(stderr, "FreedroidRPG will terminate now to draw attention to the problems it could\n\
-not resolve. We are sorry if that interrupts a major game of yours.\n");
-	} else {
-		fprintf(stderr, "The problem mentioned above is not fatal, we continue the execution.\n");
-		fprintf(stderr, "---------------------------------------------------------------------------------\n");
-	}
+	fprintf(stderr, "\n");
 
 	va_end(args);
 
-	if (IsFatal)
-		Terminate(EXIT_FAILURE, TRUE);
+	if (error_type & PLEASE_INFORM) {
+		fprintf(stderr, "\n\n"
+		                "If you encounter this message, please inform the FreedroidRPG developers about it!\n"
+		                "You can\n"
+		                "  send an e-mail to                    freedroid-discussion AT lists.sourceforge.net\n"""
+		                "  mention it on our IRC channel        #freedroid on irc.freenode.net\n"
+		                "  or report the bug on our tracker at  http://bugs.freedroid.org/\n"
+		                "\n"
+		                "Thank you!\n");
+	}
 
-};				// void ErrorMessage ( ... )
+	if (error_type & IS_FATAL) {
+		fprintf(stderr, "\nFreedroidRPG will terminate now to draw attention to the problems it could\n"
+		                "not resolve. We are sorry if that interrupts a major game of yours.\n"
+		                "---------------------------------------------------------------------------------\n");
+	} else {
+		fprintf(stderr, "---------------------------------------------------------------------------------\n");
+	}
+
+	if ((error_type & IS_FATAL) && !(error_type & NO_TERMINATE))
+		Terminate(EXIT_FAILURE);
+}
+
+/**
+ * Wrapper on error_message() that avoids to redisplay several times the same error.
+ * The first parameter defines if an error should never be redisplayed at
+ * all (ONCE_PER_RUN), or if it has to be redisplayed once if a new savegame was
+ * loaded (ONCE_PER_GAME).
+ */
+void error_once_message(int when, const char *fn, const char *fmt, int error_type, ...)
+{
+	if (error_type & SILENT)
+		return;
+
+	// Compute the error message
+
+	va_list args;
+	struct auto_string *buffer = alloc_autostr(256);
+	va_start(args, error_type);
+	autostr_vappend(buffer, fmt, args);
+	va_end(args);
+
+	// If the message is already in the store, immediately return
+
+	if (!_first_use_of_msg(when, buffer)) {
+		free_autostr(buffer);
+		return;
+	}
+
+	// This error was not yet displayed.
+
+	error_message(fn, "%s", error_type, buffer->value);
+}
+
+/**
+ * In some cases it will be necessary to inform the user of something in
+ * a big important style.  Then a popup window is suitable, with a mouse
+ * click to confirm and make it go away again.
+ */
+void alert_window(const char *text, ...)
+{
+	if (!SDL_WasInit(SDL_INIT_VIDEO))
+		return;
+
+	va_list args;
+	struct auto_string *buffer = alloc_autostr(256);
+	va_start(args, text);
+	autostr_vappend(buffer, text, args);
+	va_end(args);
+
+	Activate_Conservative_Frame_Computation();
+	StoreMenuBackground(1);
+
+	int w = 440;   // arbitrary
+	int h = 60;    // arbitrary
+	int x = (GameConfig.screen_width  - w) / 2;	// center of screen
+	int y = (GameConfig.screen_height - h) / 5 * 2; // 2/5 of screen from top
+
+	SDL_Event e;
+	while (1) {
+		SDL_Delay(1);
+		RestoreMenuBackground(1);
+
+		int r_height = show_backgrounded_text_rectangle(buffer->value, FPS_Display_Font, x, y, w, h);
+		show_backgrounded_text_rectangle(_("Click to continue..."), Red_Font, x, y + r_height, w, 10);
+
+		blit_mouse_cursor();
+		our_SDL_flip_wrapper();
+		save_mouse_state();
+
+		SDL_WaitEvent(&e);
+
+		switch (e.type) {
+		case SDL_QUIT:
+			Terminate(EXIT_SUCCESS);
+			break;
+		case SDL_KEYDOWN:
+			if (e.key.keysym.sym == SDLK_SPACE || e.key.keysym.sym == SDLK_RETURN || e.key.keysym.sym == SDLK_ESCAPE)
+				goto wait_click_and_out;
+			break;
+		case SDL_MOUSEBUTTONDOWN:
+			if (e.button.button == 1)
+				goto wait_click_and_out;
+			break;
+		}
+	}
+
+wait_click_and_out:
+	while (SpacePressed() || EnterPressed() || EscapePressed() || MouseLeftPressed());
+
+	our_SDL_flip_wrapper();
+	RestoreMenuBackground(1);
+
+	free_autostr(buffer);
+}
+
+/**
+ * Wrapper on alert_window() that avoids to redisplay several times the same alert.
+ * The first parameter defines if an alert should never be redisplayed at
+ * all (ONCE_PER_RUN), or if it has to be redisplayed once if a new savegame was
+ * loaded (ONCE_PER_GAME).
+ */
+void alert_once_window(int when, const char *text, ...)
+{
+	if (!SDL_WasInit(SDL_INIT_VIDEO))
+		return;
+
+	// Compute the alert message
+
+	va_list args;
+	struct auto_string *buffer = alloc_autostr(256);
+	va_start(args, text);
+	autostr_vappend(buffer, text, args);
+	va_end(args);
+
+	// If the message is already in the store, immediately return
+
+	if (!_first_use_of_msg(when, buffer)) {
+		free_autostr(buffer);
+		return;
+	}
+
+	// This alert was not yet displayed.
+
+	alert_window("%s", buffer->value);
+}
 
 /**
  * This function does something similar to memmem.  Indeed, it would be
@@ -180,33 +417,28 @@ char *ReadAndMallocStringFromDataOptional(char *SearchString, const char *StartI
 {
 	char *SearchPointer;
 	char *EndOfStringPointer;
-	char *ReturnString = "";
+	char *ReturnString = NULL;
 	int StringLength;
 
 	SearchPointer = strstr(SearchString, StartIndicationString);
 
 	if (!SearchPointer)
 		return 0;
-	else {
-		// Now we move to the beginning
-		SearchPointer += strlen(StartIndicationString);
 
-		// Now we move to the end with the end pointer
-		EndOfStringPointer = strstr(SearchPointer, EndIndicationString);
-		if (!EndOfStringPointer)
-			return 0;
+	// Now we move to the beginning
+	SearchPointer += strlen(StartIndicationString);
+
+	// Now we move to the end with the end pointer
+	EndOfStringPointer = strstr(SearchPointer, EndIndicationString);
+	if (!EndOfStringPointer)
+		return 0;
+
+	// Now we allocate memory and copy the string (even empty string)...
+	StringLength = EndOfStringPointer - SearchPointer;
+	ReturnString = MyMalloc(StringLength + 1);
+	strncpy(ReturnString, SearchPointer, StringLength);
+	ReturnString[StringLength] = 0;
 
-		// Now we allocate memory and copy the string...
-		// delete_one_dialog_option() doesn't free empty strings so don't
-		// malloc those.
-		if ((StringLength = (EndOfStringPointer - SearchPointer))) {
-			ReturnString = MyMalloc(StringLength + 1);
-			strncpy(ReturnString, SearchPointer, StringLength);
-			ReturnString[StringLength] = 0;
-		} else {
-			ReturnString = "";
-		}
-	}
 	return ReturnString;
 }
 
@@ -222,16 +454,16 @@ char *ReadAndMallocStringFromData(char *SearchString, const char *StartIndicatio
 	char *result = ReadAndMallocStringFromDataOptional(SearchString, StartIndicationString, EndIndicationString);
 	if (!result) {
 		fprintf(stderr, "\n\nStartIndicationString: '%s'\nEndIndicationString: '%s'\n", StartIndicationString, EndIndicationString);
-		ErrorMessage(__FUNCTION__, "\
+		error_message(__FUNCTION__, "\
 The string that is supposed to prefix an entry in a text data file\n\
 of FreedroidRPG was not found within this text data file.\n\
-This indicates some corruption in the data file in question.", PLEASE_INFORM, IS_FATAL);
+This indicates some corruption in the data file in question.", PLEASE_INFORM | IS_FATAL);
 	}
 	return result;
 };				// char* ReadAndMallocStringFromData ( ... )
 
 /**
- * This function counts the number of occurences of a string in a given
+ * This function counts the number of occurrences of a string in a given
  * other string.
  */
 int CountStringOccurences(char *SearchString, const char *TargetString)
@@ -268,10 +500,10 @@ char *ReadAndMallocAndTerminateFile(const char *filename, const char *File_End_S
 	if ((DataFile = fopen(filename, "rb")) == NULL) {
 		fprintf(stderr, "\n\nfilename: '%s'\n", filename);
 
-		ErrorMessage(__FUNCTION__, "\
+		error_message(__FUNCTION__, "\
 FreedroidRPG was unable to open a given text file,\n\
 that should be there and should be accessible.\n\
-This indicates a serious bug in this installation of FreedroidRPG.", PLEASE_INFORM, IS_FATAL);
+This indicates a serious bug in this installation of FreedroidRPG.", PLEASE_INFORM | IS_FATAL);
 	} else {
 		DebugPrintf(1, "\nchar* ReadAndMallocAndTerminateFile ( char* filename ) : Opening file succeeded...");
 	}
@@ -281,22 +513,22 @@ This indicates a serious bug in this installation of FreedroidRPG.", PLEASE_INFO
 	Data = (char *)MyMalloc(MemoryAmount);
 
 	if (fread(Data, 1, MemoryAmount, DataFile) < filelen && ferror(DataFile)) {
-		ErrorMessage(__FUNCTION__, "\
+		error_message(__FUNCTION__, "\
 		FreedroidRPG was unable to read a given text file, that should be there and\n\
 		should be accessible.\n\
-		Filename: %s", PLEASE_INFORM, IS_FATAL, filename);
+		Filename: %s", PLEASE_INFORM | IS_FATAL, filename);
 	} else {
 		DebugPrintf(1, "\n%s(): Reading file succeeded...", __FUNCTION__);
 	}
 
 	if (fclose(DataFile) == EOF) {
 		fprintf(stderr, "\n\nfilename: '%s'\n", filename);
-		ErrorMessage(__FUNCTION__, "\
+		error_message(__FUNCTION__, "\
 		FreedroidRPG was unable to close a given text file, that should be there and\n\
 		should be accessible.\n\
 		This indicates a strange bug in this installation of Freedroid, that is\n\
 		very likely a problem with the file/directory permissions of the files\n\
-		belonging to FreedroidRPG.", PLEASE_INFORM, IS_FATAL);
+		belonging to FreedroidRPG.", PLEASE_INFORM | IS_FATAL);
 	} else {
 		DebugPrintf(1, "\n%s(): file closed successfully...\n", __FUNCTION__);
 	}
@@ -304,15 +536,12 @@ This indicates a serious bug in this installation of FreedroidRPG.", PLEASE_INFO
 	// NOTE: Since we do not assume to always have pure text files here, we switched to
 	// MyMemmem, so that we can handle 0 entries in the middle of the file content as well
 	if (File_End_String) {
-		if ((ReadPointer =
-		     MyMemmem(Data, (size_t) MemoryAmount, (char *)File_End_String, (size_t) strlen(File_End_String))) == NULL) {
-			fprintf(stderr, "\n\nfilename: '%s'\n", filename);
-			fprintf(stderr, "File_End_String: '%s'\n", File_End_String);
-
-			ErrorMessage(__FUNCTION__, "\
-		    FreedroidRPG was unable to find the string, that should indicate the end of\n\
-		    the given text file within this file.\n\
-		    This indicates a corrupt or outdated data or saved game file.", PLEASE_INFORM, IS_FATAL);
+		if ((ReadPointer = MyMemmem(Data, (size_t) MemoryAmount, (char *)File_End_String, (size_t) strlen(File_End_String))) == NULL) {
+			error_message(__FUNCTION__, "FreedroidRPG was unable to find the string, that should indicate the end of\n"
+			                            "the given text file within this file.\n"
+			                            "This indicates a corrupt or outdated data or saved game file."
+			                            "  filename: '%s' - File_End_String: '%s'",
+			              PLEASE_INFORM | IS_FATAL, filename, File_End_String);
 		} else {
 			ReadPointer[0] = 0;
 		}
@@ -336,9 +565,9 @@ char *LocateStringInData(char *SearchBeginPointer, const char *SearchTextPointer
 
 	if ((temp = strstr(SearchBeginPointer, SearchTextPointer)) == NULL) {
 		fprintf(stderr, "\n\nSearchTextPointer: '%s'\n", SearchTextPointer);
-		ErrorMessage(__FUNCTION__, "\
+		error_message(__FUNCTION__, "\
 The string that was supposed to be in the text data file could not be found.\n\
-This indicates a corrupted or seriously outdated game data or saved game file.", PLEASE_INFORM, IS_FATAL);
+This indicates a corrupted or seriously outdated game data or saved game file.", PLEASE_INFORM | IS_FATAL);
 	}
 
 	return (temp);
@@ -373,11 +602,10 @@ ReadValueFromStringWithDefault(char *SearchBeginPointer, const char *ValuePrecee
 	// Now we try to read in the value!!!
 	//
 	if (sscanf(SourceLocation, FormatString, TargetValue) == EOF) {
-		fprintf(stderr, "\n\nFormatString: '%s'\n", FormatString);
-		fprintf(stderr, "ValuePreceedText: '%s'\n", ValuePreceedText);
-		ErrorMessage(__FUNCTION__, "\
-sscanf using a certain format string failed!\n\
-This indicates a corrupted or seriously outdated game data or saved game file.", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "sscanf using a certain format string failed!\n"
+		                            "This indicates a corrupted or seriously outdated game data or saved game file.\n"
+		                            "  FormatString: '%s' - ValuePreceedText: '%s'",
+		              PLEASE_INFORM | IS_FATAL, FormatString, ValuePreceedText);
 	}
 	// Now that we are done, we restore the given SearchArea to former glory
 	if (EndOfSearchSectionPointer) {
@@ -409,11 +637,10 @@ ReadValueFromString(char *SearchBeginPointer, const char *ValuePreceedText, cons
 	// Now we try to read in the value!!!
 	//
 	if (sscanf(SourceLocation, FormatString, TargetValue) == EOF) {
-		fprintf(stderr, "\n\nFormatString: '%s'\n", FormatString);
-		fprintf(stderr, "ValuePreceedText: '%s'\n", ValuePreceedText);
-		ErrorMessage(__FUNCTION__, "\
-sscanf using a certain format string failed!\n\
-This indicates a corrupted or seriously outdated game data or saved game file.", PLEASE_INFORM, IS_FATAL);
+		error_message(__FUNCTION__, "sscanf using a certain format string failed!\n"
+		                            "This indicates a corrupted or seriously outdated game data or saved game file.\n"
+		                            "  FormatString: '%s' - ValuePreceedText: '%s'",
+		              PLEASE_INFORM | IS_FATAL, FormatString, ValuePreceedText);
 	}
 
 	if (EndOfSearchSectionPointer) {
@@ -424,56 +651,103 @@ This indicates a corrupted or seriously outdated game data or saved game file.",
 };				// void ReadValueFromString( ... )
 
 /** 
- * This function is for reading in a range of positive values from a string with a default.
- * 
- * If there is no string, the default value is assigned and it returns 1, with no warning.
+ * \deprecated Use get_range_from_string.
+ * \brief Read a range of integer values from a string with a default.
  *
- * If there is a string with garbled input (negative minimum value, or minimum>maximum), it warns,
- * and assigns the default, and a 2 is returned.
+ * Read the range between two indications. The function don't accept negative value.
  *
- * A valid string, with no errors means it returns a 0.
-*/
-
+ * \param SearchString The global string to search the range.
+ * \param StartIndicationString The start indication.
+ * \param EndIndicationString The end indication.
+ * \param min,max The result of the conversion.
+ * \param default_val The value used as default.
+ * \return FALSE if the conversion failed and set min and max to default_val, else return TRUE.
+ */
 int ReadRangeFromString(char *SearchString, const char *StartIndicationString, const char *EndIndicationString, int *min, int *max, int default_val)
 {
-	char *ptr;
-	int return_value = 0;
+	int result;
 
-	// First, see if we have a valid search string
+	// First, we have to get the range string.
 	char *search_ptr = ReadAndMallocStringFromDataOptional(SearchString, StartIndicationString, EndIndicationString);
-	if (!search_ptr) {
-		//Search string was not valid, set everything to default value.
-		*min = *max = default_val;
-		return 1;
-	}
 
-	*min = strtol(search_ptr, &ptr, 10);
-	if (*ptr == '-') {
-		ptr++;
-		*max = strtol(ptr, NULL, 10);
-	} else {
-		*max = *min;
-	}
+	result = get_range_from_string(search_ptr, min, max, default_val);
 
-	//Handle corrupted values
+	//Handle corrupted values.
 	if (*min < 0) {
-		ErrorMessage(__FUNCTION__, "\
+		error_message(__FUNCTION__, "\
 The value read in as a minimum (%d) is a negative number.\n\
 This most likely means corrupted data.\n\
-Setting both the maximum and minimum to the default value (%d).", NO_NEED_TO_INFORM, IS_WARNING_ONLY, *min, default_val);
+Setting both the maximum and minimum to the default value (%d).", NO_REPORT, *min, default_val);
 		*min = *max = default_val;
-		return_value = 2;
-	} else if (*max < *min) {
-		ErrorMessage(__FUNCTION__, "\
-The value read in as a maximum (%d) is less than the minimum (%d).\n\
-This most likely means corrupted data.\n\
-Setting both the maximum and minimum to the default value (%d).", NO_NEED_TO_INFORM, IS_WARNING_ONLY, *max, *min, default_val);
-		*min = *max = default_val;
-		return_value = 2;
+		return FALSE;
 	}
 
 	free(search_ptr);
-	return return_value;
+
+	return result;
+}
+
+/**
+ * \brief Read a range of integer values from a string with a default.
+ * \param str The constant string to convert.
+ * \param min,max The result of the conversion.
+ * \param default_value The value used in case of error or NULL.
+ * \return FALSE if the conversion failed and set min and max to default_val, else return TRUE.
+ */
+int get_range_from_string(const char *str, int *min, int *max, int default_value)
+{
+	char *ptr;
+
+	if (!str) {
+		// String is NULL, set everything to default value.
+		*min = *max = default_value;
+		return TRUE;
+	}
+
+	*min = strtol(str, &ptr, 10);
+
+	if (str == ptr) {
+		goto read_error;
+	}
+
+	if (*ptr == '\0') {
+		// Conversion succeeded and the string contains only one value.
+		*max = *min;
+		return TRUE;
+	}
+
+	if (*ptr != ':') {
+		goto read_error;
+	}
+
+	// The string contains potentially another value.
+	str = ++ptr;
+	*max = strtol(str, &ptr, 10);
+
+	if (str == ptr) {
+		goto read_error;
+	}
+	// Conversion of max value succeeded
+
+	if (*max >= *min) {
+		return TRUE; // Values are in the good order.
+	}
+
+	error_message(__FUNCTION__, "The value read in as a maximum (%d) is less than the minimum (%d).\n"
+	                           "This most likely means corrupted data.\n"
+	                           "Setting both the maximum and minimum to the default value (%d).",
+	                           NO_REPORT, *max, *min, default_value);
+	*min = *max = default_value;
+	return FALSE;
+
+read_error:
+	// Failed to read the string.
+	error_message(__FUNCTION__, "The string (\"%s\") cannot be legaly converted to a range of integers.\n"
+	                           "This most likely means corrupted data.\n"
+	                           "Setting both the maximum and minimum to the default value (%d).",
+	                           NO_REPORT, str, default_value);
+	*min = *max = default_value;
+	return FALSE;
 }
 
 /**
@@ -537,7 +811,7 @@ int inflate_stream(FILE * DataFile, unsigned char **DataBuffer, int *size)
 	int cursz = 1048576;	//start with 1MB
 	unsigned char *temp_dbuffer = malloc(cursz);
 	if (fread(src, filelen, 1, DataFile) != 1) {
-		ErrorMessage(__FUNCTION__, "Error reading compressed data stream.", PLEASE_INFORM, IS_WARNING_ONLY);
+		error_message(__FUNCTION__, "Error reading compressed data stream.", PLEASE_INFORM);
 		free(temp_dbuffer);
 		free(src);
 		return -1;
@@ -546,7 +820,7 @@ int inflate_stream(FILE * DataFile, unsigned char **DataBuffer, int *size)
 	int ret;
 	z_stream strm;
 
-	/* copy paste of public domain tool "zpipe" of which a copy is in fdRPG source tree */
+	/* copy paste of public domain tool "zpipe" */
 	/* big thanks to zlib authors */
 	/* allocate inflate state */
 	strm.zalloc = Z_NULL;
@@ -559,9 +833,9 @@ int inflate_stream(FILE * DataFile, unsigned char **DataBuffer, int *size)
 
 	ret = inflateInit2(&strm, 47);
 	if (ret != Z_OK) {
-		ErrorMessage(__FUNCTION__, "\
+		error_message(__FUNCTION__, "\
 		zlib was unable to start decompressing a stream.\n\
-		This indicates a serious bug in this installation of FreedroidRPG.", PLEASE_INFORM, IS_WARNING_ONLY);
+		This indicates a serious bug in this installation of FreedroidRPG.", PLEASE_INFORM);
 		free(temp_dbuffer);
 		free(src);
 		return -1;
@@ -583,9 +857,9 @@ int inflate_stream(FILE * DataFile, unsigned char **DataBuffer, int *size)
 		case Z_MEM_ERROR:
 			(void)inflateEnd(&strm);
 
-			ErrorMessage(__FUNCTION__, "\
+			error_message(__FUNCTION__, "\
 			zlib was unable to decompress a stream\n\
-			This indicates a serious bug in this installation of FreedroidRPG.", PLEASE_INFORM, IS_WARNING_ONLY);
+			This indicates a serious bug in this installation of FreedroidRPG.", PLEASE_INFORM);
 			free(temp_dbuffer);
 			free(src);
 			return -1;
@@ -624,8 +898,8 @@ int deflate_to_stream(unsigned char *source_buffer, int size, FILE *dest)
 
     ret = deflateInit2(&strm, Z_DEFAULT_COMPRESSION, Z_DEFLATED, 31, 8, Z_DEFAULT_STRATEGY);
 	if (ret != Z_OK) {
-		ErrorMessage(__FUNCTION__, "\
-		zlib was unable to start compressing a string.", PLEASE_INFORM, IS_WARNING_ONLY);
+		error_message(__FUNCTION__, "\
+		zlib was unable to start compressing a string.", PLEASE_INFORM);
 		return -1;
 	}
 
@@ -634,7 +908,10 @@ int deflate_to_stream(unsigned char *source_buffer, int size, FILE *dest)
 	do {
 		strm.avail_out = CHUNK;
 		strm.next_out = out;
-		ret = deflate(&strm, Z_FINISH);    /* no bad return value */
+		ret = deflate(&strm, Z_FINISH);
+		if (ret == Z_STREAM_ERROR) {
+			error_message(__FUNCTION__, "Stream error while deflating a buffer", IS_FATAL | PLEASE_INFORM);
+		}
 		have = CHUNK - strm.avail_out;
 		if (fwrite(out, 1, have, dest) != have || ferror(dest)) {
 			(void)deflateEnd(&strm);
