@@ -33,6 +33,8 @@
 
 tux_t Me;
 
+// We want those names to be in the translation template file, so
+// we mark them with the 'neutral i18n macro'.
 char *AllSkillTexts[NUMBER_OF_SKILL_LEVELS] = {
 	N_("untrained"),
 	N_("novice"),
@@ -41,9 +43,9 @@ char *AllSkillTexts[NUMBER_OF_SKILL_LEVELS] = {
 	N_("expert"),
 	N_("master"),
 	N_("god"),
-	("7"),
-	("8"),
-	("9")
+	N_("haxx0r"),
+	N_("l33t haxx0r"),
+	N_("cheater")
 };
 
 //--------------------
@@ -65,21 +67,13 @@ float MeleeRechargeMultiplierTable[] = { 1.0, 0.9, 0.81, 0.73, 0.66, 0.59, 0.53,
 float RangedDamageMultiplierTable[] = { 1.0, 1.1, 1.21, 1.33, 1.46, 1.61, 1.77, 1.95, 2.14, 2.36 };
 float RangedRechargeMultiplierTable[] = { 1.0, 0.9, 0.81, 0.73, 0.66, 0.59, 0.53, 0.48, 0.43, 0.39 };
 
-spell_skill_spec *SpellSkillMap;
 int number_of_skills;
 
-druidspec *Druidmap;
-
-bulletspec *Bulletmap;
-
-blastspec Blastmap[ALLBLASTTYPES];
-
-char font_switchto_red[2] = { 1, 0 };
-char font_switchto_blue[2] = { 2, 0 };
-char font_switchto_neon[2] = { 3, 0 };
-char font_switchto_msgstat[2] = { 4, 0 };
-char font_switchto_msgvar[2] = { 5, 0 };
+char font_switchto_red[] = { 1, 0, 'r' };
+char font_switchto_blue[] = { 2, 0, 'w' };
+char font_switchto_neon[] = { 3, 0, 'n' };
+char font_switchto_msgstat[] = { 4, 0, 's' };
+char font_switchto_msgvar[] = { 5, 0, 'v' };
 
 int skip_initial_menus = FALSE;
 
-enum { INSIDE_MENU = 0, INSIDE_GAME, INSIDE_LVLEDITOR } game_status;
